import os, re, string
from ConfigurationManager.Warning import Warning
from ConfigurationManager.BuildExecutable import BuildExecutable

class Doxygen:
	def __init__(self, directory=".",doxyfile="Doxyfile",**extras):
		self.inputDir = directory
		self.inputFile = doxyfile
		self.executable = "doxygen"
		self.config = None;
		self.outputFile = "doxygen_out.txt"
		self.errFile = "doxygen_err.txt"

	def Execute(self,debug=None):
		line = self.executable + " " + self.inputFile
		executable = BuildExecutable(self.inputDir,line)
		print line,"...",
		executable.Execute(self.outputFile,self.errFile,debug=debug)
		outputFile = os.path.join( self.inputDir, self.outputFile )
		if debug: print 'Doxygen:Execute ouputFile', outputFile,'pwd',os.getcwd()
		try:
			size = os.path.getsize( outputFile )
		except:
			size = 0
		if debug: print size
		print "Done"
		if not size:
			raise IOError('Could not execute doxygen.')

	def GetWarnings(self):
		return self.GetWarningsFromFile()

	def GetOutputFiles(self):
		return ((self.inputDir, self.outputFile))

	def SetInput(self, directory, file="Doxyfile"):
		self.inputDir = directory
		self.inputFile = file

	def SetOutput(self, output):
		self.outputFile = output

	def SetConfiguration(self, filename):
		self.config = filename

	def GenerateTemplate(self, newFileName):
		line = self.executable + " -g " + newFileName
		system( line )

	def GetWarningFile(self):
		return self.inputDir + '/' + self.errFile

	def GetWarningsFromFile(self,inputDir=None,errFile=None):
		if not inputDir: inputDir = self.inputDir
		if not errFile: errFile = self.errFile
		try:
			ERR = open( "%s/%s" % (inputDir, errFile) )
		except:
			return None
		w = Warning()
		warnings = []
		reWarn = re.compile('(Warning): ')
		reError = re.compile('(Error): ')
		reLineWarn = re.compile(':(\d+): Warning: ')
		reLineError = re.compile(':(\d+): Error: ')
		cumulative = None
		line = ERR.readline()
		while line:
			warnMatch = reWarn.search(line,0)
			if not warnMatch:
				warnMatch = reError.search(line,0)
			if warnMatch:
				# If there was a previous warning, append it to list.
				if w.message: warnings.append( w )
				w = Warning()
				w.type = warnMatch.groups()[0]
				w.message = line[warnMatch.end():]

				lineWarnMatch = reLineWarn.search(line,0)
				if not lineWarnMatch: lineWarnMatch = reLineError.search(line,0)
				if lineWarnMatch:
					w.line = string.atoi(lineWarnMatch.groups()[0])
					w.file = line[:lineWarnMatch.start()]
			else:
				if w.message: w.message = w.message + " " + string.strip(line)
			line = ERR.readline()
		if w.message: warnings.append(w)
		return warnings

	def __str__(self):
		return "Doxygen object"

