import sys, os
from traceback import print_exc
from copy import copy
from ConfigurationManager.NameToFilename import NameToFilename

class Target:
	"""This class is responsible for compiling a target of source code."""

	def __init__(self, name):
		self.name = name
		self.tools = {}

	def SetTool(self, targetPlatform, toolDict ):
		self.tools[targetPlatform] = toolDict

	def InstantiateTool(self, targetPlatform,debug=None):
		tool = self.GetTool(targetPlatform)
		if debug: print 'Target:InstantiateTool',targetPlatform,tool
		if not tool: return None
		toolName = tool['tool']
		if debug: print 'Target:InstantiateTool',toolName, os.getcwd()
		dict = eval('self._CombineWithDefaults(%s)' % tool)
		str='''from ConfigurationManager.%s import %s\ncompiler=%s(**dict)\n''' % (toolName,toolName,toolName)
		exec(str)
		return compiler

	def GetTool(self, targetPlatform):
		tools = self.tools
		if tools.has_key(targetPlatform):
			return tools[targetPlatform]
		else:
			return None
	
	def SetSystem(self, makeSystem):
		"""The system is the makefile object for the current platform."""
		self.makeSystem = makeSystem

	def Construct(self):
		self.makeSystem.Execute('.', self.asName,self.outfile)

	def _CombineWithDefaults(self,specificToolDict):
		if self.tools.has_key('default'):
			combined = copy(self.tools['default'])
			combined.update(specificToolDict)
		else:
			combined = specificToolDict
		return combined

	def __getstate__(self):
		"""Keep the platform-dependent object out of pickles of this object."""
		odict = self.__dict__
		del odict['makeSystem']
		return odict
	
	def __str__(self):
		return self.name

class Library(Target):
	def __init__(self,name):
		Target.__init__(self,name)
		self.errFile = None

	def Construct(self,targetPlatform,debug=None):
		self.errFile = None
		compiler = self.InstantiateTool(targetPlatform)
		try:
			compiler.Execute(rebuild=1)
		except StandardError, e:
			if debug: print e
			self.errFile = NameToFilename(self.name)
			self.errFile = self.errFile + "_err.txt"
			f = open(self.errFile,'w')
			#print >>f, repr(e), e.args
			print_exc(None, self.errFile)
			#print >>f, sys.exc_info()
			f.close()
		self.warnings = compiler.GetWarnings()
		self.outputFile = compiler.GetOutputFile()

	def GetStats(self):
		if self.errFile:
			return [('ran',self.errFile, 'no')]
		if not self.warnings:
			stats = [('warnings',self.outputFile,0)]
			stats.append( ('errors',self.outputFile,0) )
			return stats
		import re
		warnCount = 0
		for warning in self.warnings:
			if re.match(warning.type,"warning"): warnCount = warnCount + 1
		stats = [('warnings',self.outputFile,warnCount)]
		stats.append( ('errors',self.outputFile,len(self.warnings)-warnCount) )
		return stats

class Documentation(Target):
	def __init__(self,name):
		Target.__init__(self,name)
		self.errFile = None
		
	def Construct(self,targetPlatform=None,debug=None):
		doxygen = self.InstantiateTool(targetPlatform,debug)
		try:
			doxygen.Execute(debug)
		except StandardError, e:
			if debug: print e
			self.errFile = NameToFilename(self.name)
			self.errFile = self.errFile + "_err.txt"
			f = open(self.errFile,'w')
			print_exc(None, f)
			#print >>f, repr(e), e.args
			#print >>f, sys.exc_info()
			f.close()
		self.warnings = doxygen.GetWarnings()
		self.warningFile = doxygen.GetWarningFile()

	def GetStats(self):
		if self.errFile:
			return [('ran',self.errFile,'no')]
		import re
		warnCount = 0
		for warning in self.warnings:
			if re.match(warning.type,"Warning"): warnCount = warnCount + 1
		stats = [('warnings',self.warningFile,warnCount)]
		stats.append( ('errors',self.warningFile,len(self.warnings)-warnCount) )
		return stats

class ExecutableTest(Target):
	def __init__(self,name,directory, execString, compareFile=None):
		Target.__init__(self,name)
		self.directory = directory
		self.execString = execString
		self.compareFile = compareFile

	def Construct(self,targetPlatform=None,debug=None):
		import os
		if debug: print "Name",self.name,"dir",self.directory
		self.errMessage = None
		if self.compareFile:
			self.outputFile = NameToFilename(self.name)+"_comp.txt"
		else: self.outputFile = None
		executer = self.InstantiateTool(targetPlatform,debug)

		try:
			executer.Execute(self.outputFile,debug=debug)
		except StandardError, e:
			if debug: print e
			self.errMessage = repr(e)
			for piece in e.args:
				self.errMessage = self.errMessage+repr(piece)
		curDir = os.getcwd()
		os.chdir(self.directory)
		if not self.errMessage and self.compareFile:
			from filecmp import cmp
			self.equal = cmp(self.outputFile, self.compareFile)
			self.cmpFile = NameToFilename(self.name)
			self.cmpFile = self.cmpFile + "_cmp.html"
			if debug: print self.cmpFile
			f = open(self.cmpFile,"w")
			print >>f, """<FRAMESET cols=50\%,50\%>"""
			print >>f, """<FRAME src="%s"><FRAME src="%s"></FRAMESET>""" % (self.outputFile,self.compareFile)
			f.close()
		if self.errMessage:
			self.errFile = NameToFilename(self.name)
			self.errFile = self.errFile + "_err.txt"
			if debug: print self.errFile
			f = open( self.errFile, "w" )
			print >>f, self.errMessage
			f.close()
		os.chdir( curDir )

	def GetStats(self):
		"""Returns whether the file executed and whether its output matched
		what it was supposed to."""
		returnTuple = []
		if self.errMessage:
			returnTuple.append(['ran',self.directory+'/'+self.errFile,'No'])
		else:
			returnTuple.append(['ran',None,'Yes'])
		if self.errMessage: return returnTuple
		if self.compareFile:
			returnTuple.append(['verified',self.directory+'/'+self.cmpFile,'Yes'])
		else:
			returnTuple.append(['verified',self.directory+'/'+self.cmpFile,'No'])
		return returnTuple

