import copy, sys, re, os.path

# Add Python root to path so we can load ConfigurationManger.*
sys.path.append(os.path.abspath('..'))

from Target import Library, Documentation, ExecutableTest
from Project import Project


if len(sys.argv)>1 and re.match('dir=',sys.argv[1]):
	homedir = split(sys.argv[1],'=')[1]
else:
	homedir = '../../'

# Create our overall container.
MD = Project("MD", homedir)

win32 = 'win32'
linux = 'linux2'

# Make a target to add to that container.
######## DMSystem docs
docs = Documentation("DMSystem Documentation")
docDict = { 'tool' : 'Doxygen',
			'directory' : 'src/DMSystem/doc',
			'doxyfile' : 'Doxyfile' }
docs.SetTool(  win32, docDict )
docs.SetTool(  linux, docDict )
MD.AddTarget( docs )

######## MD docs
docs = Documentation("MD Documentation")
docDict = { 'tool' : 'Doxygen',
			'directory' : 'src/MD/doc',
			'doxyfile' : 'Doxyfile' }
docs.SetTool( win32, docDict )
#MD.AddTarget( docs )

######## DMSystem Release
DMSystem = Library("DMSystem Release")
dmDefDict = { 'directory' : "src/DMSystem" }
DMSystem.SetTool( 'default' , dmDefDict )
dmVSDict = { 'tool' : 'VisualStudio',
			'project' : "DMSystem.dsp",
			'target' : "DMSystem - Win32 Release" }
DMSystem.SetTool( win32, dmVSDict )
#MD.AddTarget( DMSystem )

######### DMSystem Debug
DMSystem = Library("DMSystem Debug")
dmDefDict = { 'directory' : "src/DMSystem" }
DMSystem.SetTool( 'default' , dmDefDict )
dmVSDict = { 'tool' : 'VisualStudio',
			'project' : "DMSystem.dsp",
			'target' : "DMSystem - Win32 Debug" }
DMSystem.SetTool( win32, dmVSDict )
#MD.AddTarget( DMSystem )


def AddNDSet(project,name,directory,makefile,target):
	for setting in ('Debug 3D','Release 3D','Debug 2D','Release 2D'):
		MDLibrary = Library(name+" "+setting)
		mdDefDict = { 'directory' : directory }
		MDLibrary.SetTool( 'default', mdDefDict )
		mdlDict = { 'tool' : 'VisualStudio',
					'project' : makefile,
		            'target' : "%s - Win32 %s" % (target,setting) }
		MDLibrary.SetTool( win32, mdlDict )
		project.AddTarget( MDLibrary )

#AddNDSet(MD,"MDLibrary","src/MD/MDFiles","MDLibrary.dsp","MDLibrary")
#AddNDSet(MD,"MDSwig","src/MD/SWIGBuild","MDswig.dsp","MDswig")
#AddNDSet(MD,"MDConsole","src/MD/Console","MDConsole.dsp","MDConsole")

########## Testing the python interface
SimpleTest = ExecutableTest("SimpleTest","Python/MD","python simple.py norasmol",
			"simple_out.txt")
dict = { 'tool' : 'WinExecutable',
	'directory' : "Python/MD",
	'command' : "python simple.py norasmol",
	'comparison' : "simple_out.txt" }
SimpleTest.SetTool( win32, dict )
dict2 = copy.deepcopy(dict)
dict2['tool'] = 'UnixExecutable'
SimpleTest.SetTool( 'linux2', dict2 )
MD.AddTarget( SimpleTest )


########## Hysteresis Documentation
HystDoc = Documentation("Hysteresis Docs")
HystDocDict = { 'tool' : 'Doxygen',
				'directory' : 'src/Hysteresis/doc',
				'doxyfile' : 'doxyfile' }
HystDoc.SetTool( win32, HystDocDict )
#MD.AddTarget( HystDoc )


########## Run it all
MD.Construct()

