
##     Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
##     Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

##     This library is free software; you can redistribute it and/or modify it
##     under the terms of the GNU Lesser General Public License as published by the
##     Free Software Foundation; either version 2.1 of the License, or (at your
##     option) any later version.

##     This library is distributed in the hope that it will be useful, but WITHOUT
##     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
##     FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
##     for more details.

##     You should have received a copy of the GNU Lesser General Public License
##     along with this library; if not, write to the Free Software Foundation,
##     Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
##     http://www.gnu.org/licenses/lgpl.html

##     Author Contact Information:
##     Jim Sethna      
##     sethna@lassp.cornell.edu  
##     117 Clark Hall
##     Cornell University
##     Ithaca, NY 14850


# This is a simple application.

import sys
# Unix
sys.path.append('../../src/MD/SWIGBuild/i686/obj3D')
# Windows
sys.path.append('../../src/MD/SWIGBuild/')

from MD3D import *

# this is a simple script for a few Verlet steps with three atoms.

# Three atoms
atoms = DynamicListOfAtoms(3)

# Set positions and velocities. (Both are zero at creation time)
atoms.SetCartesianPosition(0,0,0.8)
atoms.SetCartesianPosition(1,1,1.0)
atoms.SetCartesianPosition(2,2,1.0)
atoms.SetCartesianVelocity(0,1,0.3)
atoms.SetCartesianVelocity(2,1,-0.3)

# Lennard Jones potential, cut off smoothly at 2.7 units
potential = CutLennardJonesPotential()
cutoff = potential.GetCutoffDistance()

# neighborLocator will give atoms up to cutoff (default noNeighborLocator)
neighborLocator = SimpleNeighborLocator(cutoff)
atoms.SetNeighborLocator(neighborLocator)

# Time step given by simple Verlet algorithm
# Warning: our version of Verlet isn't standard!
mover = VerletAtomsMover(potential, 0.01, 10)

# Add graphics
if "norasmol" not in sys.argv:
	sys.path.append('Observers')
	from RasMolObserver import RasMolObserver3D
	ras = RasMolObserver3D(radius=100)
	atoms.AddObserver(ras)

# 200 time steps: 20 groups of ten, runs to 200*0.01 = 2 time units
for i in range(200):
    atoms.Notify()
    PE = potential.CalculateEnergy(atoms)
    KE = atoms.GetKineticEnergy()
    print PE + KE
    mover.Move(atoms)
    
