
##     Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
##     Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

##     This library is free software; you can redistribute it and/or modify it
##     under the terms of the GNU Lesser General Public License as published by the
##     Free Software Foundation; either version 2.1 of the License, or (at your
##     option) any later version.

##     This library is distributed in the hope that it will be useful, but WITHOUT
##     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
##     FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
##     for more details.

##     You should have received a copy of the GNU Lesser General Public License
##     along with this library; if not, write to the Free Software Foundation,
##     Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
##     http://www.gnu.org/licenses/lgpl.html

##     Author Contact Information:
##     Jim Sethna      
##     sethna@lassp.cornell.edu  
##     117 Clark Hall
##     Cornell University
##     Ithaca, NY 14850


# This is a simple parallel application. It can be run by a command like
# mpirun -localonly 2 C:\Python21\mpipython.exe simpleParallel.py

import sys
# Unix path
sys.path.append('../../src/MD/SWIGBuild/i686/obj3D')
# Windows path
sys.path.append('../../src/MD/SWIGBuild/')

from MD3D import *
from Numeric import *

# this is a simple script for a few Verlet steps with a cube of atoms.

# dmProtocol is a global variable. It is set to a MPIProtocol if we
# use mpipython, or a LocalProtocol otherwise.
nProcs = cvar.dmProtocol.GetNumberOfProcessors()
myProc = cvar.dmProtocol.GetProcessorNumber()

# Choose a cpu layout (ie how many processors in each direction)
tmp0 = int(nProcs**(1/3.))
tmp1 = int(sqrt(nProcs/tmp0))
cpulayout = array([tmp0, tmp1, nProcs/tmp0/tmp1])

if myProc == 0: print 'the cpu layout is ', cpulayout

#potential = CutLennardJonesPotential()
potential = EMTPotential()
#the decorator of the potential
parapotential = ParallelPotential(potential)
cutoff = potential.GetCutoffDistance()
lattSpacing = potential.GetLengthScale()*sqrt(2)
lattice = FCCLattice(lattSpacing)

lattice.SetOrigin([0.1,0.03,0.14])

# the number of lattice spacings
cubeSize = 10.

cube = array([1, 1, 1.])*cubeSize*lattSpacing
rect0 = RectangularClusterInitializer(cube, lattice)
rect = ParallelAtomsInitializer(rect0)

atoms = DynamicListOfAtoms()

bc = SimplePeriodicBoundaryConditions()
bc.SetLength(array([1, 1, 1.])*cubeSize*lattSpacing)
atoms.SetBoundaryConditions(bc)
bc.thisown = 0

rect.Create(atoms)

domain = HomogeneousDomainSubdivision(cpulayout,
			   cvar.dmProtocol.GetProcessorNumber(), cutoff)

synchronizer = Synchronizer(atoms.GetCartesianPositions())
migrator = Migrator(domain, atoms.GetCartesianPositions())

nl = CellNeighborList(cutoff, 0.05*cutoff)
nl.thisown = 0
paranl = ParallelNeighborLocator(migrator, nl)

atoms.SetNeighborLocator(paranl)   # this will migrate the atoms

    
print 'on processor ', cvar.dmProtocol.GetProcessorNumber(), ', we have:', \
    len(atoms.GetCartesianPositions()), ' and ', \
    len(atoms.GetCartesianPositions().GetExtended().GetBranch(1)), \
    ' ghost atoms after migration'
    
E = parapotential.CalculateEnergy(atoms)
if myProc == 0: print E

#exit
mover = VerletAtomsMover(parapotential, 0.01, 10)

for i in range(3):
    mover.Move(atoms)
    E = parapotential.CalculateEnergy(atoms)
    if myProc == 0: print E
 
nonl = NoNeighborLocator()
atoms.SetNeighborLocator(nonl)
