import tempfile
from Delaunay import *
import os
from Numeric import *
import string

class Boundary:
    def __init__(self, positions, elements, Lz = 1000000.0):
	bonds = {}
	for indices in elements:
	    for n1 in range(2):
		node1 = indices[n1]
		for n2 in range(n1 + 1, 3):
		    node2 = indices[n2]
		    if abs(positions[node2][2] - 
			   positions[node1][2]) > Lz / 2.0 :
			continue
		    if(node2 > node1):
			bond = (node1, node2)
		    else:
			bond = (node2, node1)
		    bonds[bond] = 0
	self.file = tempfile.mktemp()
	alchemy = open(self.file, 'w')
	alchemy.write("%5d ATOMS, %5d BONDS,     0 CHARGES,\n" %
		      (len(positions), len(bonds)))
	mx = max(max(positions.flat), -min(positions.flat))
	s = 0.529177
	if(mx * s >= 99.0):
	    s = 99.0 / mx
	n = 0
	for R in positions:
	    n = n + 1
	    alchemy.write("%5d CU   %9.4f%9.4f%9.4f     0.0000\n" %
			  (n, R[0] * s, R[1] * s, R[2] * s))
	n = 0
	for bond in bonds.keys():
	    n = n + 1
	    alchemy.write("%5d %5d %5d  SINGLE\n" % 
			  (n, bond[0] + 1, bond[1] + 1))
	alchemy.close()
	self.rasmol = os.popen("rasmol -alchemy " + self.file)
	
    def __del__(self):
	self.rasmol.close()
	os.unlink(self.file)

def faces(filename, scale = 1.0, Lz = 10000000.0):
    f = []
    input = open(filename, 'r')
    n = int(input.readline())
    pos = zeros((n, 3), Float)
    for x in pos:
	n = 0
	for s in string.split(input.readline()):
	    x[n] = float(s) * scale
	    n = n + 1
    n = int(input.readline())
    f = zeros((n, 3))
    for x in f:
	n = 0
	for s in string.split(input.readline()):
	    x[n] = int(s)
	    n = n + 1
    return Boundary(pos, f, Lz)

def fac(filename, masters, scale = 1.0, Lz = 10000000.0):
    pos = masters.GetCartesianPositions()[:]
    input = open(filename, 'r')
    n = int(input.readline())
    f = zeros((n, 3))
    for x in f:
	n = 0
	for s in string.split(input.readline()):
	    x[n] = int(s)
	    n = n + 1
    return Boundary(pos, f, Lz)

def tet(filename, scale = 1.0, Lz = 10000000.0):
    f = []
    input = open(filename, 'r')
    n = int(input.readline())
    pos = zeros((n, 3), Float)
    for x in pos:
	n = 0
	for s in string.split(input.readline()):
	    x[n] = float(s) * scale
	    n = n + 1
    n = int(input.readline())
    for x in xrange(n):
	input.readline()
    n = int(input.readline())
    t = zeros((n, 4))
    for x in t:
	n = 0
	for s in string.split(input.readline()):
	    x[n] = int(s)
	    n = n + 1
    return Delaunay(pos, t, Lz)
