import sys
sys.path.append('../../src/QuasiContinuum/SWIGBuild')
sys.path.append('../../src/QuasiContinuum/PythonScripts')

from Numeric import *
import QC3D
from IPAF import *
from plot import *
from Delaunay import *
from Thompson import Thompson_110

potential = QC3D.QCEMTPotential("Cu")
d = potential.GetLengthScale()
a = d * sqrt(2)
rc = d * (sqrt(5) + sqrt(4)) / 2
rc1 = d * (sqrt(2) + sqrt(1)) / 2
(A, B, C, D, alpha, beta, gamma, delta) = Thompson_110(a)
lat = array([B - A, C - A, D - A])
lattice = QC3D.GeneralLattice()
lattice.SetLatticeVectors(lat)
N = 3
lattice2 = QC3D.GeneralLattice()
lattice2.SetLatticeVectors(N * lat)
dx = a / 2
dy = d / 2
dz = dy
Lx = (3 * 2 * N + 1) * dx
Ly = (3 * 2 * N + 1) * dy
Lz = (3 * 2 * N + 0) * dz
Lx2 = (9 * 2 * N + 0) * dx
Ly2 = (9 * 2 * N + 0) * dy
nonLocalAtoms = QC3D.PrimitiveListOfAtoms()
QC3D.RectangularClusterInitializer([Lx, Ly, Lz], lattice).Create(nonLocalAtoms)

localAtoms = QC3D.PrimitiveListOfAtoms()
QC3D.RectangularClusterInitializer([Lx2, Ly2, Lz], lattice2).Create(localAtoms)
QC3D.OverlapPruner(0.1, nonLocalAtoms.GetCartesianPositionsPtr()).Transform(
    localAtoms)

burgers = D - C
direction = C - D
center = 0.5 * (delta - A)

screw = QC3D.DislocationMaker(burgers, direction, center, 0.3)

masters = QC3D.QCListOfAtoms(nonLocalAtoms, localAtoms)
#screw.Transform(masters)

bc = QC3D.SmartPeriodicBoundaryConditions([0, 0, 1])
bc.SetLength([Lx2, Ly2, Lz])
masters.SetBoundaryConditions(bc)
nl = QC3D.CellNeighborList(rc, 0.05 * rc)
masters.SetNeighborLocator(nl)
tetrahedra = IPAFMesh(masters, array([[Lx2, 0.0, 0.0],
				      [0.0, Ly2, 0.0],
				      [0.0, 0.0, Lz]]), [0, 0, 1],
		      N * 2 * a)

meshData = QC3D.MeshData()

for tetrahedron in tetrahedra:
    if sort(tetrahedron)[3] >= nonLocalAtoms.GetNumber():
	meshData.CreateElement(tetrahedron, masters, lattice)

meshData.InitializeSlaves(masters)

potential.SetMeshData(meshData)


e0 = potential.CalculateEnergy(masters) 
print "E0: ", e0
