import sys
DMPATH = 'T:/dchen/QCDEV/DigitalMaterial'
sys.path.append(DMPATH+'/src/QuasiContinuum/SWIGBuild')
sys.path.append(DMPATH+'/src/QuasiContinuum/WindowsBuild')
sys.path.append(DMPATH+'/src/QuasiContinuum/PythonScripts')
sys.path.append(DMPATH+'/src/QuasiContinuum/Mesher3D')
sys.path.append(DMPATH + '/Python/MD/Observers')


from Numeric import *
from QC3D import *
from Detri import *
from exgcd import commonMillerIndices
from IPAF import *
#from RasMolObserver import RasMolObserver

class Initializer:
	""" Initializes a bicrystal with the grain boundary plane orthogonal to the
	x axis. The orientation of the crystal surfaces is given by the miller indices .
	"""
	def __init__(self, lattice1, size1, xsurface1,  xsurface2, lattice2, \
				 femLength, nodesDensity, size2=None, ysurface1 = None, \
				 ysurface2 = None, boundaryConditions=None, overlap = None, \
				 prunerParameter = None):
		self.lattice1 = lattice1
		self.lattice2 = lattice2
		self.size1 = size1 
		if size2 is None: self.size2 = size1[:]  # copy the values
		else :self.size2 = size2
		self.xsurface1 = xsurface1
		self.xsurface2 = xsurface2
		self.ysurface1 = ysurface1
		self.ysurface2 = ysurface2
		self.boundaryConditions = boundaryConditions
		if overlap is not None: self.overlap = overlap
		else : self.overlap = 0.5*lattice1.GetLatticeSpacing() # the overlapping region
		if prunerParameter is not None: self.prunerParameter = prunerParameter
		else: self.prunerParameter = 0.7*lattice1.GetMinInteratomicSpacing()
		self.femLength = femLength
		self.nodesDensity = nodesDensity
			
	def Create(self, atoms):
		shiftOrigin = [1.0e-6,1.3e-6]
		if len(self.size1) == 3: shiftOrigin.append(1.7e-6)  # for 3D
		self.lattice1.SetOrigin(self.lattice1.GetOrigin() + shiftOrigin)  # avoid having atoms right on the boundaries by shifting the origin of the lattice a tiny random amount 
		self.lattice2.SetOrigin(self.lattice2.GetOrigin() + shiftOrigin)  # avoid having atoms right on the boundaries by shifting the origin of the lattice a tiny random amount 
		self.rotation1, rectSize1 = self._CalculateGrainParameters(self.size1, self.xsurface1, self.lattice1, self.ysurface1)
		self.rotation2, rectSize2 = self._CalculateGrainParameters(self.size2, self.xsurface2, self.lattice2, self.ysurface2)
		# declare the rectangular cluster initializer
		rectangle = RectangularClusterInitializer(rectSize1, self.lattice1)
		# build the first crystal
		rectCenter1 = zeros(len(rectSize1), Float)
		rectCenter1[0] = -(rectSize1[0] - self.overlap)/2.
		rectSize1[0] = rectSize1[0] + self.overlap
		grain1 = PrimitiveListOfAtoms();
		self._CreateCrystal(rectangle, self.lattice1, self.rotation1, rectSize1,
					        rectCenter1, grain1)
		# build the constrainted zone
		tmpLattice = GeneralLattice(self.nodesDensity * self.lattice1.GetLatticeVectors())
		tmpLattice.SetOrigin(self.lattice1.GetOrigin())
		rectCenter1[0] = -rectSize1[0] + self.overlap - self.femLength/2.
		rectSize1[0] = self.femLength
		self.locals1 = PrimitiveListOfAtoms();
		self._CreateCrystal(rectangle, tmpLattice, self.rotation1,
				    rectSize1, rectCenter1, self.locals1)
		# now build the second crystal
		rectCenter2 = zeros(len(rectSize1), Float)
		rectCenter2[0] =  (rectSize2[0] - self.overlap)/2.
		rectSize2[0] = rectSize2[0] + self.overlap
		grain2 = PrimitiveListOfAtoms()
		self._CreateCrystal(rectangle, self.lattice2, self.rotation2,
				    rectSize2, rectCenter2, grain2)
		# build the constrainted zone
		tmpLattice = GeneralLattice(self.nodesDensity * self.lattice2.GetLatticeVectors())
		tmpLattice.SetOrigin(self.lattice2.GetOrigin())
		rectCenter2[0] = rectSize2[0] - self.overlap + self.femLength/2.
		rectSize2[0] = self.femLength
		self.locals2 = PrimitiveListOfAtoms();
		self._CreateCrystal(rectangle, tmpLattice, self.rotation2,
				    rectSize2, rectCenter2, self.locals2)
		# if we passed periodic boundary conditions, check that we can set a box size that is compatible with both lattices
		if isinstance(self.boundaryConditions, PeriodicBoundaryConditionsPtr):
			boxSize = zeros(len(rectSize1), Float) + 1.e8
			for i in range(1, len(rectSize1)): # the x direction is irrelevant.
				if (abs(rectSize1[i] - rectSize2[i])/(rectSize1[i]+rectSize2[i])) < 1.0e-5 : boxSize[i]=rectSize1[i]
			#boxSize[0] = 2*(self.femLength + self.size1[0])
			self.boundaryConditions.SetLength(boxSize)
			print 'bc Size = ', boxSize
		atoms.Merge(grain1)
		atoms.Merge(grain2)
		del grain1, grain2
		print 'removing overlap', atoms.GetNumber()
		OverlapPruner(self.prunerParameter).Transform(atoms)
		print 'done', atoms.GetNumber()

	def _CalculateGrainParameters(self, size, xsurface, lattice, ysurface=None): # returns the rotation matrix and the rectangle size
		rectangleSize = size
		latSpacing = lattice.GetLatticeSpacing()
		# we have to set the grain size such that it can accomodate the periodic boundary conditions
		# along y and z (or z alone).			
		if len(size) == 2: # first the 2D case
			k,l = xsurface
			if isinstance(lattice, TriangularLattice):
				# TRIANGULAR LATTICE    
				tmp = 2*sqrt(k*k+l*l-k*l)
				rotation = array([[sqrt(3)*k/tmp, (2*l-k)/tmp], [(k-2*l)/tmp, sqrt(3)*k/tmp]])
				# in the lattice's unit the periodicity arrays are [-l, k] along x and [2k-l,2l-k] along y
				rectangleSize = [sqrt(3)*tmp/2*latSpacing, tmp/2*latSpacing]
			if isinstance(lattice, SquareLattice):
				# 2D SQUARE LATTICE
				tmp = sqrt(k*k+l*l)
				rotation = array([[k/tmp,l/tmp],[-l/tmp, k/tmp]])
				rectangleSize = [latSpacing*tmp, latSpacing*tmp]
		if len(size) == 3: # 3D case
#			if not isinstance(lattice, CubicLatticePtr): raise TypeError, "need a cubic lattice"
			# 3D CUBIC LATTICE
			[kx,lx,mx] = xsurface
			if ysurface is None: [ky, ly, my] = [-lx, ky, 0]
			else:
				[ky, ly, my] = ysurface
				if abs(kx*ky+lx*ly+mx*my) > 1.0e-7: raise Exception, "xsurface is not orthogonal to ysurface" 
			tmpx = sqrt(kx**2+lx**2+mx**2) ; tmpy = sqrt(ky**2+ly**2+my**2);
			kz = lx*my-ly*mx; lz = -kx*my+mx*ky; mz = kx*ly-lx*ky
			cDenom = commonMillerIndices(kz,lz,mz)
			print "common Denominator", cDenom
			kz = kz / cDenom; lz = lz / cDenom; mz = mz / cDenom 
			tmpz = sqrt(kz**2+lz**2+mz**2)
			rotation = array([ [kx/tmpx, lx/tmpx, mx/tmpx], [ky/tmpy, ly/tmpy, my/tmpy],
												 [kz/tmpz, lz/tmpz, mz/tmpz]])
			rectangleSize = [latSpacing*tmpx, latSpacing*tmpy, latSpacing*tmpz]
		if isinstance(self.boundaryConditions, PeriodicBoundaryConditionsPtr):
			for i in range(1, len(rectangleSize)):
				tmp = rectangleSize[i]
				while rectangleSize[i] < size[i]: rectangleSize[i] = rectangleSize[i] + tmp
			rectangleSize[0] = size[0] # because we do not need periodic bc on the x axis
		else:
			rectangleSize = size
		return rotation, rectangleSize
			
	def _CreateCrystal(self, rect, lattice, rotation, size, center, crystal):
		rect.SetCenter(center)
		rect.SetRectangleSize(size)
		rect.SetBravaisLattice(lattice)
		lattice.ResetOrientation()
		lattice.RotateLatticeVectors(rotation)
		rect.Create(crystal) 


###########################################################################

potential = QCEMTPotential('Cu')
rc = potential.GetCutoffDistance()
lattice = FCCLattice(potential.GetLengthScale()*sqrt(2))
lattice2 = FCCLattice(potential.GetLengthScale()*sqrt(2))
# NEIGHBOR LOCATORS
neighborLocator = CellNeighborList(potential.GetCutoffDistance(), 0.05*potential.GetCutoffDistance())
bc = SmartPeriodicBoundaryConditions([0, 1, 1])
lengthX = potential.GetLengthScale()*sqrt(2)*3
gbLength = 200 # old was 200
minWidthZ = potential.GetCutoffDistance()*12.
nodesDensity = 2
gb = Initializer(lattice, [lengthX, gbLength,minWidthZ],\
		    xsurface1 = [2, 2, 1.], xsurface2 = [-2, 2, 1.], lattice2 = lattice2, \
		    femLength = 100, nodesDensity = nodesDensity, ysurface1 = [-1, -1, 4.],
		    ysurface2 = [-1, 1, -4.],boundaryConditions=bc)

##bc.SetLength(array([0, 5, 5.])*lattice2.GetLatticeSpacing())

nonLocals = PrimitiveListOfAtoms(0, bc)
gb.Create(nonLocals)
locals1 = gb.locals1
locals2 = gb.locals2
locals = PrimitiveListOfAtoms(0,bc)
locals.Attach(locals1)
locals.Attach(locals2)
gb.locals1.thisown = gb.locals2.thisown = 0
masters = QCListOfAtoms(nonLocals, locals, bc)

##rectSize = lattice.GetLatticeSpacing()*array([5, 5, 5]);
##lattice.SetOrigin([0.01,0.03,0.02])
##RectangularClusterInitializer(rectSize, lattice).Create(nonLocals)
##rectSize = lattice2.GetLatticeSpacing()*array([5, 5, 5]);
##lattice2.SetOrigin([0.01,0.03,0.02])
##RectangularClusterInitializer(rectSize, lattice2).Create(locals)

##OverlapPruner(1.0, nonLocals.GetCartesianPositionsPtr()).Transform(locals)

#plot = RasMolObserver(masters)
#masters.AddObserver(plot)
#masters.Notify()
nl = CellNeighborList(rc, 0.05 * rc)
masters.SetNeighborLocator(nl)
#[numNeighbor, indicesNeighbor, distances, separations] = nl.Neighbors(0)
atomicCutoff = lattice.GetLatticeSpacing()*sqrt(2.)
posNonLocals = nonLocals.GetCartesianPositions()[:]
posLocals = locals.GetCartesianPositions()[:]
posAll = zeros((len(posNonLocals)+len(posLocals),3),Int)
magic = 33
maxB = [0, 0,  0]
minB = [0, 0 , 0]
i =0

for p in posNonLocals:
    for j in range(3):
        posAll[i][j] = int(p[j]*magic)
        if(posAll[i][j] > maxB[j]): maxB[j] = posAll[i][j]
        if(posAll[i][j] < minB[j]): minB[j] = posAll[i][j]
    i = i + 1

for p in posLocals:
    for j in range(3): 
        posAll[i][j] = int(p[j]*magic)
        if(posAll[i][j] > maxB[j]): maxB[j] = posAll[i][j]
        if(posAll[i][j] < minB[j]): minB[j] = posAll[i][j]
    i = i + 1
box = [int((minB[0]-maxB[0])*1.05), int(magic*bc.GetLength(1)), \
	   int(magic*bc.GetLength(2))]
print box
adv = AdvancingFront(None,posAll,box,int(magic*nodesDensity*sqrt(2.)*atomicCutoff), \
				   len(posNonLocals), int(magic*atomicCutoff))
adv.Advance()
tetrahedra = adv.GetTetrahedra()

meshData = MeshData()

limit0 = len(nonLocals) 
limit1 = len(nonLocals) + len(locals1)

print limit1, locals.GetNumber(), nonLocals.GetNumber(), locals1.GetNumber()


##l1 = lattice.GetLatticeVectors()
##l2 = lattice2.GetLatticeVectors()
##lattice = GeneralLattice(array([l1[1]+l1[2], l1[0]+l1[2],l1[0]+l1[1]])/2.)
##lattice2 = GeneralLattice(array([l2[1]+l2[2], l2[0]+l2[2],l2[0]+l2[1]])/2.)
lattice = PrimitiveFCCLattice(potential.GetLengthScale()*sqrt(2))
lattice2 = PrimitiveFCCLattice(potential.GetLengthScale()*sqrt(2))
lattice.RotateLatticeVectors(gb.rotation1)
lattice2.RotateLatticeVectors(gb.rotation2)
for tetrahedron in tetrahedra:
    if(max(tetrahedron) >= limit0):
        if (max(tetrahedron) < limit1):
            meshData.CreateElement(tetrahedron, masters, lattice)
        else:
            meshData.CreateElement(tetrahedron, masters, lattice2)	

meshData.InitializeSlaves(masters)
print "ok"
potential.SetMeshData(meshData)
print "ok"

e0 = potential.CalculateEnergy(masters) 
print "E0: ", e0

def relax(atoms, potential, tolerance = 1e-2):
    e = potential.CalculateEnergy(atoms)
    mover = QuickMinAtomsMover(potential, 0.04, tolerance)
    mover.Verbose()
    mover.SetMaxIterations(5)
    mover.Move(atoms)
    mover.SetDt(0.15)
    mover.SetMaxIterations(100)
    while mover.GetFinalTFS() > tolerance:
        mover.Move(atoms)
        de = potential.CalculateEnergy(atoms) - e
        e = e + de
        print mover.GetFinalTFS(), e, de
    return potential.CalculateEnergy(atoms)

relax(masters, potential)
