import CAMPpath
import MD
import sys
import time
import Numeric
from MD.InitializerFromCmFile import InitializerFromCmFile
from MD.MakeParallelListOfAtoms import MakeParallelListOfAtoms
from DMSystem.Parallel.NetProtocol import NetProtocol
from DMSystem.Parallel.DomainSubdivision import DomainSubdivision
from DMSystem.Parallel.Migrator import Migrator
from DMSystem.Parallel.Synchronizer import Synchronizer

def PrintListOfAtoms(loa, offset=""):
    print offset, loa
    print offset, "this =", loa.this
    print offset, "thisown =", loa.thisown
    try:
        issimple = loa.IsSimple()
    except AttributeError:
        issimple = (loa.GetNumberOfChildren() == 0)
    print offset, "Is simple:", issimple
    print offset, "Shape:", loa.GetShape()
    print offset, "Length =", len(loa)
    if not issimple:
        try:
            n = loa.GetNumberOfTypes()
        except AttributeError:
            n = loa.GetNumberOfChildren()
        for i in xrange(n):
            try:
                PrintListOfAtoms(loa.GetSubList(i), offset+"    ")
            except AttributeError:
                PrintListOfAtoms(loa.SubArray(i), offset+"    ")

protocol = NetProtocol()
atoms = MD.PrimitiveListOfAtoms()
if (protocol.GetProcessorNumber() == 0):
    init = InitializerFromCmFile("sim037B-prep.cm", verbose=1)
    init.Create(atoms)
    basis = init.GetOriginalBasis()
    sys.stderr.write("\n")
else:
    basis = Numeric.zeros((3,3), Numeric.Float)

sys.stdout = open("output.%d" % (protocol.GetProcessorNumber(),), "w")

sys.stderr.write("MakeParallelListOfAtoms() ...\n")
atoms = MakeParallelListOfAtoms(atoms, protocol)
sys.stderr.write("PrintListOfAtoms() ...\n")
PrintListOfAtoms(atoms)
PrintListOfAtoms(atoms.GetCartesianPositionsDM())
PrintListOfAtoms(atoms.GetCartesianVelocitiesDM())

sys.stderr.write("Prepare migration ...\n")
protocol.Broadcast(basis, 0)
domain = DomainSubdivision(basis, (1,1,2), 1.0)
# synch = Synchronizer(atoms.GetCartesianPositionsDM(), domain, protocol)
migrator = Migrator(atoms.GetCartesianPositionsDM(), domain, protocol, None)
sys.stderr.write("Migrate() ...\n")
migrator.Migrate()

sys.stderr.write("PrintListOfAtoms() ...\n")
PrintListOfAtoms(atoms)
PrintListOfAtoms(atoms.GetCartesianPositionsDM())
PrintListOfAtoms(atoms.GetCartesianVelocitiesDM())

sys.stderr.write("Barrier() ...\n")
protocol.Barrier()
sys.exit(0)

emt = MD.EMTPotential("Cu")
nblist = MD.CellNeighborList(emt.GetCutoffDistance(), 0.135*emt.GetCutoffDistance())
# nblist = MD.NeighborList(emt.GetCutoffDistance(), 1.135*emt.GetCutoffDistance())

sys.stderr.write("*** atoms.SetNeighborLocator(nblist)\n")
atoms.SetNeighborLocator(nblist)

energy = emt.CalculateEnergy(atoms)/len(atoms)
ekin = atoms.GetKineticEnergy()/len(atoms)
print "Energy per atom:", energy, '+', ekin ,"=", energy+ekin
print "Atoms:", len(atoms)

print "Running ten verlet timesteps"
t1 = time.clock()
mover = MD.VerletAtomsMover(emt, 0.928, 5)
mover.Move(atoms)
print "Time elapsed:", time.clock() - t1

energy = emt.CalculateEnergy(atoms)/len(atoms)
ekin = atoms.GetKineticEnergy()/len(atoms)
print "Energy per atom:", energy, '+', ekin ,"=", energy+ekin
print "Atoms:", len(atoms)

sys.stderr.write("Done  (Entering ~)\n")

del atoms

sys.stderr.write("Done 2 (Entering ~)\n")
