// AttributeError.h interface for the AttributeError class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _ATTRIBUTE_ERROR_H_
#define _ATTRIBUTE_ERROR_H_

#include "DMFirstHeader.h"
#include <string>
#include "DigitalMaterial.h"
#include "DMException.h"

//! Error when a class does not support a method.
/*! We have some fat interfaces in base classes which do no more than throw
  this exception if the concrete subclass does not support the method.
  If you get this error, you can assume that the class purposefully does
  not support this method.

  Caught in Python as an AttributeError.
*/
class AttributeError : public DMException
{
	std::string sFunction, sFile, sNCMessage;
public:
	//! Constructor.
	/*! Maybe this constructor should allow you to give a reason why you
	  don't support the method.
	*/
	AttributeError( const std::string& sFunction, const std::string& sFile );
	//! virtual destructor
	virtual ~AttributeError();
};

#endif //_ATTRIBUTE_ERROR_H_
