// DMException.h interface for the DMException class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _DMEXCEPTION_H_
#define _DMEXCEPTION_H_

#include "DMFirstHeader.h"
#include <iostream>
#include <string>
#include "DigitalMaterial.h"

//! Base exception class providing basic ostream services.
/*! This exception stores the class and function that threw it
	and any message you supply.  It also prints itself.  It does
	not know how to collect a stack trace or anything of that sort.

	We need to teach SWIGged modules to catch this exception in python.

	\date 15 Sept 2000
	\author ajd27
	\ingroup dmsystem
	*/
class DMException
{
protected:
	std::string sClass; //!< Class that threw exception.
	std::string sFunction; //!< Function where thrown.
	std::string sMessage; //!< Whatever you want to say.
public:
	//! Constructor
	DMException( const std::string& sClass, const std::string& sFunction,
		const std::string& sMessage );
	//! virtual destructor.
	virtual ~DMException();
	//! Get a string describing the error.
	virtual std::string GetString( ) const;
	//! Permit ostream to write us.
	friend std::ostream& operator<<(std::ostream& s, const DMException& r);
protected:
	//! Constructor
	DMException( const std::string& sClass, const std::string& sFunction );

private:
	//! Write message to ostream.
	virtual std::ostream& Put(std::ostream& s) const;
};


#endif //_DMEXCEPTION_H_
