// Protocol.h: interface for the Protocol class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef DMPROTOCOL
#define DMPROTOCOL

#include "DMFirstHeader.h"
#include <vector>
#include <string>
#include "DigitalMaterial.h"
#include "DMReader.h"
#include "DMWriter.h"

class DMProtocol;

extern DMProtocol *dmProtocol;

class DMProtocol  
{
protected:
    DMProtocol() {}
public:
    virtual ~DMProtocol() {}
    
    virtual int GetProcessorNumber() const = 0;
    virtual int GetNumberOfProcessors() const = 0;
    
    virtual void Broadcast(double *values, int size, int master) = 0;
    virtual void Broadcast(int *values, int size, int master) = 0;
    virtual void Broadcast(std::string &string, int master) = 0;
    
    virtual void Barrier() = 0;
    
    virtual std::vector<int> GatherValue(int number, int root) = 0;
    
    
    enum Operation{SUM, MAX, MIN} ;
    virtual void AllReduce(const double *in, double *out, int size, Operation op) = 0;
    virtual void AllReduce(const int *in   , int    *out, int size, Operation op) = 0;

    class Buffers
    {
    public:
        virtual ~Buffers() {}
        //! an alltoall communication using buffers.
        /*!Useful for mixed-type data and/or variable length messages
        */
        virtual void AllToAll() = 0;
        virtual DMWriter *GetSendBuffer(int proc) = 0;
        virtual DMReader *GetRecvBuffer(int proc) = 0;
    protected:
        Buffers() {}
    };

    virtual DMProtocol::Buffers *NewBuffers() = 0;
};
    

#endif // !defined PROTOCOL_H
