// DMReader.h: interface for the DMReader class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef DMREADER_H
#define DMREADER_H

#include "DMFirstHeader.h"
#include <string>
#include <vector>
#include "DigitalMaterial.h"
#include "Serializable.h"

class DMReader
{
public:
    DMReader(const std::string &filename) : filename(filename) {}
    virtual ~DMReader() {}
    virtual void Open() = 0;
    virtual void Close() = 0;
    virtual std::string GetType(const std::string &name="root") = 0; 
    virtual bool Fill(Serializable *toFill, const std::string &name="root") = 0;
    virtual Serializable *Get(const std::string &name="root") = 0;
    virtual bool GetBool( const std::string& name ) = 0;
    virtual int GetInt(const std::string &name) = 0;
    virtual double GetDouble(const std::string &name) = 0;
    virtual std::string GetString(const std::string &name) = 0;
    virtual int *GetIntArray(std::vector<int> &dims, const std::string &name) = 0;
    virtual double *GetDoubleArray(std::vector<int> &dims, const std::string &name) = 0;

    //! Returns the number of elements to allocate.
    /*! If you call this with a shape which has zero size, it will throw
        an exception.
    */
    static size_t DimsToSize( const std::vector<int>& dims )
    {
        std::vector<int>::const_iterator begin;
        
        size_t len = 1;
        for ( begin = dims.begin(); begin < dims.end(); begin++ )
            len *= *begin;
        
        return len;
    }


    template <class Type> 
        void GetArray(std::vector<int> &dims, const std::string &name, Type *&ip)
    { throw DMException("DMReader", "GetArray<>", "This type not implemented yet");}
    void GetArray(std::vector<int> &dims, const std::string &name, int *&ip)
                { ip = GetIntArray(dims, name);}
    void GetArray(std::vector<int> &dims, const std::string &name, double *&xp)
                { xp = GetDoubleArray(dims, name);}

protected:
    std::string filename;
};


#endif // DMREADER_H
