// Minimizable.h: interface for the Minimizable class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef MINIMIZABLE_H
#define MINIMIZABLE_H
#include "DMFirstHeader.h"

#include "DigitalMaterial.h"
#include "AttributeError.h"

//! This class provides the functions needed by Minimizer classes. ObjectiveFunction is the function being minimized and must always be provided. Gradient and Hessian must be provided if needed by the Minimizer algorithm, otherwise an exception is thrown.

/**
 *
 * \author nbailey 
 * \date 6/11/2000
 * \ingroup moleculardynamics
 */


class Minimizable
{
 public:
  
  virtual double ObjectiveFunction(double *parameters)=0;
  virtual  double *Gradient(double *parameters) {
	  throw AttributeError("Minimizable","Gradient");
  }
  virtual double **Hessian(double *parameters) {
	  throw AttributeError("Minimizable","Hessian");
  }
  virtual int GetNParameters()=0;

};



#endif // MINIMIZABLE_H
