// NetCDFStructure.h interface for the NetCDFStructure class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _NETCDF_STRUCTURE_H_
#define _NETCDF_STRUCTURE_H_

#include "DMFirstHeader.h"
#include <stack>
#include <string>
#include "DigitalMaterial.h"

//! Static class which knows conventions for storing serialized NetCDF data.
class NetCDFStructure
{
public:
	typedef std::stack<std::string> PathType;
	enum Access { AINVALID, READONLY, WRITE };

	/*! NetCDF file access. */
	//@{
	static void CreateFile( const std::string& filename, int& fileID );
	static void Open( const std::string& filename, Access access, int& fileID );
	static void Close( int fileID );
	//@}

	/*! Conventions for mangling names. */
	//@{
	static void CheckItemName( const std::string& sItem );
	static std::string LengthMangle( const std::string& sItem );
	static std::string DimMangle( const std::string& sItem, int dimIdx );
	static std::string DataMangle( const std::string& sItem );
	static std::string FakeDimMangle( const std::string& sItem );
	//@}

	/*! These modify the directory structure.  The PathType object,
		owned by the client class, is a token this class manipulates
		in order to set the current directory in the NetCDF file.
		*/
	//@{
	static std::string CurDir( const PathType& path );
	static std::string AbsolutePath( const PathType& path, const std::string& fileName );
	static void UpDir( PathType& path );
	static void DownDir( PathType& path, const std::string& dirName );
	static bool IsAtRoot( const PathType& path );
	//@}
private:
	//! Makes sure a filename is valid.
	static void CheckFilename( const std::string& filename,
		const std::string& function );
};

#endif // _NETCDF_STRUCTURE_H_
