// NetCDFWriter.h: interface for the netCDFWriter class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef NETCDFWRITER_H
#define NETCDFWRITER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "DMWriter.h"
#include "NetCDFStructure.h"


// XXX This uses the C interface to netCDF. 
// There is a C++ interface, which keeps track of the ncxxxID numbers and such.
// The binaries aren't available for Windows, though, and somewhere in the
// documentation it suggests that the C++ interface isn't as well used and debugged
// This does seem pretty ugly, though: we should explore using C++ more effectively

//! Writes serial data to disk using the Unidata NetCDF format.
class NetCDFWriter: public DMWriter
{
	bool bFileOpen; //!< Whether we have a currently open file.
	NetCDFStructure::PathType path; //!< Represents virtual directory structure.
	int ncID; //!< NetCDF ID of the open data file.
public:
	//! Creates a new file.
	NetCDFWriter(const std::string &filename = "data.nc");
	virtual ~NetCDFWriter();

	//! Open the file you created.
	void Open();
	//! Close the open file.
	void Close();

	//! Store a serializable object with a given name.
	void Put(const Serializable *serializable, const std::string &name="root");

	void PutBool(bool b, const std::string &name);
	void PutInt(int n, const std::string &name);
	void PutDouble(double x, const std::string &name);
	void PutString(const std::string &str, const std::string &name);
	void PutIntArray(const int *data, const std::vector<int> &dims,
		const std::string &name);
	void PutDoubleArray(const double *data, const std::vector<int> &dims,
		const std::string &name);

	int GetMaxNumDimensions() const;
private:
	static void BeginDefinition( int ncID );
	static void EndDefinition( int ncID );
	static int DefineVariable( int ncID, const std::string& sItemName,
		int type );
	static int DefineDimension( int ncID, const std::string& sDimItem, int size );
	static int DefineArray( int ncID, const std::vector<int>& dims,
		const std::string sItem, int type );
	void WriteZeroDimensions( const std::vector<int>& dims,
		const std::string& sItemName );
	static size_t DimsToSize( const std::vector<int>& dims );
	static bool CheckDims( bool isNonZeroData, const std::vector<int>& dims,
		const std::string& name );
};


#endif //NETCDFWRITER
