// NotEnoughMemory.h interface for the NotEnoughMemory class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _NOT_ENOUGH_MEMORY_H_
#define _NOT_ENOUGH_MEMORY_H_

#include <string>
#include "DigitalMaterial.h"
#include "DMException.h"

//! This exception signals that a method failed to allocate and therefore failed to initialize.
/*! When you fail to allocate, often new() throws std::bad_alloc, but by the time
	the thread of execution has left the method, you usually have caught bad_alloc
	and freed any previous allocations, so std::bad_alloc is no longer the right exception
	to pass up the call stack.  We use this class as a simple sign that initialization
	has failed due to a lack of memory.  It may even be a recoverable exception.

	Caught in Python as a MemoryError.
	*/
class NotEnoughMemory : public DMException
{
public:
	//! Constructor
	/*! This exception says there was a memory error, so you just
		tell it what kind of allocation failed.

		@param sClass Class where occurred.
		@param sFunction Function where occurred.
		@param sVariable What variables you couldn't allocate.
		*/
	NotEnoughMemory( const std::string& sClass, const std::string& sFunction,
		const std::string& sVariable );
	//! Destructor.
	virtual ~NotEnoughMemory();
};

#endif //_NOT_ENOUGH_MEMORY_H_
