# -*- makefile -*-
# Processing
# CXX is C++ files.
CFILES = $(filter %.c, $(ALLFILES))
CXXFILES = $(filter %.cpp, $(ALLFILES))
IFILES = $(filter %.i, $(ALLFILES))

COBJECTS = $(CFILES:%.c=$(ARCH)/$(OBJDIR)/%.o)
CXXOBJECTS = $(CXXFILES:%.cpp=$(ARCH)/$(OBJDIR)/%.o)
IOBJECTS = $(IFILES:%.i=$(ARCH)/$(OBJDIR)/$(SWIGPREFIX)%_wrap.o)
SWIGMODULES = $(IFILES:%.i=$(ARCH)/$(OBJDIR)/$(SWIGPREFIX)%cmodule$(SO))

# Dependencies get their own variable so we don't try to include
# an empty list of dependencies.
DFILES = $(CFILES:%.c=deps/%.d) $(CXXFILES:%.cpp=deps/%.d) 

CFLAGS += $(OPTS)

.PHONY: ALL clean $(SUBDIRS)

ECHO = @# set to @ to hide everything.  Unset to see it all.

# You have to make the library before the subdirectories or the
# $(ARCH)  $(OBJDIR) subdirectory won't exist.
# ALL: $(LIBRARY) $(SUBDIRS)

ALL: dirs $(COBJECTS) $(CXXOBJECTS) SWIG

dirs:
	$(ECHO)-mkdir -p $(ARCH)
	$(ECHO)-mkdir -p $(ARCH)/$(OBJDIR)


# If you don't like this funny target, check out the CLEAN target below.
# $@ is the target subdirectory.  -C tells make to change to that dir and run make.
$(SUBDIRS):
	$(ECHO)(cd $@; $(MAKE) ALL)


# Ar only the updated objects. $? are files among $(OBJECTS) newer than $(LIBRARY).
$(LIBRARY): $(COBJECTS) $(CXXOBJECTS)
	$(ECHO)-mkdir -p $(ARCH)
	$(ECHO)-mkdir -p $(ARCH)/$(OBJDIR)
	$(ECHO)-$(AR) -r $@ $?

deps/%.d: %.cpp
	$(ECHO)echo Making dependencies for $<
	$(ECHO)-mkdir -p deps
	$(ECHO)-rm -f $@
	$(ECHO)$(CPP) $(CFLAGS) $(INCLUDE) $< \
	| sed 's/\($*\)\.o[ :]*/$(ARCH)\/obj2D\/\1.o $(ARCH)\/obj3D\/\1.o deps\/$(notdir $@) : /g' | grep -v '/usr/include' > $@
#	$(ECHO)$(SHELL) -ec '$(CPP) $(CFLAGS) $(INCLUDE) $< \
#	| sed '\''s/\($*\)\.o[ :]*/$(ARCH)\/obj2D\1.o $(ARCH)\/obj3D\/1.o $(notdir $@) : /g'\'' > $@'

deps/%.d: %.c
	$(ECHO)echo Making dependencies for $<
	$(ECHO)-mkdir -p deps
	$(ECHO)$(CPP) $(CFLAGS) $(INCLUDE) $< \
	| sed 's/\($*\)\.o[ :]*/$(ARCH)\/obj2D\/\1.o $(ARCH)\/obj3D\/\1.o deps\/$(notdir $@) : /g' | grep -v '/usr/include' > $@

ifneq ($(DFILES),)
include $(DFILES)
endif

$(ARCH)/$(OBJDIR)/%.o: %.cpp
	$(ECHO)echo Compiling $<
	$(ECHO)-mkdir -p $(ARCH)
	$(ECHO)-mkdir -p $(ARCH)/$(OBJDIR)
	$(ECHO)$(CXX) $(CFLAGS) -c $< $(INCLUDE) -o $@

$(ARCH)/$(OBJDIR)/%.o: %.c
	$(ECHO)echo Compiling $<
	$(ECHO)-mkdir -p $(ARCH)
	$(ECHO)-mkdir -p $(ARCH)/$(OBJDIR)
	$(ECHO)$(CC) $(CFLAGS) -c $< $(INCLUDE) -o $@

$(ARCH)/$(OBJDIR)/$(SWIGPREFIX)%cmodule$(SO): $(ARCH)/$(OBJDIR)/$(SWIGPREFIX)%_wrap.o $(LIBRARY) 
	@echo Linking shared object $@
	$(ECHO)$(CXXSHARED) $(CFLAGS) $(CPP_DLLIBS) -o $@ $< -L./$(ARCH)/$(OBJDIR) -l$(LIBRARY_SHORT) $(LIBS)

$(SWIGPREFIX)%_wrap.cpp: %.i
	@echo Swigging $<
	$(ECHO)$(SWIG) -c++ -python $(SWIGOPT) $(INCLUDE) -o $(SWIGPREFIX)$(<:%.i=%_wrap.cpp) $<
	$(ECHO)mv -f $(@:%_wrap.cpp=%.py) $(ARCH)/$(OBJDIR)

SWIG:  $(SWIGMODULES)

fido:
	@echo $(SWIGMODULES)
	@echo $(ARCH)/$(OBJDIR)/$(SWIGPREFIX)%cmodule$(SO)

clean::
	-rm -rf deps
	-rm -rf $(ARCH)
	-rm -f so_locations

clean::
	$(foreach dir, $(SUBDIRS), (cd $(dir); $(MAKE) clean; ))

distclean:: clean
	-rm -rf $(ARCH) deps



