// ProtocolFactory.cpp: implementation of the ProtocolFactory class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "ProtocolFactory.h"

#include "LocalProtocol.h"

#ifdef DMPARALLEL
#include "Parallel/MPIProtocol.h"
#endif //DMPARALLEL


using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ProtocolFactory *ProtocolFactory::_instance = 0;

DMProtocol *dmProtocol = 0;

// make sure the factory is instantiated
ProtocolFactory *ProtocolFactorySingleton = ProtocolFactory::Instance();


ProtocolFactory::ProtocolFactory()
{
    dmProtocol = new LocalProtocol();
#ifdef DMPARALLEL
    int doneInit;
    int res = MPI_Initialized(&doneInit);
    if (doneInit) SetProtocol(World); 
#endif // DMPARALLEL
}

ProtocolFactory::~ProtocolFactory()
{
    while(!protocolStack.empty())
    {
        delete protocolStack.top();
        protocolStack.pop();
    }
}

ProtocolFactory* ProtocolFactory::Instance()
{
    if(0 == _instance) _instance = new ProtocolFactory();
    return _instance;
}


void ProtocolFactory::SetProtocol(Comm comm)
{
    protocolStack.push(dmProtocol);
    if(comm == World) 
{
#ifdef DMPARALLEL
        dmProtocol = new MPIProtocol();
#else
        throw DMException("ProtocolFactory","SetProtocol",
                          "asked for MPI, but compiled without DMPARALLEL flag");
#endif // DMPARALLEL
}
    else dmProtocol = new LocalProtocol();
}

void ProtocolFactory::Reset()
{
    delete dmProtocol;
    dmProtocol = protocolStack.top();
    protocolStack.pop();
}
