// Runnable.h interface for the Runnable class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _RUNNABLE_H_
#define _RUNNABLE_H_

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif

//! A class inherits from this if it would like to run in a thread.
/**
 * This is just like the Java Runnable class.  A class inherited from
 * Runnable can be run in a thread until the class exits from its 
 * Run() method.  There is no graceful way to stop, pause, or resume
 * a thread created for a Runnable class.  The best way to use it is
 * for something that can just run to completion.
 *
 * \code
 * class MyCalc : public Runnable {
 *     int i;
 *     void Run() {
 *         while ( i < 20 ) {
 *             i = i + 1; // Do calculation here
 *             // Then notify display you have done a step.
 *         }
 *     }
 * };
 *
 * int main(void) {
 *     MyCalc sim();
 *     Thread lightning(&sim);
 *     lightning.Start();
 *     DoUserInterface();
 * }
 * \endcode
 * \ingroup dmsystem
 * \author Drew Dolgert
 * \date 21 March 2000
 * \sa Threadable, Thread
 */
class Runnable
{
public:
	virtual void Run() = 0;
};

#endif //_RUNNABLE_H_
