/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * r = r + a * b
 */

#include "vectools.h"

/* r[] = r[] + a[] * b */
void vec_self_add_mul_scal(double *res, const double *a, double b, int n)
{
    int i, limit;
    double a0, a1, a2, a3, a4, a5, a6, a7;
    double s0, s1, s2, s3, s4, s5, s6, s7;

    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	s0 = res[i];
	s1 = res[i+1];
	s2 = res[i+2];
	s3 = res[i+3];
	s4 = res[i+4];
	s5 = res[i+5];
	s6 = res[i+6];
	s7 = res[i+7];
	res[i] = a0 * b + s0;
	res[i+1] = a1 * b + s1;
	res[i+2] = a2 * b + s2;
	res[i+3] = a3 * b + s3;
	res[i+4] = a4 * b + s4;
	res[i+5] = a5 * b + s5;
	res[i+6] = a6 * b + s6;
	res[i+7] = a7 * b + s7;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	s0 = res[i];
	res[i] = a0 * b + s0;
    }
}


/* r[][1..3] = r[][1..3] + a[1..3] * b[offset[]] */
void vec3_self_add_mul_indir(double (*res3)[3], const double (*a3)[3], const double *b,
			     const int *offset, int n)
{
    int i, limit;
    double *res, *a;
    double a0, a1, a2;
    double s0, s1, s2;
    double bx;
    
#ifdef AIX
#pragma disjoint (*res3, *a3, *b)
#pragma disjoint (*res, *a, *b)
#endif

    res = (double *) res3;
    a = (double *) a3;

    limit = 3*n;
    for (i = 0; i < limit; i+=3)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	s0 = res[i];
	s1 = res[i+1];
	s2 = res[i+2];
	bx = b[*offset++];
	res[i] = a0 * bx + s0;
	res[i+1] = a1 * bx + s1;
	res[i+2] = a2 * bx + s2;
    }
}
