/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * r = (a * b + c) * a * d
 */

#include "vectools.h"

/* res[] = (a[] * b + c) * a[] * d[] */
void vec_P_av_times_b_plus_c_p_times_av_times_dv(double *res, const double *a,
						 double b, double c, const double *d,
						 int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, b, c, *d)
#endif
    
    int i,limit;

    double a0, a1, a2, a3, a4, a5, a6, a7;
    double d0, d1, d2, d3, d4, d5, d6, d7;

    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	d0 = d[i];
	d1 = d[i+1];
	d2 = d[i+2];
	d3 = d[i+3];
	d4 = d[i+4];
	d5 = d[i+5];
	d6 = d[i+6];
	d7 = d[i+7];
	res[i] = (a0 * b + c) * a0 * d0;
	res[i+1] = (a1 * b + c) * a1 * d1;
	res[i+2] = (a2 * b + c) * a2 * d2;
	res[i+3] = (a3 * b + c) * a3 * d3;
	res[i+4] = (a4 * b + c) * a4 * d4;
	res[i+5] = (a5 * b + c) * a5 * d5;
	res[i+6] = (a6 * b + c) * a6 * d6;
	res[i+7] = (a7 * b + c) * a7 * d7;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	d0 = d[i];
	res[i] = (a0 * b + c) * a0 * d0;
    }
}

