/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * Vectorized calculation of dEds
 */

#include "vectools.h"


/*  res[] = (d * a[] + e) * b[] + f * c[]  */
void vec_dEds(double *res, const double *a, const double *b, const double *c, double d,
	      double e, double f, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, *b, *c)
#endif

    double a0, a1, a2, a3;
    double b0, b1, b2, b3;
    double c0, c1, c2, c3;
    int i, limit;

    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	a0 = a[i];
	b0 = b[i];
	c0 = c[i];
	a1 = a[i+1];
	b1 = b[i+1];
	c1 = c[i+1];
	a2 = a[i+2];
	b2 = b[i+2];
	c2 = c[i+2];
	a3 = a[i+3];
	b3 = b[i+3];
	c3 = c[i+3];
	res[i] = (d * a0 + e) * b0 + f * c0;
	res[i+1] = (d * a1 + e) * b1 + f * c1;
	res[i+2] = (d * a2 + e) * b2 + f * c2;
	res[i+3] = (d * a3 + e) * b3 + f * c3;
    }
    for (i = n - n % 4; i < n; i++)
    {
	a0 = a[i];
	b0 = b[i];
	c0 = c[i];
	res[i] = (d * a0 + e) * b0 + f * c0;
    }
}

#if 0
/*  res[] = (d[offset[]] * a[] + e[offset[]]) * b[] + f[offset[]] * c[]  */
void vec_dEds(double *res, const double *a, const double *b, const double *c, const double *d,
	      const double *e, const double *f, const int *offset, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, *b, *c, *d, *e, *f)
#endif

    double a0, a1, a2, a3;
    double b0, b1, b2, b3;
    double c0, c1, c2, c3;
    double d0, d1, d2, d3;
    double e0, e1, e2, e3;
    double f0, f1, f2, f3;
    int i, limit;

    limit = n-3;
    for (i = 0; i < limit; i+=4)
    {
	a0 = a[i];
	b0 = b[i];
	c0 = c[i];
	d0 = d[offset[i]];
	e0 = e[offset[i]];
	f0 = f[offset[i]];
	a1 = a[i+1];
	b1 = b[i+1];
	c1 = c[i+1];
	d1 = d[offset[i+1]];
	e1 = e[offset[i+1]];
	f1 = f[offset[i+1]];
	a2 = a[i+2];
	b2 = b[i+2];
	c2 = c[i+2];
	d2 = d[offset[i+2]];
	e2 = e[offset[i+2]];
	f2 = f[offset[i+2]];
	a3 = a[i+3];
	b3 = b[i+3];
	c3 = c[i+3];
	d3 = d[offset[i+3]];
	e3 = e[offset[i+3]];
	f3 = f[offset[i+3]];
	res[i] = (d0 * a0 + e0) * b0 + f0 * c0;
	res[i+1] = (d1 * a1 + e1) * b1 + f1 * c1;
	res[i+2] = (d2 * a2 + e2) * b2 + f2 * c2;
	res[i+3] = (d3 * a3 + e3) * b3 + f3 * c3;
    }
    for (i = n - n % 4; i < n; i++)
    {
	a0 = a[i];
	b0 = b[i];
	c0 = c[i];
	d0 = d[offset[i]];
	e0 = e[offset[i]];
	f0 = f[offset[i]];
	res[i] = (d0 * a0 + e0) * b0 + f0 * c0;
    }
}
#endif
    
