/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * r = a * b
 */

#include "vectools.h"

/* r[] = a[] * b[] */
void vec_mul(double *res, const double *a, const double *b, int n)
{
#ifdef AIX
#pragma disjoint (*res, *a, *b)
#endif
    
    int i, limit;
    double a0, a1, a2, a3, a4, a5, a6, a7;
    double b0, b1, b2, b3, b4, b5, b6, b7;

    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	b0 = b[i];
	b1 = b[i+1];
	b2 = b[i+2];
	b3 = b[i+3];
	b4 = b[i+4];
	b5 = b[i+5];
	b6 = b[i+6];
	b7 = b[i+7];
	res[i] = a0 * b0;
	res[i+1] = a1 * b1;
	res[i+2] = a2 * b2;
	res[i+3] = a3 * b3;
	res[i+4] = a4 * b4;
	res[i+5] = a5 * b5;
	res[i+6] = a6 * b6;
	res[i+7] = a7 * b7;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	b0 = b[i];
	res[i] = a0 * b0;
    }
}

/* r[] = a[] * b */
void vec_mul_scal(double *res, const double *a, double b, int n)
{
    int i, limit;
    double a0, a1, a2, a3, a4, a5, a6, a7;

    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	res[i] = a0 * b;
	res[i+1] = a1 * b;
	res[i+2] = a2 * b;
	res[i+3] = a3 * b;
	res[i+4] = a4 * b;
	res[i+5] = a5 * b;
	res[i+6] = a6 * b;
	res[i+7] = a7 * b;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	res[i] = a0 * b;
    }
}

/* r[] = a[] * b[offset[]] */
void vec_mul_indir(double *res, const double *a, const double *b, const int *offset, int n)
{
    int i, limit;
    double a0, a1, a2, a3, a4, a5, a6, a7;
    double b0, b1, b2, b3, b4, b5, b6, b7;

    limit = n-7;
    for (i = 0; i < limit; i+=8)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	a3 = a[i+3];
	a4 = a[i+4];
	a5 = a[i+5];
	a6 = a[i+6];
	a7 = a[i+7];
	b0 = b[offset[i]];
	b1 = b[offset[i+1]];
	b2 = b[offset[i+2]];
	b3 = b[offset[i+3]];
	b4 = b[offset[i+4]];
	b5 = b[offset[i+5]];
	b6 = b[offset[i+6]];
	b7 = b[offset[i+7]];
	res[i] = a0 * b0;
	res[i+1] = a1 * b1;
	res[i+2] = a2 * b2;
	res[i+3] = a3 * b3;
	res[i+4] = a4 * b4;
	res[i+5] = a5 * b5;
	res[i+6] = a6 * b6;
	res[i+7] = a7 * b7;
    }
    for (i = n - n % 8; i < n; i++)
    {
	a0 = a[i];
	b0 = b[offset[i]];
	res[i] = a0 * b0;
    }
}

/* r[][0..2] = a[][0..2] * b[offset[]] */
void vec3_mul_indir(double (*res3)[3], const double (*a3)[3], const double *b, const int *offset,
		    int n)
{
    int i, limit;
    double *res, *a;
    double a0, a1, a2;
    double bx;
#ifdef AIX
#pragma disjoint (*res3, *a3, *b)
#pragma disjoint (*res, *a, *b)
#endif

    res = (double *) res3;
    a = (double *) a3;
    
    limit = 3*n;
    for (i = 0; i < limit; i+=3)
    {
	a0 = a[i];
	a1 = a[i+1];
	a2 = a[i+2];
	bx = b[*offset++];
	res[i] = a0 * bx;
	res[i+1] = a1 * bx;
	res[i+2] = a2 * bx;
    }
}
