/*
 * SimQA: Copyright (C) Jakob Schiotz 1996, 1997
 *
 * IBM POWER2 optimized vector library:
 * Calculates the square of the norm of an array of 3-vectors.
 */

#include "vectools.h"

/* res[] = v[][0]*v[][0] + v[][1]*v[][1] + v[][2]*v[][2] */
void vecnorm(int n, double *res, const double (*v)[3])
{
    int i, limit;
    double v00, v01, v02, v10, v11, v12, v20, v21, v22, v30, v31, v32;
    double v40, v41, v42, v50, v51, v52, v60, v61, v62, v70, v71, v72;

    /* Unroll loop 8 times */
    limit = n-7;
    for(i = 0; i < limit; i+= 8)
    {
	v00 = v[i][0];
	v01 = v[i][1];
	v02 = v[i][2];
	v10 = v[i+1][0];
	v11 = v[i+1][1];
	v12 = v[i+1][2];
	v20 = v[i+2][0];
	v21 = v[i+2][1];
	v22 = v[i+2][2];
	v30 = v[i+3][0];
	v31 = v[i+3][1];
	v32 = v[i+3][2];
	v40 = v[i+4][0];
	v41 = v[i+4][1];
	v42 = v[i+4][2];
	v50 = v[i+5][0];
	v51 = v[i+5][1];
	v52 = v[i+5][2];
	v60 = v[i+6][0];
	v61 = v[i+6][1];
	v62 = v[i+6][2];
	v70 = v[i+7][0];
	v71 = v[i+7][1];
	v72 = v[i+7][2];
	res[i] = v00 * v00 + v01 * v01 + v02 * v02;
	res[i+1] = v10 * v10 + v11 * v11 + v12 * v12;
	res[i+2] = v20 * v20 + v21 * v21 + v22 * v22;
	res[i+3] = v30 * v30 + v31 * v31 + v32 * v32;
	res[i+4] = v40 * v40 + v41 * v41 + v42 * v42;
	res[i+5] = v50 * v50 + v51 * v51 + v52 * v52;
	res[i+6] = v60 * v60 + v61 * v61 + v62 * v62;
	res[i+7] = v70 * v70 + v71 * v71 + v72 * v72;
    }
    for(; i < n; i++)
    {
	v00 = v[i][0];
	v01 = v[i][1];
	v02 = v[i][2];
	res[i] = v00 * v00 + v01 * v01 + v02 * v02;
    }
}
