// SerialFactory.h interface for the SerialFactory class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _SERIALFACTORY_H_
#define _SERIALFACTORY_H_

#ifndef SWIG
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#endif // SWIG

#include "DMFirstHeader.h"
#include <string>
#include <map>
#include "DigitalMaterial.h"

class SerialCreator;
class Serializable;

//! This singleton returns a factory object given a subtype of Serializable.
/**
 * If you know the type of the class you want to create, here the
 * string name, this creates a new instance of that class.
 * It can create any class which has registered itself using the
 * SerialCreator class.  This is a kind of Pluggable Factory, related
 * to the ProductTrader pattern.
 *
 * \code
 * Serializable* LOA = SerialFactory::Instance()->New("ListOfAtoms");
 * ListOfAtoms* ploa = dynamic_cast<ListOfAtoms*>(LOA);
 * \endcode
 *
 * \author ajd27
 * \date 27 July 2000
 * \ingroup dmsystem
 **/

class SerialFactory
{
	static SerialFactory* _instance;
	typedef std::map<std::string, const SerialCreator*> creatorMap;
	creatorMap registry;
public:
	//! Retrieve an instance of the Singleton SerialFactory from anywhere in the code.
	static SerialFactory* Instance();
	//! Register a creator object with the factory.
	bool Register( const SerialCreator* const creator );
	//! Request an instance of a given type.
	Serializable* New( const std::string& type );
	~SerialFactory();
private:
	SerialFactory();
};

#endif //_SERIALFACTORY_H_
