/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
/*	SerialTest application

	This application tests serializer readers and writers.

	author - Drew Dolgert
	date - 25 Feb 2001
	*/

#include <iostream>
#include "DynSer.h"
#include "NetCDFWriter.h"
#include "NetCDFReader.h"
#include "ASCIIReader.h"
#include "ASCIIWriter.h"
#include "DMException.h"
#include "DMLastHeader.h"

using namespace std;

//typedef ASCIIReader TheReader;
//typedef ASCIIWriter TheWriter;

typedef NetCDFReader TheReader;
typedef NetCDFWriter TheWriter;

int main( int argc, char* argv[] )
{
	bool bPassed = true;
	try {
		int iSeed;
		for ( iSeed = 0; iSeed < 20; iSeed++ ) {
			DynSerA t0( iSeed );
			
			{
				TheWriter writer( "test.dat" );
				writer.Put( &t0 );
			}
			
			TheReader reader( "test.dat" );
			Serializable* s1 = reader.Get();
			DynSerA* t1 = dynamic_cast<DynSerA*>( s1 );
			
			if ( t1->IsEqual( t0 ) ) {
				cout << "They are equal." << endl;
			} else {
				cout << "They are not equal." << endl;
				bPassed = false;
			}

			delete t1;
		}
	} catch ( DMException& e ) {
		bPassed = false;
		cout << e;
	}

	if ( bPassed ) {
		cout << "Passed test." << endl;
	} else {
		cout << "Failed test." << endl;
	}

	return 0;
}