// Serializable.h: interface for the Serializable class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef SERIALIZABLE_H
#define SERIALIZABLE_H

#include "DMFirstHeader.h"
#include <string>
#include "DigitalMaterial.h"

class DMWriter;
class DMReader;

//! Abstract base class for classes which Writer and Reader can serialize.
class Serializable  
{
public:
	//! Save the class.
	virtual void Save(DMWriter *writer) const = 0;
	//! Load the class.
	virtual void Load(DMReader *reader) = 0;
	//! Get the string name of the class for saving or loading.
	virtual const std::string &GetType() const = 0;
};

#endif // !defined(SERIALIZABLE_H)
