// SimpleMatrixInverter.h  ---  inverts small matrices (often DIMENSION x DIMENSION)

/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef SIMPLEMATRIXINVERTER_H
#define SIMPLEMATRIXINVERTER_H

#include "DigitalMaterial.h"
#include "SprintString.h"
#include "DMException.h"
#include "lapack.h"

template<int N>
class SimpleMatrixInverter
{
public:
	void Invert(double matrix[N][N])
		{
			int size = N;
			int ipiv[N];
			int info;
			double fMatrix[ N * N ];
			int lwork = N * N;
			double *work = new double[lwork];
			int index = 0;
			for(int idx2=0;idx2<N;++idx2)
				for(int idx=0;idx<N;++idx)
				{
					fMatrix[index++] = matrix[idx][idx2];
				}
			

			// LU factorize the fMatrix
			dgetrf_( &size, &size, fMatrix, &size,  ipiv,  &info );
			
			ASSERT(info == 0);
			if(info < 0) throw DMException("SimpleMatrixInverter", 
										   "Invert", SprintString("dgetrf: %d-th argument had an illegal value", -info));
			if(info > 0) throw DMException("SimpleMatrixInverter", 
										   "Invert", SprintString("dgetrf: U(%d,%d) is exactly zero", info,info));
			
			// compute the inverse of the matrix from the LU factorization
			dgetri_( &size, fMatrix, &size, ipiv, work, &lwork, &info );
			
			ASSERT(info == 0);
			if(info < 0) throw DMException("SimpleMatrixInverter", 
										   "Invert", SprintString("dgetri: %d-th argument had an illegal value", -info));
			if(info > 0) throw DMException("SimpleMatrixInverter", 
										   "Invert", SprintString("dgetri: U(%d,%d) is exactly zero", info,info));
			
			delete [] work;
			
			index = 0;
			for(int idx=0;idx<N;++idx)
				for(int idx2=0;idx2<N;++idx2)
					matrix[idx2][idx] = fMatrix[index++];
	
		}
};

#endif // SIMPLEMATRIXINVERTER_H
