// Timer.h: interface for the Timer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/


#if !defined(AFX_TIMER_H__82563A7E_F599_11D2_8A2A_00A02489285A__INCLUDED_)
#define AFX_TIMER_H__82563A7E_F599_11D2_8A2A_00A02489285A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <windows.h>
#include "time.h"

//! This timer allows you to time short sections of code with high accuracy.
/** Adapted from "Developing Optimized Code with Microsoft Visual C++ 6.0"
  * by Martin Heller, March 1999 (http://www.msdn.microsoft.com/library/techart/optcode.htm)
  *
  * This timer currently works only on Windows.  As something which times
  * threads, it might go in the ProcessFactory with RWLock and Thread.
  *
  * \ingroup dmsystem
  */
class Timer  
{
public:
	void Reset();
	Timer();
	virtual ~Timer();
	bool IsTiming() const {return isTiming;}
	void StartClock();
	double StopClock();
	double TotalTime() const {return totalTime;}
	double LatestTime() const {return latestTime;}
	double AverageTime() const {return totalTime/numCalls;}

protected:
	LARGE_INTEGER perfFreq;
	double totalTime;
	int numCalls;
	double latestTime;
	bool isTiming;
	void DetermineTimer();	
	bool ttuHiRes;	// True if high-res available
	int perfFreqAdjust; // in case Freq is too big
	int overheadTicks;   // overhead  in calling timer
	LARGE_INTEGER ticksStart, ticksStop;	// used if HiRes
	long timeStart, timeStop;	// used if no HiRes
	
};

#endif // !defined(AFX_TIMER_H__82563A7E_F599_11D2_8A2A_00A02489285A__INCLUDED_)