// UnixThread.h - 21 November 1999

/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

/* Windows needs to put the simulation in a thread.
 * If we derive the simulation from a thread, we would
 * have trouble using the simulation class on different
 * platforms because threads are a hairy subject.
 * Therefore this thread will own a Threadable object.
 *
 * The rules of this thread class are as follows:
 * It is synchronous, so the methods do what they claim before they leave.
 * Only one client thread gets access at a time, so the thread is thread-safe.
 * Note no functions tell you how the thread is doing.  This is a thread class
 * for the decisive.
 */

#ifndef _UnixThread_
#define _UnixThread_ 1

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Threadable.h" // includes Runnable
#include "ThreadImp.h"

class UnixThread : public ThreadImp
{
public:
	// Pass by reference because UnixThread is not
	// responsible for threadable deletion.
	UnixThread();
	virtual void SetTask(Runnable *thread);
	virtual void SetTask(Threadable *thread);
	virtual int Start();
	virtual int Stop();
	virtual int Pause();
	virtual int Resume();
//	virtual void Lock();
//	virtual void Unlock();
	virtual int Toggle();
	// interface to simulation
	virtual bool IsRunning();
	virtual ~UnixThread();
};


#endif // _UnixThread_
