// WinProcessFactory.h interface for the WinProcessFactory class
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _WinProcessFactory_H_
#define _WinProcessFactory_H_

#include "DigitalMaterial.h"
#include "ProcessFactory.h"

//! A concrete ProcessFactory for Windows.
/**
 * This is a concrete implementation of the abstract ProcessFactory.
 * In order to use ProcessFactory::Instance() in a program, you must
 * first instantiate a WinProcessFactory with the call
 * WinProcessFactory::Create().
 *
 * Note: Imp as in ThreadImp or RWLockImp mean implementation
 *
 * \author Drew Dolgert
 * \date 21 March 2000
 * \bug There is an easier way to instantiate the ProcessFactory.
 *		Because only the WinProcessFactory will be included in the
 *		build on a Windows machine (no reason to include UnixProcessFactory),
 *		we could have WinProcessFactory instantiate itself automatically
 *		using a static member variable.  Cute, but quiet and effective.
 * \sa ProcessFactory, UnixProcessFactory
 * \ingroup dmsystem
 */
class WinProcessFactory : public ProcessFactory {
public:
	static void Create() {
		ProcessFactory::_instance = new WinProcessFactory();
	}
	virtual ThreadImp* GetThread();
	virtual RWLockImp* GetRWLock();
	virtual EggTimer*  GetEggTimer();
protected:
	WinProcessFactory() {}
};

#endif // _WinProcessFactory_H_
