#include <iostream>
#include <sstream> // for stringstream
#include "DiscreteAvalanche.h"
#include "CmdStreamReader.h"
#include "DMLastHeader.h"

CmdStreamReader::CmdStreamReader()
{
}

CmdStreamReader::~CmdStreamReader()
{
	DestroyCmdParsers();
}


void CmdStreamReader::AddParser( CmdParser* pParser )
{
	cmdParsers.push_back( pParser );
}


void CmdStreamReader::DestroyCmdParsers( )
{
	int parserIdx;
	for ( parserIdx=0; parserIdx<cmdParsers.size(); parserIdx++ ) {
		delete cmdParsers[parserIdx];
	}
}


CmdParser* CmdStreamReader::ParserWhichMatches( const std::string& inputToken )
{
	int parserIdx;
	for ( parserIdx=0; parserIdx<cmdParsers.size(); parserIdx++ ) {
		// If a parser object claims this command
		if (cmdParsers[parserIdx]->CmdStringMatches( inputToken )) {
			return cmdParsers[parserIdx];
		}
	}

	return 0;
}


bool CmdStreamReader::ReadCmdStream( std::istream& cmdStream )
{
	bool bQuit = false;
	const int MAX_LINE = 500;
	char inputLine[MAX_LINE];

	while (cmdStream.good() && !bQuit) {
		std::string inputToken;
		cmdStream.getline( inputLine, MAX_LINE, '\n' );
		std::istringstream cmdLine( inputLine );
		cmdLine >> inputToken;
		
		if (inputToken.size() > 0) {
			// Only read non-comment tokens.
			if ( inputToken[0] != '#' ) {
				CmdParser* cmdParser = ParserWhichMatches( inputToken );
				if ( 0 != cmdParser ) {
					bQuit = cmdParser->AcceptCmdLine( cmdLine );
				} else {
					std::cout << "Could not identify that command." << std::endl;
				}
			} // else ignore the line
		}
	}
	return bQuit;
}


const CmdStreamReader::ParserVector& CmdStreamReader::GetCmdParsers( ) const
{
	return cmdParsers;
}