#include "DMFirstHeader.h"
#include <algorithm>
#include "BruteAvalancheStart.h"
#include "FixedFieldAvalanche.h"
#include "SpinLattice.h"
#include "FailedVerify.h"
#include "OrderByFields.h"
#include "DMLastHeader.h"

BruteAvalancheStart::BruteAvalancheStart()
{
	lattice = 0;

	pdRandomField = 0;
	pcNUp = 0;

	iVerify = 0;
}

BruteAvalancheStart::~BruteAvalancheStart()
{
}

/** You need to give it an AvalancheAlgorithm that has already
  * done its internal initializations, because this function will
  * steal const copies of some of the internal pointers.
  */
void BruteAvalancheStart::SetAlgorithm( FixedFieldAvalanche* avalanche )
{
	this->lattice = avalanche->GetLattice();
	this->pdRandomField = avalanche->GetRandomField();
	this->pcNUp = avalanche->GetNUp();
	this->Z = lattice->GetCoordinationNumber();
	int L = lattice->GetLength();
	this->N = L*L;
	if (pdRandomField && pcNUp) iVerify |= ALGORITHM;
}

void BruteAvalancheStart::Initialize()
{
	if ( (iVerify & ALGORITHM) == 0)
		throw FailedVerify("BruteAvalancheStart", "Can\'t call initialize until algorithm is set.");
	iVerify |= INITIALIZE;
}

void BruteAvalancheStart::Verify() const
{
	if (iVerify != 3) throw FailedVerify( "BruteAvalancheStart","Failed to verify" );
}

/**
  * @param H returns the magnetic field where the next avalanche starts.
  * @param iFirstSpin returns the spin index where the avalanche starts.
  * @return false if you can't have any more avalanches.  true if you can.
  */
AvalancheAlgorithm::Status
BruteAvalancheStart::FindNextStart( double *H, SpinIndex *iFirstSpin )
{
	SpinIndex iMinSpin = -1, i;
	double dTestH, dMinH = 1.0e30;

	for (i=0; i<N; i++) {
		if (lattice->Get(i)==-1) {
			dTestH = -(pdRandomField[ i ] +2*pcNUp[i] - Z);
			if (dTestH < dMinH) {
				iMinSpin = i;
				dMinH = dTestH;
			}
		}
	}
	if (iMinSpin == -1) return AvalancheAlgorithm::ALLFLIPPED;

	*H = dMinH;
	*iFirstSpin = iMinSpin;
	return AvalancheAlgorithm::SUCCESS;
}


// ******* Member functions below here are private. *************/
