#ifndef _CHECKPOINTER_H_
#define _CHECKPOINTER_H_

#include "DMFirstHeader.h"
#include "DiscreteAvalanche.h"

//! Abstract class to decide when to checkpoint and do the saving.
class Checkpointer
{
public:
	//! Set how often we checkpoint.
	virtual void SetFrequency( int uiSeconds ) = 0;
	//! Set how often we checkpoint in minutes.
	virtual void SetFrequencyMinutes( int iMinutes ) = 0;

	//! Get the time for the next checkpoint.
	/*! Time is measured in seconds.  It returns zero when 
		there is no more time before the end to do a checkpoint.
		That may be time to quit.

		\param uiCurrentTime Time since time=0.
		\param uiEndTime May be zero if there is no end time
		\return Time of next checkpoint beyond current time or 
		zero if there are no checkpoints past the current time.
		*/
	virtual int GetNextCheckpoint( int uiCurrentTime,
		int uiEndTime ) = 0;
	//! Do the checkpointing.
	virtual void Checkpoint() = 0;
};

#endif //_CHECKPOINTER_H_
