#ifndef _FLIP_LISTENER_H_
#define _FLIP_LISTENER_H_

#include "DMFirstHeader.h"
#include "DiscreteAvalanche.h"
#include "SpinIndex.h"
#include "SpinLattice.h"

//! A flip listener is notified after each spin flips.
/*! Abstract interface to observers notified at the end of every spin flip.
*/
class FlipListener
{
public:
	//! Call this method at the end of every spin flip.
	/*! You do not need to make FlipCompleted virtual in 
	classes derived from FlipListener.  That way, the derived
	class need not have a vftable, so the FlipCompleted function
	can be inlined.
	
	  @param lattice The lattice whose spin was flipped.
	  @param coords The coordinates of the flipped spin.
	  @param Dimension The number of coordinates.
		*/
	virtual void FlipCompleted
		(
		SpinLattice*	lattice,
		const int*		coords,
		int				Dimension
		) PUREVIRTUAL;

	virtual void BeginSweep( bool bKeepData ) PUREVIRTUAL;
};

#endif //_FLIP_LISTENER_H_