
#include "DMFirstHeader.h"
#include "Rand.h"
#include "GaussianInitializer.h"
#include "DMReader.h"
#include "DMWriter.h"
#include "DMException.h"
#include "SerialBuilder.h"
#include "DMLastHeader.h"

const std::string sType =
	std::string("GaussianInitializer");

/*! This static object permits the DMReader to create an empty GaussianInitializer. */
const SerialBuilder<GaussianInitializer> *_creator = new
	SerialBuilder<GaussianInitializer>(sType);


GaussianInitializer::GaussianInitializer() {}

void GaussianInitializer::SetDisorder( double dDisorder )
{
	ASSERT( dDisorder > 0 );
	if (dDisorder <= 0) throw
		DMException("GaussianInitializer","SetDisorder",
		"The disorder has to be greater than zero." );
	this->dDisorder = dDisorder;
	invSqrt2R = 1/( sqrt(2.0)*dDisorder );
}

void GaussianInitializer::SetRand( Rand* rand )
{
	ASSERT( 0 != rand );
	if (0 ==rand) throw
		DMException("GaussianInitializer","SetRand",
		"The pointer passed to SetRand should not be null.");

	this->rand = rand;
}

void GaussianInitializer::Initialize( double *pdRandomField, int iLength )
{
	ASSERT( 0 != pdRandomField );
	if ( 0 == pdRandomField )
		throw DMException("GaussianInitializer","Initialize",
		"Passed a null pointer.");

	int i;
	for (i=0; i<iLength; i++)
		pdRandomField[i] = rand->gaussian( dDisorder );
}

void GaussianInitializer::Save( DMWriter *writer ) const
{
	writer->PutDouble( dDisorder, "disorder" );
}

void GaussianInitializer::Load( DMReader *reader )
{
	double dDisorder = reader->GetDouble( "disorder" );
	this->SetDisorder( dDisorder );
}

const std::string GaussianInitializer::GetType() const
{
	return sType;
}