#include "DigitalMaterial.h"
#include "WholeGridDD.h"
#include "WholeGridSource.h"
#include "SlowPointer.h"
#include "DMLastHeader.h"

WholeGridDD::WholeGridDD()
{
	this->lpDD7 = 0;
	this->storage = 0;
	gridhelp = new WholeGridBase();
	if (gridhelp==0) return;
	cWidth = cHeight = 0;
	::SetRectEmpty(&displayRect);
	bDDInitialized = false;
	bOffscreenBuffer = false;
	lpDDSBuffer = 0;
	lpDDSPrimary = 0;
	palette = 0;
}

WholeGridDD::~WholeGridDD()
{
	ReleaseAllObjects();
	if (storage) delete[] storage;
	delete gridhelp;
	if (palette) delete palette;
}


int WholeGridDD::CreateWindowedDD()
{
	HRESULT result = DirectDrawCreateEx(NULL, (VOID**)&(this->lpDD7),
			IID_IDirectDraw7, NULL);
	if (result != DD_OK) {
		return FAILURE; //return false; // fail
	}
	// Tell dd that we will work in windowed mode.
	result = lpDD7->SetCooperativeLevel(hwnd, DDSCL_NORMAL);
	if (result != DD_OK) {
		return FAILURE; // fail
	}
	return SUCCESS;
}

int WholeGridDD::CreatePrimarySurface()
{
	memset((void *)&(this->ddsd),0,sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE; 
	HRESULT result = lpDD7->CreateSurface( &ddsd, &lpDDSPrimary, NULL);
	if (result != DD_OK) {
		return FAILURE; //fail
	}
	return SUCCESS;
}

int WholeGridDD::CreateOffscreenSurface(int cWidth, int cHeight)
{
	memset((void *)&(ddsd2),0,sizeof(ddsd2));
	ddsd2.dwSize = sizeof(ddsd2);
	ddsd2.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd2.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
	ddsd2.dwHeight = cHeight;
	ddsd2.dwWidth = cWidth;

	HRESULT result;
	result = lpDD7->CreateSurface ( &ddsd2, &lpDDSBuffer, NULL);
	if (result != DD_OK) return FAILURE;
	return SUCCESS;
}

int WholeGridDD::CheckPixelFormat()
{
	DDPIXELFORMAT ddpf;
	memset((void *)&ddpf,0,sizeof(ddpf));
	ddpf.dwSize = sizeof(ddpf);
	HRESULT result = lpDDSPrimary->GetPixelFormat( &ddpf );
	if (result != DD_OK) {
		return FAILURE; // fail
	}
	if ( ddpf.dwFlags & DDPF_PALETTEINDEXED8 ) {
		return FAILURE; // we have to use a palette
	}
	dwRGBBitCount = ddpf.dwRGBBitCount;
	dwRBitMask = ddpf.dwRBitMask;
	dwGBitMask = ddpf.dwGBitMask;
	dwBBitMask = ddpf.dwBBitMask;

	return SUCCESS;
}

int WholeGridDD::CreateClipper()
{
	LPDIRECTDRAWCLIPPER lpDDClipper;
	HRESULT result = lpDD7->CreateClipper( 0, &lpDDClipper, NULL );
	if (result != DD_OK) {
		return FAILURE; // fail
	}
	// Now associate the clipper with our window
	result = lpDDClipper->SetHWnd( 0, this->hwnd );
	if (result != DD_OK) {
		return FAILURE; // fail
	}

	result = lpDDSPrimary->SetClipper( lpDDClipper );
	if (result != DD_OK) {
		return FAILURE; // fail
	}
	lpDDClipper->Release();  // reduce ref count.  Doesn't delete.
	return SUCCESS;
}

int WholeGridDD::Initialize(HWND hwnd)
{
	if (hwnd == 0) return BADPARAM;
	if (gridhelp==0) return FAILURE;
	this->hwnd = hwnd;

	int res = this->CreateWindowedDD();
	if (res != SUCCESS) return res;

	res = this->CreatePrimarySurface();
	if (res != SUCCESS) return res;

	res = this->CheckPixelFormat();
	if (res != SUCCESS) return res;
	
	res = this->CreateClipper();
	if (res != SUCCESS) return res;
	
	bDDInitialized = true;

	return SUCCESS;
}

int WholeGridDD::SetGridDimensions(int iX, int iY, int iMaxVal)
{
	if (iX<=0 || iY<=0) return BADPARAM;
	this->xDim = iX;
	this->yDim = iY;
	this->iMaxVal = iMaxVal;
	if (storage) delete[] storage;
	this->storage = new int[iX*iY];
	if (storage==0) return NOMEM;
	::memset(storage,0,iX*iY*sizeof(int));
	iDecimate = gridhelp->CalculateDecimation(cWidth,cHeight,xDim,yDim);
	return SUCCESS;
}

int WholeGridDD::GetPreferredSize(RECT& bounds, RECT& preferred)
{
	if (xDim==0 || yDim==0) return DIMNOTSET;
	if (gridhelp->GetPreferredSize(bounds, preferred) == 0)
			return FAILURE;
	return SUCCESS;
}

int WholeGridDD::SetDisplayRect(RECT &dispRect)
{
	if (hwnd==0) return NOTINITIALIZED;
	if (dispRect.top-dispRect.bottom>0) return BADPARAM;
	if (dispRect.right-dispRect.left<0) return BADPARAM;
	::CopyRect(&displayRect,&dispRect);
	::CopyRect(&bufferRect,&displayRect);
	// Move display rectangle to screen coordinates.
	::ClientToScreen(hwnd,(LPPOINT)&displayRect.left);
	::ClientToScreen(hwnd,(LPPOINT)&displayRect.right);
	int width = dispRect.right-dispRect.left;
	int height = dispRect.bottom-dispRect.top;
	if (this->cWidth != width || this->cHeight != height) {
		this->cWidth = width;
		this->cHeight = height;
	} else {
		return SUCCESS;
	}

	if (lpDDSBuffer) lpDDSBuffer->Release();
	
	int res = this->CreateOffscreenSurface(cWidth, cHeight);

	if (res != SUCCESS) {
		bOffscreenBuffer = false;
	} else {
		bOffscreenBuffer = true;
	}
	iDecimate = gridhelp->CalculateDecimation(cWidth,cHeight,xDim,yDim);
	return SUCCESS;
}

// Decimate preserving aspect ratio.
/*
void WholeGridDD::SetDecimation()
{
	if (cWidth==0 || cHeight == 0 || xDim==0 || yDim==0) return;
	int decX = xDim/cWidth + 1;
	int decY = yDim/cHeight + 1;
	this->iDecimate = (decX<decY) ? decY : decX;
	if (iDecimate==1) {
		decX = cWidth/xDim;
		decY = cHeight/yDim;
		this->iDecimate = (decX<decY) ? -decX : -decY;
		if (iDecimate==0) iDecimate = 1;
	}
}
*/
bool WholeGridDD::RestoreAll()
{
	HRESULT result;

	// restore primary surface
	result = lpDDSPrimary->Restore();
	if (result == DDERR_WRONGMODE) {
		// display mode changed.  destroy and recreate
		;//ReleaseObjects() then Init
		return false;
	} else if (result != DD_OK) {
		return false;
	}
	if (bOffscreenBuffer) {
		result = lpDDSBuffer->Restore();
		if (result == DDERR_WRONGMODE) {
			// display mode changed.  destroy and recreate
			;//ReleaseObjects() then Init
			return false;
		} else if (result != DD_OK) {
			return false;
		}
	}
	return true;
}

void WholeGridDD::ReleaseAllObjects()
{
	if (lpDD7 != NULL) {
		if (lpDDSPrimary != NULL) {
			lpDDSPrimary->Release();
			lpDDSPrimary = NULL;
			if (lpDDSBuffer) {
				lpDDSBuffer->Release();
				lpDDSBuffer = NULL;
			}
		}
		lpDD7->Release();
		lpDD7 = NULL;
	}
}

int WholeGridDD::SetData(WholeGridSource *data)
{
	// We should decimate with an average color of some sort.
	// For now, lose extra pixels.
	// The plus one is for data ranges from -1 to 254.

	// We can't copy the pixels until someone set our display rect
	// and we allocate a bitmap to draw into.
	if (data == 0) return BADPARAM;
	if (storage == 0) return DIMNOTSET;
	int i,j;
	for (j=0; j<yDim; j++) {
		for (i=0; i<xDim; i++) {
			storage[i+j*xDim] = data->GetGridPoint(i,j);
		}
	}
	// Invalidate our part of the rectangle and 
	// ask that the background not be cleared for us.
//	if (::InvalidateRect(hwnd, &displayRect, false)==0) {
//		i=3; // say what?
//	}
	return true;
}

int WholeGridDD::DrawToDC(HDC hDC, RECT& rect)
{
	gridhelp->SetData(storage, xDim, yDim, iMaxVal);
	gridhelp->Draw(hDC, displayRect);
	return SUCCESS;
}

HGLOBAL WholeGridDD::CreatePackedDIB(RECT& bounds)
{
	gridhelp->SetData(storage, xDim, yDim, iMaxVal);
	return gridhelp->CreatePackedDIB(bounds);
}

void WholeGridDD::InitializePalette()
{
	// This won't fail.
	RGBQUAD* colors = gridhelp->GetPalette();
	int nColors = gridhelp->GetPaletteSize();
	palette = new DWORD[nColors];

	int i;
	DWORD redMin, blueMin, greenMin;
	redMin = this->BitsToMin(dwRBitMask);
	greenMin = this->BitsToMin(dwGBitMask);
	blueMin = this->BitsToMin(dwBBitMask);

	DWORD r, g, b;
	for (i=0; i<nColors; i++) {
		r = redMin+(colors[i].rgbRed*(dwRBitMask-redMin))/255;
		g = greenMin+(colors[i].rgbGreen*(dwGBitMask-greenMin))/255;
		b = blueMin+(colors[i].rgbBlue*(dwBBitMask-blueMin))/255;
		palette[i] = (r&dwRBitMask) | (g&dwGBitMask) | (b&dwBBitMask);
	}
}

DWORD WholeGridDD::BitsToMin(DWORD bitMask)
{
	return (bitMask & (~(bitMask-1)))-1;
}

void WholeGridDD::DrawToBuffer(LPVOID lpSurface, long lPitch, POINT& upperLeft)
{
	int i,j;
	DWORD *dest = (DWORD *)lpSurface;
	DWORD color;

	if (palette==0) this->InitializePalette();

	// fine for 32 bit colors.
	dest += upperLeft.y*(lPitch/4) + upperLeft.x;
	if (iDecimate>0) {
		DWORD *line;
		for (j=0; j<yDim; j += iDecimate) {
			line = dest;
			for (i=0; i<xDim; i += iDecimate) {
				color = palette[storage[i+xDim*j]];
				*line = color;
				line++;
			}
			dest += lPitch / 4;
		}
	} else {
		int cPix = -iDecimate;
		int cx, cy; // pixel size of blocks.
		for (j=0; j<yDim; j++) {
			for (cy=0; cy<cPix; cy++) {
				for (i=0; i<xDim; i++) {
					for (cx=0; cx<cPix; cx++) {
						color = palette[storage[i+xDim*j]];
						dest[i*cPix+cx] = color;
					}
				}
				dest += lPitch / 4;
			}
		}
	}
}

void WholeGridDD::DrawToBuffer16(LPVOID lpSurface, long lPitch, POINT& upperLeft)
{
	int i,j;
	DWORD *dest = (DWORD *)lpSurface;
	DWORD color;

	if (palette==0) this->InitializePalette();
	// fine for 16 bit colors.

	dest += (upperLeft.y*lPitch)/4+ upperLeft.x/2;
	bool bOddStart = (upperLeft.x % 2) == 1;
	bool bOddFinish;
	int *data = storage;
	DWORD *line;
	int midLen;
	
	if (iDecimate>0) {
		// bOddFinish = (there are odd # points) != had an odd start.
		midLen = (xDim+1)/iDecimate;
		if (bOddStart) midLen--;
		bOddFinish = ( (midLen % 2) == 1 );
		if (bOddFinish) midLen--;
		midLen /= 2; // it should be small at this point.
		for (j=0; j<yDim; j += iDecimate) {
			line = dest;
			if ( bOddStart ) {
				(*line++) &= palette[*data];
			}
			data += iDecimate;
			// may need to start at 1
			for (i=0; i < midLen; i++) {
				color = palette[*data] << 16;
				data += iDecimate;
				*line++ = (color << 16) | palette[*data];
				data += iDecimate;
			}
			if ( bOddFinish ) {
				(*line) &= palette[*data] << 16;
			}
			dest += lPitch / 4;
		}
	} else {
		int cPix = -iDecimate;
		int cy; // pixel size of blocks.
		// bOddFinish = (there are odd # points) != had an odd start.
		midLen = xDim*cPix;
		if (bOddStart) midLen--;
		bOddFinish = (( midLen % 2)==1);
		if (bOddFinish) midLen--;
		midLen /= 2;
		for (j=0; j<yDim; j ++) {
			for (cy=0; cy<cPix; cy++) {
				SlowPointer ptr(data, cPix);
				line = dest;
				if ( bOddStart ) {
					(*line++) &= palette[*ptr++];
				}
				// may need to start at 1
				for (i=0; i < midLen; i++) {
					int dat = *ptr;
					color = palette[*ptr] << 16;
					ptr++;
					*line++ = (color << 16) | palette[*ptr];
					ptr++;
				}
				if ( bOddFinish ) {
					(*line) &= palette[*ptr] << 16;
				}
				dest += lPitch / 4;
			}
			data += xDim;
		}
	}
}


// can't do decimation
// packs wrong side of bytes
// relies on black background.
void WholeGridDD::DrawToBufferN(LPVOID lpSurface, long lPitch, POINT& upperLeft)
{
	int j, cx, cy;
	DWORD *dest = (DWORD *)lpSurface;
	DWORD *line = dest;

	if (palette==0) this->InitializePalette();
	// fine for 16 bit colors.

	dest += (upperLeft.y*lPitch)/4;
	int bitCount = dwRGBBitCount;
	int nBitsRight = upperLeft.x*dwRGBBitCount;
	int nWordsRight = nBitsRight/32;
	int nBitsStart = nBitsRight % 32;
	dest += nWordsRight;
	int ptrSpeed;
	int dec;
	if (iDecimate>0) {
		ptrSpeed = 1;
		dec = iDecimate;
	} else {
		ptrSpeed = -iDecimate;
		dec = 1;
	}

	DWORD dcopy;
	int *data = storage;
	int bitsFree;
	for (j=0; j<yDim; j+= dec) {
		for (cy=0; cy<ptrSpeed; cy++) {
			SlowPointer ptr(data, ptrSpeed);
			bitsFree = nBitsStart;
			line = dest;
			
			cx = 0;
			while (cx<xDim*ptrSpeed) {
				dcopy = palette[*ptr];
				if (bitsFree<0) {
					dcopy <<= (32-(dwRGBBitCount+bitsFree));
					*line |= dcopy;
					bitsFree = 32-(dwRGBBitCount+bitsFree);
					ptr++; cx+=dec;
				} else if (bitsFree>=bitCount) {
					dcopy <<= (bitsFree-dwRGBBitCount);
					*line |= dcopy;
					bitsFree -= dwRGBBitCount;
					ptr++; cx+=dec;
				} else if (bitCount>bitsFree) {
					dcopy >>= (dwRGBBitCount-bitsFree);
					*line |= dcopy;
					bitsFree -= dwRGBBitCount;
				}
				if (bitsFree<=0) line++;
				if (bitsFree==0) bitsFree = 32;
			}				
			dest += lPitch / 4;
		}
		data += xDim;
	}
}

void WholeGridDD::DrawToSurface(LPDIRECTDRAWSURFACE7 lpDDS,
								DDSURFACEDESC2 ddsd, RECT& displayRect)
{
	HRESULT result;
	if ( FAILED( lpDDS->IsLost() ) ) {
		if ( !RestoreAll() ) {
			; // fail
		}
	}
	
	// Turn it black, black as night.
	int i;
	DDBLTFX bltfx;
	::memset(&bltfx, 0, sizeof(bltfx));
	bltfx.dwSize = sizeof(bltfx);
	bltfx.dwFillColor = 0x00000000;
	result = lpDDS->Blt(&displayRect, 0, 0, DDBLT_COLORFILL | DDBLT_WAIT, &bltfx);
	if (result != DD_OK) {
		if (result==DDERR_GENERIC) {
			i= 2;
		} else if (result==DDERR_INVALIDCLIPLIST) {
			i=2;
		} else if (result==DDERR_INVALIDOBJECT) {
			i=2;
		} else if (result==DDERR_INVALIDPARAMS) {
			i=2;
		} else if (result==DDERR_INVALIDRECT) {
			i=2;
		} else if (result==DDERR_NOALPHAHW) {
			i=2;
		} else if (result==DDERR_NOBLTHW) {
			i=2;
		} else if (result==DDERR_NOCLIPLIST) {
			i=2;
		} else if (result==DDERR_NODDROPSHW) {
			i=2;
		} else if (result==DDERR_NOMIRRORHW) {
			i=2;
		} else if (result==DDERR_NORASTEROPHW) {
			i=2;
		} else if (result==DDERR_NOROTATIONHW) {
			i=2;
		} else if (result==DDERR_NOSTRETCHHW) {
			i=2;
		} else if (result==DDERR_NOZBUFFERHW) {
			i=2;
		} else if (result==DDERR_SURFACEBUSY) {
			i=2;
		} else if (result==DDERR_SURFACELOST) {
			i=2;
		} else if (result==DDERR_UNSUPPORTED) {
			i=2;
		} else if (result==DDERR_WASSTILLDRAWING) {
			i=2;
		}
	}

	result = lpDDS->Lock(&displayRect, &ddsd, DDLOCK_WAIT, NULL);
	if ( result != DD_OK ) {
		if (result == DDERR_INVALIDOBJECT) {
			result = 1;
		} else if (result == DDERR_INVALIDPARAMS) {
			result = 1;
		} else if (result == DDERR_OUTOFMEMORY) {
			result = 1;
		} else if (result == DDERR_SURFACEBUSY) {
			result = 1;
		} else if (result == DDERR_SURFACELOST) {
			result = 1;
		} else if (result == DDERR_WASSTILLDRAWING) {
			result = 23; // fail
		}
		return;
	}

	POINT upperLeft;
	if (iDecimate>0) {
		upperLeft.x = (cWidth-(xDim/iDecimate))/2;
		upperLeft.y = (cHeight-(yDim/iDecimate))/2;
	} else {
		upperLeft.x = (cWidth+(xDim*iDecimate))/2;
		upperLeft.y = (cHeight+(yDim*iDecimate))/2;
	}

	if (dwRGBBitCount==32) {
		this->DrawToBuffer(ddsd.lpSurface, ddsd.lPitch, upperLeft);
	} else if (dwRGBBitCount==16) {
		this->DrawToBuffer16(ddsd.lpSurface, ddsd.lPitch, upperLeft);
	} else {
		this->DrawToBufferN(ddsd.lpSurface, ddsd.lPitch, upperLeft);
	}

	result = lpDDS->Unlock(&displayRect);
	if (result != DD_OK) {
		; // fail
	}
}

void WholeGridDD::Show()
{
	HRESULT result;
	int i;
	if (this->lpDD7 == 0) return;
	if (!bDDInitialized) return;
	if (cWidth==0 || cHeight == 0) return;
	if (storage == 0) return;
	if (bOffscreenBuffer) {
		this->DrawToSurface(lpDDSBuffer, ddsd2, bufferRect);
		DDBLTFX ddblt;
		memset((void *)&ddblt,0,sizeof(ddblt));
		ddblt.dwSize = sizeof(ddblt);
		ddblt.dwDDFX = DDBLTFX_NOTEARING;
		//  | DDBLT_KEYSRC
//		result = lpDDSPrimary->Blt(&displayRect,lpDDSBuffer,
//				&bufferRect,DDBLT_WAIT | DDBLT_DDFX, &ddblt);
		result = lpDDSPrimary->Blt(&displayRect,lpDDSBuffer,
				&bufferRect,DDBLT_WAIT, 0);
		if (result != DD_OK) {
			if (result==DDERR_UNSUPPORTED) {
				i=1;
			} else if (result == DDERR_INVALIDRECT) {
				i=2;
			} else if (result== DDERR_EXCEPTION) {
				i=3; 
			} else if (result== DDERR_GENERIC) {
				i=4; 
			} else if (result== DDERR_INVALIDOBJECT) {
				i=5; 
			} else if (result== DDERR_INVALIDPARAMS) {
				i=6; 
			} else if (result== DDERR_NOBLTHW) {
				i=7; 
			} else if (result== DDERR_SURFACEBUSY) {
				i=8; 
			} else if (result== DDERR_SURFACELOST) {
				i=9; 
			} else if (result== DDERR_WASSTILLDRAWING) {
				i=10; 
			}
		}
	} else {
		this->DrawToSurface(lpDDSPrimary, ddsd, displayRect);
	}
}
