#ifndef _SORTEDLISTAVALANCHESTART_H_
#define _SORTEDLISTAVALANCHESTART_H_

#include "DMFirstHeader.h"
#include "DiscreteAvalanche.h"
#include "SpinIndex.h"
#include "SpinLattice.h"
#include "FixedFieldAvalancheStart.h"

class FixedFieldAvalanche;

//! Responsible for finding the start of the next avalanche.
/** 
  * This AvalancheStart uses the Sorted-List algorithm to decide at what
  * field the the next spin wants to flip and which spin that is.
  *
  * \author ajd27
  * \date 15 August 2000
  */

class SortedListAvalancheStart : public FixedFieldAvalancheStart
{
	SpinIndex *piSortedSpins;
	SpinIndex *piNextSpin;
	bool *pbSpinDone;
public:
	SortedListAvalancheStart();
	~SortedListAvalancheStart();
	//! Tell this which algorithm it's working with before we begin.
	void SetAlgorithm( FixedFieldAvalanche* avalanche );
	//! You must call Initialize after setting the algorithm.
	void Initialize();
	//! Ask whether everything is ready to rumble.
	void Verify() const; // throw (FailedVerify);
	//! Finds the start of the next avalanche.
	AvalancheAlgorithm::Status
		FindNextStart( double* H, SpinIndex *iFirstSpin );
private:
	void AllocateArrays();
	void DeleteArrays();
	void InitializeSortedSpins();
	void InitializeSortedList();
	enum { ALGORITHM = 0x1,
		INITIALIZE = 0x2
	};
	int iVerify;
	//! Cached from AvalancheAlgorithm
	const SpinLattice* lattice;
	//! Cached from AvalancheAlgorithm
	const double *pdRandomField;
	//! Cached from AvalancheAlgorithm
	const char *pcNUp;
	//! Cached from AvalancheAlgorithm
	int Z;
	//! Cached from AvalancheAlgorithm
	int N;
};

#endif
