#include <iostream>

#include "DiscreteAvalanche.h"
#include "HysteresisSimulation.h"
#include "DMException.h"
#include "NetCDFReader.h"
#include "NetCDFWriter.h"
#include "SpinLattice.h"
#include "DebugDefines.h"
#include "Timer.h"
#include "Rand.h"


int main( int argc, char *argv[] )
{
	//_crtBreakAlloc=241;
#ifndef _DEBUG
	_CrtSetReportMode( _CRT_WARN, _CRTDBG_MODE_FILE );
	_CrtSetReportFile( _CRT_WARN, _CRTDBG_FILE_STDOUT );
	_CrtSetReportMode( _CRT_ERROR, _CRTDBG_MODE_FILE );
	_CrtSetReportFile( _CRT_ERROR, _CRTDBG_FILE_STDOUT );
	_CrtSetReportMode( _CRT_ASSERT, _CRTDBG_MODE_FILE );
	_CrtSetReportFile( _CRT_ASSERT, _CRTDBG_FILE_STDOUT );
#endif //_DEBUG
#ifdef _DEBUG
	// Get the current state of the flag
	// and store it in a temporary variable
	int tmpFlag = _CrtSetDbgFlag( _CRTDBG_REPORT_FLAG );
	
	// Turn On (OR) - Keep freed memory blocks in the
	// heaps linked list and mark them as freed
//	tmpFlag |= _CRTDBG_DELAY_FREE_MEM_DF;
	tmpFlag |= _CRTDBG_ALLOC_MEM_DF; // use checking at all
//	tmpFlag |= _CRTDBG_LEAK_CHECK_DF; // end of program
	tmpFlag |= _CRTDBG_CHECK_ALWAYS_DF; // end of program

	// Turn Off (AND) - prevent _CrtCheckMemory from
	// being called at every allocation request
	tmpFlag &= ~_CRTDBG_CHECK_ALWAYS_DF;
	tmpFlag &= ~_CRTDBG_DELAY_FREE_MEM_DF;
	
	// Set the new state for the flag
	_CrtSetDbgFlag( tmpFlag );
#endif //_DEBUG
	
	int len = 153;

	try {
		SpinLattice lat;
		Rand rand;
		rand.SetSeed( 37 );
		lat.SetLength( len );
		int i,j,res;
		for (i=0; i<len*len; i++) {
			j = rand.discrete(0,1);
			res = j+j-1;
			lat.flip( i, res );
		}
		int totalUp = lat.GetTotalNumberUp();
		{
			NetCDFWriter writer("lattice.cdf");
			writer.Put( &lat, "lattice" );
		}
		SpinLattice* pLat;
		{
			NetCDFReader read("lattice.cdf");
			pLat = dynamic_cast<SpinLattice*>(read.Get( "lattice" ) );
		}
		int totalUpTwo = pLat->GetTotalNumberUp();
		for (i=0; i<len*len; i++) {
			int j,k;
			j = lat.Get(i); k = pLat->Get(i);
			ASSERT( lat.Get( i ) == pLat->Get(i) );
		}
		ASSERT( totalUp == totalUpTwo );
		delete pLat;
		
	} catch (DMException e) {
		std::cout << e;
	}
	TRACE( "Leaks before 100 aren\'t my fault, darn it.\n");
	return 0;
}