#ifndef _SWEEPLOGIC_H_
#define _SWEEPLOGIC_H_

#include "DMFirstHeader.h"
#include <vector>
#include "DiscreteAvalanche.h"
#include "AvalancheListener.h"
#include "ObserverList.h"
#include "Serializable.h"
#include "AvalancheAlgorithm.h"

class AvalancheStart;

//! Calls Avalanche methods in the correct order to increase the field.
/*! This class keeps track of the current field and number of runs.
	It calls the AvalancheAlgorithm and AvalancheStart in turn.
	
	\author ajd27
	\date 15 Aug 2000
	*/
class SweepLogic : public Serializable
{
	AvalancheAlgorithm* avalanche;
	AvalancheStart* starter;
	double startingField;
	double dField; //!< The current value of the applied field.
	int iTotalRuns; //!< How many runs have happened so far.
	//! Our list of observers.
	ObserverList<AvalancheListener> listeners;
public:
	//! Constructor.
	SweepLogic();

	//! Set the starting mag field for the simulation.
	void SetStartingField( double dField );

	//! We need a pointer to an avalanche.
	void SetAvalanche( AvalancheAlgorithm * avalanche );

	//! Returns the number of runs completed.
	int GetAvalancheCount( void ) { return iTotalRuns; }

	//! Call before doing a new avalanche
	void BeginSweep( bool bKeepData = false );

	//! Do some runs.  Quit early if we finish.
	AvalancheAlgorithm::Status Run( int SweepCount );

	//! Add an AvalancheListener.
	bool AddListener( AvalancheListener *listener );

	//! Remove an AvalancheListener.
	bool RemoveListener( AvalancheListener *listener );

	/*! Serializable interface */
	//@{
	void Save( DMWriter *writer ) const;
	void Load( DMReader *reader );
	const std::string GetType() const;
	//@}
private:
	//! Notify our AvalancheListeners that we are done.
	void Notify( double dField );
};

#endif // _SWEEPLOGIC_H_
