#include "DMFirstHeader.h"
#include <iostream>

#include "DiscreteAvalanche.h"
#include "HysteresisSimulation.h"
#include "DMException.h"
#include "Timer.h"
#include "ExceptionHandler.h"
#include "NetCDFReader.h"
#include "BatchTimer.h"
#include "NetCDFCheckpointer.h"
#include "SimpleTracer.h"
#include "DMLastHeader.h"

int main( int argc, char *argv[] )
{
	//_crtBreakAlloc=1630;
	
	bool bFromFile = false;
	bool bBatchTimer = true;
	int iLength = 5;
	int iSeed = 1;

	try {
		BatchTimer bTimer;
		bTimer.SetEndTimeMinutes( (int)(1) );
		SimpleTracer tracer;
		tracer.SetTraceFrequency( 300 );
		bTimer.SetTracer( &tracer );

		HysteresisSimulation* pSim;
//		while ((!bBatchTimer || !bTimer.IsTimeToQuit())
//			&& iLength<100000)
		{
			Timer clock;
			if (bFromFile) {
				NetCDFReader lector("start.dat");
				pSim = dynamic_cast<HysteresisSimulation*>(lector.Get("checkpoint"));
			} else {
				pSim = new HysteresisSimulation();
				pSim->SetLength( iLength );
				pSim->SetSeed( iSeed );
				pSim->SetDisorder( 0.7 );
			}
			pSim->SetBatchTimer( &bTimer );
			pSim->Initialize();
			pSim->Verify();
			clock.StartClock();

			NetCDFCheckpointer cp( pSim );
			bTimer.SetCheckpointer( &cp );
			std::cout << "Starting to run for " << iLength << " points." << std::endl;
			pSim->Run();
			clock.StopClock();
			if (bBatchTimer && bTimer.IsTimeToQuit()) {
				std::cout << iLength << " killed after " << clock.TotalTime() << std::endl;
			} else {
				std::cout << iLength << " \t" << clock.TotalTime() << std::endl;
			}
			pSim->SaveResults();
			delete pSim;
		//	iLength *= 2;
			++iSeed;
		}
	} catch (DMException e) {
		std::cout << e;
	}
	TRACE( "Leaks before 100 aren\'t my fault, darn it.\n");
	return 0;
}