#ifndef _HYSTERESIS_CANVAS_H_
#define _HYSTERESIS_CANVAS_H_

#include <vector>
#include "wx/wx.h"
#include "ColoringObserver.h"

class HysteresisCanvas : public wxScrolledWindow
{
	const ColoringObserver* m_Coloring;
	int m_CellSize;

	// sc stands for SpinCount
	int m_scBoardX;
	int m_scBoardY;

	static const int m_ColorCnt;

	struct Stylus {
		wxPen* pen;
		wxBrush* brush;
	};

	std::vector<Stylus> Colors;

public:
	HysteresisCanvas( const ColoringObserver* coloring, wxWindow* parent,
		wxWindowID, const wxPoint& pos, const wxSize& size );
	virtual ~HysteresisCanvas();

	int GetCellSize() const { return m_CellSize; }
	void SetCellSize( int cellsize );
	void Recenter( wxInt32 i, wxInt32 j );

	void DrawCell( wxInt32 i, wxInt32 j, int color );
	void DrawCell( wxInt32 i, wxInt32 j, wxDC& dc );

private:
	DECLARE_EVENT_TABLE();

	void OnPaint( wxPaintEvent& event );
	void OnEraseBackground( wxEraseEvent& event );

	// Cell coordinates are i, j.
	// Screen coordinates are x, y.
	wxInt32 XToCell( wxCoord x ) const {
		return (x / m_CellSize);
	}

	wxInt32 YToCell( wxCoord y ) const {
		return (y / m_CellSize );
	}

	wxCoord CellToX( wxInt32 i ) const {
		return i * m_CellSize;
	}

	wxCoord CellToY( wxInt32 j ) const {
		return j * m_CellSize;
	}
};


#endif // _HYSTERESIS_CANVAS_H_
