/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
#include "DMFirstHeader.h"
#include <iostream>
#include "DigitalMaterial.h"
#include "MDMain.h"

#include "DMLastHeader.h"


using namespace std;

void main(int argc, char **argv)
{
try
	{
#ifdef WIN32
	WinProcessFactory::Create();
#else
	UnixProcessFactory::Create();
#endif

#if DMPARALLEL == 1
	MPI_Init(&argc, &argv);

	//MPIProtocol protocol;
	//dmProtocol = &protocol;

	ProtocolFactory::Instance()->SetProtocol(ProtocolFactory::World);
#endif	
	int nProcs = dmProtocol->GetNumberOfProcessors();
	int myProc = dmProtocol->GetProcessorNumber();

	cout.precision(12);
	//DynamicListOfAtoms *dyn = new DynamicListOfAtoms(10);
	//cout << "Dyn: " << dyn->GetNumber() << endl;
	
	// get a filename

	// Create a CheckPointRestarter using the filename.

	CheckPointRestarter restarter("siliconCube.00006");

	// Call recreate
	restarter.Recreate();

	DynamicListOfAtoms *atoms = dynamic_cast<DynamicListOfAtoms*> (restarter.GetListOfAtoms());
	StillingerWeberPotential *potential = dynamic_cast<StillingerWeberPotential*> (restarter.GetPotential());
	LangevinAtomsMover *mover = dynamic_cast<LangevinAtomsMover*> (restarter.GetMover());
	CellNeighborList *neighborLocator = dynamic_cast<CellNeighborList*> (restarter.GetNeighborLocator());
	SimplePeriodicBoundaryConditions *pbc = dynamic_cast<SimplePeriodicBoundaryConditions*> (restarter.GetBoundaryConditions());
	
	int nMajorSteps = 3;
	for ( int step = 0; step < nMajorSteps; ++step )
	{
		mover->Move(atoms);
		double potentialEnergy = potential->CalculateEnergy(atoms);
		double kineticEnergy = atoms->GetKineticEnergy();
		if( 0 == myProc )
		{
			cout << "Potential Energy = " << potentialEnergy << endl;
			cout << "Kinetic Energy = " << kineticEnergy << endl;
			cout << "Total Energy = " << kineticEnergy + potentialEnergy << endl;
		}

	}

	//  attach a Plot observer

	// Run and plot energy



	delete atoms;
	delete pbc;
	delete neighborLocator;
	delete potential;

	}
	catch (DMException e){
		std::cout << e;}

#if DMPARALLEL == 1
	MPI_Finalize();
#endif
}
