#ifndef PARALLELEPIPEDCLUSTERINITIALIZER_H
#define PARALLELEPIPEDCLUSTERINITIALIZER_H
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
#include "DMFirstHeader.h"

#include "ClusterInitializer.h"
#include "math.h"

//! Creates a cluster in the form of a parallelepiped according to the vectors passed

/*!

  The cluster is centered using the SetCenter variable as usual. The corners will 
  be the center plus/minus the three boundary vectors. The format of the Boundary
  Vectors is as for SuperCellVectors in SkewPeriodicBoundaryConditions (allowing
  the latter to be directly used here) namely the coord index is first and the
  vector index second.


  \author nbailey
  \date August 2001

 */

#if DIMENSION == 3

class ParallelepipedClusterInitializer : public ClusterInitializer
{
public:
	ParallelepipedClusterInitializer(const double boundaryVectors[DIMENSION][DIMENSION],
                                     BravaisLatticeWithBasis *bravais);
	virtual ~ParallelepipedClusterInitializer();
	void SetBoundaryVectors(const double boundaryVectors[DIMENSION][DIMENSION]); 
    //const double (&GetBoundaryVectors() const)[DIMENSION][DIMENSION] {return boundaryVectors;}
	
protected:
	double boundaryVectors[DIMENSION][DIMENSION];
	double inverseBoundaryVectors[DIMENSION][DIMENSION];
	double volume;
	int SetMaxSize();
	bool Inside(double X[DIMENSION]);
};

#endif // DIMENSION == 3

#endif // !defined(PARALLELEPIPEDCLUSTERINITIALIZER_H)
