// CompositeAtomsMover.h: interface for the CompositeAtomsMover class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef COMPOSITEATOMSMOVER_H
#define COMPOSITEATOMSMOVER_H


#include "DigitalMaterial.h"
#include "AtomsMover.h"


class CompositeAtomsMover : public AtomsMover  
{
public:
	CompositeAtomsMover(AtomsMover *mover1,AtomsMover *mover2,double tolerance);
	~CompositeAtomsMover();
	void Move(ListOfAtoms *atoms);
	
	void SetInterestingLeaves(int numberOfInterestingLeaves1, int numberOfInterestingLeaves2, int *interestingLeaves1, int *interestingLeaves2);
	
	double GetTolerance() const {return tolerance;}
	void SetTolerance(double t) {tolerance = t;}
	int GetMaxSteps() const {return maxSteps;}
	void SetMaxSteps(int n) {maxSteps = n;}
	
	void Verbose() {verbose = true;}
	void Quiet() {verbose = false;}
	
	int GetNIterations() const {return nIterations;}

	
	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);	
	const std::string &GetType() const
		{
			return className;
		}
	
private:
	// eventually could have an arbitrary numberof movers.
	AtomsMover *mover1;
	AtomsMover *mover2;

	//! The shape of the ghosts
	Shape ghostShape1;
	Shape ghostShape2;

	// which leaves on each branch are interesting to the other branch
	int numberOfInterestingLeaves1;
	int numberOfInterestingLeaves2;
	int *interestingLeaves1;
	int *interestingLeaves2;
	

	int nParameters;
	double tolerance;
	int maxSteps;
	int nAtoms;
	int nIterations;
	bool verbose;
	static const std::string className;
};

#endif //  COMPOSITEATOMSMOVER_H
