// MinimizerAtomsMover.h: interface for the MinimizerAtomsMover class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef MINIMIZERATOMSMOVER_H
#define MINIMIZERATOMSMOVER_H

#include <math.h>
#include "DigitalMaterial.h"
#include "AtomsMover.h"
#include "ListOfAtoms.h"
#include "DMArray.h"
#include "Minimizer.h"
#include "Minimizable.h"
class BoundaryConditions;
class NeighborLocator;
class ExternalWork;

//! Performs ConjugateGradient minimization on a ListOfAtoms.
/*! Needs to be passed a potential when constructed.
	*/

class MinimizerAtomsMover : public AtomsMover
{
public:
	void Move(ListOfAtoms *atoms);
	MinimizerAtomsMover(Minimizer *minimizer = 0, Potential *potential = 0,ExternalWork *work = 0);
	virtual ~MinimizerAtomsMover();
	Minimizer *GetMinimizer() {return minimizer;}

	void Verbose() {verbose = true;}
	void Quiet() {verbose = false;}
	

#ifndef SWIG
	class AtomsMinimizable : public Minimizable
	{
	public:
		AtomsMinimizable(ListOfAtoms *atoms,Potential *potential,ExternalWork *work, bool verbose) 
			{
				this->atoms = atoms;
				// ajd27 forces are a sibling to the positions.
				forces = new DMArray<double, DIMENSION>(
					(atoms->GetCartesianPositions()));
				this->potential = potential;
				this->work = work;
				bc = atoms->GetBoundaryConditions();
				neighborLocator = atoms->GetNeighborLocator();
				gradients = new double[atoms->GetNumber()*DIMENSION];
				this->verbose = verbose;
			}
		~AtomsMinimizable()
			{
				delete gradients;
				delete forces; 
			}
		double ObjectiveFunction(double *parameters);
		double *Gradient(double *parameters);
		int GetNParameters() {return atoms->GetNumber()*DIMENSION;}
	private:
		ListOfAtoms *atoms;
		DMArray<double, DIMENSION> *forces;
		double *gradients;
		Potential *potential;
		const BoundaryConditions *bc;
		NeighborLocator *neighborLocator;
		ExternalWork *work;
		bool verbose;
	};
	Minimizable *atomsMinimizable;


	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	const std::string &GetType() const
		{
			return className;
		}
#endif // SWIG

private:
	Minimizer 	*minimizer;
	Potential *potential;
	int nIterations;
	ExternalWork *work;
	static const std::string className;
	bool verbose;
};

#endif // !defined(MINIMIZERATOMSMOVER_H)
