// Prefactor.h: interface for the Prefactor class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef PREFACTOR_H
#define PREFACTOR_H

#include "DigitalMaterial.h"

class ListOfAtoms;
class Potential;
class AtomsExtension;

// functions to ask how many zero/positive/negative eigenvalues
// possibly will have a function for vibrational entropy and the prefactor will be a combination of two calls to that function (one for saddle one for minimum)


class Prefactor
{
public:
	class AsymmetryError
    {
    public:
		AsymmetryError(double mxAsym,int nAsym) : maxAsymmetry(mxAsym),nAsymmetric(nAsym) {}
		double maxAsymmetry;
		int nAsymmetric;
    };
	Prefactor(Potential *potential,double eps = 0.00001, AtomsExtension *atomsExtension = 0);
	~Prefactor();
	void SetEpsilon(double eps) {epsilon = eps;}
	double GetEpsilon() const{return epsilon;}
	double CalculatePrefactor(ListOfAtoms *minimum, ListOfAtoms *saddle);

	// if using AtomsExtension
	void SetParameters(const double *parametersMin,const double *parametersSadd)
		{ 
			this->parametersMin = parametersMin;
			this->parametersSadd = parametersSadd;
		}

protected:
	void CalculateHessian(int nDOFs, double *hessian, ListOfAtoms *atoms, DMArray<double, DIMENSION> *forces, double *reducedForcesPlus, double *reducedForcesMinus);
	void CalculateExtendedHessian(int nDOFs, double *hessian, ListOfAtoms *atoms, const double *parameters, DMArray<double, DIMENSION> *forces, double *reducedForcesPlus, double *reducedForcesMinus);
	int CheckSymmetry(int size, double *matrix, double tol);
	double CalculateMassGradientFactor(double *hessSadd, int nFreeAtomsDOFs, int unstableIdx, double *currentForces, double *saddForceSq);
	Potential *potential;
	int nParameters;
	AtomsExtension *atomsExtension;
	double *invMassMatrix;
	double epsilon;
	const double *parametersMin;
	const double *parametersSadd;
	double maxAsymmetry;
};



#endif // PREFACTOR_H
