// BoundaryConditions.h: interface for the BoundaryConditions class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef BOUNDARYCONDITIONS_H
#define BOUNDARYCONDITIONS_H


// Nick Bailey
// I haven't figured out who should own this class/have pointers to it. NeighborLocator
// should at least have a pointer to it. Force calculators also need to be able to access it

// Default is free boundary conditions

#include <iostream>
#include "DigitalMaterial.h"
#include "Cloneable.h"
#include "Serializable.h"

template<class Type, int N> class DMArray;

class BoundaryConditions  : public Cloneable, public Serializable
{
public:
	BoundaryConditions(){ }
	virtual ~BoundaryConditions(){ /* cout << "BoundaryConditions deleted" << endl; */ }
	virtual void EnforceBoundaryConditions(DMArray<double, DIMENSION> *positions) const=0;
	virtual void EnforceBoundaryConditions(double *position) const=0; // Can we have (*)[DIMENSION] ?
	virtual void DifferenceBoundaryConditions(DMArray<double, DIMENSION> *displacements) const=0;
	virtual void DifferenceBoundaryConditions(double (*displacements)[DIMENSION],int nAtoms) const=0;
	virtual BoundaryConditions *Copy() const=0;
};

#endif // !defined(BOUNDARYCONDITIONS_H)
