// PeriodicBoundaryConditions.h: interface for the PeriodicBoundaryConditions class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef PERIODICBOUNDARYCONDITIONS_H__
#define PERIODICBOUNDARYCONDITIONS_H__

#include "BoundaryConditions.h"

#include "DMArray.h"
#include "DMWriter.h"
#include "DMReader.h"
#include "DMException.h"

class PeriodicBoundaryConditions : public BoundaryConditions
{
public:
	// overriding BoundaryConditions methods
	PeriodicBoundaryConditions() {}
	virtual ~PeriodicBoundaryConditions(){}
	virtual void EnforceBoundaryConditions(DMArray<double, DIMENSION> *positions) const =0;
	virtual void DifferenceBoundaryConditions(DMArray<double, DIMENSION> *positions) const=0;
	virtual void DifferenceBoundaryConditions(double (*displacements)[DIMENSION],int nAtoms) const=0;
	virtual void EnforceBoundaryConditions(double *position) const =0;
	// Interface special to PeriodicBoundaryConditions
	virtual void SetLength(const double l[DIMENSION]) = 0;
	double GetLength(int coord) const {return length[coord];}
	const double *GetLength() const {return (const double *)length;}

	virtual void Save(DMWriter *writer) const 
	{
		std::vector<int> dim;
		dim.push_back(DIMENSION);
		writer->PutDoubleArray(length, dim, "length");
	}
	virtual void Load(DMReader *reader)
	{
		std::vector<int> dim;
		double *data = reader->GetDoubleArray(dim, "length");
		if((dim.size() != 1) || (dim[0] != DIMENSION)) 
			throw DMException("PeriodicBoundaryConditions", "Load", "Problem while reading length");
		for(int coord=0; coord<DIMENSION; ++coord) length[coord] = data[coord];
		delete data;
	}

protected:
	double length[DIMENSION];
};


#endif // !defined(PERIODICBOUNDARYCONDITIONS_H__)
