// SimplePeriodicBoundaryConditions.h: interface for the SimplePeriodicBoundaryConditions class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef SIMPLEPERIODICBOUNDARYCONDITIONS_H
#define SIMPLEPERIODICBOUNDARYCONDITIONS_H

#include "PeriodicBoundaryConditions.h"

#include "DigitalMaterial.h"

#include "DMArray.h"

class SimplePeriodicBoundaryConditions : public PeriodicBoundaryConditions  
{
public:
	// interface overridden from BoundaryConditions
	SimplePeriodicBoundaryConditions();
	virtual ~SimplePeriodicBoundaryConditions();

	void EnforceBoundaryConditions(DMArray<double, DIMENSION> *positions) const ;
#ifndef SWIG
	SimplePeriodicBoundaryConditions(const double l[DIMENSION]);
	void DifferenceBoundaryConditions(double (*displacements)[DIMENSION],int nAtoms) const;
	void EnforceBoundaryConditions(double *position) const;
	void DifferenceBoundaryConditions(DMArray<double, DIMENSION> *displacements) const;

	void Save(DMWriter *writer) const {PeriodicBoundaryConditions::Save(writer);}
	void Load(DMReader *reader) {PeriodicBoundaryConditions::Load(reader);}
	//! Get the string name of the class for saving or loading.
#endif // SWIG
	const std::string &GetType() const 
	{
		return className;	
	}

	
	
	// interface overridden from PeriodicBoundaryConditions
	void SetLength(const double l[DIMENSION])
	{
		// somewhere it should be checked that the length of the box is not too 
		// small compared with the cutoffs used in the simulation (maybe at outer level)
		for (int coord = 0; coord < DIMENSION; coord++)
		{
			length[coord] = l[coord];
		}
	} 
	BoundaryConditions *Copy() const;
private:
	static const std::string className;
};

#endif // !defined(SIMPLEPERIODICBOUNDARYCONDITIONS_H)
