// SmartPeriodicBoundaryConditions.h: interface for the SmartPeriodicBoundaryConditions class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef SMARTPERIODICBOUNDARYCONDITIONS_H
#define SMARTPERIODICBOUNDARYCONDITIONS_H

#include "DigitalMaterial.h"

#include "PeriodicBoundaryConditions.h"
#include "DMArray.h"
/**
 * This is a "smarter" periodic boundary conditions class where you can choose not to have 
 * the periodicity along some directions. This is specified with the "wrap" variable in the 
 * constructor. We can also define the center of the box, of set its corners.
 * \date 11 May 2000
 * \author Thierry Cretegny
 * \ingroup moleculardynamics
**/
class SmartPeriodicBoundaryConditions : public PeriodicBoundaryConditions  
{
public:
	SmartPeriodicBoundaryConditions(const bool wrap[DIMENSION] = 0);

	virtual ~SmartPeriodicBoundaryConditions();

	void SetBoxCenter(const double center[DIMENSION]);
	void SetLength(const double length[DIMENSION]);
	void SetLowerEdge(const double lowerEdges[DIMENSION]);
	void SetUpperEdge(const double upperEdges[DIMENSION]);
	bool GetWrap(int coord) const {return wrap[coord];}

	virtual void EnforceBoundaryConditions(DMArray<double ,DIMENSION> *positions) const;
	virtual void EnforceBoundaryConditions(double *position) const;
	void DifferenceBoundaryConditions(double (*displacements)[DIMENSION],int nAtoms) const;
	void DifferenceBoundaryConditions(DMArray<double, DIMENSION> *displacements) const;

	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	//! Get the string name of the class for saving or loading.
	const std::string &GetType() const
	{
		return className;
	}
	BoundaryConditions *Copy() const;
private:
	bool wrap[DIMENSION];

	double lowerEdge[DIMENSION];
	double upperEdge[DIMENSION];
	double center[DIMENSION];
	static const std::string className;
};

#endif // !defined(SMARTPERIODICBOUNDARYCONDITIONS_H
