// 2DLattices.cpp: implementations for the 2DLattices
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DMFirstHeader.h"

#include "DigitalMaterial.h"

#if DIMENSION == 2

#include "2DLattices.h"
#include "SerialBuilder.h"
#include "DMWriter.h"
#include "DMReader.h"

#include "DMLastHeader.h"


static SerialBuilder<SquareLattice> *_SquareLatticecreator = 
    new SerialBuilder<SquareLattice>("SquareLattice");

static SerialBuilder<TriangularLattice> *_TriangularLatticecreator = 
    new SerialBuilder<TriangularLattice>("TriangularLattice");


const std::string SquareLattice::className = std::string("SquareLattice");

const std::string TriangularLattice::className = std::string("TriangularLattice");

///////////////////  SQUARE LATTICE  /////////////////////////



SquareLattice::SquareLattice(double a)
{
	latticeConstant = a;
    for(int vec=0; vec<DIMENSION; vec++)
    {
        for(int coord=0; coord<DIMENSION;coord++)
            latticeVectors[vec][coord] = (vec==coord? a : 0.);
    }
    UpdateMemberVariables();
}

void SquareLattice::SetLatticeConstant(double a)
{
    double factor = a/latticeConstant;
    for(int vec=0; vec<2; vec++)
    {
        for(int coord=0; coord<DIMENSION; coord++)
            latticeVectors[vec][coord] *= factor;
    }
    latticeConstant = a;
    UpdateMemberVariables();
}


///////////////////  TRIANGULAR LATTICE  /////////////////////////

TriangularLattice::TriangularLattice(double a)
{
	latticeConstant = a;
    latticeVectors[0][0] = a;
    latticeVectors[0][1] = 0.;
    latticeVectors[1][0] = 0.5*a;
    latticeVectors[1][1] = 0.5*sqrt(3.)*a;
    UpdateMemberVariables();
}

void TriangularLattice::SetLatticeConstant(double a)
{
    double factor = a/latticeConstant;
    for(int vec=0; vec<2; vec++)
    {
        for(int coord=0; coord<DIMENSION; coord++)
            latticeVectors[vec][coord] *= factor;
    }
    latticeConstant = a;
    UpdateMemberVariables();
}

#endif // DIMENSION == 2
