
//     Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
//     Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

//     This library is free software; you can redistribute it and/or modify it
//     under the terms of the GNU Lesser General Public License as published by the
//     Free Software Foundation; either version 2.1 of the License, or (at your
//     option) any later version.

//     This library is distributed in the hope that it will be useful, but WITHOUT
//     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//     FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
//     for more details.

//     You should have received a copy of the GNU Lesser General Public License
//     along with this library; if not, write to the Free Software Foundation,
//     Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
//     http://www.gnu.org/licenses/lgpl.html

//     Author Contact Information:
//     Jim Sethna      
//     sethna@lassp.cornell.edu  
//     117 Clark Hall
//     Cornell University
//     Ithaca, NY 14850


%{
#include "BravaisLatticeWithBasis.h"
#if DIMENSION == 3
#include "3DLattices.h"
#endif //DIMENSION == 3
#if DIMENSION == 2
#include "2DLattices.h"
#endif //DIMENSION == 2
%}

%include "BravaisLatticeWithBasis.h"
%extend BravaisLatticeWithBasis
{
	PyObject *LatticeCoordFromCartCoord(double cartesian[DIMENSION])
	{
		int dims = DIMENSION;
	    PyObject *o = PyArray_FromDims(1, &dims, 'd');
		double *ptr = (double *)(((PyArrayObject *)o)->data); 
		self->LatticeCoordinatesFromCartesianCoordinates(ptr, cartesian); 
		return o;
	}
	PyObject *CartCoordFromLatticeCoord(double latticeCoord[DIMENSION])
	{
		int dims = DIMENSION;
	    PyObject *o = PyArray_FromDims(1, &dims, 'd');
		double *ptr = (double *)(((PyArrayObject *)o)->data); 
		self->CartesianCoordinatesFromLatticeCoordinates(ptr, latticeCoord); 
		return o;
	}
	PyObject *GetOrigin()
	{
		int dims = DIMENSION;
	    PyObject *o = PyArray_FromDims(1, &dims, 'd');
		memcpy(((PyArrayObject *)o)->data, self->GetOrigin(), DIMENSION*sizeof(double));
		return o;		
	}
	PyObject *GetLatticeVectors()
	{
		int dims[2] = {DIMENSION, DIMENSION};
	    PyObject *o = PyArray_FromDims(2, dims, 'd');
		memcpy(((PyArrayObject *)o)->data, self->GetLatticeVectors(),
			DIMENSION*DIMENSION*sizeof(double));
		return o;		
	}

    PyObject *GetAtomsCoordinatesInCell(int atom)
	{
		int dims = DIMENSION;
	    PyObject *o = PyArray_FromDims(1, &dims, 'd');
		const double *tmp = self->GetAtomsCoordinatesInCell(atom); 
		memcpy(((PyArrayObject *)o)->data, tmp, DIMENSION*sizeof(double));
		return o;
	}

	void SetAtomsCoordinatesInCell(PyObject *atomCoords, PyObject *typeNames = 0)
	{	
		std::vector<std::string> *types = 0;
		if(typeNames != 0)
		{
			if(!PyList_Check(typeNames))
			{
				PyErr_SetString(PyExc_TypeError,"typeNames should be a list or None");
				return;
			}
			types = new std::vector<std::string> ;
			const int listSize = PyList_Size(typeNames);
			PyObject *o;
			for(int i=0; i<listSize; ++i)
			{
				o = PyList_GetItem(typeNames,i);
				if(!PyString_Check(o))
				{
					PyErr_SetString(PyExc_TypeError,"an item in typeNames isn't a list");
					return;
				}
				types->push_back(PyString_AsString(o));
			}
		
		}
		if(!PyArray_Check(atomCoords))
			PyErr_SetString(PyExc_TypeError,"non-array passed to SetAtomsCoordinatesInCell");
		else
		{
			PyArrayObject *o = (PyArrayObject *)(atomCoords);
			if(o->nd != 2)
			PyErr_SetString(PyExc_TypeError,"wrong number of dimensions in array passed to SetAtomsCoordinatesInCell");
			if(o->dimensions[1] != DIMENSION)
				PyErr_SetString(PyExc_TypeError,"wrong size of second dimension (should be DIMENSION) in array passed to SetAtomsCoordinatesInCell");
			double (*ptr)[DIMENSION] = (double (*)[DIMENSION])(((PyArrayObject *)o)->data);
			std::vector<TinyVector<double, DIMENSION> > coordinates;
			for(int i=0; i<o->dimensions[0]; ++i) coordinates.push_back(ptr[i]);	
			self->SetAtomsCoordinatesInCell(coordinates, types);
		}
		if(types) delete types;
	}	

}

# we provide our own constructor for GeneralLattices
%extend GeneralLattice
{
	GeneralLattice(PyObject *o = 0)
	{
		if(0 == o) return new GeneralLattice();
		if(PyFloat_Check(o)) return new GeneralLattice(PyFloat_AsDouble(o));
		if(PyArray_Check(o))
		{
			PyArrayObject *arr = (PyArrayObject *)(o);
			if(arr->nd == 2 && arr->dimensions[0] == DIMENSION  && arr->dimensions[1] == DIMENSION
				&& arr->descr->type == 'd')
					return new GeneralLattice((double (*)[DIMENSION]) (arr->data));
		
		}
		PyErr_SetString(PyExc_TypeError,"not a NumPy Array or a double");
		return NULL;
	}
}


#if DIMENSION == 3
class CubicLattice: public BravaisLatticeWithBasis
{
public:
	CubicLattice(double a);
	void SetLatticeConstant(double a);
	double GetLatticeConstant();
};

class FCCLattice: public CubicLattice
{
public:
	FCCLattice(double a);
};

class BCCLattice: public CubicLattice
{
public:
	BCCLattice(double a);
};

class PrimitiveFCCLattice: public BravaisLatticeWithBasis
{
public:
	PrimitiveFCCLattice(double a);
	void SetLatticeConstant(double a);
	double GetLatticeConstant();
};

class DiamondLattice: public FCCLattice
{
public:
	DiamondLattice(double a);
};

class TriclinicLattice: public GeneralLattice
{
public:
    TriclinicLattice(const double latticeConstants[DIMENSION],
                     const double anglesInUnitCell[DIMENSION]);

    void SetLatticeConstants(const double latticeConstants[DIMENSION]);
    void SetAnglesInUnitCell(const double anglesInUnitCell[DIMENSION]);

    PyObject *GetLatticeConstants()
	{
		int dims = DIMENSION;
	    PyObject *o = PyArray_FromDims(1, &dims, 'd');
		const double (&tmp)[DIMENSION] = self->GetLatticeConstants();
		memcpy(((PyArrayObject *)o)->data, tmp, DIMENSION*sizeof(double));
		return o;
	}

    PyObject *GetAnglesInUnitCell()
	{
		int dims = DIMENSION;
	    PyObject *o = PyArray_FromDims(1, &dims, 'd');
		const double (&tmp)[DIMENSION] = self->GetAnglesInUnitCell();
		memcpy(((PyArrayObject *)o)->data, tmp, DIMENSION*sizeof(double));
		return o;
	}
};

#endif 

#if DIMENSION == 2

class SquareLattice: public BravaisLatticeWithBasis
{
public:
    SquareLattice(double a=1.0);
    void SetLatticeConstant(double a);
    double GetLatticeConstant() const {return latticeConstant;}
};

class TriangularLattice: public BravaisLatticeWithBasis
{
public:
    TriangularLattice(double a=1.0);
    void SetLatticeConstant(double a);
    double GetLatticeConstant() const {return latticeConstant;}
};

#endif
