// AndersonDynamics.cpp: implementation of the AndersonDynamics class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"

#include "AndersonDynamics.h"
#include "DMArray.h"

#include "DMLastHeader.h"

const std::string AndersenDynamics::className = std::string("AndersenDynamics");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


AndersenDynamics::AndersenDynamics(double frequencyDt, double sigma, int seed) : 
	rand(seed), sigma(sigma), frequencyDt(frequencyDt)
{
}

void AndersenDynamics::AdjustVelocities(DMArray<double, DIMENSION> *velocities)
{
	DMArrayIterator<double, DIMENSION> veloIter(velocities);
	int coord;
	for(veloIter.Init(); !veloIter.IsDone(); veloIter.Next())
	{
		for(coord=0; coord<DIMENSION; ++coord)
			if(rand.uniform() < frequencyDt) veloIter.Current()[coord] = rand.gaussian(sigma);
	}
}
