// PartialFixedBody.cpp: implementation of the PartialFixedBody class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"

#include "PartialFixedBody.h"
#include "DMArray.h"

#include "DMLastHeader.h"
#include "ListOfAtoms.h"


const std::string PartialFixedBody::className = std::string("PartialFixedBody");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////



PartialFixedBody::PartialFixedBody(const DMArray<double, DIMENSION> *positions, const int directions[DIMENSION])
{
	fixedPositions = 0;
	SetFixedPositions(positions);
	for(int i = 0; i < DIMENSION; i++)
		totalForce[i] = 0.0;
	SetFixedDirections(directions);
}

PartialFixedBody::~PartialFixedBody()
{
	if(fixedPositions) delete fixedPositions;
}

Constraint *PartialFixedBody::Copy() const
{
	return new PartialFixedBody(fixedPositions,fixedDirections);
}

void PartialFixedBody::SetFixedPositions(const DMArray<double, DIMENSION> *positions)
{
	if(fixedPositions) delete fixedPositions;
	fixedPositions = new DMArray<double, DIMENSION> (positions);
	fixedPositions->Copy(positions);
}

void PartialFixedBody::SetFixedDirections(const int directions[DIMENSION])
{
	for(int coord=0;coord<DIMENSION;coord++)
		fixedDirections[coord] = directions[coord];
}

void PartialFixedBody::AdjustPositions(DMArray<double, DIMENSION> *positions)
{
	if(!(positions->GetShape() == fixedPositions->GetShape()))
		throw DMException("PartialFixedBody","AdjustPositions","Shape of positions differs from that of fixedPositions");
	ASSERT(positions->GetShape() == fixedPositions->GetShape());
	for(int i=0; i < atoms->GetNumber(); i++){
		for(int coord=0;coord<DIMENSION;coord++){
			if(fixedDirections[coord]){
				(*positions)[i][coord] = (*fixedPositions)[i][coord];
			}
		}
	}	  
}

void PartialFixedBody::AdjustForces(DMArray<double, DIMENSION> *forces)
{
	forces->Sum(totalForce);
	for(int i=0; i < atoms->GetNumber(); i++){
		for(int coord=0;coord<DIMENSION;coord++){
			if(fixedDirections[coord]){
				(*forces)[i][coord]=0.0;
			}
		}
	}
}

double PartialFixedBody::GetTotalForce(int coord)
{
	return totalForce[coord];
}

int PartialFixedBody::NumberGeneralizedCoordinates()
{
	return 0;
}

void PartialFixedBody::SetGeneralizedCoordinates(double *generalizedCoordinates)
{

}


void PartialFixedBody::GetGeneralizedCoordinates(double *generalizedCoordinates)
{

}


void PartialFixedBody::IncrementGeneralizedCoordinate(int n,double inc)
{
	
}


void PartialFixedBody::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces)
{

}

void PartialFixedBody::DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION])
{

}
