// PartialFixedCOM.cpp: implementation of the PartialFixedCOM class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"

#include "PartialFixedCOM.h"
#include "ListOfAtoms.h"
#include "DMArray.h"

#include "DMLastHeader.h"


const std::string PartialFixedCOM::className = std::string("PartialFixedCOM");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


PartialFixedCOM::PartialFixedCOM(bool fixedDir[DIMENSION], const double center[DIMENSION])
{
	nFixed = 0;
	for(int coord=0 ; coord<DIMENSION; ++ coord) 
	{
		this->fixedDir[coord] = fixedDir[coord];
		if (fixedDir[coord])
			++nFixed;
	}
    this->SetCenter(center);
}


PartialFixedCOM::PartialFixedCOM(bool fixedDir[DIMENSION], const DMArray<double, DIMENSION> *initialPositions)
{
	double avg[DIMENSION];
	initialPositions->Average(avg);
	nFixed = 0;
	for(int coord=0 ; coord<DIMENSION; ++ coord) 
	{
		this->fixedDir[coord] = fixedDir[coord];
		if (fixedDir[coord])
			++nFixed;
		this->center[coord] = avg[coord];
	}
}

void PartialFixedCOM::SetCenter(const double center[DIMENSION])
{
    for(int coord=0 ; coord<DIMENSION; ++ coord) 
        this->center[coord] = center[coord];
}

void PartialFixedCOM::AdjustPositions(DMArray<double, DIMENSION> *positions)
{
    double avg[DIMENSION];
    positions->Average(avg);
    for(int coord=0; coord<DIMENSION; ++coord)
	{
		if(fixedDir[coord])
			avg[coord] = center[coord] - avg[coord];
		else
			avg[coord] = 0.;
	}
    positions->Increment(avg);
}


int PartialFixedCOM::NumberGeneralizedCoordinates()
{
    return DIMENSION*atoms->GetNumber() - nFixed;
}

void PartialFixedCOM::SetGeneralizedCoordinates(double *generalizedCoordinates)
{
    throw DMException("PartialFixedCOM","SetGeneralizedCoordinates","Not Implemented Yet!");
}


void PartialFixedCOM::GetGeneralizedCoordinates(double *generalizedCoordinates)
{
    throw DMException("PartialFixedCOM","GetGeneralizedCoordinates","Not Implemented Yet!");
    
}


void PartialFixedCOM::IncrementGeneralizedCoordinate(int n,double inc)
{
    throw DMException("PartialFixedCOM","IncrementGeneralizedCoordinate",
        "Not Implemented Yet!");
}


void PartialFixedCOM::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces,
                                                   double *generalizedForces)
{
    throw DMException("PartialFixedCOM","CalculateGeneralizedForces","Not Implemented Yet!");
}

void PartialFixedCOM::DerivativesWRTGeneralizedCoordinates(int atom,
                                                             double (*gradient)[DIMENSION])
{
    throw DMException("PartialFixedCOM","DerivativesWRTGeneralizedCoordinates",
        "Not Implemented Yet!");
}
