// UniformlyMovingCenterOfMass.cpp: implementation of the UniformlyMovingCenterOfMass class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"

#include "UniformlyMovingCenterOfMass.h"
#include "DMArray.h"
#include "ListOfAtoms.h"

#include "DMLastHeader.h"

const std::string UniformlyMovingCenterOfMass::className = std::string("UniformlyMovingCenterOfMass");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

UniformlyMovingCenterOfMass::UniformlyMovingCenterOfMass(const double velocity[DIMENSION])
{
    this->SetVelocity(velocity);
}

void UniformlyMovingCenterOfMass::SetVelocity(const double velocity[DIMENSION])
{
    for(int coord=0 ; coord<DIMENSION; ++coord) 
        this->velocity[coord] = velocity[coord];
}

void UniformlyMovingCenterOfMass::AdjustVelocities(DMArray<double, DIMENSION> *velocities)
{
    double avg[DIMENSION];
    velocities->Average(avg);
    for(int coord=0; coord<DIMENSION; ++coord)
        avg[coord] = velocity[coord] - avg[coord];
    velocities->Increment(avg);
}

int UniformlyMovingCenterOfMass::NumberGeneralizedCoordinates()
{
    return DIMENSION*(atoms->GetCartesianPositions()->Size() - 1);
}

void UniformlyMovingCenterOfMass::SetGeneralizedCoordinates(double *generalizedCoordinates)
{
    throw DMException("UniformlyMovingCenterOfMass","SetGeneralizedCoordinates","Not Implemented Yet!");
}


void UniformlyMovingCenterOfMass::GetGeneralizedCoordinates(double *generalizedCoordinates)
{
    throw DMException("UniformlyMovingCenterOfMass","","Not Implemented Yet!");
}


void UniformlyMovingCenterOfMass::IncrementGeneralizedCoordinate(int n,double inc)
{
    throw DMException("UniformlyMovingCenterOfMass","IncrementGeneralizedCoordinate","Not Implemented Yet!");
}


void UniformlyMovingCenterOfMass::CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces)
{
    throw DMException("UniformlyMovingCenterOfMass","CalculateGeneralizedForces","Not Implemented Yet!");
}

void UniformlyMovingCenterOfMass::DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION])
{
    throw DMException("UniformlyMovingCenterOfMass","DerivativesWRTGeneralizedCoordinates","Not Implemented Yet!");
}





