// UniformlyMovingCenterOfMass.h: interface for the UniformlyMovingCenterOfMass class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef UNIFORMLYMOVINGCENTEROFMASS_H
#define UNIFORMLYMOVINGCENTEROFMASS_H

#include "DigitalMaterial.h"
#include "Constraint.h"

class UniformlyMovingCenterOfMass: public Constraint
{
public:
	UniformlyMovingCenterOfMass(const double velocity[DIMENSION]);
	void AdjustVelocities(DMArray<double, DIMENSION> *velocities);
	void SetVelocity(const double velocity[DIMENSION]);

	// GeneralizedCoordinate functions: most throw errors right now (March 01)
	int NumberGeneralizedCoordinates();
	void SetGeneralizedCoordinates(double *generalizedCoordinates) ;
    void GetGeneralizedCoordinates(double *generalizedCoordinates); 
	void IncrementGeneralizedCoordinate(int n,double inc);
#ifndef SWIG
    void CalculateGeneralizedForces(DMArray<double, DIMENSION> *forces, double *generalizedForces);
    void DerivativesWRTGeneralizedCoordinates(int atom, double (*gradient)[DIMENSION]);
#endif // SWIG

	const std::string &GetType() const
	{
		return className;
	}


private:
	double velocity[DIMENSION];
	static const std::string className;
};

#endif // !defined(UNIFORMLYMOVINGCENTEROFMASS_H)
