// DMArray_AnyType.cpp: Implementation of the base class for DMArrays
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"
#include "DMArray_AnyType.h"
#include "Shape.h"
#include "SynchronizationToken.h"
#include "DMException.h"
#include "DMLastHeader.h"

DMArray_AnyType::DMArray_AnyType( )
{
	//TRACE("DMArray_AnyType::DMArray_AnyType void constructor");
	shape = Shape(0);
	siblings = new SynchronizationToken(this);
}

/*! This constructor may be of the wrong signature.  It should
	be possible to construct a shape only through manipulation
	of smaller objects you create, so you should only be able
	to make leaves of the DMArray.  That is, the argument
	to this constructor should be an integer.  More complicated
	trees are made by addition or cloning.
	*/
DMArray_AnyType::DMArray_AnyType( const Shape& shape )
{
	//TRACE("DMArray_AnyType::DMArray_AnyType Shape constructor");
	this->shape = shape;
	this->siblings = new SynchronizationToken(this);
}

DMArray_AnyType::DMArray_AnyType( const DMArray_AnyType* array )
{
	//TRACE("DMArray_AnyType::DMArray_AnyType sibling constructor");
	bool bSuccess;
	this->shape = array->shape;
	this->siblings = array->siblings;
	bSuccess = this->siblings->AddSibling(this);
	ASSERT( bSuccess );
	if (false == bSuccess) throw DMException("DMArray_AnyType",
		"DMArray_AnyType","Could not add self to list of siblings.");
}

DMArray_AnyType::~DMArray_AnyType()
{
	bool bSuccess;
	if (this->siblings->GetSiblings().size()==1) {
		ASSERT( siblings->Contains(this) );
		delete siblings;
	} else {
		bSuccess = siblings->RemoveSibling(this);
		ASSERT( bSuccess );
		if (false == bSuccess) throw DMException("DMArray_AnyType",
			"~DMArray_AnyType","I wasn't in my own list of siblings.");
	}
}

int DMArray_AnyType::Size() const
{
	return shape.GetSize();
}

const Shape& DMArray_AnyType::GetShape() const
{
	return shape;
}

const DMArray_AnyType::siblingContainer& DMArray_AnyType::GetSiblings() const
{
	return siblings->GetSiblings();
}
