//! Leaf-to-leaf iterator for objects with tree structure.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/



/*! Designed for DMArrays.  It works for anything which has the
  following methods.
  \begincode
  int NumberOfBranches();
  bool IsLeaf();
  Type* GetBranch(int i);
  \endcode
  \author ajd27
  \date 4 October 2000
 */
#include "DMFirstHeader.h"
#include <stack>
#include "DigitalMaterial.h"

template <class Type>
class LeafToLeafIterator
{
	Type* head;
	//! The index of the current leaf.
	int leaf;
	//! The number of leaves in the tree.
	int nLeaves;
 public:
	LeafToLeafIterator( Type* head )
	{
		this->head = head;
		Init(); //  for compatibility with previous version. Should maybe be removed in the long run.
	}
	void Init()
	{
		leaf	   = 0;
		nLeaves    = head->NumberOfLeaves();
	}
	void Next()
	{
		++leaf;
	}
	Type *Current()
	{
		return head->GetLeaf(leaf);
	}
	bool Done() { return leaf >= nLeaves; }
};
