//SynchronizationKit_AnyType.h
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef SYNCHRONIZATIONKIT_ANYTYPE_H
#define SYNCHRONIZATIONKIT_ANYTYPE_H

#include "DMFirstHeader.h"
#include <vector>

#include "DigitalMaterial.h"
class DMArray_AnyType;
class Synchronizer;

//! A class that is responsible to allocate ghost arrays and call the synchronizer
/*!
The synchronizer kit is an extension of a regular \a DMArray that allows them to 
be synchronized in a parallel simulation. As for DMArrays, we have the untyped 
(abstract)version (\a Synchronization_AnyType) and the template version. The 
concrete synchronization kit is built by the DMArrays.

If you want an array to be synchronized, you first have to call
\code
	array->GetSynchronizationKit()->Init();
\endcode
If this hasn't been done, this allocates the ghost array. Note that it does so
only at the time when a \a Synchronizer is set. A \a Synchronizer sets itself
(on all the siblings) when it receives a position array. 
Then, at the time of synchronization you just call
\code 
	array->GetSynchronizationKit()->Synchronize();
\endcode

*/

class SynchronizationKit_AnyType
{
public:
	//! Constructor. There's no default synchonizer.
	SynchronizationKit_AnyType(){ synchronizer = 0;}
	//! Empty destructor.
	virtual ~SynchronizationKit_AnyType() {}
	//! Set the synchronizer and allocate the ghost array if asked before
	virtual void SetSynchronizer(Synchronizer *sync) = 0;
	//! Returns the current synchronizer. 
	Synchronizer *GetSynchronizer() const {return this->synchronizer;}
	//! Returns the ghost arrays as a \a DMArray_AnyType
	virtual DMArray_AnyType *GetUntypedGhosts() const = 0;
	virtual void WriteItems(const std::vector<Index> &indices, DMWriter *writer) = 0;
	virtual void ReadItems(const std::vector<Index> &indices, DMReader *reader) = 0;
	virtual void AddItems(const std::vector<Index> &indices, DMReader *reader) = 0;
        virtual const Shape &ExtendedShape() const = 0;
protected:
	//! The synchronizer
	Synchronizer *synchronizer;
};


#endif // !defined SYNCHRONIZATIONKIT_ANYTYPE_H
