
//     Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
//     Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

//     This library is free software; you can redistribute it and/or modify it
//     under the terms of the GNU Lesser General Public License as published by the
//     Free Software Foundation; either version 2.1 of the License, or (at your
//     option) any later version.

//     This library is distributed in the hope that it will be useful, but WITHOUT
//     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//     FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
//     for more details.

//     You should have received a copy of the GNU Lesser General Public License
//     along with this library; if not, write to the Free Software Foundation,
//     Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
//     http://www.gnu.org/licenses/lgpl.html

//     Author Contact Information:
//     Jim Sethna      
//     sethna@lassp.cornell.edu  
//     117 Clark Hall
//     Cornell University
//     Ithaca, NY 14850


%{
#include "ListOfAtoms.h"
#include "DynamicListOfAtoms.h"
%}

%feature("shadow") ListOfAtoms::Attach(ListOfAtoms*) %{
def Attach(*args):
	cmd = 'val = apply(_' + __name__ + '.ListOfAtoms_Attach,args)'
	exec(cmd)
	# Now C++ owns the attached object, python should not destroy it
	# when the corresponding python object is destroyed ....
	# The attached object is refered to as _args[1].
	args[1].thisown = 0
	# .... and the C++ object should not be destroyed before the
	# corresponding Python object has disappeared.  This is achieved by
	# keeping a reference to the owning Python object in the attached
	# Python object, thus preventing the owning Python object (self) from
	# being destroyed before the attached one (branch).  (J. Schiotz)
	args[1].attachedTo = args[0]
	return val
%}

%feature("shadow") ListOfAtoms::RemoveBranch %{
def RemoveBranch(*args):
	cmd = 'val = apply(_' + __name__ + '.ListOfAtoms_RemoveBranch, args)'
	exec cmd
	args[1].thisown = 1
	return val
%}

%feature("shadow") ListOfAtoms::SetConstraint %{    
def SetConstraint(*args):
	# This is a workaround, because addtomethod is broken in swig 1.3.6
	cmd = 'val = apply(_' + __name__ + '.ListOfAtoms_SetConstraint,args)'
	exec cmd
	# Now C++ owns the constraint, python should not destroy it
	# when the corresponding python object is destroyed ....
	# The constraint is refered to as _args[1].
	args[1].thisown = 0
	return val
%}
%feature("shadow") ListOfAtoms::SetNeighborLocator %{
def SetNeighborLocator(*args):
	# This is a workaround, because addtomethod is broken in swig 1.3.6
	cmd = 'val = apply(_' + __name__ + '.ListOfAtoms_SetNeighborLocator,args)'
	exec cmd
	# we want to add a reference to the neighbor locator such that it isn't 
	# deleted before the list of atoms, which refers to it.
	args[0]._NL = args[1]
	return val
%}


%include "ListOfAtoms.h"
%extend ListOfAtoms
{
public:
    ListOfAtoms(int length=0, const BoundaryConditions *bc=0, NeighborLocator *nl=0)
    {
        return new ListOfAtoms(length, bc, nl);
    }
    const DMArray<double,DIMENSION> *GetCartesianPositionsPtr() 
    {return self->GetCartesianPositions();}
    const ArrayDoubleVector *GetCartesianPositions() 
    {return self->GetCartesianPositions();}
    void SetCartesianPositions(const ArrayDoubleVector *newpositions) 
    {self->SetCartesianPositions(newpositions);}
    const DMArray<double,DIMENSION> *GetCartesianVelocitiesPtr() 
    {return self->GetCartesianVelocities();}
    const ArrayDoubleVector *GetCartesianVelocities() 
    {return self->GetCartesianVelocities();}
    void SetCartesianVelocitiesPtr(const DMArray<double,DIMENSION> *newvelocities)
    {self->SetCartesianVelocities(newvelocities);}
    void SetCartesianVelocities(const ArrayDoubleVector *newvelocities) 
    {self->SetCartesianVelocities(newvelocities);}

    // for compatibility with CAMP's standards	
    int __len__() {return self->GetNumber();}

    ArrayDoubleVector *GetSameShapeEmptyArray()
    {return new DMArray<double,DIMENSION>(self->GetShape());}	

    void RemoveAtoms(PyObject *pyObj)
    {
	    std::set<int> toRemove;
	    if(PyList_Check(pyObj))
	    {
		    int size = PyList_Size(pyObj);
    		for(int i=0; i<size; ++i)
        	{
            	PyObject *o = PyList_GetItem(pyObj,i);
                if(PyInt_Check(o)) toRemove.insert(PyInt_AsLong(o));
                else
                {
                    PyErr_SetString(PyExc_TypeError,"list must contain int");
                }
            }
        }
        else if (PyArray_Check(pyObj))
        {
            PyArrayObject *o = (PyArrayObject *)(pyObj);
            if((o->nd == 1) && (o->descr->type_num == PyArray_INT || 
                                o->descr->type_num == PyArray_LONG))
            {
                int size = o->dimensions[0];
                int *data = (int *)(o->data);
                for(int i=0; i<size; ++i) toRemove.insert(data[i]);
            }
            else
            {
                PyErr_SetString(PyExc_TypeError,"not an Array of correct size and/or type");
            }
        }
	    else
	    {
		    PyErr_SetString(PyExc_TypeError,"not a list nor an array");
	    }
        self->RemoveAtoms(toRemove);
    }
};




// There's no need to include "DynamicListOfAtoms.h" since it has the same interface as ListOfAtoms


class DynamicListOfAtoms : public ListOfAtoms
{};
%extend DynamicListOfAtoms
{
public:
 DynamicListOfAtoms(int length=0, const BoundaryConditions *bc=0, NeighborLocator *nl=0)
   {
        return new DynamicListOfAtoms(length, bc, nl);
   }
}



