// SimpleNeighborLocator.h: interface for the SimpleNeighborLocator class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef SIMPLENEIGHBORLOCATOR_H
#define SIMPLENEIGHBORLOCATOR_H

#include "DigitalMaterial.h"
#include "NeighborLocator.h"

/*!
  SimpleNeighborLocator is the most basic kind of neighbor locator that you could think of:
  Each time it recieves a request to calculate the neighbors of atom (or point) atomNumber,
  it look at all the other points and explicitely computes the distance. It works perfectly
  but it's of course extremely slow. The only case where you could think of using this neighbor 
  locator is if there are a few particles, of if the range of the forces is the same size 
  of the system (or for debugging purposes, if you do not trust the other neighbor locators!).  
*/

class SimpleNeighborLocator : public NeighborLocator 
{
public:
	SimpleNeighborLocator(double cutoffDistance = 1.);
	virtual ~SimpleNeighborLocator();
#ifndef SWIG
	bool HalfNeighbors(int atomNumber, int *neighbors, int &numNeighbors,
							double (*displacements)[DIMENSION], double *displacementsSquared,
							int &spacesLeftInArrays);
	bool Neighbors(int atomNumber, int *neighbors, int &numNeighbors,
							double (*displacements)[DIMENSION], double *displacementsSquared,
							int &spacesLeftInArrays);
	bool Neighbors(const double *center, int* neighbors, int& numNeighbors,
							double (*displacements)[DIMENSION],	double* displacementsSquared,
							int& spacesLeftInArrays, double cutoff=-1.);
	bool Neighbors(int atomNumber, int type1, int type2,
							int* neighbors, int& numNeighbors,
							double (*displacements)[DIMENSION],
							double* displacementsSquared,
							int& spacesLeftInArrays);
	bool HalfNeighbors(int atomNumber, int type1, int type2,
							int* neighbors, int& numNeighbors,
							double (*displacements)[DIMENSION],
							double* displacementsSquared,
							int& spacesLeftInArrays);

	NeighborLocator *Copy() const;
	
	void Save(DMWriter *writer) const {NeighborLocator::Save(writer);}
	void Load(DMReader *reader) {NeighborLocator::Load(reader);}
	//! Get the string name of the class for saving or loading.
	const std::string &GetType() const
	{
		return className;
	}

#endif // SWIG
	void UpdatePosition();

private:
	static const std::string className;
};

#endif // !defined(SIMPLENEIGHBORLOCATOR_H)
