// CheckPointObserver.h: interface for the CheckPointObserver class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef CHECKPOINTOBSERVER_H
#define CHECKPOINTOBSERVER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include <string>
#include "ListOfAtomsObserver.h"

class DMWriter;
class Potential;
class AtomsMover;

//! An observer which outputs the LOA to a file whenever notify/update is called

/*!

  This is an observer which, whenever Update is called, increments a counter.
  When the counter is a multiple of the member variable countInterval the LOA
  and its boundary conditions are saved to disk, using the NetCDF format.

  
  \author nbailey
  \date 7/10/2001
  \ingroup moleculardynamics

 */

class CheckPointObserver : public ListOfAtomsObserver
{
public:
	CheckPointObserver(Potential *potential, AtomsMover *mover, std::string baseFileName, bool oneFile, int notifyLevel = 0);
	void Update(ListOfAtoms *atoms);
	virtual ~CheckPointObserver();
	void ResetCounter(int count = 0) { counter = count; }
	std::string GetLastFileName() { return lastFileName; }

private:
	DMWriter *writer;
	Potential *potential;
	AtomsMover *mover;
	int counter;
	bool oneFile;
	std::string baseFileName;
	std::string lastFileName;
};

#endif // !defined(CHECKPOINTOBSERVER_H)
