
//     Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
//     Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

//     This library is free software; you can redistribute it and/or modify it
//     under the terms of the GNU Lesser General Public License as published by the
//     Free Software Foundation; either version 2.1 of the License, or (at your
//     option) any later version.

//     This library is distributed in the hope that it will be useful, but WITHOUT
//     ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//     FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
//     for more details.

//     You should have received a copy of the GNU Lesser General Public License
//     along with this library; if not, write to the Free Software Foundation,
//     Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
//     http://www.gnu.org/licenses/lgpl.html

//     Author Contact Information:
//     Jim Sethna      
//     sethna@lassp.cornell.edu  
//     117 Clark Hall
//     Cornell University
//     Ithaca, NY 14850


%{
#include "CheckPointObserver.h"
#include "ColorMethod.h"
#include "CrackPlaneObserver.h"
#include "EnergyObserver.h"
#include "LocalForceObserver.h"
#include "PythonLOAObserver.h"
#include "StressIntensityObserver.h"
%}


%nodefault ListOfAtomsObserver;
class ListOfAtomsObserver{};

%nodefault ColorMethod;
class ColorMethod 
{
public:
    %extend
    {
        double __call__(ListOfAtoms *atoms,int i) {return (*self)(atoms,i);}
    }
};

class EnergyColorMethod : public ColorMethod
{
public:
	EnergyColorMethod(Potential *potential);
};

class StressColorMethod : public ColorMethod
{
public:
	StressColorMethod(Potential *potential);
	void SetComponent(int comp);
};

class ForceColorMethod : public ColorMethod
{
public:
    ForceColorMethod(Potential *potential);
    void SetComponent(int comp);
};


class CoordinationColorMethod : public ColorMethod
{
public:
    CoordinationColorMethod();
};

class CentroSymmetryColorMethod : public ColorMethod
{
public:
    CentroSymmetryColorMethod(BravaisLatticeWithBasis *undeformedLattice, 
        int numberNearestNeighbors, ArrayDoubleVector *nearestNeighborVectors);
};

%include "CheckPointObserver.h"
%include "CrackPlaneObserver.h"

%extend CrackPlaneObserver
{
	PyObject *GetPairDistances()
	{
		const std::vector<double > &pairDist = self->GetPairSeparations();
		int size = pairDist.size();
		PyObject *pResult = PyList_New(0);
		for(int idx = 0; idx < size; ++idx )
		{
		PyList_Append(pResult, PyFloat_FromDouble(pairDist[idx]) );
		}
	return pResult;
	}
} // end extend


%include "EnergyObserver.h"
%include "LocalForceObserver.h"
%include "PythonLOAObserver.h"
%include "StressIntensityObserver.h"
