// Synchronizer.h: interface for the Synchronizer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef SYNCHRONIZER_H
#define SYNCHRONIZER_H

#include "DMFirstHeader.h"
#include <vector>

#include "DigitalMaterial.h"
#include "DMProtocol.h"
#include "DMArray.h"
class DomainSubdivision;

class Synchronizer
{
public:
    Synchronizer(DMArray<double, DIMENSION> *positions=0, DMProtocol *protocol = dmProtocol);
    virtual ~Synchronizer();
    void SetPositionsArray(DMArray<double, DIMENSION> *positions);
    void Rebuild(const DomainSubdivision *domain);
    void Synchronize(DMArray_AnyType *array) const;
    void SynchronizeAndAdd(DMArray<double, DIMENSION> *array) const;
    const Shape &GetGhostsShape() const {return ghostShape;}
private:
    //! The message passing protocol
    DMProtocol *protocol;
    //! The buffers we use for the communication.(See comment in Migrator).
    DMProtocol::Buffers *buffers;
    //! The number of precessors
    int nProcs;
    //! The index of my process
    int myProc;
    //! The positions array that says which atoms are to be synchronized.
    DMArray<double, DIMENSION> *positions;
    //! The number of leaves of the positions array
    int numberOfLeaves;
    //! we store the index of the atoms  we have to send in vectors (one per processor).
    std::vector<Index> *interestingAtomsToSend;
    //! we store the index of the ghost atoms that are received in the receiveBuffers (one per processor).
    std::vector<Index> *interestingAtomsToRecv;
    //! the size of the leaves of the array that has to be sent (one per processor per leaf)
    std::vector<int> *sizeOfLeavesToSend;
    //! the size of the leaves of the array that has to be received (one per processor per leaf)
    std::vector<int> *sizeOfLeavesToRecv;
    //! The length of each leaves of the ghost array
    std::vector<int> sizeOfGhostLeaves;
    //! The shape of the ghosts
    Shape ghostShape;
    //! the size of the array that has to be sent (one per processor per leaf)
    int *sizeToSend;
    //! the size of the array that has to be received (one per processor per leaf)
    int *sizeToRecv;

};

#endif // !defined SYNCHRONIZER_H
