// Composite Potential
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
#include "DigitalMaterial.h"
#include <iostream>
#include "CompositePotential.h"

#include "ListOfAtoms.h"
#include "DMArray.h"
#include "IndexError.h"
#include "NotEnoughMemory.h"
#include "NeighborLocator.h"
#include "SerialBuilder.h"
#include "BoundaryConditions.h"

#include "DMLastHeader.h"

using namespace std;

#define LEN_NBR_ARRAYS 1500


static SerialBuilder<CompositePotential> *_Gcreator = 
    new SerialBuilder<CompositePotential>("CompositePotential");

const string CompositePotential::className = string("CompositePotential");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CompositePotential::CompositePotential(Potential *potential1, Potential *potential2) 
{
	this->potential1 = potential1;
	this->potential2 = potential2;
	forces2 = 0;
}

CompositePotential::~CompositePotential()
{
	if (forces2) delete forces2;
}
 
void CompositePotential::CalculateForces(ListOfAtoms *atoms, DMArray<double, DIMENSION> *forces)
{

	if (!(forces2 && (forces2->GetShape() == atoms->GetShape())))
		Allocate(atoms->GetCartesianPositions());
	potential1->CalculateForces(atoms,forces);
	potential2->CalculateForces(atoms,forces2);
	forces->Axpy(forces2,1);

}

double CompositePotential::CalculateEnergy(ListOfAtoms *atoms)
{
	return (potential1->CalculateEnergy(atoms)) + (potential2->CalculateEnergy(atoms));
}

double CompositePotential::GetLengthScale() const 
{
	return min(potential1->GetLengthScale(),potential2->GetLengthScale());
}
	
double CompositePotential::GetEnergyCurvatureScale() const
{
	return min(potential1->GetEnergyCurvatureScale(),potential2->GetEnergyCurvatureScale());
}

// double CompositePotential::GetInnerCutoffDistance() const
// {
// 	return min(potential1->GetInnerCutoffDistance(),potential2->GetInnerCutoffDistance());
// }

double CompositePotential::CalculateAtomicEnergy(ListOfAtoms *atoms, int i)
{
	return (potential1->CalculateAtomicEnergy(atoms,i)) + (potential2->CalculateAtomicEnergy(atoms,i));
}

void CompositePotential::Save(DMWriter *writer) const
{
	potential1->Save(writer);
	potential2->Save(writer);
}

void CompositePotential::Load(DMReader *reader)
{
	potential1->Load(reader);
	potential2->Load(reader);
}

void CompositePotential::Allocate(const DMArray<double, DIMENSION> *array)
{
	if (forces2) delete forces2;
	forces2 = new DMArray<double, DIMENSION>(array);
}
