// EMTAtomicUnitParameterProvider.cpp  --  provides the default EMT parameters.

/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
#include <string.h>
#include <cmath>
#include <iostream>
#include "DigitalMaterial.h"
#include "EMTAtomicUnitParameterProvider.h"
#include "DMException.h"
#include "DMLastHeader.h"

using namespace std;

emt_parameters *EMTAtomicUnitParameterProvider::GetNewParameters(const string element)
{
	double E0, S0, n0, V0, eta2, kappa, lambda, mass;
	double ls;
	int Z;
	emt_parameters *p;
	
	if(element == "Al"){
		E0=-3.280; S0=3.000; V0=1.493; Z=13; eta2=1.240; kappa=2.000; lambda=1.169; mass=26.98 ; n0=0.007  ; 
		ls = 7.54871784;
	} 
	else if(element == "Cu"){
		E0=-3.510; S0=2.67 ; V0=2.476; Z=29; eta2=1.652; kappa=2.74 ; lambda=1.906; mass=63.54 ; n0=0.0091 ;
		ls = 6.789382809;
	}
	else if(element == "Ag"){
		E0=-2.96 ; S0=3.01 ; V0=2.132; Z=47; eta2=1.652; kappa=2.790; lambda=1.892; mass=107.87; n0=0.00547;
		ls = 7.6790043;
	}
	else if(element == "Au"){ 
		E0=-3.80 ; S0=3.00 ; V0=2.321; Z=79; eta2=1.674; kappa=2.873; lambda=2.182; mass=196.97; n0=0.00703; 
		ls = 7.66504117182;
	}
	else if(element == "Ni"){
		E0=-4.44 ; S0=2.60 ; V0=3.673; Z=28; eta2=1.669; kappa=2.757; lambda=1.948; mass=58.71 ; n0=0.0103 ;
		ls = 6.598896;
	}
	else if(element == "Pd"){
		E0=-3.90 ; S0=2.87 ; V0=2.773; Z=46; eta2=1.818; kappa=3.107; lambda=2.155; mass=106.4 ; n0=0.00688;
		ls = 7.330378;
	}
	else if(element == "Pt"){
		E0=-5.85 ; S0=2.90 ; V0=4.067; Z=78; eta2=1.812; kappa=3.145; lambda=2.192; mass=195.09; n0=0.00802;
		ls = 7.41119853;
	}
	//else if(!strcmp(element,"H")){
	//	E0=-2.15 ; S0=1.9  ; V0=0.   ; Z=1 ; eta2=3.2  ; kappa=4.1  ; lambda=2.096; mass=1.    ; n0=0.007;
	//}
	else throw DMException("EMTAtomicUnitParameterProvider","GetNewParameters",
						   "This element isn't defined in EMT ("+element+")");
	
	p = new emt_parameters;
	p->e0 = E0;
	p->seq = S0;
	p->neq = n0;
	p->V0 = V0;
	p->eta2 = eta2;
	p->kappa = kappa;
	p->lambda = lambda;
	p->mass = mass;
	p->invmass = 1.0/mass;
	p->gamma1 = 0.0;        // These are calculated later!
	p->gamma2 = 0.0;
	p->Z = Z;
	p->name = element;
	p->lengthscale = ls / sqrt(2.0);

	return p;
}
