// Gravity Potential
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
#include "DigitalMaterial.h"
#include <iostream>
#include "GravityPotential.h"

#include "ListOfAtoms.h"
#include "DMArray.h"
#include "IndexError.h"
#include "NotEnoughMemory.h"
#include "NeighborLocator.h"
#include "SerialBuilder.h"
#include "BoundaryConditions.h"

#include "DMLastHeader.h"

using namespace std;

#define LEN_NBR_ARRAYS 1500


static SerialBuilder<GravityPotential> *_Gcreator = 
    new SerialBuilder<GravityPotential>("GravityPotential");

const string GravityPotential::className = string("GravityPotential");

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


GravityPotential::GravityPotential() 
{
	g = 0.01;
	direction[0]=0;
	direction[1]=-1;
	if(DIMENSION==3) direction[2]=0;	// Direction for gravity
}


void GravityPotential::CalculateForces(ListOfAtoms *atoms, DMArray<double, DIMENSION> *forces)
{
	double grav[DIMENSION];
	for(int coord=0; coord<DIMENSION; coord++) 
		grav[coord] = g * direction[coord];
	forces->Set(grav);
}
	

void GravityPotential::SetG(double g) {this->g = g;}

void GravityPotential::SetDirection(double direction[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; ++coord)
		this->direction[coord]=direction[coord];	// Direction for gravity
}

double GravityPotential::CalculateEnergy(ListOfAtoms *atoms)
{
	double energy = 0;
	for (int atom=0; atom < atoms->GetNumber(); atom++)
	{	
		for(int coord=0; coord<DIMENSION; coord++) 
			energy += g * direction[coord] * atoms->GetCartesianPosition(atom, coord);
	}
	return energy;
}

