// HolianCutLennardJonesPotential.h: interface for the HolianCutLennardJonesPotential class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef HOLIANCUTLENNARDJONESPOTENTIAL_H
#define HOLIANCUTLENNARDJONESPOTENTIAL_H

#include "DigitalMaterial.h"
#include "CutLennardJonesPotential.h"

//! Like CutLennardJones, except much shorter cutoff (1.711 sigma).


/*!

 THe shorter cutoff is good in 3D where the other Lennard-Jones potential has 86 neighbors within the cutoff! (1st through 6th neighbors). Taken from Holian et al.....

*/

class HolianCutLennardJonesPotential : public CutLennardJonesPotential  
{
public:
	HolianCutLennardJonesPotential();
	virtual ~HolianCutLennardJonesPotential();
	void CalculateForces(ListOfAtoms *atoms, DMArray<double, DIMENSION> *forces);
	double CalculateEnergy(ListOfAtoms *atoms);
	double CalculateEnergyAndForces(ListOfAtoms *atoms, DMArray<double, DIMENSION> *forces);
	double CalculatePairForceEnergy(double *displacement, double displacementSq, double *force);

	
	double GetLengthScale() const
	{
		if(DIMENSION==3) return 1.10725081873;  //   the energy per atom in 3D is : -6.2952423313
		else return pow(2.0, 1.0/6.0);	// the energy per atom in 2D is : -3.0
	}

	void Save(DMWriter *writer) const;
	void Load(DMReader *reader);
	//! Get the string name of the class for saving or loading.
	const std::string &GetType() const
	{
		return className;
	}

private:
#ifndef SWIG	
	// parameters of the potential (relating the polynomial part)
	//! \a rSpl marks the boundary between the standard LJ part and the polynomial part
	static const std::string className;
	const double	HCLJ_rSpl;
	//! r2Spl is the square of \a rSpl
	double			r2Spl;
	const double	HCLJ_a2;
	const double	HCLJ_a3;
	double			HCLJ_cutoffDistance;

	//! Do the main work for calculating atomic stress--we don't need to overload the CalculateStress functions, just this
	void StressTensorKernel(ListOfAtoms *atoms, int i, double stress[(DIMENSION+1)*DIMENSION/2]);

#endif // SWIG
};

#endif // !defined(HOLIANCUTLENNARDJONESPOTENTIAL_H)
