// AnisotropicDislocationMaker.h: interface for the AnisotropicDislocationMaker class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef ANISOTROPICDISLOCATIONMAKER_H
#define ANISOTROPICDISLOCATIONMAKER_H

#include "DMFirstHeader.h"
#include <math.h>
#include "DigitalMaterial.h"
#include <complex>


#include "DislocationMaker.h"
#include "ListOfAtoms.h"

//! Introduces straight dislocations into materials with cubic anisotropy

/*!

  AnisotropicDislocationMaker inherits from DislocationMaker, overloading the
  ElasticDisplacement and GradientWRTPosition functions. At the moment (Feb 
  2001), it only deals with cubic anisotropy, and only when the dislocation 
  line is along a cubic axis of the material.

  \date January 2001
  \author Nicholas Bailey

 */

class AnisotropicDislocationMaker : public DislocationMaker
{
public:
	AnisotropicDislocationMaker(const double burgers[DIMENSION],const double direction[DIMENSION], double center[DIMENSION],double *elasticConstats,double thetaLowerCutoff = -PI);
	virtual ~AnisotropicDislocationMaker();

	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);


	void GradientWRTPosition(const double position[DIMENSION],  double gradientWRTPosition[DIMENSION][DIMENSION]);

	
#ifndef SWIG

private:
//	void UpdateMemberVariables();
	double elasticConstants[3]; // assuming cubic symmetry: 3 elastic constants (C11, C12, C44)	
	std::complex<double> anisotropyPhaseFactor; // e^(i alpha) in Eshelby, Sinclair papers
	std::complex<double> logCoefficient; // D/(2 Pi i) in the papers
	std::complex<double> yCompFactor; // A in Eshelby, Sinclair papers
#endif //SWIG
};

#endif // !defined(ANISOTROPICDISLOCATIONMAKER_H)
