// AtomsLocator.cpp: implementation of the AtomsLocator class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DigitalMaterial.h"
#include "AtomsLocator.h"
#include "ListOfAtoms.h"

#include "DMLastHeader.h"


/*!
  In the base class we implement the most direct, but possibly not the most
  efficient, method of finding atoms in the specified region: iterate through
  all positions and use the Inside function to identify the ones we keep.


 */

const std::vector<int> &AtomsLocator::GetAtomNumbers(ListOfAtoms *atoms)
{
	listOfAtomNumbers.clear();

	int nAtoms = atoms->GetNumber(),atomIdx;
	DMArrayIterator<double, DIMENSION> posIter(atoms->GetCartesianPositions());
	

	for( posIter.Init(), atomIdx = 0; !posIter.IsDone(); posIter.Next(), ++atomIdx)
	{
		double *position = posIter.Current();
		
		if (Inside(position))
			listOfAtomNumbers.push_back(atomIdx);
	}

	return listOfAtomNumbers;
}
