// ExternalStress.h: interface for the ExternalStress class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef EXTERNALSTRESS_H
#define EXTERNALSTRESS_H

#include "DMFirstHeader.h"
#include <vector>

#include "ElasticFieldTransformer.h"
#define N_CPTS (DIMENSION*(DIMENSION+1)/2) // number of independent stress or strain components

/**
 * \author Nick Bailey
 * \date October 2000
 * \ingroup moleculardynamics
**/

class ListOfAtoms;

class ExternalStress : public ElasticFieldTransformer  
{
public:
	ExternalStress(double *C,double *origin,ListOfAtoms *undeformed = NULL);
	virtual ~ExternalStress();
	void SetStress(double stress[DIMENSION][DIMENSION]);
	const double GetStress(int a, int b) { return stress[a][b]; }

	void SetStrain(double strain[DIMENSION][DIMENSION]);
	const double GetStrain(int a, int b) { return strain[a][b]; }


	void SetUndeformed(ListOfAtoms *undeformed) {this->undeformed = undeformed;}
	void SetOrigin(double *origin)
	  { for(int coord=0;coord<DIMENSION;++coord) this->origin[coord] = origin[coord];}
	const double *GetOrigin() const {return origin; }
	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);
	void GradientWRTPosition(const double position[DIMENSION], double gradientWRTPosition[DIMENSION][DIMENSION]);

private:
	double stress[DIMENSION][DIMENSION];
	double strain[DIMENSION][DIMENSION];
	double origin[DIMENSION]; // in principle not necessary but important if I'm applying this transformer to the boundary atoms and not the central atoms---a large shift in origin would be bad!
	ListOfAtoms *undeformed;
	double C[3]; // assuming cubic symmetry: 3 elastic constants (C11, C12, C44)	
};

#endif // !defined(EXTERNALSTRESS_H)
