// LinearPositionTransformer.h: interface for the LinearPositionTransformer class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef LINEARPOSITIONTRANSFORMER_H
#define LINEARPOSITIONTRANSFORMER_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include <math.h>
#include <assert.h>

#include "ElasticFieldTransformer.h"
#include "ListOfAtoms.h"

//! Applies a given linear transformation to the coordinates of each atom in the ListOfAtoms

/* \ingroup moleculardynamics
 
 Takes a DIMENSIONxDIMENSION matrix, and operates on the coordinates of
 each atom in turn (using an iterator), giving the displacement, which is then 
 added to the position. This is different to how the class was originally
 implemented--then the given matrix was to multiply the position giving the
 new position rather than the change in position. You can think of it as this:
 Before, the passed in matrix was the deformation gradient. Now it is the 
 strain matrix (in the context of infinitesimal transformations)

 Can be used to scale the positions etc. 
 
 /author Nicholas Bailey
 
 */


class LinearPositionTransformer : public ElasticFieldTransformer
{
public:
	LinearPositionTransformer(const double transformationMatrix[DIMENSION][DIMENSION]);
	virtual ~LinearPositionTransformer();
	
	void SetTransformationMatrix(const double transformationMatrix[DIMENSION][DIMENSION]);
	void ElasticDisplacement(const double position[DIMENSION], double displacement[DIMENSION]);

	void GradientWRTPosition(const double position[DIMENSION],double gradientWRTPosition[DIMENSION][DIMENSION]);
  

	void SetCenter(double center[DIMENSION]){for (int coord = 0; coord < DIMENSION; ++coord) this->center[coord] = center[coord];}
	const double *GetCenter() { return center; }

private:
	double transformationMatrix[DIMENSION][DIMENSION];
	double center[DIMENSION];
};

#endif // !defined(LINEARPOSITIONTRANSFORMER_H)
