// NotchMaker.cpp: implementation of the NotchMaker class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
#include "DigitalMaterial.h"
#include "NotchMaker.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

NotchMaker::NotchMaker(double openingAngle, double tipLocation[DIMENSION], double dirAngle)
{
	this->openingAngle = openingAngle;
	for(int coord=0; coord<DIMENSION; coord++) this->tipLocation[coord] = tipLocation[coord];
	this->dirAngle = dirAngle;
	UpdateMemberVariables();
}

NotchMaker::~NotchMaker()
{

}

void NotchMaker::SetDirAngle(double dirAngle)
{
	this->dirAngle = dirAngle;
	UpdateMemberVariables();
}

double NotchMaker::GetDirAngle() const
{
	return dirAngle;
}

void NotchMaker::SetOpeningAngle(double openingAngle)
{
	this->openingAngle = openingAngle;
	UpdateMemberVariables();
}

double NotchMaker::GetOpeningAngle() const
{
	return openingAngle;
}

void NotchMaker::SetTipLocation(const double tipLocation[DIMENSION])
{
	for(int coord=0; coord<DIMENSION; coord++) this->tipLocation[coord] = tipLocation[coord];
}

void NotchMaker::UpdateMemberVariables()
{
	normal1[0] = -sin(dirAngle-openingAngle/2.);
	normal1[1] =  cos(dirAngle-openingAngle/2.);
	normal2[0] =  sin(dirAngle+openingAngle/2.);
	normal2[1] = -cos(dirAngle+openingAngle/2.);
}

bool NotchMaker::Inside(const double X[DIMENSION])
{
	bool inside;
	inside = ( ((X[0]-tipLocation[0])*normal1[0] + (X[1]-tipLocation[1])*normal1[1] >= 0.) ||
		   ((X[0]-tipLocation[0])*normal2[0] + (X[1]-tipLocation[1])*normal2[1] >= 0.)   )	;
	return inside;
}

