// Quasi2DElasticField.h: interface for the Quasi2DElasticField class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef QUASI2DELASTICFIELD_H
#define QUASI2DELASTICFIELD_H


#include "DMFirstHeader.h"
#include <math.h>
#include "DigitalMaterial.h"
#include <assert.h>

#include "ElasticFieldTransformer.h"
#include "ListOfAtoms.h"


class Quasi2DElasticField : public ElasticFieldTransformer
{
public:
	Quasi2DElasticField(const double direction[DIMENSION], double center[DIMENSION],double thetaLowerCutoff = 0);

	virtual void CalculateGeometry(const double position[DIMENSION]);

	virtual void SetDirection(const double direction[DIMENSION]); 
	virtual const double *GetDirection() const {return (const double *)direction;}
	virtual void SetCenter(const double center[DIMENSION]); 
	virtual const double *GetCenter() const {return (const double *)center;}
	void SetCutoff(const double cutoff){this->cutoff = cutoff;}
	double GetCutoff() const {return cutoff;}

	void SetOrientingVector(int a,const double vector[DIMENSION]){for(int coord=0;coord<DIMENSION;++coord)orientingVector[a][coord] = vector[coord];}
	const double *GetOrientingVector(int a){return orientingVector[a];}

	void GradientWRTCenter(const double position[DIMENSION],double gradientWRTCenter[DIMENSION][DIMENSION]);

protected:
	double direction[DIMENSION];
	double center[DIMENSION];
	double orientingVector[2][DIMENSION];
	double cutoff; // cutoff the singularity---don't displace atoms closer than this to the line
	double parallelComp[DIMENSION];
	double perpComp[DIMENSION];
	double rPerp;
	double rPerpInverse;
	double theta;
	double thetaLowerCutoff; // determines which 2 PI range of theta is allowed
	double cosTheta;
	double sinTheta;
	double orient1Comp;
	double orient2Comp;	

};

#endif // !defined(QUASI2DELASTICFIELD_H)
