// RectangleAtomsLocator.h: interface for the RectangleAtomsLocator class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef RECTANGLEATOMSLOCATOR_H
#define RECTANGLEATOMSLOCATOR_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "AtomsLocator.h"


//! Identifies atoms in a rectangular regoin



/*!
  Constructor arguments include the bounding faces and a boolean 'bounded' for 
  each direction. If this is false then the bounds in that direction are
  ignored.
  

  \author nbailey
  \date 2/25/2002
  \ingroup moleculardynamics

 */


class RectangleAtomsLocator : public AtomsLocator
{
public:
	RectangleAtomsLocator(double lower[DIMENSION], double upper[DIMENSION], bool bounded[DIMENSION] );
	virtual ~RectangleAtomsLocator();
	bool Inside(const double *position);


	void SetLower(double lower[DIMENSION]) 
		{
			for(int coord = 0; coord < DIMENSION; ++coord ) 
				this->lower[coord] = lower[coord];
			CalculateCenterRadius();
		}
	void SetUpper(double upper[DIMENSION])
		{
			for(int coord = 0; coord < DIMENSION; ++coord )
				this->upper[coord] = upper[coord];CalculateCenterRadius();
			CalculateCenterRadius();
		}
	
	void SetBounded(bool bounded[DIMENSION])
		{
			for(int coord = 0; coord < DIMENSION; ++coord )
				this->bounded[coord] = bounded[coord];	
		}

#ifndef SWIG
	const std::vector<int> &GetAtomNumbers(ListOfAtoms *atoms);
	
private:
	void AllocateForNeighborLocator(int nSpaces);
	void CalculateCenterRadius();
	double lower[DIMENSION];
	double upper[DIMENSION];
	bool bounded[DIMENSION]; 
	double center[DIMENSION];
	double radius;
	
	int nSpaces;
	double *displacementsSq;
	double (*displacements)[DIMENSION];
	int *neighbors;
#endif // SWIG
};

#endif // !defined(RECTANGLEATOMSLOCATOR_H)
