// SliceAtomsLocator.h: interface for the SliceAtomsLocator class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef SLICEATOMSLOCATOR_H
#define SLICEATOMSLOCATOR_H

#include "DMFirstHeader.h"
#include "DigitalMaterial.h"
#include "AtomsLocator.h"
#include <cmath>

//! Identifies atoms in "slice" region---specified by a center, a normal and a thickness



/*!
  Constructor arguments include a center, which is any point in the mid-plane 
  of the slice, a unit vector which is the normal to the mid-plane, and the 
  slice thickness.
  

  \author nbailey
  \date 7/12/2002
  \ingroup moleculardynamics

 */


class SliceAtomsLocator : public AtomsLocator
{
public:
	SliceAtomsLocator(double center[DIMENSION], double normal[DIMENSION], double thickness);
	virtual ~SliceAtomsLocator();
	bool Inside(const double *position);


	void SetCenter(double center[DIMENSION]) 
		{
			for(int coord = 0; coord < DIMENSION; ++coord ) 
				this->center[coord] = center[coord];
			//CalculateCenterRadius();
		}
	void SetNormal(double normal[DIMENSION])
		{
			int coord;
			double norm = 0.;
			for(coord = 0; coord < DIMENSION; ++coord )
			{
				this->normal[coord] = normal[coord];
				norm += this->normal[coord] * this->normal[coord];
			}
			norm = sqrt(norm);

			for(coord = 0; coord < DIMENSION; ++coord )
				this->normal[coord] /= norm;
		}
	
	void SetThickness(double thickness)
		{
			this->thickness = fabs(thickness);	
		}

#ifndef SWIG
private:
	double center[DIMENSION];
	double normal[DIMENSION];
	double thickness;

#endif // SWIG
};

#endif // !defined(SLICEATOMSLOCATOR_H)
