// UndercoordinatedPruner.cpp: implementation of the UndercoordinatedPruner class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "DMFirstHeader.h"
#include <vector>
#include <set>

#include "DigitalMaterial.h"

#include "UndercoordinatedPruner.h"
#include "CellNeighborList.h"
#include "ListOfAtoms.h"
#include "BoundaryConditions.h"
#include "DMException.h"

#include "DMLastHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

UndercoordinatedPruner::UndercoordinatedPruner(double cutoff, int minCoordination):cutoff(cutoff), minCoordination(minCoordination)
{

}

UndercoordinatedPruner::~UndercoordinatedPruner()
{
}

void UndercoordinatedPruner::SetCutoff(double cutoff)
{
	this->cutoff = cutoff;
}


void UndercoordinatedPruner::Transform(ListOfAtoms *atoms)
{


	const BoundaryConditions *bc = atoms->GetBoundaryConditions();
	int i;
	double drift = 0.05*cutoff;
	const DMArray<double, DIMENSION> *positions = atoms->GetCartesianPositions();
	const DMArray<double, DIMENSION> *velocities = atoms->GetCartesianVelocities();
	int nAtoms = atoms->GetNumber();
	NeighborLocator *tempNList = new CellNeighborList(cutoff, drift);
	tempNList->SetBoundaryConditions(atoms->GetBoundaryConditions());
	tempNList->SetPositionArray(atoms->GetCartesianPositions());
	int spaceLeftInArray, numberNeighbors;
	int *neighbors = new int[nAtoms];
	double (*displacement)[DIMENSION] = new double [nAtoms][DIMENSION];
	double *distSQ = new double[nAtoms];
	
	tempNList->UpdatePosition();
	atomsRemoved.clear();

	for(i=0;i<nAtoms;i++)
	{
		spaceLeftInArray = nAtoms;
		
		bool ok = tempNList->Neighbors(i, neighbors, numberNeighbors, displacement, distSQ, spaceLeftInArray);
		assert(ok);
		if(numberNeighbors < minCoordination)
			atomsRemoved.insert(i);

	}
	atoms->RemoveAtoms(atomsRemoved);

	delete [] neighbors;
	delete [] displacement;
	delete [] distSQ;
	delete tempNList;

}
