// UndercoordinatedPruner.h: interface for the UndercoordinatedPruner class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef UNDERCOORDINATEDPRUNER_H
#define UNDERCOORDINATEDPRUNER_H

#include "DMFirstHeader.h"
#include <vector>

#include "Transformer.h"

/*
  The duty of this class is to remove atoms which are undercoordinated, e.g. 
  Silicon atoms with only one neighbor.

  \author Nicholas Bailey
  \date August 2001
  \ingroup moleculardynamics
*/

class ListOfAtoms;

class UndercoordinatedPruner : public Transformer  
{
public:
	UndercoordinatedPruner(double cutoff, int minCoordination = 2);
	virtual ~UndercoordinatedPruner();
	void SetCutoff(double cutoff);
	double GetCutoff() {return cutoff;}
	void Transform(ListOfAtoms *atoms);
#ifndef SWIG
	const std::set<int> &GetAtomsRemoved() {return atomsRemoved;}
private:

	double cutoff;
	int minCoordination;
	std::set<int> atomsRemoved;
#endif //SWIG
};

#endif // !defined(UNDERCOORDINATEDPRUNER_H)
