// WedgeMaker.h: interface for the WedgeMaker class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef WEDGEMAKER_H
#define WEDGEMAKER_H

#include "DMFirstHeader.h"
#include <math.h>
#include "DigitalMaterial.h"

#include "VoidMaker.h"

class WedgeMaker : public VoidMaker  

{
public:
	WedgeMaker(double direction[DIMENSION], double center[DIMENSION], double normal1[DIMENSION], double normal2[DIMENSION]);
	virtual ~WedgeMaker();


	void SetDirection(const double direction[DIMENSION]);
	const double *GetDirection() const {return (const double *) direction; };

	void SetCenter(const double center[DIMENSION]);
	const double *GetCenter() const {return (const double *) center; };

	void SetNormal1(const double normal1[DIMENSION]);
	const double *GetNormal1() const {return (const double *) normal1; };

	void SetNormal2(const double normal2[DIMENSION]);
	const double *GetNormal2() const {return (const double *) normal2; };

private:
	bool Inside(const double X[DIMENSION]);
	double direction[DIMENSION];
	double center[DIMENSION];
	double normal1[DIMENSION];
	double normal2[DIMENSION];

};

#endif // !defined(WEDGEMAKER_H)
