// PlotAtomsObserver.h: interface for the PlotAtomsObserver class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/



#ifndef PLOTATOMSOBSERVER_H
#define PLOTATOMSOBSERVER_H


#include <stdio.h> // for FILE
#include <map>

#include "ListOfAtomsObserver.h"
#include "ColorMethod.h"


//! Uses LASSPTools PlotAtoms to plot ListOfAtoms.
/*! The PlotAtomsObserver class is from LASSPTools.  It is 
  an observer of ListOfAtoms.

  It was written to use stdiostream, but that has been removed because
  it can conflict with standard iostream libraries.  It now uses
  regular stdlib library functions.
*/
class PlotAtomsObserver : public ListOfAtomsObserver  
{
//#ifndef SWIG
public:
	//! Constructor.
	PlotAtomsObserver(double xMin, double xMax, double yMin, double yMax, int radius, int defaultColor = 0, ColorMethod *colorMethod=0);
	//! Destructor.
	virtual ~PlotAtomsObserver();
	//! Called by the subject to tell us when atoms move.
	void Update(ListOfAtoms *atoms);
	//! Alternative syntax for Update().
	void Plot(ListOfAtoms *atoms);

	void SetColorScale ( double colorScale ) { this->colorScale = colorScale; colorScaleSetByUser = true; }
	void UnSetColorScale() { colorScaleSetByUser = false; }
	double GetColorScale() { return colorScale; }

	void SetColor(int leaf,int color)
		{
			colorMap.insert(std::make_pair(leaf,color));
		}
	
	void SetLengthScale ( double lengthScale ) { this->lengthScale = lengthScale; UpdateRotateScale(); }
	double GetLengthScale() { return lengthScale; }

	void SetRotationMatrix(double rotationMatrix[DIMENSION][DIMENSION]);

//#endif // SWIG

protected:
#ifndef SWIG
	void UpdateRotateScale();
	FILE*	        pipe_file; //!< Pipe to PlotAtoms.
	ColorMethod*    colorMethod;
	bool colorScaleSetByUser;
	double colorScale;
	std::map<int, int> colorMap; // maps leaf index to colour index
	int nColors;

	double lengthScale;
	double rotationMatrix[DIMENSION][DIMENSION];
	double rotateScaleMatrix[DIMENSION][DIMENSION];
	
#endif // SWIG
};

#endif // !defined(PLOTATOMSOBSERVER_H)

