

/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#ifndef _IDRESOURCEMANAGER_H_
#define _IDRESOURCEMANAGER_H_

#include <utility> // for pair
#include <list> // for list

//! Lets you check in/out ranges of integer ID numbers.
/**
 * You create this class with a range of integer IDs which it
 * will parcel out when asked.  If something releases the ID
 * later, then it goes back into the pool of IDs.
 *
 * This class is not a singleton and should not be one.  (We
 * could make it a singleton by making it a template object
 * on the ID min and max.)  We don't want it to be a singleton
 * because it is possible two separate pools of ID numbers have
 * the same minimum and maximum range.  Instead, this class
 * can be used as the implementation of a singleton.
 *
 * \author Drew Dolgert
 * \date 19 Feb 2000
 * \ingroup dmsystem
 */
class IDResourceManager {
public:
	/*! Construct this with the min and max allowable IDs. */
	IDResourceManager(int IDMin, int IDMax);
	int GetID();
	void ReleaseID(int ID);
	virtual ~IDResourceManager();
private:
	int IDMin, IDMax;  // The min and max allowable IDs.
	typedef std::pair<int,int> intpair;
	typedef std::list<intpair> IDList;
	IDList freeIDs;
	static IDResourceManager *_instance;

	// This is a function object (functor) used in the
	// algorithms.
	struct CouldJoinRange {
		CouldJoinRange(int ID) : ID(ID) {}
		int operator()(const intpair& idrange) {
			return (ID <= idrange.second+1);
		}
		int ID;
	};
};

#endif // _IDRESOURCEMANAGER_H_
