// TransformerMenu.cpp: implementation of the TransformerMenu class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "stdafx.h"
#include "MD.h"
#include "TransformerMenu.h"
#include "MDDoc.h"
#include "CommandIDManager.h"
#include <algorithm>
#include "Thread.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// XXX need to have a singleton idd_user base class to store these!
// Initialize static member variables for IDD

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TransformerMenu::TransformerMenu(MDSimulation *sim, Thread *simThread)
{
	// "File Edit ..." menu at top of screen does not belong to the view, but to MainFrame
	// MainFrame, at least for this program, is the parent of the CMDView class

	pSim = sim;

	pSimThread = simThread;
	selectMenu = 0;
}

TransformerMenu::~TransformerMenu()
{
	CommandToComponentMap::const_iterator begin = menuMap.begin();
	CommandToComponentMap::const_iterator end = menuMap.end();
	CommandIDManager* idmanager = CommandIDManager::Instance();
	while (begin != end) {
		idmanager->ReleaseCommandID(begin->first);
		begin++;
	}

}

HMENU TransformerMenu::GetMenu()
{
	if (selectMenu==0) selectMenu = this->MakeComponentList(pSim);
	return selectMenu;
}

// Responds to windows command
void TransformerMenu::OnSelect(UINT idd)
{
	// Should catch if idd is not in menuMap.
	CommandToComponentMap::const_iterator found = menuMap.find(idd);
	if (found==menuMap.end()) return;
	ComponentIdentifier index = found->second;

	int ret = pSimThread->Pause();	// Wait for thread to pause
	bool wasRunning = !(ret==Thread::ALREADY_PAUSED);
	int numTransformers = pSim->listOfTransformers.size();
	int numMovers = pSim->listOfAtomsMovers.size();
	if (index.iBaseType < pSim->listOfTransformers.size())
	{
		int numTransformer = index.iBaseType;
		// Transformer
		Transformer *pTransformer = 
			((Transformer *)pSim->listOfTransformers[numTransformer]);
		pTransformer->Transform(pSim->GetListOfAtoms());
	}
	else
	{
		int numMover = index.iBaseType - numTransformers;
		assert(numMover < numMovers);
		AtomsMover *pMover = ((AtomsMover *)pSim->listOfAtomsMovers[numMover]);
		pMover->Move(pSim->GetListOfAtoms());
	}
	if (wasRunning) pSimThread->Resume();	// Resume thread	
}

// responds to windows update command
void TransformerMenu::OnUpdateComponentUI (CCmdUI* pCmdUI)
{
	return;
}

// Private methods
// overrideable
bool TransformerMenu::IsSelected(UINT idd) const
{
	return false;
}

HMENU TransformerMenu::MakeComponentList(MDSimulation *sim)
{
	// empty the current list of IDs.
	this->menuMap.clear();

	int nTransformers = pSim->listOfTransformers.size();
	int nMovers = pSim->listOfAtomsMovers.size();

	ComponentIdentifier index;

	CMenu wholeMenu;
	wholeMenu.CreateMenu();

	DM_ComponentList * pTransformers = &(pSim->listOfTransformers);
	DM_ComponentList * pMovers = &(pSim->listOfAtomsMovers);

	for (int transformerNumber = 0; transformerNumber < nTransformers; transformerNumber++)
	{
		Transformer * pTransformer = (Transformer *)(pTransformers->at(transformerNumber));
		index.iBaseType = transformerNumber;

		int idd_current_user_number = CommandIDManager::Instance()->GetCommandID();
		this->menuMap[idd_current_user_number] = index;

		std::string sBangName = pTransformer->GetName();
		sBangName.append(std::string("!"));
		wholeMenu.AppendMenu(MF_STRING|MF_UNCHECKED,
			idd_current_user_number,_T(sBangName.c_str()) );
	}
	for (int moverNumber = 0; moverNumber < nMovers; moverNumber++)
	{
		AtomsMover * pMover = (AtomsMover *)(pMovers->at(moverNumber));
		index.iBaseType = moverNumber+nTransformers;

		int idd_current_user_number = CommandIDManager::Instance()->GetCommandID();
		this->menuMap[idd_current_user_number] = index;

		std::string sOnceName = pMover->GetName();
		sOnceName.append(std::string(" (one call)"));
		wholeMenu.AppendMenu(MF_STRING|MF_UNCHECKED,
			idd_current_user_number,_T(sOnceName.c_str()) );
	}
	return wholeMenu.Detach();
}

