// TransformerMenu.h: interface for the TransformerMenu class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

// Jim Sethna 19 Feb 2000
// The ComponentSelector is responsible for associating commands (ID numbers
// for Windows commands here) with which objects in the simulation
// are selected to run.
//
// This is separate from any menus, which associate a graphical selection
// with an ID through an ID to MenuHandle map.
//
// There is one ComponentSelector for each simulation.  There can be many
// menus which refer to the ComponentSelector.
//////////////////////////////////////////////////////////////////////

#if !defined(TRANSFORMERMENU_H)
#define TRANSFORMERMENU_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>
#include <map>
#include "Introspective.h"
#include "MDSimulation.h"

class Thread;

/*! \ingroup windows */
class TransformerMenu  
{
public:
	TransformerMenu(MDSimulation *sim, Thread *simThread);
	virtual ~TransformerMenu();
	virtual HMENU GetMenu(); // get the menu to append to title menu
	virtual void OnSelect(UINT idd); // responds to menu command
	virtual void OnUpdateComponentUI (CCmdUI* pCmdUI); // Called to decide whether to put a check mark.
private:
	virtual bool IsSelected(UINT idd) const; // How we query the sim to ask whether to put a check.
	virtual HMENU MakeComponentList(MDSimulation *sim);

	struct ComponentIdentifier {
		int iBaseType;
		int iType;
	};
	MDSimulation *pSim;
	Thread * pSimThread;
	HMENU selectMenu; // the menu folks ask for.

	// Assign own identification number to dialog box class
	typedef std::map<UINT,ComponentIdentifier> CommandToComponentMap;
	CommandToComponentMap menuMap;
};

#endif // !defined(TRANSFORMERMENU_H)
