// SimThreadController.cpp: implementation of the SimThreadController class.
//
/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/

#include "stdafx.h"
#include "MD.h"
#include "SimThreadController.h"
#include <assert.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SimThreadController::SimThreadController()
{
	pauseThread = false;
	stopThread = true;	// Must announce to SimThreadController just before starting thread: call Started()
}

SimThreadController::~SimThreadController()
{

}

void SimThreadController::Pause()
{
	if (ThreadRunning()) // Don't try to pause already halted threads
	{
		pauseThread = true;
		threadPaused.Lock();	// Wait for thread to pause
		threadPaused.Unlock();	// Probably unnecessary: conforms to Microsoft tradition
	}
}

void SimThreadController::Resume()
{
	assert(pauseThread=true);	// Should know thread is paused
	threadResume.SetEvent();	// Signal thread to start again
}

void SimThreadController::Paused()
{
	threadPaused.SetEvent();	// Thread announces that it's paused
	threadResume.Lock();		// Wait for main program to allow restart
	threadResume.Unlock();		// Probably unnecessary: conforms to Microsoft tradition
	pauseThread = false;
}

bool SimThreadController::PauseNow() const
{
	return pauseThread;
}

void SimThreadController::Stop()
{
	if (!stopThread)	// Check if thread is still running
	{
		stopThread=true;
		if (pauseThread)	// Check if thread is paused: need to restart to successfully end
			Resume();
		threadStopped.Lock();	// Wait for thread to stop
		threadStopped.Unlock();	// Probably unnecessary: conforms to Microsoft tradition
	}
}

void SimThreadController::Stopped()
{
	threadStopped.SetEvent();	// Thread announces it's demise
}

bool SimThreadController::StopNow() const
{
	return stopThread;
}

void SimThreadController::Started()
{
	stopThread = false;
}

bool SimThreadController::ThreadRunning()
{
	return (!stopThread && !pauseThread);
}