/*********************************************************************************

    Copyright (C) 2005 Nicholas P. Bailey, Thierry Cretegny, Andrew J. Dolgert,
    Christopher R. Myers, Jakob Schiotz, James P. Sethna, Valerie R. Coffman

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by the
    Free Software Foundation; either version 2.1 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
    for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit
    http://www.gnu.org/licenses/lgpl.html

    Author Contact Information:
    Jim Sethna      
    sethna@lassp.cornell.edu  
    117 Clark Hall
    Cornell University
    Ithaca, NY 14850

**********************************************************************************/
/*
 * vtkSphereFilter - Drew Dolgert 14 Feb 2000
 * This class is an observer of ListOfAtoms.  It is responsible
 * for translating a pointer to atom coordinates into 
 * a vtk representation.
 */
#ifndef _VTKSPHEREFILTER_H_
#define _VTKSPHEREFILTER_H_

#include "vtkPolyDataMapper.h"
#include "vtkPolyData.h"
#include "vtkFloatPoints.h"
#include "vtkSphereSource.h"
#include "vtkGlyph3D.h"

#include "SubjectObserver.h"
#include "vtkPropSource.h"
class MDSimulation;
class ListOfAtoms;
class SphereListener;
#include <vector>

/*! \ingroup windows */
class vtkSphereFilter : public Observer, public vtkPropSource
{
public:
	vtkSphereFilter(MDSimulation *sim);
	void Update(Subject *theSubject);
	virtual ~vtkSphereFilter();
	// Override from vtkPropSource
	virtual vtkPropCollection *GetProps();
private:
	void ChainIntegrity();
	void GetCurrentPolyData();
	ListOfAtoms* pAtoms;
	bool bDirty;
	vtkPolyDataMapper *Mapper;
	vtkActor *Actor;
	vtkPolyData *pAtomPolyData;
	vtkFloatPoints *pAtomCoords;
	vtkSphereSource *ball;
	vtkGlyph3D *balls;
};

#endif //_VTKSPHEREFILTER_H_