import math, Numeric, string
from DigitalMaterial.PX.Geometry.Common.Vectors import Vector
from Scientific.Geometry import Transformation, Quaternion

def DegreesToRadians(degrees):
    return (2.*math.pi)*(degrees/360.)

class Orientation:

    def __init__(self, EulerAngles=None, AxisAngle=None, RotationMatrix=None):
        self.rot = RotationMatrix

        if self.rot is None:
            self.rot = Numeric.zeros((3,3), Numeric.Float)

        if EulerAngles is not None:
            fi1 = EulerAngles[0]
            fi  = EulerAngles[1]
            fi2 = EulerAngles[2]
            from math import sin, cos, acos
            self.rot[0][0] = cos(fi1)*cos(fi2)-sin(fi1)*sin(fi2)*cos(fi);
            self.rot[0][1] = sin(fi1)*cos(fi2)+cos(fi1)*sin(fi2)*cos(fi);
            self.rot[0][2] = sin(fi2)*sin(fi);
            self.rot[1][0] = -cos(fi1)*sin(fi2)-sin(fi1)*cos(fi2)*cos(fi);
            self.rot[1][1] = -sin(fi1)*sin(fi2)+cos(fi1)*cos(fi2)*cos(fi);
            self.rot[1][2] = cos(fi2)*sin(fi);
            self.rot[2][0] = sin(fi1)*sin(fi);
            self.rot[2][1] = -cos(fi1)*sin(fi);
            self.rot[2][2] = cos(fi);

        elif AxisAngle is not None:

            pass
##            U = vec(1);
##            V = vec(2);
##            W = vec(3);
##            theta = scal;
   
##            g(1,1) = U^2*(1-cos(theta)) + cos(theta);
##            g(1,2) = U*V*(1-cos(theta)) - W*sin(theta);
##            g(1,3) = U*W*(1-cos(theta)) + V*sin(theta);
##            g(2,1) = V*U*(1-cos(theta)) + W*sin(theta);
##            g(2,2) = V^2*(1-cos(theta)) + cos(theta);
##            g(2,3) = V*W*(1-cos(theta)) - U*sin(theta);
##            g(3,1) = W*U*(1-cos(theta)) - V*sin(theta);
##            g(3,2) = W*V*(1-cos(theta)) + U*sin(theta);
##            g(3,3) = W^2*(1-cos(theta)) + cos(theta);


##    def axis(self):
##        return self.__axis

##    def angle(self):
##        return self.__angle

    def RotationMatrix(self):
        return self.rot

    def __mul__(self, other):
        m1 = self.RotationMatrix()
        m2 = other.RotationMatrix()
        m3 = Numeric.matrixmultiply(m1, m2)
        return Orientation(RotationMatrix = m3)

    def __div__(self, other):
        import LinearAlgebra
        m1 = self.RotationMatrix()
        m2 = LinearAlgebra.inverse(other.RotationMatrix())
        m3 = Numeric.matrixmultiply(m1, m2)
        return Orientation(RotationMatrix = m3)


def AxisAngle(orientation):

    """function [ax,ang] = ax_ang(a)
    %AX_ANG   determines the axis - angle representation of the
    %(mis)orientation from the rotation matrix formulation.
    %
    %       [axis,angle] = ax_ang(a)
    %               a = rotation matrix representation of (mis)orientation
    
    ax(1,1) = a(3,2) - a(2,3);
    ax(2,1) = a(1,3) - a(3,1);
    ax(3,1) = a(2,1) - a(1,2);

    ang = acos(((trace(a) -1)/2));
    """

    if type(orientation) == Numeric.ArrayType:
        r = orientation
    else:
        r = orientation.RotationMatrix()
    ax = Numeric.zeros((3,), Numeric.Float)
    ax[0] = r[2,1] - r[1,2]
    ax[1] = r[0,2] - r[2,0]
    ax[2] = r[1,0] - r[0,1]

    import math
    ang = math.acos(((Numeric.trace(r)-1.)/2.))

    return ax, ang


def Angle(orientation):

    if type(orientation) == Numeric.ArrayType:
        r = orientation
    else:
        r = orientation.RotationMatrix()

    ang = math.acos((0.5*(Numeric.trace(r)-1.)))
    return ang


def MinimumAxisAngle(orientation):

    """function [min_ax,min_ang] = ax_ang_min(a)
    %AX_ANG_MIN   determines the axis - angle representation of the
    %(mis)orientation from the rotation matrix formulation. All 24 symmetry
    %relations are investigated, and the one with minimum angle is chosen by
    %convention
    %
    %       calls function AX_ANG
    %
    %       [axis,angle] = ax_ang(a)
    %               a = rotation matrix representation of (mis)orientation
    # T(:,:,1) = eye(3);
# T(:,:,2) = [0 0 -1;0 -1 0;-1 0 0];%-eye(3);
# T(:,:,3) = [0 0 -1;0 1 0;1 0 0];
# T(:,:,4) = [-1 0 0;0 1 0;0 0 -1];
# T(:,:,5) = [0 0 1;0 1 0;-1 0 0];
# T(:,:,6) = [1 0 0;0 0 -1;0 1 0];
# T(:,:,7) = [1 0 0;0 -1 0;0 0 -1];
# T(:,:,8) = [1 0 0;0 0 1;0 -1 0];
# T(:,:,9) = [0 -1 0;1 0 0;0 0 1];
# T(:,:,10) =[-1 0 0;0 -1 0;0 0 1];
# T(:,:,11) =[0 1 0;-1 0 0;0 0 1];
# T(:,:,12) =[0 0 1;1 0 0;0 1 0];
# T(:,:,13) =[0 1 0;0 0 1;1 0 0];
# T(:,:,14) =[0 0 -1;-1 0 0;0 1 0];
# T(:,:,15) =[0 -1 0;0 0 1;-1 0 0];
# T(:,:,16) =[0 1 0;0 0 -1;-1 0 0];
# T(:,:,17) =[0 0 -1;1 0 0;0 -1 0];
# T(:,:,18) =[0 0 1;-1 0 0;0 -1 0];
# T(:,:,19) =[0 -1 0;0 0 -1;1 0 0];
# T(:,:,20) =[0 1 0;1 0 0;0 0 -1];
# T(:,:,21) =[-1 0 0;0 0 1;0 1 0];
# T(:,:,22) =[0 0 1;0 -1 0;1 0 0];
# T(:,:,23) =[0 -1 0;-1 0 0;0 0 -1];
# T(:,:,24) =[-1 0 0;0 0 -1;0 -1 0];
    """

    T = Numeric.zeros((24,3,3), Numeric.Float)
    T[0]  = Numeric.identity(3)
    T[1]  = Numeric.array([[0, 0, -1],[0, -1, 0],[-1, 0, 0]])
    T[2]  = Numeric.array([[0, 0, -1],[0, 1, 0],[1, 0, 0]])
    T[3]  = Numeric.array([[-1, 0, 0],[0, 1, 0],[0, 0, -1]])
    T[4]  = Numeric.array([[0, 0, 1],[0, 1, 0],[-1, 0, 0]])
    T[5]  = Numeric.array([[1, 0, 0],[0, 0, -1],[0, 1, 0]])
    T[6]  = Numeric.array([[1, 0, 0],[0, -1, 0],[0, 0, -1]])
    T[7]  = Numeric.array([[1, 0, 0],[0, 0, 1],[0, -1, 0]])
    T[8]  = Numeric.array([[0, -1, 0],[1, 0, 0],[0, 0, 1]])
    T[9]  = Numeric.array([[-1, 0, 0],[0, -1, 0],[0, 0, 1]])
    T[10] = Numeric.array([[0, 1, 0],[-1, 0, 0],[0, 0, 1]])
    T[11] = Numeric.array([[0, 0, 1],[1, 0, 0],[0, 1, 0]])
    T[12] = Numeric.array([[0, 1, 0],[0, 0, 1],[1, 0, 0]])
    T[13] = Numeric.array([[0, 0, -1],[-1, 0, 0],[0, 1, 0]])
    T[14] = Numeric.array([[0, -1, 0],[0, 0, 1],[-1, 0, 0]])
    T[15] = Numeric.array([[0, 1, 0],[0, 0, -1],[-1, 0, 0]])
    T[16] = Numeric.array([[0, 0, -1],[1, 0, 0],[0, -1, 0]])
    T[17] = Numeric.array([[0, 0, 1],[-1, 0, 0],[0, -1, 0]])
    T[18] = Numeric.array([[0, -1, 0],[0, 0, -1],[1, 0, 0]])
    T[19] = Numeric.array([[0, 1, 0],[1, 0, 0],[0, 0, -1]])
    T[20] = Numeric.array([[-1, 0, 0],[0, 0, 1],[0, 1, 0]])
    T[21] = Numeric.array([[0, 0, 1],[0, -1, 0],[1, 0, 0]])
    T[22] = Numeric.array([[0, -1, 0],[-1, 0, 0],[0, 0, -1]])
    T[23] = Numeric.array([[-1, 0, 0],[0, 0, -1],[0, -1, 0]])

    r = orientation.RotationMatrix()

    #a = map(lambda t, r=r: Numeric.matrixmultiply(t,r), T)
    #axesangles = map(AxisAngle, a)

    min_ang = 9999999999.
    ind = 0;
    for i in range(24):
        atmp = Numeric.matrixmultiply(T[i], r)
        ax,ang = AxisAngle(atmp);
        if ang < min_ang:
            min_ang = ang
            min_ax = ax

    return min_ax, min_ang


def MinimumAngle(orientation):

    T = Numeric.zeros((24,3,3), Numeric.Float)
    T[0]  = Numeric.identity(3)
    T[1]  = Numeric.array([[0, 0, -1],[0, -1, 0],[-1, 0, 0]])
    T[2]  = Numeric.array([[0, 0, -1],[0, 1, 0],[1, 0, 0]])
    T[3]  = Numeric.array([[-1, 0, 0],[0, 1, 0],[0, 0, -1]])
    T[4]  = Numeric.array([[0, 0, 1],[0, 1, 0],[-1, 0, 0]])
    T[5]  = Numeric.array([[1, 0, 0],[0, 0, -1],[0, 1, 0]])
    T[6]  = Numeric.array([[1, 0, 0],[0, -1, 0],[0, 0, -1]])
    T[7]  = Numeric.array([[1, 0, 0],[0, 0, 1],[0, -1, 0]])
    T[8]  = Numeric.array([[0, -1, 0],[1, 0, 0],[0, 0, 1]])
    T[9]  = Numeric.array([[-1, 0, 0],[0, -1, 0],[0, 0, 1]])
    T[10] = Numeric.array([[0, 1, 0],[-1, 0, 0],[0, 0, 1]])
    T[11] = Numeric.array([[0, 0, 1],[1, 0, 0],[0, 1, 0]])
    T[12] = Numeric.array([[0, 1, 0],[0, 0, 1],[1, 0, 0]])
    T[13] = Numeric.array([[0, 0, -1],[-1, 0, 0],[0, 1, 0]])
    T[14] = Numeric.array([[0, -1, 0],[0, 0, 1],[-1, 0, 0]])
    T[15] = Numeric.array([[0, 1, 0],[0, 0, -1],[-1, 0, 0]])
    T[16] = Numeric.array([[0, 0, -1],[1, 0, 0],[0, -1, 0]])
    T[17] = Numeric.array([[0, 0, 1],[-1, 0, 0],[0, -1, 0]])
    T[18] = Numeric.array([[0, -1, 0],[0, 0, -1],[1, 0, 0]])
    T[19] = Numeric.array([[0, 1, 0],[1, 0, 0],[0, 0, -1]])
    T[20] = Numeric.array([[-1, 0, 0],[0, 0, 1],[0, 1, 0]])
    T[21] = Numeric.array([[0, 0, 1],[0, -1, 0],[1, 0, 0]])
    T[22] = Numeric.array([[0, -1, 0],[-1, 0, 0],[0, 0, -1]])
    T[23] = Numeric.array([[-1, 0, 0],[0, 0, -1],[0, -1, 0]])

    r = orientation.RotationMatrix()

    #a = map(lambda t, r=r: Numeric.matrixmultiply(t,r), T)
    #axesangles = map(AxisAngle, a)

    min_ang = 9999999999.
    ind = 0;
    for i in range(24):
        atmp = Numeric.matrixmultiply(T[i], r)
        ang = Angle(atmp);
        if ang < min_ang:
            min_ang = ang

    return min_ang




class OrientationSource:

    pass


class RandomOrientationSource:

    def __init__(self):
        pass

    def SetNumberOfOrientations(self, number):
        self.NumberOfOrientations = number

    def GetOrientations(self):

        import math, whrandom
        orientations = []
        for n in range(self.NumberOfOrientations):
            angle1 = whrandom.uniform(0.,2.*math.pi)
            angle2 = math.acos(whrandom.uniform(-1., 1.))
            angle3 = whrandom.uniform(0.,2.*math.pi)
            orientations.append([angle1, angle2, angle3])
        return orientations
        

class BungeOrientationReader:

    def __init__(self, filename=None):
        self.filename = filename

    def SetFileName(self, filename):
        self.filename = filename

    def SetNumberOfOrientations(self, number):
        self.NumberOfOrientations = number
        
    def GetOrientations(self):

        f = open(self.filename, 'r')
        orientations = []
        n = self.NumberOfOrientations
        while n > 0:
            o = map(string.atof, string.split(f.readline()))
            angle1 = DegreesToRadians(o[0])
            angle2 = DegreesToRadians(o[1])
            angle3 = DegreesToRadians(o[2])
            orientations.append([angle1, angle2, angle3])
            n = n-1
        return orientations
        

# class OrientationSampler:

#     import ODFSampling

#     def __init__(self, texture=None):
#         self.texture = texture

#     def SetTexture(self, texture):
#         self.texture = texture

#     def SetNumberOfOrientations(self, number):
#         self.NumberOfOrientations = number
        
#     def GetOrientations(self):

#         self.sampler = ODFSampling.WrapSampling()
#         orientations = self.sampler.do_Sampling()
#         return orientations
        
