import Numeric
import GeoFmt
import xdrlib

PATCHTYPE_TRIANGLE = 0
PATCHTYPE_QUAD = 1


class GeoPackerImplem:

    def pack_id(self, t):
	pass

    def pack_control_pnt_indx(self, t, num_pts):
	pass
    
    def pack_properties(self, t):
	pass

    def pack_Cptc_GPoint(self, point):
	pass

    def pack_Cptc_GCurve(self, curve):
	pass

    def pack_Cptc_GPatch(self, patch):
	pass
	
    def pack_Cptc_TVertex(self, vertex):
	pass

    def pack_Cptc_TEdge(self, edge):
	pass

    def pack_Cptc_TSurface(self, surface):
	pass

    def pack_Cptc_TRegion(self, region):
	pass

    def pack_Cptc_GeomDesc(self, geomdesc):
	pass


class XDR_GeoPackerImplem(GeoPackerImplem, xdrlib.Packer):

    def pack_id(self, t):
	stringlen = len(t.id)+1
	self.pack_uint(stringlen)
	self.pack_string(t.id)
	return None

    def pack_control_pnt_indx(self, t, num_pts):
	self.pack_farray(num_pts, t.control_pnt_indx, self.pack_int)
	return None
    
    def unpack_properties(self):
	num_properties = self.unpack_int()
	props = [''] * num_properties
	values = [''] * num_properties
	for i in range(num_properties):
	    stringlen = self.unpack_uint()
	    props[i] = self.unpack_string()
	for i in range(num_properties):
	    stringlen = self.unpack_uint()
	    values[i] = self.unpack_string()
	return num_properties, props, values

    def pack_properties(self, t):
	self.pack_int(t.num_properties)
	for i in range(t.num_properties):
	    stringlen = len(t.props[i])+1
	    self.pack_uint(stringlen)
	    self.pack_string(t.props[i])
	for i in range(t.num_properties):
	    stringlen = len(t.values[i])+1
	    self.pack_uint(stringlen)
	    self.pack_string(t.values[i])
	return None

    def pack_Cptc_GPoint(self, p):
	self.pack_farray(3, p.coord, self.pack_double)
	return None

    def pack_Cptc_GCurve(self, c):
	self.pack_int(c.order)
	self.pack_control_pnt_indx(c, c.order+1)
	return None

    def pack_Cptc_GPatch(self, p):
        self.pack_int(p.type)
        self.pack_int(p.order_u)
        self.pack_int(p.order_v)
        if (p.type == PATCHTYPE_TRIANGLE):
            num = (p.order_u + 1) * (p.order_u + 2) / 2
        else:
            num = (p.order_u + 1) * (p.order_v + 1)
	self.pack_control_pnt_indx(p, num)
	return None

    def pack_Cptc_TVertex(self, v):
	self.pack_id(v)
	self.pack_int(v.control_pnt_indx)
	self.pack_properties(v)
	return None

    def pack_Cptc_TEdge(self, e):
	self.pack_id(e)

	# curves
	self.pack_int(e.num_curves)
	for i in xrange(e.num_curves):
	    c = e.curves[i]
	    self.pack_Cptc_GCurve(c)

	# bound_vertices
	self.pack_int(e.num_bound_vertices)
	self.pack_farray(e.num_bound_vertices, e.bound_vertices, self.pack_int)

	# properties
	self.pack_properties(e)
	return None

    def pack_Cptc_TSurface(self, s):
	self.pack_id(s)

	# patches
	self.pack_int(s.num_patches)
	for i in xrange(s.num_patches):
	    pa = s.patches[i]
	    self.pack_Cptc_GPatch(pa)

	# bound_edges
	self.pack_int(s.num_bound_edges)
	self.pack_farray(s.num_bound_edges, s.bound_edges, self.pack_int)

	# bound_vertices
	self.pack_int(s.num_bound_vertices)
	self.pack_farray(s.num_bound_vertices, s.bound_vertices, self.pack_int)

	# properties
	self.pack_properties(s)
	return None

    def pack_Cptc_TRegion(self, r):
	
	self.pack_id(r)
	
	# bound_surfaces
	self.pack_int(r.num_bound_surfaces)
	self.pack_farray(r.num_bound_surfaces, r.bound_surfaces, self.pack_int)
	# surface_orientations
	self.pack_farray(r.num_bound_surfaces, r.surface_orientation, \
			 self.pack_int)
	
	# bound_edges
	self.pack_int(r.num_bound_edges)
	self.pack_farray(r.num_bound_edges, r.bound_edges, self.pack_int)
	
	# bound_vertices
	self.pack_int(r.num_bound_vertices)
	self.pack_farray(r.num_bound_vertices, r.bound_vertices, self.pack_int)

	
	# properties
	self.pack_properties(r)
	return None

    def pack_Cptc_GeomDesc(self, g):

	self.pack_int(g.intrinsic_dimension)
	self.pack_int(g.embedded_dimension)
	self.pack_properties(g)

	self.pack_int(g.num_control_pts)
	self.pack_int(g.num_top_vertices)
	self.pack_int(g.num_top_edges)
	self.pack_int(g.num_top_surfaces)
	self.pack_int(g.num_top_regions)

	self.pack_farray(g.num_control_pts, g.control_points, 
			 self.pack_Cptc_GPoint)

	self.pack_farray(g.num_top_vertices, g.top_vertices, 
			 self.pack_Cptc_TVertex)

	self.pack_farray(g.num_top_edges, g.top_edges, 
			 self.pack_Cptc_TEdge)

	self.pack_farray(g.num_top_surfaces, g.top_surfaces, 
			 self.pack_Cptc_TSurface)

	self.pack_farray(g.num_top_regions, g.top_regions, 
			 self.pack_Cptc_TRegion)


class GeoPacker:

    def __init__(self, PackerImplem=XDR_GeoPackerImplem()):
	self.__PackerImplem = PackerImplem
	return None

    def __getattr__(self, attr):
	return getattr(self.__PackerImplem, attr)

    def pack_id(self, topoElement):
	self.__PackerImplem.pack_id(topoElement)

    def pack_control_pnt_indx(self, topoElement, num_pts):
	self.__PackerImplem.pack_control_pnt_indx(topoElement, num_pts)
    
    def pack_properties(self, topoElement):
	self.__PackerImplem.pack_properties(topoElement)

    def pack_Cptc_GPoint(self, point):
	self.__PackerImplem.pack_Cptc_GPoint(point)

    def pack_Cptc_GCurve(self, curve):
	self.__PackerImplem.pack_Cptc_GCurve(curve)

    def pack_Cptc_GPatch(self, patch):
	self.__PackerImplem.pack_Cptc_GPatch(patch)
	
    def pack_Cptc_TVertex(self, vertex):
	self.__PackerImplem.pack_Cptc_TVertex(vertex)

    def pack_Cptc_TEdge(self, edge):
	self.__PackerImplem.pack_Cptc_TEdge(edge)

    def pack_Cptc_TSurface(self, surface):
	self.__PackerImplem.pack_Cptc_TSurface(surface)

    def pack_Cptc_TRegion(self, region):
	self.__PackerImplem.pack_Cptc_TRegion(region)

    def pack_Cptc_GeomDesc(self, geomdesc):
	self.__PackerImplem.pack_Cptc_GeomDesc(geomdesc)

# -------------------------------------------------------------------------



class GeoUnpackerImplem:

    def unpack_id(self, t):
	pass

    def unpack_control_pnt_indx(self, num_pts):
	pass
    
    def unpack_properties(self):
	pass

    def unpack_Cptc_GPoint(self):
	pass

    def unpack_Cptc_GCurve(self):
	pass

    def unpack_Cptc_GPatch(self):
	pass
	
    def unpack_Cptc_TVertex(self):
	pass

    def unpack_Cptc_TEdge(self):
	pass

    def unpack_Cptc_TSurface(self):
	pass

    def unpack_Cptc_TRegion(self):
	pass

    def unpack_Cptc_GeomDesc(self):
	pass

class XDR_GeoUnpackerImplem(GeoUnpackerImplem, xdrlib.Unpacker):

    def unpack_id(self):
	stringlen = self.unpack_uint()
	id = self.unpack_string()
	return id

    def unpack_control_pnt_indx(self, num_pts):
	indx = self.unpack_farray(num_pts, self.unpack_int)
	return indx
    
    def unpack_properties(self):
	num_properties = self.unpack_int()
	props = [''] * num_properties
	values = [''] * num_properties
	for i in range(num_properties):
	    stringlen = self.unpack_uint()
	    props[i] = self.unpack_string()
	for i in range(num_properties):
	    stringlen = self.unpack_uint()
	    values[i] = self.unpack_string()
	return num_properties, props, values

    def unpack_Cptc_GPoint(self):
	p = GeoFmt.Cptc_GPoint()
	p.coord = Numeric.array(self.unpack_farray(3, self.unpack_double))
	return p

    def unpack_Cptc_GCurve(self):
	c = GeoFmt.Cptc_GCurve()
	c.order = self.unpack_int()
	c.control_pnt_indx = self.unpack_control_pnt_indx(c.order+1)
	return c

    def unpack_Cptc_GPatch(self):
	p = GeoFmt.Cptc_GPatch()
        p.type = self.unpack_int()
        p.order_u = self.unpack_int()
        p.order_v = self.unpack_int()
        if (p.type == PATCHTYPE_TRIANGLE):
            num = (p.order_u + 1) * (p.order_u + 2) / 2
        else:
            num = (p.order_u + 1) * (p.order_v + 1)
	p.control_pnt_indx = self.unpack_control_pnt_indx(num)
	return p
	
    def unpack_Cptc_TVertex(self):
	v = GeoFmt.Cptc_TVertex()
	v.id = self.unpack_id()
	v.control_pnt_indx = self.unpack_int()
	v.num_properties, v.props, v.values = self.unpack_properties()
	return v

    def unpack_Cptc_TEdge(self):
	e = GeoFmt.Cptc_TEdge()

	e.id = self.unpack_id()

	# curves
	e.num_curves = self.unpack_int()
	e.curves = []
	for i in xrange(e.num_curves):
	    c = self.unpack_Cptc_GCurve()
	    e.curves.append(c)

	# bound_vertices
	e.num_bound_vertices = self.unpack_int()
	e.bound_vertices = self.unpack_farray(e.num_bound_vertices, \
					      self.unpack_int)

	# properties
	e.num_properties, e.props, e.values = self.unpack_properties()
	return e


    def unpack_Cptc_TSurface(self):
	s = GeoFmt.Cptc_TSurface()

	s.id = self.unpack_id()

	# patches
	s.num_patches = self.unpack_int()
	s.patches = []
	for i in xrange(s.num_patches):
	    p = self.unpack_Cptc_GPatch()
	    s.patches.append(p)

	# bound_edges
	s.num_bound_edges = self.unpack_int()
	s.bound_edges = self.unpack_farray(s.num_bound_edges, self.unpack_int)

	# bound_vertices
	s.num_bound_vertices = self.unpack_int()
	s.bound_vertices = self.unpack_farray(s.num_bound_vertices, self.unpack_int)

	# properties
	s.num_properties, s.props, s.values = self.unpack_properties()
	return s


    def unpack_Cptc_TRegion(self):
	r = GeoFmt.Cptc_TRegion()
	
	r.id = self.unpack_id()
	
	# bound_surfaces
	r.num_bound_surfaces = self.unpack_int()
	r.bound_surfaces = self.unpack_farray(r.num_bound_surfaces, \
					      self.unpack_int)
	# surface_orientations
	r.surface_orientation = self.unpack_farray(r.num_bound_surfaces, \
						self.unpack_int)

	# bound_edges
	r.num_bound_edges = self.unpack_int()
	r.bound_edges = self.unpack_farray(r.num_bound_edges, self.unpack_int)
	
	# bound_vertices
	r.num_bound_vertices = self.unpack_int()
	r.bound_vertices = self.unpack_farray(r.num_bound_vertices, \
					      self.unpack_int)

	# properties
	r.num_properties, r.props, r.values = self.unpack_properties()
	return r

    def unpack_Cptc_GeomDesc(self):

	g = GeoFmt.Cptc_GeomDesc()

	g.intrinsic_dimension = self.unpack_int()
	g.embedded_dimension = self.unpack_int()
	g.num_properties, g.props, g.values = self.unpack_properties()

	g.num_control_pts = self.unpack_int()
	g.num_top_vertices = self.unpack_int()
	g.num_top_edges = self.unpack_int()
	g.num_top_surfaces = self.unpack_int()
	g.num_top_regions = self.unpack_int()

	g.control_points = self.unpack_farray(g.num_control_pts, 
					      self.unpack_Cptc_GPoint)

	g.top_vertices = self.unpack_farray(g.num_top_vertices, 
			 self.unpack_Cptc_TVertex)

	g.top_edges = self.unpack_farray(g.num_top_edges, 
					 self.unpack_Cptc_TEdge)

	g.top_surfaces = self.unpack_farray(g.num_top_surfaces, 
					    self.unpack_Cptc_TSurface)

	g.top_regions = self.unpack_farray(g.num_top_regions, 
					   self.unpack_Cptc_TRegion)
	
	g.update_properties()
	return g

class GeoUnpacker:

    def __init__(self, file='', UnpackerImplem=XDR_GeoUnpackerImplem):
	self.file = file
	f = open(self.file, 'rb')
	buf = f.read()
	self.__UnpackerImplem = UnpackerImplem(buf)
	return None

    def __getattr__(self, attr):
	return getattr(self.__UnpackerImplem, attr)

    def unpack_id(self):
	return self.__UnpackerImplem.unpack_id()

    def unpack_control_pnt_indx(self, num_pts):
	return self.__UnpackerImplem.unpack_control_pnt_indx(self, num_pts)
    
    def unpack_properties(self):
	return self.__UnpackerImplem.unpack_properties()

    def unpack_Cptc_GPoint(self):
	return self.__UnpackerImplem.unpack_Cptc_GPoint()

    def unpack_Cptc_GCurve(self):
	return self.__UnpackerImplem.unpack_Cptc_GCurve()

    def unpack_Cptc_GPatch(self):
	return self.__UnpackerImplem.unpack_Cptc_GPatch()
	
    def unpack_Cptc_TVertex(self):
	return self.__UnpackerImplem.unpack_Cptc_TVertex()

    def unpack_Cptc_TEdge(self):
	return self.__UnpackerImplem.unpack_Cptc_TEdge()

    def unpack_Cptc_TSurface(self):
	return self.__UnpackerImplem.unpack_Cptc_TSurface()

    def unpack_Cptc_TRegion(self):
	return self.__UnpackerImplem.unpack_Cptc_TRegion()

    def unpack_Cptc_GeomDesc(self):
	return self.__UnpackerImplem.unpack_Cptc_GeomDesc()




def unpack_geo_file(filename):
    u = GeoUnpacker(filename)
    hdr = u.unpack_fstring(256)
    geo = u.unpack_Cptc_GeomDesc()
    return hdr, geo


def print_geo_file_info(filename):
    u = GeoUnpacker(filename + '.geo')
    hdr = u.unpack_fstring(256)
    print hdr
    g = GeoFmt.Cptc_GeomDesc()
    g.intrinsic_dimension = u.unpack_int()
    g.embedded_dimension = u.unpack_int()
    g.num_properties, g.props, g.values = u.unpack_properties()
    g.num_control_pts = u.unpack_int()
    g.num_top_vertices = u.unpack_int()
    g.num_top_edges = u.unpack_int()
    g.num_top_surfaces = u.unpack_int()
    g.num_top_regions = u.unpack_int()
    print g


