# This module defines objects in 3D-space. Current implementation includes 
# point, line, polygon
#
# Written by David Chen
# last revision: 2001-3-16
#
import Numeric
import geoPoly3D
from Scientific.Geometry.VectorModule import Vector

class Polygon:
    def __init__(self, points = []):
        """Polygon
        
           Constructor: Polygon(points), where points is a list 
           of Vector.
        """
        if (len(points) < 3):
            raise ValueError, "Polygon must have at least three points"
        self.geoPoly = geoPoly3D.new_GeoPolys(1)
        self.geoPts = geoPoly3D.new_GeoPoints(len(points))
        i = 0        
        for pt in points:
            geoPoly3D.set_GeoPoint(self.geoPts, i, pt.x(), pt.y(), pt.z())
            i = i + 1
        if (geoPoly3D.GeoGenPoly(len(points), self.geoPts, self.geoPoly) == 0):
            raise ValueError, "Cannot generate polygon from geoPoly3D"
        #geoPoly3D.delete_GeoPoints(pts)        
    
    def __del__(self):
        geoPoly3D.delete_GeoPoints(self.geoPts)        
        geoPoly3D.delete_GeoPolys(self.geoPoly)        

    def lineIntersection(self, line):
        p = geoPoly3D.new_GeoPoints(1)
        if (geoPoly3D.GeoIntscLinePoly(line, self.geoPoly, p)):
            point = Vector( geoPoly3D.GeoPoint_x_get(p), \
                            geoPoly3D.GeoPoint_y_get(p), \
                            geoPoly3D.GeoPoint_z_get(p) )        
            geoPoly3D.delete_GeoPoints(p)
            return point
        else: return None    
        
class Line:
    def __init__(self, head = Vector(0,0,0), tail = Vector(0,0,0)):
        """Polygon
        
           Constructor: Line(head, tail), where head and tail are 
           starting and ending points of the line.
        """
        if (head == tail):
            raise ValueError, "Two points cannot be identical for a line"
        self.head = head
        self.tail = tail
        self.geoPt0 = geoPoly3D.new_GeoPoints(1)
        geoPoly3D.set_GeoPoint(self.geoPt0, 0, head.x(), \
                                               head.y(), \
                                               head.z())
        self.geoPt1 = geoPoly3D.new_GeoPoints(1)
        geoPoly3D.set_GeoPoint(self.geoPt1, 0, tail.x(), \
                                               tail.y(), \
                                               tail.z())
        self.geoLine = geoPoly3D.new_GeoLines(1)
        geoPoly3D.GeoGenLine(self.geoPt0, self.geoPt1, self.geoLine)
        
    def __del__(self):
        geoPoly3D.delete_GeoPoints(self.geoPt0)
        geoPoly3D.delete_GeoPoints(self.geoPt1)
        geoPoly3D.delete_GeoLines(self.geoLine)    
