/*
**      A strip version of Geo.h from FRANC3D 
**      Written by David Chen
**      last revision: 2001-3-16
**      
**  [Description from Geo.h] header file for the computational geometry package
**
**      Version:        1.0 Geo.h
**
**      History:        Created 28-Mar-88       Luiz F. Martha
**                      Modified 7-jun-88       PWW     version 1.0
**
*/

#ifndef _Geo_h
#define _Geo_h

/*      --------------------------------------------------
**      computational geometry data types :     
*/


typedef struct _GeoPoint { float x, y, z ; }        GeoPoint ;

typedef struct _GeoLine { GeoPoint d, e ; }         GeoLine ;

typedef struct _GeoPlane { GeoPoint a, b, c ; }     GeoPlane ;

typedef struct _GeoNormPlane { GeoPoint a, u, 
                                        v, n ; }    GeoNormPlane ;

typedef struct _GeoPoly { GeoPoint a, u, v, n ;
                  int n_verts; GeoPoint *verts ; }  GeoPoly ;

typedef struct _GeoPlanEqn { float a, b, c, d ; }   GeoPlanEqn ;

typedef struct _GeoSurfPoint { float x, y, z ;
                               float xn, yn, zn ; } GeoSurfPoint ;

typedef float                                       GeoPointPar ;

typedef struct _GeoSurfPar { float u, v ; }         GeoSurfPar ;

typedef enum _GeoStatus { GeoInvalid, GeoValid }    GeoStatus ;

typedef enum _GeoPredicate { GeoFalse, GeoTrue }    GeoPredicate ;

typedef enum _GeoPtTest { GeoAbove,
                          GeoOn,
                          GeoBelow         }        GeoPtTest ;

/*      --------------------------------------------------
**      function prototypes :   
*/

/* C/SWIG/Python Interfaces */
GeoPoint* new_GeoPoints(int num_pts) ;
void set_GeoPoint(GeoPoint* pts, int ptnum, float x, float y, float z) ;
void delete_GeoPoints(GeoPoint* pts) ;

GeoLine* new_GeoLines(int num_lines) ;
void delete_GeoLines(GeoLine* lines) ;

GeoPoly* new_GeoPolys(int num_polys) ;
void delete_GeoPolys(GeoPoly* polys) ;

/* prototypes */
GeoPredicate    GeoIntscPolyLine ( GeoSurfPar *line, GeoSurfPar *spar ) ;
GeoPredicate    GeoIsInsidePoly  ( GeoPoly *poly, GeoSurfPar *spar ) ;

void      GeoGenLine  ( GeoPoint *pt0, GeoPoint *pt1, GeoLine *line ) ;
void      GeoGenPlane ( GeoPoint *pt0, GeoPoint *pt1, GeoPoint *pt2,
                        GeoPlane *plane ) ;
void      GeoGenPlanEqn( GeoPoint *pt0, GeoPoint *pt1, GeoPoint *pt2,
                         GeoPlanEqn *plan_eqn ) ;

GeoStatus GeoGenNormPlane  ( int *n_verts, GeoPoint *verts, GeoNormPlane *plane);
int GeoGenPoly  ( int n_verts, GeoPoint *verts, GeoPoly *poly ) ;

void      GeoEvalLine    ( GeoPointPar *par, GeoLine *line, GeoPoint *pt ) ;
void      GeoEvalPlane   ( GeoSurfPar  *par, GeoPlane *pln, GeoPoint *pt ) ;

float        GeoDotProd    ( GeoPoint *in0, GeoPoint *in1 ) ;
void         GeoCrossProd  ( GeoPoint *in0, GeoPoint *in1, GeoPoint *out ) ;
float        GeoTripleProd ( GeoPoint *in0, GeoPoint *in1, GeoPoint *in2 ) ;
GeoPredicate GeoNullVec    ( GeoPoint *vec, float *tolerance ) ;
float        GeoVecLen     ( GeoPoint *vec ) ;
void         GeoVecNormalize ( GeoPoint *vec, GeoPoint *n ) ;
float        GeoDist       ( GeoPoint *in0, GeoPoint *in1 ) ;
void         GeoDiffVec    ( GeoPoint *in0, GeoPoint *in1, GeoPoint *out ) ;
void         GeoSumVec     ( GeoPoint *in0, GeoPoint *in1, GeoPoint *out ) ;
void         GeoProdScalVec ( GeoPoint *in, float *csi, GeoPoint *out ) ;

void      GeoRotateX ( float *angle, GeoPoint *in, GeoPoint *out ) ;
void      GeoRotateY ( float *angle, GeoPoint *in, GeoPoint *out ) ;
void      GeoRotateZ ( float *angle, GeoPoint *in, GeoPoint *out ) ;
void      GeoRotateAxis ( GeoPoint *axis, float *angle, 
                          GeoPoint *in, GeoPoint *out ) ;

GeoStatus GeoIntscLinePlane ( GeoLine *line, GeoPlane *plane, 
                              GeoPoint *pt, GeoPointPar *ptpar,
                              GeoSurfPar *spar ) ;
int GeoIntscLinePoly ( GeoLine *line, GeoPoly *poly, 
                              GeoPoint *pt ) ;

GeoStatus GeoPolyNormal  ( int *n_verts, GeoPoint *verts,  GeoPoint *norm ) ;

GeoStatus  GeoLstSqrPlanEqn( int *num_pts, GeoPoint *pts, 
                             GeoPlanEqn *plan_eqn ) ;

void      GeoClstPtLine  ( GeoLine *line, GeoPoint *pt, GeoPoint *clst,
                           GeoPointPar *par ) ;
void      GeoClstPtNormPlane  ( GeoNormPlane *plane, GeoPoint *pt, 
                                GeoPoint *clst, GeoSurfPar *spar ) ;
GeoStatus GeoClstPtPoly  ( GeoPoly *poly, GeoPoint *pt, 
                           GeoPoint *clst, GeoSurfPar *spar ) ;

GeoPredicate GeoIsOnPlane( GeoPoint  *pt, GeoNormPlane  *pla ) ;

GeoStatus GeoInvert2x2 ( float a[2][2], float b[2][2] ) ;
GeoStatus GeoInvert3x3 ( float a[3][3], float b[3][3] ) ;
GeoStatus GeodInvert3x3 ( double a[3][3], double b[3][3] ) ;

void    GeoCentroid( int *num_pts,  GeoPoint *pts,  GeoPoint *cntrd ) ;

GeoStatus GeoPerpLine ( GeoLine *in0, GeoLine *in1, GeoLine *out,
                        GeoPointPar *par0, GeoPointPar *par1,
                        GeoPointPar *parout ) ;

void GeoPtNormToParPlan( GeoPoint *pt, GeoPoint *normal, GeoNormPlane *plane ) ;

void GeoPlanEqnToParPlan( GeoPlanEqn *plan_eqn, GeoNormPlane *plane ) ;

GeoPtTest GeoSideofPlan( GeoPlanEqn *plan_eqn, GeoPoint *pt ) ;

void GeoSurfInterp( int *num_pts, float *map_func,GeoSurfPar *pts_val, 
                        GeoSurfPar *interp_val ) ;

void GeoSurfJacob( int *num_pts, float *map_deriv_s, float *map_deriv_t, 
                        GeoSurfPar *pts_val, float jac[2][2] ) ;

float GeoAngVecAboutLine( GeoLine *l, GeoPoint *v0, GeoPoint *v1 ) ;
float GeoAngPtAboutLine ( GeoLine *l, GeoPoint *p0, GeoPoint *p1 ) ;
float GeoAngPtAboutPt( GeoPoint *pivot, GeoPoint *p0, GeoPoint *p1 ) ;

GeoStatus GeoProjVecPlane( GeoPoint *vec, GeoPlane *plane, GeoPoint *proj ) ;

#endif
