import random, math, Numeric, string

class NonuniformPointGenerator:

    def __init__(self, inputfile=None, seed=None):
        self.inputfile = inputfile

        if seed is None:
            self.seed = random.randint(0,1000000)
        else:
            self.seed = seed

        self.poissrv = None

    def _readInputFile(self):

        if self.inputfile is None:
            raise ValueError, "input file not specified"

        f = open(self.inputfile, 'r')
        self.xmax, self.ymax, self.zmax = \
                   map(string.atof, string.split(f.readline()))
        self.nx, self.ny, self.nz = \
                 map(string.atoi, string.split(f.readline()))
        self.dx = self.xmax/self.nx
        self.dy = self.ymax/self.ny
        self.dz = self.zmax/self.nz

        self.numsubdiv = self.nx * self.ny * self.nz
        print self.numsubdiv
        self.p  = Numeric.zeros((self.numsubdiv,)*3, Numeric.Float)
        self.pp = Numeric.zeros((self.numsubdiv+1, 4))


        for i in xrange(self.numsubdiv):
            xtmp, ytmp, ztmp, savg = \
                       map(string.atof, string.split(f.readline()))
            ii = int(math.floor(xtmp/self.dx))
            jj = int(math.floor(ytmp/self.dy))
            kk = int(math.floor(ztmp/self.dz))

            self.p[ii][jj][kk] = (self.dx*self.dy*self.dz)/savg
            print i, ii, jj, kk
            self.pp[i+1][0] = ii
            self.pp[i+1][1] = jj
            self.pp[i+1][2] = kk
            self.pp[i+1][3] = self.pp[i][3] + self.p[ii][jj][kk]

        f.close()

    def _initialize(self):

        self._readInputFile()
        self.sp = self.pp[self.numsubdiv][3]

        for i in xrange(self.numsubdiv):
            self.pp[i][3] = self.pp[i][3]/self.sp

    def _generatePoisson_rv(self):
        self.poissrv = poissrnd(self.sp)

    def _generatePoints(self):

        if self.poissrv is None:
            self._generatePoisson_rv()
        self.pts = Numeric.zeros((self.poissrv, 3), Numeric.Float)

        for i in xrange(self.poissrv):
            index = 0
            r = random.random()
            while (self.pp[index][3] < r):
                index = index + 1

            ii, jj, kk = self.pp[index][0:3]
            self.pts[i][0] = self.dx*random.random() + ii*self.dx
            self.pts[i][1] = self.dy*random.random() + jj*self.dy
            self.pts[i][2] = self.dz*random.random() + kk*self.dz


    def _write(self, outputFile=None):
        
        f = open(outputFile, 'w')
        f.write('3 nonuniformPoints\n')
        f.write('%d\n' % (self.poissrv))
        for i in xrange(self.poissrv):
            f.write('%20.16f %20.16f %20.16f\n' % \
                    (self.pts[i][0]-0.5*self.xmax, \
                     self.pts[i][1]-0.5*self.ymax, \
                     self.pts[i][2]-0.5*self.zmax))
        f.close()



def poissrnd(lam):

    t = 0
    N = 0
    laminv = 1./lam

    while t < 1:
        r = random.random()
        r2 = -laminv*math.log(r)
        t = t + r2
        N = N+1

    return N
