from DigitalMaterial.PX.Geometry.Common \
     import Points, Vectors, GeoFmt, GeoFmtIO, PointSetSource
from DigitalMaterial.PX.Tools.Utils.SystemCall import SystemCall
from DigitalMaterial.PX.Tools.Utils.FileIO import *
import Numeric
import string
import tempfile
import UserDict, UserList
Vector = Vectors.Vector

qh_INFINITE_VERTEX = 0


def PointsOutside(pointSet, geom):
    """return list of point IDs in pointSet outside of geom, based on use
    of geom.inside()"""
    pointIDlist = []
    for pi in range(len(pointSet)):
        if not geom.isInside(pointSet[pi]):
            pointIDlist.append(pi)
    return pointIDlist

def PointsInside(pointSet, geom):
    """return list of point IDs in pointSet inside of geom, based on use
    of geom.inside()"""
    pointIDlist = []
    for pi in range(len(pointSet)):
        if geom.isInside(pointSet[pi]):
            pointIDlist.append(pi)
    return pointIDlist

def pointReflect(point, hyperplane):

    p = Vector(point)  # point should be a Vector
    h = Vector(hyperplane.coeffs) 
    p_dot_n = p*h + hyperplane.offset
    reflectedPoint = p - 2.*p_dot_n*h
    return Vector(reflectedPoint)

def pointProject(point, hyperplane):

    p = Vector(point)  # point should be a Vector
    h = Vector(hyperplane.coeffs) 
    p_dot_n = p*h + hyperplane.offset
    projectedPoint = p - p_dot_n*h
    return Vector(projectedPoint)


def isUnboundedRegion(region):
    if region.vertices.count(qh_INFINITE_VERTEX) > 0: return 1
    else: return 0
        
def isUnboundedInterface(interface):
    if interface.vertices.count(qh_INFINITE_VERTEX) > 0: return 1
    else: return 0      

# -------------------------------------------------------------------------

class qhullGenerator(SystemCall):

    def __init__(self, optionString):
        SystemCall.__init__(self, 'qhull', optionString)

    def SetPointSet(self, pointSet):
        self.pointSet = pointSet
        self.infile = tempfile.mktemp()
        self.pointSet.toFile(self.infile)

    def generate(self):
        self.SetTemporaryOutputFile()
        if self.infile is None:
            raise ValueError, "input data not provided"
        self.run()

# -------------------------------------------------------------------------


class qhull_GeomElement:

    def nvertices(self):
        return (len(self.vertices))

    def containsVertex(self, v):
        if self.vertices.count(v) > 0: return 1
        else: return 0


class HullFacet(qhull_GeomElement):

    def __init__(self, ID=None, vertices=[]):
        self.ID        = ID
        self.vertices  = vertices
        
    def __repr__(self):
        s = 'HullFacet( %s ): ' % (self.ID)
        s = s + 'vertices = %s' % (self.vertices)
        s = s + '\n'
        return s

    def __cmp__(self, other):
        status = 0
        status = status + cmp(self.ID, other.ID)
        status = status + cmp(self.vertices,  other.vertices)
        return status
        
class Hyperplane:

    def __init__(self, inputSites=(), coeffs=[], offset=0., nindices=0):
        self.inputSites = inputSites
        self.coeffs     = coeffs
        self.offset     = offset
        self.nindices   = nindices

# -------------------------------------------------------------------------

class ElementDict(UserDict.UserDict):

    def __init__(self, data=None):
        UserDict.UserDict.__init__(self, data)

    def __repr__(self):
        s = ''
        for key, val in self.data.items():
            s = s + '%s %s\n' % (key, val)
        return s

#    def __call__(self, index):
#        return self.data[index]

#    def __getitem__(self, index):
#        return self.data.items()[index]


# -------------------------------------------------------------------------

class qhull_ConvexHull:

    def __init__(self, pointSet):

        self.qhull_options = 'o n'
        self.qhull = qhullGenerator(self.qhull_options)
        self.qhull.SetPointSet(pointSet)
        self.qhull.generate()
        self.infile = self.qhull.outfile

        f = open(self.infile, 'r')
        self.__parse_o(f)
        self.__parse_n(f)
        f.close()
        import os
        os.remove(self.infile)


    def __parse_o(self, fp):
        
        self.dim = parse_atoi(fp)
        self._nvertices, self._nregions, ignore = parse_map_atoi(fp)

        # vertices/points
        self.Vertices = []
        for n in range(self._nvertices):
            p = parse_map_atof(fp)
            self.Vertices.append(Vector(p))

        # facetSet
        self.Facets = []
        for n in range(self._nregions):
            line = parse_map_atoi(fp)
            nvertices = line[0]
            vertices = line[1:]
            fac = HullFacet(n, vertices)
            self.Facets.append(fac)


    def __parse_n(self, fp):
        
        dim = parse_atoi(fp)-1
        if dim != self.dim:
            raise ValueError, "model dimension not consistently defined"
        self.nnormals = parse_atoi(fp)
        self.normalSet = {}
        for n in range(self.nnormals):
            line = string.split(fp.readline())
            coeffs = map(string.atof,line[0:self.dim])
            offset = string.atof(line[-1])
            inputSites = (None,)
            self.normalSet[n]=Hyperplane(inputSites, coeffs, offset)
        return

# -------------------------------------------------------------------------






    
