from DigitalMaterial.PX.Geometry.Common.Vectors import *
import umath

class Polygon:

    def __init__(self, npoints, points=[]):
        """constructs a polygon from the specified set of points;
        points is a list of Vector objects"""
        
        self.npoints = npoints
        self.points  = points
        return None

    def inside(self,point):
        """returns 1 if point is inside the polygon, 0 if not;
        point is a Vector object"""

        sum = 0
        tol = 0.01
        dp0 = self.points[-1] - point

        for p in self.points:
            dp1 = p - point
            if dp0.length() == 0. or dp1.length() == 0.:
                return 1
            ang = dp0.angle(dp1)
            if dp0[0]*dp1[1]-dp0[1]*dp1[0] > 0:
                ang = -ang
            sum = sum + ang
            dp0 = dp1
            
        if abs(sum) > tol:
            return 1
        return 0


class RegularPolygon(Polygon):

    def __init__(self, numSides, outerRadius=1., orientation=0., 
                 center=Vector([0.,0.])):
        """constructs a RegularPolygon of the specified number of sides,
        with the specified outer radius, orientation, and center"""

        self.numSides     = numSides
        self.outerRadius  = outerRadius
        self.orientation  = orientation
        self.center       = center
        self.sidelen      = 2.*self.outerRadius*(umath.pi/self.numSides)
        self.innerRadius  = 0.5*self.sidelen/umath.tan(umath.pi/self.numSides)

        pts = []
        npts = self.numSides
        ang = self.orientation
        for i in range(npts):
            p = self.center + \
                self.outerRadius * Vector([umath.cos(ang), umath.sin(ang)])
            pts.append(p)
            ang = ang + 2.*umath.pi/npts
        Polygon.__init__(self, npts, pts)
        return None

    def boundingRadius(self):
        return self.outerRadius

