import VTK
from DigitalMaterial.PX.Attributes.DM_VTK import VTKFieldArray

class VTKPolyhedralTesselation:

    def __init__(self, polycrystal=None, vtkfilename=None):

	self.polycrystal = polycrystal
        self.ngrains = self.polycrystal.NumberOfRegions()
	self.vtkfilename = None

	if self.polycrystal is not None and vtkfilename is None:
	    import tempfile
	    self.vtkfilename = tempfile.mktemp('.poly.vtk')
            import PolycrystalIO
            writer = PolycrystalIO.VTKPolycrystalWriter(self.vtkfilename)
	    writer.write(self.polycrystal)
            self._build_VTKGBMap()
            
	if self.vtkfilename is not None:
	    self.reader = VTK.vtkPolyDataReader()
	    self.reader.SetFileName(self.vtkfilename)
	    self.GBPolyData = self.reader.GetOutput()
	    self.reader.Update()
        
            
    def _build_VTKGBMap(self):

        self._VTKGBMap = {}

        ncell = 0
        for inter in self.polycrystal.Interfaces():
            self._VTKGBMap[inter.ID] = ncell
            ncell = ncell+1


    def __repr__(self):
        s = 'VTKPolycrystal ( ' + self.polycrystal.ID + ' )\n'
        return s

    def __getattr__(self, attr):
        pass
        if 0:
            try:
                self.ExtractFieldArray(attr)
                return self.__dict__[attr]
            except:
                raise AttributeError, ""

    def ExtractFieldArray(self, arrayName):
        a = self.GBPolyData.GetCellData().GetFieldData().GetArray(arrayName)
        self.__dict__[arrayName] = VTKFieldArray(arrayName, a)


# -------------------------------------------------------------------------

class VTKPolyhedralTesselationSource:

    pass

class VTKPolyhedralTesselationFilter(VTKPolyhedralTesselationSource):

    def __init__(self):
        self.vtktesselation = None

    def SetInput(self, tesselation):
        self.tesselation = tesselation

    def GetOutput(self):
        if self.vtktesselation is None:
            self._buildTesselation()
        return self.vtktesselation

    def _buildTesselation(self):

        self.nregions = self.tesselation.NumberOfRegions()
	self.vtkfilename = None

	if self.polycrystal is not None and vtkfilename is None:
	    import tempfile
	    self.vtkfilename = tempfile.mktemp('.poly.vtk')
            import PolycrystalIO
            writer = PolycrystalIO.VTKPolycrystalWriter(self.vtkfilename)
	    writer.write(self.polycrystal)
            self._build_VTKGBMap()
            
	if self.vtkfilename is not None:
	    self.reader = VTK.vtkPolyDataReader()
	    self.reader.SetFileName(self.vtkfilename)
	    self.GBPolyData = self.reader.GetOutput()
	    self.reader.Update()
        
            
    def _build_VTKGBMap(self):

        self._VTKGBMap = {}

        ncell = 0
        for inter in self.polycrystal.Interfaces():
            self._VTKGBMap[inter.ID] = ncell
            ncell = ncell+1


def VTKGBPolyDataToPolycrystal(gbpolydata):

    from DigitalMaterial.PX.Geometry.Common.Vectors import Vector

    npoints = gbpolydata.GetNumberOfPoints()
    vertexSet = {}
    for np in range(npoints):
        vertexSet[np] = Vector(gbpolydata.GetPoint(np))

    

# -------------------------------------------------------------------------

def test():
    import Polycrystal, PolycrystalBuilders
    builder = PolycrystalBuilders.VoronoiPolycrystalBuilder("D3 5")
    poly = builder.GetPolycrystal()
    vtkpoly = VTKPolycrystal(polycrystal=poly)
    return vtkpoly

if __name__ == '__main__':
    vtkpoly = test()
