#include <stdio.h>
#include <math.h>

#define MAXFACES     10000
#define MAXFACEVERTS 30
#define MAXVERTS     10000
#define MAXGBS       10000
#define MAXLENS      10000
#define MAXRND      32768.0
#define NOINT       -1111
#define TINY        1E-5

void vecsum3 (float v1[],float v2[],float outvec[]);
void vecdiff3(float v1[],float v2[],float outvec[]);
void veccross3(float v1[],float v2[],float outvec[]);
float vecnorm3(float v1[]);
void linplanint(float plan[][3],float lin[][3],float intpt[]);
float rand1(void);
void srand1(unsigned int seed);
float matdet3 (float mat[][3]);
int round(float in);
float absval(float in);
void sortgbs(unsigned long n, float arr[][3]);

static unsigned long int next = 1;

int intersect(double* point, double *vec)
{
  int i,j;
  int faces[MAXFACES][MAXFACEVERTS];
  int nfverts[MAXFACES];
  float verts[MAXVERTS][3];
  float lin[2][3];
  float plan[2][3];
  int nfaces, nverts;

  FILE *fvert,*fface,*foutgbs,*foutlens;  
  char c, cold;

  int ngbs,finngbs,lencnt;
  float gbs[MAXGBS][3];
  float fingbs[MAXGBS][3];
  float vert1[3],vert2[3],vert3[3],e1[3],e2[3],nint[3],n[3];
  float intpt[3];

  float suminout;
  float tempvert1[3],tempvert2[3],e[3],v[3],vv[3],evx[3],temvec[3];
  float inout;
  float lentemp;
  float lens[MAXLENS];
  float gb1[3],gb2[3];

  if (argc == 5)
    srand1(atoi(argv[4]));
  else
    srand1(1);

  sscanf(argv[3],"%c %f %c %f %c %f %c %f %c %f %c %f %c", \
	 &c,&lin[0][0],&c,&lin[0][1],&c,&lin[0][2],         \
	 &c,&lin[1][0],&c,&lin[1][1],&c,&lin[1][2],&c);

/***************************
Here we read in the data files for the tessellation. 
This is currently a very inelegant way of doing it,
and invites improvement
***************************/
  fvert = fopen(argv[1],"r");  
  fface = fopen(argv[2],"r");  

  nfaces=0;
   while ((c = getc(fface)) != EOF ){ 
     if (c == '\n'){ 
       nfaces++; 
     } 
   } 
  fclose(fface);
  fface = fopen(argv[2],"r");
  for (i = 0; i < nfaces; i++){
    j=0;
    
    cold = getc(fface);
    while( (c = getc(fface)) != '\n' ){
      if (cold == ' ' && c != ' '){
	j++;
      }
      cold = c;
    }
    nfverts[i] = j+1;  
  }
  fclose(fface);
  fface = fopen(argv[2],"r");  
  for (i=0;i<nfaces;i++){
    for (j=0;j<nfverts[i];j++){
      fscanf(fface,"%d",&faces[i][j]);
    }
    faces[i][j] = faces[i][0];
  }
  fclose(fface);
/*  for(i=0;i<nfaces;i++){
    for(j=0;j<=nfverts[i];j++){
      faces[i][j] -= 1;
    }
  }*/

  nverts=0;
   while ((c = getc(fvert)) != EOF ){ 
     if (c == '\n'){ 
       nverts++; 
     } 
   }   
  fclose(fvert);
  fvert = fopen(argv[1],"r");
  for (i=0;i<nverts;i++){
    fscanf(fvert,"%f %f %f",&verts[i][0],&verts[i][1],&verts[i][2]);
  }
/***************************
Here ends the reading of the input
***************************/
  ngbs = 0;
  for (i=0;i<nfaces;i++){
    /* printf("i = %d\n",i);    */
    vert1[0] = verts[faces[i][0]][0];
    vert1[1] = verts[faces[i][0]][1];
    vert1[2] = verts[faces[i][0]][2];
    /* printf("vert1 %f %f %f\n",vert1[0],vert1[1],vert1[2]);*/

    vert2[0] = verts[faces[i][1]][0];
    vert2[1] = verts[faces[i][1]][1];
    vert2[2] = verts[faces[i][1]][2];
    /* printf("vert2 %f %f %f\n",vert2[0],vert2[1],vert2[2]);*/

    vert3[0] = verts[faces[i][2]][0];
    vert3[1] = verts[faces[i][2]][1];
    vert3[2] = verts[faces[i][2]][2];
    /* printf("vert3 %f %f %f\n",vert3[0],vert3[1],vert3[2]);*/
    
    vecdiff3(vert2,vert1,e1);
    vecdiff3(vert3,vert2,e2);
    /* printf("test1\n");  */
    /* printf("e1 %f %f %f\n",e1[0],e1[1],e1[2]);*/
    /* printf("e2 %f %f %f\n",e2[0],e2[1],e2[2]);*/

    veccross3(e1,e2,nint);
    /* printf("test1a\n %f",vecnorm3(nint));  */

    n[0] = nint[0]/vecnorm3(nint);
    n[1] = nint[1]/vecnorm3(nint);
    n[2] = nint[2]/vecnorm3(nint);
/*printf("test1b\n");  */
/*printf("n %f %f %f\n",n[0],n[1],n[2]);*/
    
    if (n[2] < 0){
      n[0] = -n[0];
      n[1] = -n[1];
      n[2] = -n[2];
    }
    else if (n[2] == 0 && n[1] < 0){
      n[0] = -n[0];
      n[1] = -n[1];
      n[2] = -n[2];
    }
    else if (n[2] == 0 && n[1] == 0 && n[0] < 0){
      n[0] = -n[0];
      n[1] = -n[1];
      n[2] = -n[2];
    }
/*printf("test2\n");  */

    plan[0][0] = vert1[0];
    plan[0][1] = vert1[1];
    plan[0][2] = vert1[2];
    plan[1][0] = n[0];
    plan[1][1] = n[1];
    plan[1][2] = n[2];
/*printf("p = %f %f %f %f %f %f\n",plan[0][0],plan[0][1],plan[0][2],plan[1][0],plan[1][1],plan[1][2]);
printf("l = %f %f %f %f %f %f\n",lin[0][0],lin[0][1],lin[0][2],lin[1][0],lin[1][1],lin[1][2]);*/

    /* printf("test3\n");  */
    linplanint(plan,lin,intpt);
    /* printf("test4\n"); */
/*printf("%f %f %f\n",intpt[0],intpt[1],intpt[2]);
scanf("%c",&c);*/
    if(intpt[0] != NOINT && intpt[1] != NOINT && intpt[2] != NOINT){
      suminout = 0;
      for(j=0;j<nfverts[i];j++){
	tempvert1[0] = verts[faces[i][j]][0];
	tempvert1[1] = verts[faces[i][j]][1];
	tempvert1[2] = verts[faces[i][j]][2];

	tempvert2[0] = verts[faces[i][j+1]][0];
	tempvert2[1] = verts[faces[i][j+1]][1];
	tempvert2[2] = verts[faces[i][j+1]][2];

	vecdiff3(tempvert2,tempvert1,e);
	vecdiff3(intpt,tempvert1,v);
	vecdiff3(intpt,tempvert2,vv);
	veccross3(e,v,evx);
/*printf("%f %f %f\n",e[0],e[1],e[2]);*/
	if(vecnorm3(vv) < TINY | vecnorm3(v) < TINY){
	  suminout = nfverts[i];
	  break;
	}
	else if (vecnorm3(evx) == 0){
	  suminout -= 1;
	}
	else {
	  temvec[0] = evx[0]/vecnorm3(evx)*n[0];
	  temvec[1] = evx[1]/vecnorm3(evx)*n[1];
	  temvec[2] = evx[2]/vecnorm3(evx)*n[2];

	  inout = temvec[0] + temvec[1] + temvec[2];
/*printf("inout = %f\n",inout);
scanf("%c",&c);*/
	  suminout += inout;
	}
      }
/*printf("test5\n");  */

      if (round(absval(suminout)) == nfverts[i]){
	gbs[ngbs][0] = intpt[0];
	gbs[ngbs][1] = intpt[1];
	gbs[ngbs][2] = intpt[2];
	ngbs++;
/*printf("%d \n",ngbs);*/
      }
    }
  }
/*printf("test6\n");  */

  sortgbs(ngbs,gbs);

/*printf("test7\n");  */

  fingbs[0][0] = gbs[0][0];
  fingbs[0][1] = gbs[0][1];
  fingbs[0][2] = gbs[0][2];
  finngbs = 1;
  for(i=0;i<ngbs-1;i++){
    gb1[0] = gbs[i+1][0];
    gb1[1] = gbs[i+1][1];
    gb1[2] = gbs[i+1][2];

    gb2[0] = fingbs[finngbs-1][0];
    gb2[1] = fingbs[finngbs-1][1];
    gb2[2] = fingbs[finngbs-1][2];

    vecdiff3(gb1,gb2,temvec);
    lentemp = vecnorm3(temvec);
    if (lentemp > TINY){
      fingbs[finngbs][0] = gbs[i+1][0];
      fingbs[finngbs][1] = gbs[i+1][1];
      fingbs[finngbs][2] = gbs[i+1][2];
      lens[finngbs-1] = lentemp;
      finngbs ++;
    }
  }
/*printf("test8\n");  */

  foutgbs = fopen("gbs","w");
  foutlens = fopen("lens","w");
  for(i=0;i<finngbs;i++){
    fprintf(foutgbs,"%f %f %f\n",fingbs[i][0],fingbs[i][1],fingbs[i][2]);
  }
  for(i=0;i<finngbs-1;i++){
    fprintf(foutlens,"%f\n",lens[i]);
  }
  fclose(foutgbs);
  fclose(foutlens);

}

void vecsum3(float v1[],float v2[],float outvec[])
{
int i;
for (i=0;i<3;i++){
  outvec[0] = v1[0]+v2[0];
  outvec[1] = v1[1]+v2[1];
  outvec[2] = v1[2]+v2[2];
}
}
void vecdiff3(float v1[],float v2[],float outvec[])
{
int i;
for (i=0;i<3;i++){
  outvec[0] = v1[0]-v2[0];
  outvec[1] = v1[1]-v2[1];
  outvec[2] = v1[2]-v2[2];
}
}
void veccross3(float v1[],float v2[],float outvec[])
{
  outvec[0] = v1[1]*v2[2] - v2[1]*v1[2];
  outvec[1] = -(v1[0]*v2[2] - v2[0]*v1[2]);
  outvec[2] = v1[0]*v2[1] - v2[0]*v1[1];
}
float vecnorm3(float v1[])
{
  float out;
  out = sqrt(v1[0]*v1[0]+v1[1]*v1[1]+v1[2]*v1[2]);
  return out;
}
void linplanint(float plan[][3],float lin[][3],float intpt[])
{

  float x0,y0,z0,a,b,c,x0p,y0p,z0p,ap,bp,cp;
  float v1int[3],v2int[3],v3int[3];
  float v1[3],v2[3],v3[3];
  float rvec[3];
  float xint,yint,zint;
  float sysmat[3][3];
  float v1norm,v2norm,v3norm;

  x0 = plan[0][0];
  y0 = plan[0][1];
  z0 = plan[0][2];
  a = plan[1][0];
  b = plan[1][1];
  c = plan[1][2];

  x0p = lin[0][0];
  y0p = lin[0][1];
  z0p = lin[0][2];
  ap = lin[1][0];
  bp = lin[1][1];
  cp = lin[1][2];

  v1[0] = ap;
  v1[1] = bp;
  v1[2] = cp;
/*printf("v1 = %f %f %f\n",v1[0],v1[1],v1[2]);*/
  rvec[0] = rand1();
  rvec[1] = rand1();
  rvec[2] = rand1();

/*  rvec[0] = .65;
  rvec[1] = -.23;
  rvec[2] = .9;*/

  veccross3(v1,rvec,v2);
/*printf("v2 = %f %f %f\n",v2[0],v2[1],v2[2]);*/

  rvec[0] = rand1();
  rvec[1] = rand1();
  rvec[2] = rand1();

/*  rvec[0] = -.22;
  rvec[1] = -.2;
  rvec[2] = .54;*/
/*printf("v1 = %f %f %f\n",v1[0],v1[1],v1[2]);
printf("rvec = %f %f %f\n",rvec[0],rvec[1],rvec[2]);*/

  veccross3(v1,rvec,v3);
/*printf("v3 = %f %f %f\n",v3[0],v3[1],v3[2]);*/
  v1norm = vecnorm3(v1);
  v1[0] = v1[0]/v1norm;
  v1[1] = v1[1]/v1norm;
  v1[2] = v1[2]/v1norm;

  v2norm = vecnorm3(v2);
  v2[0] = v2[0]/v2norm;
  v2[1] = v2[1]/v2norm;
  v2[2] = v2[2]/v2norm;

  v3norm = vecnorm3(v3);
  v3[0] = v3[0]/v3norm;
  v3[1] = v3[1]/v3norm;
  v3[2] = v3[2]/v3norm;

/*printf("\n\nv1 = %f %f %f\n",v1[0],v1[1],v1[2]);
printf("v2 = %f %f %f\n",v2[0],v2[1],v2[2]);
printf("v3 = %f %f %f\n",v3[0],v3[1],v3[2]);*/


  sysmat[0][0] = a;
  sysmat[0][1] = b;
  sysmat[0][2] = c;

  sysmat[1][0] = v2[0];
  sysmat[1][1] = v2[1];
  sysmat[1][2] = v2[2];

  sysmat[2][0] = v3[0];
  sysmat[2][1] = v3[1];
  sysmat[2][2] = v3[2];


 
  if(matdet3(sysmat) != 0){
     xint = (a *(v2[2]* v3[1] - v2[1]* v3[2])* x0 + 
            b *(-(v2[2]* v3[0]* x0p) + v2[0]* v3[2]* x0p +    
            v2[2]* v3[1]* y0 - v2[1]* v3[2]* y0 -    
            v2[2]* v3[1]* y0p + v2[1]* v3[2]* y0p) +   
            c* (v2[1]* v3[0]* x0p - v2[0]* v3[1]* x0p +    
            v2[2]* v3[1]* z0 - v2[1]* v3[2]* z0 -    
            v2[2]* v3[1]* z0p + v2[1]* v3[2]* z0p)) /   
            (c* v2[1]* v3[0] - b* v2[2]* v3[0] - c* v2[0]*   
            v3[1] + a* v2[2]* v3[1] + b* v2[0]* v3[2] -   
            a* v2[1]* v3[2]);

     yint = (b *(v2[2]* v3[0] - v2[0] *v3[2])* y0 +
         a *(v3[2]* (-(v2[0]* x0) + v2[0]* x0p + v2[1] *y0p) +
            v2[2]* (v3[0]* x0 - v3[0]* x0p - v3[1]* y0p)) +
         c* (-(v2[1]* v3[0] *y0p) + v2[0]* v3[1]* y0p + v2[2]* 
	     v3[0]* z0 - v2[0]* v3[2]* z0 -
            v2[2]* v3[0]* z0p + v2[0]* v3[2] *z0p)) /
       (-(c* v2[1]* v3[0]) + b *v2[2]* v3[0] + c* v2[0]* 
	v3[1] - a *v2[2]* v3[1] - b* v2[0]* v3[2] +
         a* v2[1]* v3[2]);

     zint =(c* (-(v2[1]* v3[0]) + v2[0]* v3[1])* z0 +
         b *(-(v2[1]* v3[0]* y0) + v2[0]* v3[1] *y0 + 
	     v2[1]* v3[0]* y0p - v2[0]* v3[1]* y0p +
            v2[2]* v3[0]* z0p - v2[0]* v3[2]* z0p) +
         a *(v3[1]* (v2[0]* x0 - v2[0] *x0p - v2[2]* z0p) +
            v2[1] *(-(v3[0]* x0) + v3[0]* x0p + v3[2]* z0p))) /
       (-(c* v2[1]* v3[0]) + b* v2[2]* v3[0] + c* v2[0]* v3[1] 
	 - a* v2[2]* v3[1] - b* v2[0]* v3[2] +
         a *v2[1]* v3[2]);

     intpt[0] = xint;
     intpt[1] = yint;
     intpt[2] = zint;
  }
  else {
    intpt[0] = NOINT;
    intpt[1] = NOINT;
    intpt[2] = NOINT;
  }
}

float rand1(void)
{
  int temp;
        next = next * 1103515245 + 12345;
        temp = (unsigned int) (next/65536) % 32768;
  return temp/MAXRND;
}

void srand1(unsigned int seed)
{
        next = seed;
}
float matdet3(float mat[3][3])
{
  float det = 0;
   det += mat[0][0]*(mat[1][1]*mat[2][2] - mat[2][1]*mat[1][2]); 
   det -= mat[0][1]*(mat[1][0]*mat[2][2] - mat[2][0]*mat[1][2]); 
   det += mat[0][2]*(mat[1][0]*mat[2][1] - mat[2][0]*mat[1][1]); 
  return det;
}
int round(float in)
{
  float rem,fout;
  int out;

  if (in<0){
    in = -in;
    rem = in - ((int) in/1);
    if(rem<=0.5)
      fout = in;
    else
      fout = in + 0.5;
    out = fout;
    return(-out);
  }
  else {
    rem = in - ((int) in/1);
    if(rem<=0.5)
      fout = in;
    else
      fout = in + 0.5;
    out = fout;
    return(out);

  }
}
float absval(float in)
{
  if (in<0)
    return -in;
  else 
    return in;
}
void sortgbs(unsigned long n, float arr[][3])
{
  int i,j;
  float a1,a2,a3;

  for (j=1;j<n;j++){
    a1 = arr[j][0];
     a2 = arr[j][1]; 
     a3 = arr[j][2]; 
    i = j-1;
    while (i>=0 && arr[i][0] > a1){
      arr[i+1][0] = arr[i][0];
       arr[i+1][1] = arr[i][1]; 
       arr[i+1][2] = arr[i][2]; 
      i--;
    }
    arr[i+1][0] = a1;
     arr[i+1][1] = a2; 
     arr[i+1][2] = a3; 
  }

  for (j=1;j<n;j++){
    a1 = arr[j][0];
     a2 = arr[j][1]; 
     a3 = arr[j][2]; 
    i = j-1;
    while (i>=0 && arr[i][1] > a2){
      arr[i+1][0] = arr[i][0];
       arr[i+1][1] = arr[i][1]; 
       arr[i+1][2] = arr[i][2]; 
      i--;
    }
    arr[i+1][0] = a1;
     arr[i+1][1] = a2; 
     arr[i+1][2] = a3; 
  }
 
  for (j=1;j<n;j++){
    a1 = arr[j][0];
     a2 = arr[j][1]; 
     a3 = arr[j][2]; 
    i = j-1;
    while (i>=0 && arr[i][2] > a3){
      arr[i+1][0] = arr[i][0];
       arr[i+1][1] = arr[i][1]; 
       arr[i+1][2] = arr[i][2]; 
      i--;
    }
    arr[i+1][0] = a1;
     arr[i+1][1] = a2; 
     arr[i+1][2] = a3; 
  }


}










