import VTK
from DigitalMaterial.PX.Attributes.DM_VTK import VTKFieldArray

class VTKPolycrystalSource:

    def __init__(self):
        self.GrainBoundaries = None
        self.polycrystal = None

class VTKPolycrystalReader(VTKPolycrystalSource):

    def __init__(self):
        VTKPolycrystalSource.__init__(self)
        self.filename = None

    def SetPolycrystal(self, polycrystal):
        self.polycrystal = polycrystal
        
    def SetFileName(self, filename):
        self.filename = filename

    def GetOutput(self):
        if self.GrainBoundaries is None:
            self.GrainBoundaries = self.GetGrainBoundaries()
        vtkpoly = VTKPolycrystal()
        vtkpoly.SetGrainBoundaries(self.GrainBoundaries)
        vtkpoly.SetPolycrystal(self.polycrystal)
        return vtkpoly

    def GetOutput(self):

        reader = VTK.vtkPolyDataReader()
        reader.SetFileName(self.filename)
        gbpolydata = reader.GetOutput()
        reader.Update()
        vtkpoly = VTKPolycrystal()
        vtkpoly.SetGrainBoundaries(gbpolydata)
        return vtkpoly


class VTKPolycrystalBuilder(VTKPolycrystalSource):

    def __init__(self):
        VTKPolycrystalSource.__init__(self)
        self.polycrystal = None
        self.vtkpolycrystal = None

    def SetPolycrystal(self, polycrystal):
        self.polycrystal = polycrystal

    def GetOutput(self):
        vtkpoly = self._buildVTKPoly()
        vtkpoly.SetPolycrystal(self.polycrystal)
        vtkpoly._build_VTKGBMap()
        return vtkpoly

    def _getGrainBoundaries(self):
        if self.polycrystal is None:
            return None
        if self.vtkpolycrystal is None:
            gbpolydata = self._buildGrainBoundaries()
            return gbpolydata

    def _buildVTKPoly(self):

        import tempfile
        self.vtkfilename = tempfile.mktemp('.poly.vtk')
        import PolycrystalIO
        writer = PolycrystalIO.VTKPolycrystalWriter(self.vtkfilename)

        writer.write(self.polycrystal)
        #self._build_VTKGBMap()
            
        reader = VTKPolycrystalReader()
        reader.SetFileName(self.vtkfilename)
        vtkpoly = reader.GetOutput()
        import os
        os.remove(self.vtkfilename)
        return vtkpoly


class VTKPolycrystal:

    def __init__(self):
        pass
            
    def __repr__(self):
        s = 'VTKPolycrystal:\n{  %s  }' % self.polycrystal
        return s

    def SetPolycrystal(self, polycrystal):
        self.polycrystal = polycrystal

    def SetGrainBoundaries(self, polydata):
        self.GrainBoundaries = polydata
        self.GrainBoundaries.Update()
        self.grainIndices = \
          self.GrainBoundaries.GetCellData().GetFieldData().GetArray('grainIndices')

            
    def ExtractFieldArray(self, arrayName):
        a = self.GrainBoundaries.GetCellData().GetFieldData().GetArray(arrayName)
        self.__dict__[arrayName] = VTKFieldArray(arrayName, a)

    def _build_VTKGBMap(self):

        self._VTKGBMap = {}
        ncell = 0
        for inter in self.polycrystal.Interfaces():
            self._VTKGBMap[inter.ID] = ncell
            ncell = ncell+1

    def __getattr__(self, attr):
        return getattr(self.polycrystal, attr)

# -------------------------------------------------------------------------

def PolycrystalToVTKPolycrystal(polycrystal):

    pass


def VTKGrainBoundariesToPolycrystal(gbpolydata):

    from DigitalMaterial.PX.Geometry.Common.Vectors import Vector

    npoints = gbpolydata.GetNumberOfPoints()
    vertexSet = {}
    for np in range(npoints):
        vertexSet[np] = Vector(gbpolydata.GetPoint(np))

    

# -------------------------------------------------------------------------


def test(ngrains):
    import PolycrystalSource
    poly = PolycrystalSource.BuildUniformGrainsWithUniformTexture(ngrains)
    builder = VTKPolycrystalBuilder()
    builder.SetPolycrystal(poly)
    vtkpoly = builder.GetOutput()
    return vtkpoly

if __name__ == '__main__':
    
    vtkpoly = test(10)
    from DigitalMaterial.PX.Tools.Viz import ViewPolyData, VTKDisplay
    #viewer = ViewPolyData.PolycrystalViewer()
    display = VTKDisplay.VTKDisplay()

