from DigitalMaterial.PX.Geometry.Common import GeoFmt, GeoFmtIO

def toGeoFmt(tesselation, filename):
    """Convert Tesselation object to CPTC GeoFmt file"""
    intrinsic_dimension = tesselation.dimension()

    if intrinsic_dimension != 3:
        raise ValueError, "GeoFmt only support for dimension=3"

    embedded_dimension = tesselation.dimension()
    num_properties = 0
    props=[]
    values=[]
    
    # build edgeList and boundingEdges
    edgeList = []
    boundingEdges = {}
    nf = 0
    for f in tesselation._interfaces.values():
        boundingEdges[nf] = []
        for i in xrange(len(f.vertices)):            
            if (i+1) == len(f.vertices): j = 0
            else: j = i+1
            if f.vertices[i] < f.vertices[j]:
                edg = (f.vertices[i], f.vertices[j])
            else:
                edg = (f.vertices[j], f.vertices[i])
            if edgeList.count(edg) == 0:
                edgeList.append(edg)      
            boundingEdges[nf].append(edgeList.index(edg))
        nf = nf + 1    

    num_control_pts = tesselation.NumberOfVertices()
    num_top_vertices = tesselation.NumberOfVertices()
    num_top_edges = len(edgeList)
    num_top_surfaces = tesselation.NumberOfInterfaces()
    num_top_regions = tesselation.NumberOfRegions()
    
    # vertices and control points
    top_vertices = []
    control_points = []
    nv = 0
    for v in tesselation._vertices.keys():
        top_vertices.append(GeoFmt.Cptc_TVertex(`v`, \
                                                nv, \
                                                0, [], []))
        control_points.append(GeoFmt.Cptc_GPoint(tesselation._vertices[v]))
        nv = nv + 1

    # edges                
    top_edges = []
    eID = 0
    for e in edgeList:
        geom_cur = []
        vlist = [tesselation._vertices.keys().index(e[0]), \
                 tesselation._vertices.keys().index(e[1])]
        geom_cur.append(GeoFmt.Cptc_GCurve(1, vlist)) 
        edg = GeoFmt.Cptc_TEdge('E'+`eID`, 1, geom_cur, 2, vlist,
                                 0, [], [])
        top_edges.append(edg)
        eID = eID + 1

    # surfaces
    top_surfaces=[]
        
    nf = 0
    for f in tesselation._interfaces.values():
        geom_surf = []
        # assume convex polygon with linear bezier triangle patch
        for i in range(len(f.vertices)-2):
            patch = []
            patch.append(tesselation._vertices.keys().index(f.vertices[0]))
            patch.append(tesselation._vertices.keys().index(f.vertices[i+1])) 
            patch.append(tesselation._vertices.keys().index(f.vertices[i+2]))
            geom_surf.append(GeoFmt.Cptc_GPatch(0, 1, 0, patch))
        
        surface = \
                GeoFmt.Cptc_TSurface(`f.ID`, len(geom_surf), geom_surf, 
                                     len(boundingEdges[nf]),
                                     boundingEdges[nf], 0, [],
                                     0, [], [])

        top_surfaces.append(surface)
        nf = nf + 1

    # regions
    top_regions=[]
    for r in tesselation._regions.values():
        nsurf = r.NumberOfInterfaces()
        boundingFaces=[]
        surfaceOrientations=[]
        for fID in r.interfaces:
            boundingFaces.append(tesselation._interfaces.keys().index(fID)) 
            orientation = (fID[0]==r.ID)
            surfaceOrientations.append(orientation)
        region = \
               GeoFmt.Cptc_TRegion(`r.ID`, 
                                   nsurf,
                                   boundingFaces, 
                                   0, [], 0, [],
                                   surfaceOrientations,
                                   0, [], [])
        top_regions.append(region)

    tesselation.geo = GeoFmt.Cptc_GeomDesc(intrinsic_dimension, 
                                    embedded_dimension,
                                    num_properties, props, values,
                                    num_control_pts,
                                    num_top_vertices,
                                    num_top_edges,
                                    num_top_surfaces,
                                    num_top_regions,
                                    control_points, top_vertices,
                                    top_edges, top_surfaces, top_regions)

    f = open(filename, 'w')
    p = GeoFmtIO.GeoPacker()
    header = "abcdgeo   voronoi tesselation"
    numchars = len(header)
    header = header + (252-numchars) * '-'
    p.pack_string(header)
    p.pack_Cptc_GeomDesc(tesselation.geo)
    f.write(p.get_buffer())
    p.reset()
    f.close()

if __name__ == '__main__':
    from DigitalMaterial.PX.Structures.Polycrystals.PolycrystalSource import test
    poly = test(500)
    toGeoFmt(poly.geometry,'test500grains.geo')
