"""Cut.py provides functionality to cut a Polycrystal along a specified
plane"""

from DigitalMaterial.PX.Structures.Polycrystals import Polycrystal, VTKPolycrystal
from VTKPolycrystalCutter import *

class PolycrystalCutter:

    def __init__(self, polycrystal=None, vtkpolycrystal=None):
        if polycrystal is not None:
            self.SetPolycrystal(polycrystal)
        if vtkpolycrystal is not None:
            self.SetVTKPolycrystal(vtkpolycrystal)

    def SetPolycrystal(self, polycrystal):
        self.polycrystal = polycrystal
        self._buildVTKPolycrystal()

    def SetVTKPolycrystal(self, vtkpolycrystal):
        self.vtkpolycrystal = vtkpolycrystal
        self.polycrystal = vtkpolycrystal.polycrystal

    def Cut(self, planeNormal, planeOffset=0.):
        self.planeNormal = planeNormal
        self.planeOffset = planeOffset
        self._cutVTKPolycrystal(planeNormal, planeOffset)
        self._vtksurfaceGrains = \
           self._vtkcutter._buildPolygonsFromSurfaceCut(self._vtksurfaceCut)

    def GetSurfaceGrains(self):
        return self._vtksurfaceGrains

    def _buildVTKPolycrystal(self):
        builder = VTKPolycrystal.VTKPolycrystalBuilder()
        builder.SetPolycrystal(self.polycrystal)
	self.vtkpolycrystal = builder.GetOutput()

    def _cutVTKPolycrystal(self, planeNormal, planeOffset):

        if self.vtkpolycrystal is None:
            self._buildVTKPolycrystal()

        self._vtkcutter = VTKPolycrystalCutter(self.vtkpolycrystal)
        self._vtksurfaceCut = self._vtkcutter.CutAlongPlane(planeNormal,
                                                            planeOffset)

    def _extractNewTesselationVertices(self, surfaceCut):
        from DigitalMaterial.PX.Geometry.Common.Vectors import Vector
        vertices = []
        numpoints = surfaceCut.GetPoints().GetNumberOfPoints()
        for n in range(numpoints):
            vertices.append(Vector(list(surfaceCut.GetPoints().GetPoint(n))))
        return vertices
        
    def _augmentPolycrystalData(self):

        newVertices = self._extractNewTesselationVertices(self,
                                                          self._vtksurfaceCut)
        


def test():
    sample = PolycrystalSample.GetSample(generator='Voronoi',
                                         generatorOptions='D3 50')
    knife = PolycrystalCutter(sample)
    return sample, knife

if __name__ == '__main__':

    from DigitalMaterial.PX.Tools.Viz.PolycrystalView import *

    actors = []
    sample, knife = test()
    knife.Cut((0.,1.,0.), 0.)

    viewer, graphics = DisplayCut(sample, knife)
