from DigitalMaterial.PX.Attributes import Orientations

class MisorientationDistribution:

    def __init__(self):
        self.misorientations = None
        self.distribution = None
        self.polycrystal = None

    def SetPolycrystal(self, polycrystal):
        self.polycrystal = polycrystal

    def ComputeMisorientationsFromPolycrystal(self, polycrystal):
        if polycrystal is None:
            self.misorientation = None
        self.misorientations = \
            GetAllMisorientationAnglesOnInternalBoundaries(self.polycrystal)

    def ComputeDistribution(self, nbins=0):
        if self.misorientations is None:
            self.ComputeMisorientationsFromPolycrystal(self.polycrystal)
        self.distribution = \
            GetMisorientationAngleDistribution(self.misorientations, nbins)

    def GetDistribution(self):
        if self.distribution is None:
            self.ComputeDistribution()
        return self.distribution


# Various misorientation computations

def GetMisorientation(polycrystal, grainIndex1, grainIndex2):
    if grainIndex1 < polycrystal.GetNumberOfGrains():
        o1 = Orientations.Orientation(polycrystal.GetOrientation(grainIndex1))
    else:
        return None
    if grainIndex2 < polycrystal.GetNumberOfGrains():
        o2 = Orientations.Orientation(polycrystal.GetOrientation(grainIndex2))
    else:
        return None
    
    o3 = o2/o1
    return o3

def GetMisorientationOnInternalBoundary(polycrystal, grainIndex1, grainIndex2):
    o1 = Orientations.Orientation(polycrystal.GetOrientation(grainIndex1))
    o2 = Orientations.Orientation(polycrystal.GetOrientation(grainIndex2))
    o3 = o2/o1
    return o3

# ---

def GetMisorientationAxisAngle(polycrystal, grainIndex1, grainIndex2):
    mis = GetMisorientation(polycrystal, grainIndex1, grainIndex2)
    axis, angle = Orientations.MinimumAxisAngle(mis)
    return axis, angle

def GetMisorientationAxisAngleOnInternalBoundary(polycrystal,
                                                 grainIndex1, grainIndex2):
    mis = GetMisorientationOnInternalBoundary(polycrystal,
                                              grainIndex1, grainIndex2)
    axis, angle = Orientations.MinimumAxisAngle(mis)
    return axis, angle

# ---

def GetMisorientationAngle(polycrystal, grainIndex1, grainIndex2):
    mis = GetMisorientation(polycrystal, grainIndex1, grainIndex2)
    angle = Orientations.MinimumAngle(mis)
    return angle

def GetMisorientationAngleOnInternalBoundary(polycrystal,
                                            grainIndex1, grainIndex2):
    mis = GetMisorientationOnInternalBoundary(polycrystal,
                                              grainIndex1, grainIndex2)
    angle = Orientations.MinimumAngle(mis)
    return angle

# ---

def GetAllMisorientationAxisAngles(polycrystal):
    numGrains = polycrystal.GetNumberOfGrains()
    axesangles = {}
    for interface in polycrystal.Interfaces():
        ID0, ID1 = interface.ID
        if ID0 < numGrains and ID1 < numGrains:
            miso = GetMisorientation(polycrystal, ID0, ID1)
            axis, angle = Orientations.MinimumAxisAngle(miso)
            axesangles[(ID0, ID1)] = (axis, angle)
    return axesangles

def GetAllMisorientationAxisAnglesOnInternalBoundaries(polycrystal):
    numGrains = polycrystal.GetNumberOfGrains()
    axesangles = {}
    for interface in polycrystal.Interfaces():
        ID0, ID1 = interface.ID
        if ID0 < numGrains and ID1 < numGrains:
            miso = GetMisorientationOnInternalBoundary(polycrystal, ID0, ID1)
            axis, angle = Orientations.MinimumAxisAngle(miso)
            axesangles[(ID0, ID1)] = (axis, angle)
    return axesangles
    
# ---

def GetAllMisorientationAngles(polycrystal):
    numGrains = polycrystal.GetNumberOfGrains()
    angles = {}
    for interface in polycrystal.Interfaces():
        ID0, ID1 = interface.ID
        if ID0 < numGrains and ID1 < numGrains:
            miso = GetMisorientation(polycrystal, ID0, ID1)
            angle = Orientations.MinimumAngle(miso)
            angles[(ID0, ID1)] = angle
    return angles

def GetAllMisorientationAnglesOnInternalBoundaries(polycrystal):
    numGrains = polycrystal.GetNumberOfGrains()
    angles = {}
    for interface in polycrystal.Interfaces():
        ID0, ID1 = interface.ID
        if ID0 < numGrains and ID1 < numGrains:
            miso = GetMisorientationOnInternalBoundary(polycrystal, ID0, ID1)
            angle = Orientations.MinimumAngle(miso)
            angles[(ID0, ID1)] = angle
    return angles

#

def GetMisorientationAngleDistribution(angles, nbins=0):
    from Scientific.Statistics.Histogram import *
    if nbins == 0:
        num = len(angles)
        if num < 1000:
            nbins = 10
        elif num < 10000:
            nbins = num/100
        else:
            nbins = 100
    angle_dist = Histogram(angles.values(), nbins)
    angle_dist.normalize()
    return angle_dist







