#
#
#
from Numeric import array
from DigitalMaterial.PX.Geometry.Common.GeomShape3D import Line
from Scientific.Geometry.VectorModule import Vector

class RaysGenerator:
    def __init__(self, nRays = 0):
        self.nRays = nRays
        self.rays = [] # rays are a list of Line objects
        
    def __len__(self):
        return len(self.rays)

    def __getitem__(self, index):
        return self.rays[index]

    def randomInCube(self, minimum = array([0.,0,0]), \
                    maximum = array([0.,0,0])):
        from RandomArray import uniform
        for i in range(self.nRays):
            pt0 = uniform(minimum, maximum, 3)
            pt1 = uniform(minimum, maximum, 3)
            r = Line(Vector(pt0), Vector(pt1))
            self.rays.append(r)
    
    def randomOnPlane(self, ex = Vector(0.,0,0), ey = Vector(0.,0,0), \
                     minimum = array([0.,0]), maximum = array([0.,0])):
        if (ex == ey): 
            raise ValueError, "Two base vectors cannot be the same"
        from RandomArray import uniform
        for i in range(self.nRays):
            pts = []
            for j in range(2):
                a = uniform(minimum, maximum, 2)
                pts.append(a[0]*ex + a[1]*ey)
            r = Line(pts[0], pts[1])
            self.rays.append(r)
    
    def regularSpacing(self, ex = Vector(0.,0,0), ey = Vector(0.,0,0), \
                      origon = Vector(0.0, 0, 0), yspacing = 0.0):
        if (ex == ey): 
            raise ValueError, "Two base vectors cannot be the same"
        for i in range(self.nRays):
            pt0 = origon + i * yspacing * ey
            pt1 = pt0 + 1.0 * ex
            r = Line(pt0, pt1)
            self.rays.append(r)       
    
    def normalToGrid(self, ex = Vector(0.,0,0), ey = Vector(0.,0,0), \
                     origon = Vector(0.0, 0, 0), xspacing = 0.0, \
                     nx = 0, yspacing = 0.0, ny = 0):
        if (ex == ey): 
            raise ValueError, "Two base vectors cannot be the same"
        ez = ex.cross(ey)
        for i in range(nx+1):
            for j in range(ny+1):
                pt0 = origon + i * xspacing * ex + j * yspacing * ey
                pt1 = pt0 + 1.0 * ez
                r = Line(pt0, pt1)
                self.rays.append(r)
    
