#
# regression test using a set of rays shooting through polyhedra
#
def Gnuplot2D(title='', *datasets):
    import Gnuplot
    plotter = Gnuplot.Gnuplot()
    plotter.title(title)
    plotter('set data style linespoints')
    plotter.plot(datasets)
    return plotter

def printGrainSizes(grainSizeSet):
    i = 0
    for sizes in grainSizeSet:
        print "Ray ", str(i)
        print "Grain sizes are:"
        print sizes  
        i = i + 1

if __name__ == '__main__':
    from DigitalMaterial.PX.Structures.Polycrystals import PolycrystalSource
    from Numeric import array
    from Scientific.Geometry.VectorModule import Vector
    from PolyhedronRayShoot import PolyhedronRayShoot, GrainSizeDistribution
    from RaysGenerator import RaysGenerator
    ngrains = 100
    poly = PolycrystalSource.test(ngrains)
    nrays = 10
    #
    # random rays in a cube
    #
    print "Generate random rays in a cube"
    rays = RaysGenerator(nrays) 
    rays.randomInCube(minimum = array([-0.5,-0.5,-0.5]), \
                      maximum = array([0.5,0.5,0.5]))
    grainSizeSet = PolyhedronRayShoot(poly, rays)
    printGrainSizes(grainSizeSet)
    p = Gnuplot2D('Grain size distribution: random rays in a cube', \
                  GrainSizeDistribution(grainSizeSet))
    raw_input('Please press Return to continue')
    #
    # random rays on a plane
    #
    print "Generate random rays on a plane"
    rays = RaysGenerator(nrays) 
    rays.randomOnPlane(ex = Vector(1., 0, 0),  \
                       ey = Vector(0., 1, 0),  \
                       minimum = array([-0.5,-0.5]), \
                       maximum = array([0.5,0.5]))       
    grainSizeSet = PolyhedronRayShoot(poly, rays)
    printGrainSizes(grainSizeSet)
    p = Gnuplot2D('Grain size distribution: random rays on a plane', \
                  GrainSizeDistribution(grainSizeSet))
    raw_input('Please press Return to continue')
    #
    # rays on a regular spacing
    #
    print "Generate rays on a regular spacing"
    rays = RaysGenerator(nrays) 
    rays.regularSpacing(ex = Vector(1., 0, 0),  \
                        ey = Vector(0., 1, 0),  \
                        origon = Vector(0.0, 0, 0), \
                        yspacing = 0.5/nrays )       
    grainSizeSet = PolyhedronRayShoot(poly, rays)
    printGrainSizes(grainSizeSet)
    p = Gnuplot2D('Grain size distribution: rays on a regular spacing', \
                  GrainSizeDistribution(grainSizeSet))
    raw_input('Please press Return to continue')
    #
    # rays perpendicular to a 2D regular grid
    #
    print "Generate rays perpendicular to a planar regular grid"
    rays = RaysGenerator() 
    rays.normalToGrid(ex = Vector(1., 0, 0),  \
                      ey = Vector(0., 1, 0),  \
                      origon = Vector(-0.5, -0.5, 0), \
                      xspacing = 1.0/3, nx = 3, \
                      yspacing = 1.0/4, ny = 4 )       
    grainSizeSet = PolyhedronRayShoot(poly, rays)
    printGrainSizes(grainSizeSet)
    p = Gnuplot2D('Grain size distribution: rays perpendicular to a grid', \
                  GrainSizeDistribution(grainSizeSet))
    raw_input('Please press Return to continue')
