import types, string

def _lookupTypeFromString(self, str):
    if type(str) != types.StringType:
        raise TypeError, "input type is neither a type object nor a string"
    typename = str + 'Type'
    if typename not in dir(types): 
        raise ValueError, "input type is not defined in builtin types"
    return eval('types.'+typename)

class Signature:

    def __init__(self, pyType, *argTypes, **kwTypes):
        self._type = pyType
        self._argTypes = argTypes
        self._kwTypes  = kwTypes

    def GetType(self): return self._type
    def GetArgTypes(self): return self._argTypes
    def GetKeywordTypes(self): return self._kwTypes

    def __repr__(self):
        if self._type == 'Int' or self._type == 'Float' or \
           self._type == 'Complex' or self._type == 'String':
            return self._type
        else:
            return '(%s, %s, %s)' % (self._type, self._argTypes, self._kwTypes)

    def GetPythonType(self):
        return _lookupTypeFrom
    

class NameValueSignature:

    def __init__(self, name, value, signature):
        self._name  = name
        self._value = value
        if type(signature) is types.StringType:
            self._signature = Signature(signature)
        else:
            self._signature = signature

    def GetName(self):      return self._name
    def GetValue(self):     return self._value
    def GetSignature(self): return self._signature

    def SetValue(self, value):    self._value = value
    def SetSignature(self, type): self._signature = type

    def __repr__(self):
        return '(%s, %s, %s)' % (`self._name`, self._value, self._signature)
    

class ConfigurationTable:

    def __init__(self):
        self.records = {}

    def __getitem__(self, key):
        return self.records[key]
    
    def __setitem__(self, key, value):
        # transform (value, type) tuple to NameValueSignature instance if necessary
        if type(value) == types.TupleType: 
            value = apply(NameValueSignature, value)
        self.records[key] = value
    
    def __getattr__(self, attr):
        """['clear', 'copy', 'get', 'has_key', 'items', 'keys', 'update',
        'values']"""
        return getattr(self.records, attr)

    def __repr__(self):
        return `self.records`
    
        
class ConfigurationTableReader:

    def __init__(self):
        self._filename = None
        self._table = None

    def SetFileName(self, filename):
        self._filename = filename

    def GetFileName(self):
        return self._filename

    def _read(self):
        self._table = ConfigurationTable()
        import sys
        if self._filename is None:
            self._input = sys.stdin
        else:
            self._input = open(self._filename, 'r')

        lines = map(string.split, self._input.readlines())
        for line in lines:
            if len(line) < 3: break
            name = line[0]
            valuestr = line[1]
            typestr = line[2:]
            if typestr[0] == 'Int' or \
               typestr[0] == 'Float' or \
               typestr[0] == 'Complex':
                value = eval(valuestr)
            else:
                value = valuestr
            self._table[name] = NameValueSignature(name, value,
                                                   apply(Signature, typestr))

    def GetTable(self):
        if self._table is None: self._read()
        return self._table

    


import Tkinter
if 'tk' not in dir(): tk = Tkinter.Tk()

class LabeledEntry(Tkinter.Frame):

    def __init__(self, master, name='', type='', variable=None):
        Tkinter.Frame.__init__(self, master)
        Tkinter.Label(self, text=name).pack(side=Tkinter.LEFT,
                                             fill=Tkinter.X)
        self.tkstr = Tkinter.StringVar()
        self.tkstr.set(variable)
        self.entry = Tkinter.Entry(self, textvariable=self.tkstr)
        self.entry.pack(side=Tkinter.RIGHT, fill=Tkinter.X)        
        self.pack()
        
class TkConfigurationTable(Tkinter.Frame):

    def __init__(self, configurationTable=None):
        self._configurationTable = configurationTable
        Tkinter.Frame.__init__(self, tk)
        self.frames = {}

    def SetConfigurationTable(self, configurationTable):
        self._configurationTable = configurationTable
        
    def _buildFrameTable(self):
        if self._configurationTable is None:
            return 
        
        for name, value in self._configurationTable.items():
            self.frames[name] = \
                  LabeledEntry(self, name=name,
                               type=value.GetSignature().GetType(),
                               variable=value.GetValue())

        self.pack()


if __name__ == '__main__':

    r = ConfigurationTableReader()
    r.SetFileName('tmpf')
    t = r.GetTable()
    gui = TkConfigurationTable(t)
    gui._buildFrameTable()
