import VTK

class GrainActor:
	def __init__(self, polyIQ,colorFunc=None):
		self.actor = self.MakeGrains( polyIQ, colorFunc )

	def GetActor(self):
		return self.actor

	def MakeGrains(self,polyIQ,colorFunc=None):
		totalGrid = VTK.vtkAppendPolyData()
		nGrains = polyIQ.GetNumberOfGrains()
		for grainIndex in range(nGrains):
			tess = self.SurfacesToPolyData(polyIQ,grainIndex)
			math = VTK.vtkMath()
			if colorFunc:
				color = colorFunc(polyIQ[grainIndex])
			else:
				color = math.Random(0,1)
			scalars = VTK.vtkFloatScalars()
			i=0
			scalars.Allocate(tess.GetNumberOfPolys(),10)
			while (i<tess.GetNumberOfPolys()):
				scalars.InsertScalar(i,color/511.0)
				i=i+1
			tess.GetCellData().SetScalars(scalars)
			totalGrid.AddInput(tess)
		grainActor = VTK.vtkActor()
		mapper = VTK.vtkDataSetMapper()
		if colorFunc:
			try:
				mapper.SetLookupTable( colorFunc.GetLookupTable() )
				mapper.ScalarVisibilityOn()
			except AttributeError:
				pass
		mapper.SetInput(totalGrid.GetOutput())
		grainActor.SetMapper(mapper)
		grainActor.GetProperty().SetOpacity(1.0)
		return grainActor

	def SurfacesToPolyData( self, polyIQ, grainIndex ):
		"""Given a list of surfaces, this constructs a VTK.VTK PolyData object."""
		vtkRegion = VTK.vtkPolyData()
		# There is one set of coordinates for all the surfaces.
		points = VTK.vtkFloatPoints()
		# The CellArray holds the faces.
		polys = VTK.vtkCellArray()

		# The list in the file has many more points
		# than required.
		polyIDList = polyIQ.GetRegionPolygons(grainIndex)
		idMap = polyIQ.GetCoordinateMap()
		control_points = polyIQ.GetCoordinateList()
		usedCoords = {}
		for poly in polyIDList:
			for id in poly:
				usedCoords[`id`] = 0

		# In python, we need to pre-allocate the number of points.
		points.SetNumberOfPoints(len(usedCoords.keys()))

		i = 0
		for coordID in usedCoords.keys():
			usedCoords[coordID] = i
			coordIndex = idMap[coordID]
			pt = control_points[coordIndex]
			coord = pt.coord
			points.InsertPoint(i,coord[0],coord[1],coord[2])
			i = i + 1
		for poly in polyIDList:
			polys.InsertNextCell(len(poly))
			for pti in poly:
				polys.InsertCellPoint(usedCoords[`pti`])

		vtkRegion.SetPoints(points)
		vtkRegion.SetPolys(polys)
		# delete points and polys?
		return vtkRegion

