from libVTKCommonPython import *
from libVTKGraphicsPython import *


class PipelinedDataSet:

    def __init__(self, constructor):
	self.data   = constructor()

    def __getattr__(self, attr):
	if hasattr(self.data, attr): return getattr(self.data, attr)
	if hasattr(self.mapper, attr): return getattr(self.mapper, attr)
	if hasattr(self.actor, attr): return getattr(self.actor, attr)

    def connect(self):

	self.mapper = vtkPolyDataMapper()
	self.mapper.SetInput(self.data)
	self.actor  = vtkActor()
	self.actor.SetMapper(self.mapper)


class PipelinedDataSource:

    def __init__(self, constructor):
	self.data   = constructor()

    def __getattr__(self, attr):
	if hasattr(self.data, attr): return getattr(self.data, attr)
	if hasattr(self.mapper, attr): return getattr(self.mapper, attr)
	if hasattr(self.actor, attr): return getattr(self.actor, attr)

    def connect(self):

	self.mapper = vtkPolyDataMapper()
	self.mapper.SetInput(self.data.GetOutput())
	self.actor  = vtkActor()
	self.actor.SetMapper(self.mapper)


class PipelinedFilter:

    def __init__(self, constructor, data):
	self.filter = constructor()
	self.filter.SetInput(data)

    def __getattr__(self, attr):
	if hasattr(self.filter, attr): return getattr(self.filter, attr)
	if hasattr(self.mapper, attr): return getattr(self.mapper, attr)
	if hasattr(self.actor, attr): return getattr(self.actor, attr)

    def connect(self):

	self.mapper = vtkDataSetMapper()
	self.mapper.SetInput(self.filter.GetOutput())
	self.actor = vtkActor()
	self.actor.SetMapper(self.mapper)
	

class PipelinedRenderWindow:

    def __init__(self, withInteractor=0):
	
	self.withInteractor = withInteractor
	self.win  = vtkRenderWindow()
	self.ren  = vtkRenderer()
	self.win.AddRenderer(self.ren)
	self.curImageNum = 0
	if self.withInteractor:
	    self.iren = vtkRenderWindowInteractor()
	    self.iren.SetRenderWindow(self.win)
 	self.ren.SetBackground(0.5,0.5,0.5)
	self.win.SetSize(500,500)

    def __getattr__(self, attr):
	if hasattr(self.ren, attr): return getattr(self.ren, attr)
	if hasattr(self.win, attr): return getattr(self.win, attr)
	if hasattr(self.iren, attr): return getattr(self.iren, attr)

    def updateRenderer(self):

	self.win.RemoveRenderer(self.ren)
	self.ren = vtkRenderer()
	self.ren.SetBackground(0.3000,0.3000,0.5000)
	self.win.AddRenderer(self.ren)

