import VTK, Tkinter

def mapDataToActor(data, mapperType=VTK.vtkDataSetMapper, lookupTable=None):

    mapper = apply(mapperType, ())
    mapper.SetInput(data)
    if lookupTable is not None: mapper.SetLookupTable(lookupTable)
    actor = VTK.vtkActor()
    actor.GetProperty().SetLineWidth(3.0)
    actor.SetMapper(mapper)
    return actor

class VTKGraphics:
    """VTKGraphics"""

    def __init__(self, title='VTKGraphics', windowSize=(400,400)):
        self.title = title
        self.Initialize(windowSize)

    def Initialize(self, windowSize):
        self.master = Tkinter.Tk()
        self.master.title(self.title)
        self.ctrl = self.master
        self.root = Tkinter.Toplevel (self.ctrl)
        self.tkwidget = VTK.vtkTkRenderWidget (self.root, width=windowSize[0],
                                               height=windowSize[1])
        self.tkwidget.pack (expand='true',fill='both')
        self.ren = VTK.vtkRenderer ()
        self.ren.TwoSidedLightingOn()
        self.renwin = self.tkwidget.GetRenderWindow ()
        self.renwin.AddRenderer (self.ren)
        self.actors  = []
        self.renwin.Render()

    def AddActor(self, actor):
        self.actors.append(actor)

    def Clear(self):
        if self.ren is not None: del self.ren
        self.ren = VTK.vtkRenderer ()
        self.renwin.AddRenderer (self.ren)
        self.actors = []

    def SaveImage(self, filename=None):
        if filename is None:
            import tempfile
            filename = tempfile.mktemp('.ppm')
            print "Saving image to %s" % filename
        self.renwin.SetFileName(filename)
        self.renwin.SaveImageAsPPM()
        return

    def Show(self):
        for a in self.actors:
            self.ren.AddActor(a)
        self.ren.GetActiveCamera().Azimuth(30)
        self.ren.GetActiveCamera().Elevation(20)
        self.ren.GetActiveCamera().Dolly(1.0)
        self.ren.ResetCameraClippingRange()
        self.renwin.Render()

if __name__ == '__main__':
	pass

	
