#!/usr/bin/env python

from Tkinter import *
import Numeric
import string
import math

def containsNumberedVertex(element, vertexNumber):
    if element.vertices.count(vertexNumber) > 0: return 1
    else: return 0


class Palette:

    def __init__(self, min=0., max=1., nlevels=10):
	self.min = min
	self.max = max
	self.range = self.max-self.min
	self.nlevels = nlevels

    def lookup(self, val):
	index = int( math.floor( ((val-self.min)/self.range)*self.nlevels) )
	if index < 0: index = 0
	if index > self.nlevels-1: index = self.nlevels-1
	return self.data[index]

class GrayscalePalette(Palette):

    def __init__(self, min=0., max=1., nlevels=10):
	Palette.__init__(self, min, max, nlevels)
	self.data = {}
	for n in range(self.nlevels):
	    level = '#%02d%02d%02d' % (3*(n*(100/self.nlevels),))
	    self.data[n] = level

def map_orientation(o):
    return o[0]+o[1]+o[2]

def map_misorientation(o):
    return o[0]+o[1]+o[2]


class PolycrystalGraphics:

    def __init__(self, tesselation, cellField=None, interfaceField=None, 
		 size=(500,500), palette=GrayscalePalette(0.,1.,10)):

	self.geom = tesselation
	self.cellField = cellField
	self.interfaceField = interfaceField
	self.palette = palette
	if self.geom.dim != 2:
	    raise ValueError, "polycrystalcrystalDraw2D.PolycrystalGraphics only appropriate for 2D tesselations"

	self.tk = tk = Tk()
	self.canvas = c = Canvas(tk, width=size[0], height=size[1])
	c.pack()
	self.width, self.height = tk.getint(c['width']), \
				  tk.getint(c['height'])

	bb = self.geom.componentGeometry.boundingBox()
	self.xmin = 1.1*bb[0][0]
	self.ymin = 1.1*bb[1][0]
	self.xmax = 1.1*bb[0][1]
	self.ymax = 1.1*bb[1][1]
	self.xrange = self.xmax-self.xmin
	self.yrange = self.ymax-self.ymin

	self.mapped_pts = {}
	for p in self.geom.vpoints.items():
	    self.mapped_pts[p[0]] = [self.px(p[1][0]), self.py(p[1][1])]

	
    def px(self, x):
	return ( self.width * ( (x-self.xmin)/self.xrange ) )
    def py(self, y):
	return ( self.height * ( 1. - ((y-self.ymin)/self.yrange) ) )

    def drawCells(self, cellField=None, update=0):

	for index in self.geom.facetSet.keys():
	    plist = []
	    if not containsNumberedVertex(self.geom.facetSet[index], 0):
		for v in self.geom.facetSet[index].vertices:
		    plist.append(self.mapped_pts[v][0])
		    plist.append(self.mapped_pts[v][1])
		
		if cellField is None: 
		    fill_color = 'white'
		else:
		    fill_color = self.palette.lookup(map_orientation(cellField(index)))
		apply(self.canvas.create_polygon, plist, 
		      {'fill': fill_color})

	if update: self.tk.update()

    def drawInterfaces(self, interfaceField=None, update=0):

	for inter in self.geom.ridgeSet.keys():
	    if not containsNumberedVertex(self.geom.ridgeSet[inter], 0):
		v0,v1 = self.geom.ridgeSet[inter].vertices
		plist = [self.mapped_pts[v0][0], self.mapped_pts[v0][1],
			 self.mapped_pts[v1][0], self.mapped_pts[v1][1]]

		if interfaceField is None: 
		    line_color = 'black'
		else:
		    line_color = self.palette.lookup(
			map_misorientation(interfaceField(inter))
			)
		apply(self.canvas.create_line, plist, 
		      {'fill': line_color, 'width':2})


	if update: self.tk.update()

    def draw(self):
	
	self.drawCells(self.cellField, update=0)
	self.drawInterfaces(self.interfaceField, update=1)



