#from DigitalMaterial.PX.Geometry.Common import rbox, qhull
#import string

class PolycrystalGraphics:

    def __init__(self, tesselation, cellField=None, interfaceField=None, 
		 size=(500,500)):

	self.tesselation = tesselation

    def draw(self, interactive=0):

	from libVTKCommonPython import *
	from libVTKGraphicsPython import *
	
	pts = vtkFloatPoints()
	pts.SetNumberOfPoints(self.tesselation.nvpoints())
	np = 0
	vmap={}
	dim = self.tesselation.dim
	for p in self.tesselation.vpoints.items():
	    if p[0] != 0:
		if dim == 3: pts.InsertPoint(np, p[1][0], p[1][1], p[1][2])
		if dim == 2: pts.InsertPoint(np, p[1][0], p[1][1], 0.)
		vmap[p[0]] = np
		np = np+1
	    
	cells = vtkCellArray()

	for f in self.tesselation.regionSet.elements():
	    iS = f.inputSite
	    for r in self.tesselation.interfaceSet.elements():
                if r.inputSites[0] == iS:
#		if r.inputSites[0] == iS or r.inputSites[1] == iS:
		    if not r.containsVertex(0):
			nV = len(r.vertices)
			cells.InsertNextCell(nV)
		    for vert in r.vertices:
			cells.InsertCellPoint(vmap[vert])


	profile = vtkPolyData()
	profile.SetPoints(pts)
	profile.SetPolys(cells)

	map = vtkPolyDataMapper()
	map.SetInput(profile)

	act = vtkActor()
	act.SetMapper(map)
	act.GetProperty().SetColor(0.3800,0.7000,0.1600)

	ren = vtkRenderer()
	renWin = vtkRenderWindow()
	renWin.AddRenderer(ren)
	ren.AddActor(act)
	ren.SetBackground(0.1,0.1,0.1)
	
	if not interactive:
	    renWin.Render()
	else:
	    iren = vtkRenderWindowInteractor()
	    iren.SetRenderWindow(renWin)
	    iren.Initialize()
	    iren.Start()


    def plot3DFromGeo(self, file='voronoi.geo', interactive=0):

	from VTK import *
	from DigitalMaterial.PX.Geometry.Common import PyGeo
	import string

	header, geo = PyGeo.read_geo_file(file)

	print geo

	dim = geo.intrinsic_dimension

	pts = vtkFloatPoints()
	pts.SetNumberOfPoints(self.tesselation.nvpoints())
	np = 0
	vmap={}
	for p in geo.control_points:
	    if dim == 3:
                pts.InsertPoint(np, p.coord[0], p.coord[1], p.coord[2])
	    np = np+1

	np = 0
	for v in geo.top_vertices:
	    vmap[string.atoi(v.id)] = np
	    np = np+1
	    
	cells = vtkCellArray()

	for s in geo.top_surfaces:
            poly = []
	    for ne in s.bound_edges:
		e = geo.top_edges[ne]
                poly.append(vmap[e.bound_vertices[0]])
            cells.InsertNextCell(len(poly))
            for pti in poly:
                cells.InsertCellPoint(geo.top_vertices[pti].control_pnt_indx)


	profile = vtkPolyData()
	profile.SetPoints(pts)
	profile.SetPolys(cells)

	map = vtkPolyDataMapper()
	map.SetInput(profile)

	act = vtkActor()
	act.SetMapper(map)
	act.GetProperty().SetColor(0.9000,0.7000,0.100)

	ren = vtkRenderer()
	renWin = vtkRenderWindow()
	renWin.AddRenderer(ren)
	ren.AddActor(act)
	ren.SetBackground(0.1,0.1,0.1)
	
	if not interactive:
	    renWin.Render()
	else:
	    iren = vtkRenderWindowInteractor()
	    iren.SetRenderWindow(renWin)
	    iren.Initialize()
	    iren.Start()

