#ifndef CONSERVEDOP_H
#define CONSERVEDOP_H

#ifdef _DEBUG
#include <assert.h>
#endif

#include "DigitalMaterial.h"
#include "Field.h"
#include "RightHandSide.h"
#include "OperatorOperand.h"
#include "BinaryOperators.h"
#include "UnaryOperators.h"


//#include "Introspective.h"
//#include "Clonable.h"

class ConservedOPParameters {
	protected:
		Real diffusivity, potential;
	public:
		ConservedOPParameters(void);
		ConservedOPParameters(const ConservedOPParameters &copp);
		ConservedOPParameters(Real d, Real p);
		virtual ~ConservedOPParameters(void);
		virtual Real GetDiffusivity(void) const { return(diffusivity); }
		virtual bool SetDiffusivity(Real dd) { diffusivity=dd; return(true); }
		virtual Real GetPotential(void) const { return(potential); }
		virtual bool SetPotential(Real dd) { potential=dd; return(true); }
};

class ConservedOP : virtual public RightHandSide, virtual public ConservedOPParameters  //, public Introspective, public Cloneable
{
	private:
		FieldCollection WorkSpace;
	protected:
		Real diffusivity, potential;
		PhaseFieldPotential PhaseFieldTerm;
		MakeEqual Equal;
		Laplacian Laplace;
		AddField Add;
		MultiplyReal Times;
	public:
		ConservedOP(void);
		ConservedOP(const ConservedOP &cop);
		virtual ~ConservedOP(void);
		virtual bool RHS(const FieldCollection &input, FieldCollection &output);
		virtual bool SetWorkSpace(const FieldCollection & Fields);
		virtual FieldCollection & GetWorkSpace() { return(WorkSpace); }
		virtual const FieldCollection & GetWorkSpace() const { return(WorkSpace); }
};


#endif
