#ifndef EXPLICITEULER_H
#define EXPLICITEULER_H

#ifdef _DEBUG
#include <assert.h>
#endif

#include <typeinfo>
#include "DigitalMaterial.h"
#include "Field.h"
#include "RightHandSide.h"
#include "FieldMover.h"
#include "OperatorOperand.h"
#include "BinaryOperators.h"
#include "UnaryOperators.h"
#include "FieldCollection.h"


//#include "Introspective.h"
//#include "Clonable.h"


class ExplicitEuler : public FieldMover //, public Introspective, public Cloneable
{
	protected:
		Real step_size;

		// Array to pointers to work space fields.
		// The workspace here contains the time derivatives.
		FieldCollection WorkSpace;

		MakeEqual Equal;
		AddField Add;
		MultiplyReal Times;

		virtual void FreeWorkSpace();

	public:
		ExplicitEuler(void);
#ifndef SWIG
		ExplicitEuler(Real step, RightHandSide *RHS);
		ExplicitEuler(const ExplicitEuler & exeuler);
#endif //SWIG
		virtual ~ExplicitEuler();

		//SWIG doesn't cope if this definition is not repeated in the subclass
		virtual bool SetRightHandSide(RightHandSide *r) { rhs=r; return(true); }

		virtual bool SetStepSize(Real step) { step_size=step; return(true); }
		virtual Real GetStepSize(void) const { return(step_size); }
		virtual bool SetWorkSpace(const FieldCollection & field);
		virtual FieldCollection & GetWorkSpace(void) { return(WorkSpace); }
#ifndef SWIG
		virtual const FieldCollection & GetWorkSpace(void) const { return(WorkSpace); }
#endif //SWIG

		virtual bool SafeMove(FieldCollection & fields);
		virtual bool Move(FieldCollection & fields);

};


#endif
