#ifndef FIELD_H
#define FIELD_H

#include "DigitalMaterial.h"
#include "BoundaryCondition.h"
#include "FieldInitializer.h"
#include "Point.h"
#include "SubjectObserver.h"
#include "OperatorOperand.h"
#ifdef _DEBUG
#include <assert.h>
#endif

//#include "Introspective.h"
//#include "Clonable.h"

class Field : public Subject, public virtual Operand //public virtual Introspective, public Cloneable // not yet!
{
protected:
#ifndef SWIG
		int number_of_fields;
		BoundaryCondition * boundary_condition;
#endif //SWIG

public:
#ifndef SWIG
		// constructors and destructor
		Field();
		Field(const Field &field);
		Field(int number, const BoundaryCondition * bc);
		virtual ~Field(void);

		virtual Operator & Apply(Operator &op) { return(op.OperateOn(this)); }
		virtual Operator & Apply(Operator &op) const { return(op.OperateOn(this)); }
		virtual void LookAtMe(FieldObserver &obs) const { obs.Update(this); }
		// set- and get-functions:
		virtual bool SetNumberOfFields(int number) =0;
#endif //SWIG
		int GetNumberOfFields() const { return(number_of_fields); }
#ifndef SWIG
		virtual bool RemoveField(int number) =0;
		virtual bool InsertField(int number) =0;
	    //virtual bool InsertField(const Field & insertee, int number) =0;
		virtual const BoundaryCondition * GetBoundaryCondition() const { return(boundary_condition); }
#endif //SWIG
		virtual BoundaryCondition * GetBoundaryCondition() { return(boundary_condition); }
		virtual const BoundaryCondition & GetBoundaryConditionReference() const { return(*boundary_condition); }
		virtual bool SetBoundaryCondition(const BoundaryCondition * bc) { 
			if (boundary_condition!=ZERO) delete boundary_condition;
			boundary_condition=bc->AllocCopy(); 
			if (boundary_condition!=ZERO) return(true); 
			else return(false);
		}

		virtual const Real GetPhysicalSize(const int d){return((boundary_condition->GetPhysicalSize())[d]); }

#ifndef SWIG
		// Access field values
		// Need a method to iterate through all datapoints. 
		// Maybe an iterator?
		virtual void GotoFirstPoint() =0;
		virtual bool GotoNextPoint() =0;
		virtual bool IsLastPoint() =0;

		virtual Point GetPointCoordinates() =0;

		virtual bool SetFieldValue(const Real * phi) =0;
		virtual bool SetFieldValue(const Real phii, int i=0) =0;

		virtual Real * GetFieldValue(Real * phi) const =0;
		virtual Real   GetFieldValue(int i=0) const =0;

		virtual Real GetFieldValue(const Point &x, int i=0) const =0;
		virtual bool GetFieldValue(const Point &x, Real *y) const;

		virtual Field* AllocCopy() const =0;

		virtual bool Initialize(const FieldInitializer & initializer, const int i=0) =0;

		// mathematical operations:
		Field & operator= (const Field &right);
		Field & operator+=(const Field &right);
		Field & operator-=(const Field &right);
		Field & operator*=(const Field &right);
		Field & operator+=(Real addend);
		Field & operator*=(Real factor);
		Field & Laplace(void);
		Field & Laplace2(void);
		Field & DirectionalDerivative(const Point & dir);
		Field & Derivative(int dir);
		//Field & Apply(Function *f);
		//Field & Multiply(Function *f);
		Field & PhaseFieldTerm(Real strength);
		
#endif //SWIG
};

#endif //FIELD_H
