#ifndef GNUPLOTTER_H
#define GNUPLOTTER_H

#include <stdio.h>
#include "SubjectObserver.h"
#include "DigitalMaterial.h"

class GnuPlotter : virtual public FieldObserver
{
	protected:
		char title[256];
		char *buffername;
		FILE* gnuplot_pipe;
		FILE* buffer;
	public:
		virtual bool SetTitle(const char * text) { 
			#ifdef _DEBUG 
				if (strlen(text)>=255) return(false);
			#endif
			strcpy(title, text);
			return(true);
		}
#ifndef SWIG
		GnuPlotter(int x=360, int y=270);
		GnuPlotter(FILE * gplotpipe);
		virtual ~GnuPlotter(void);

		virtual bool InitializeGnuplot(void)=0;
		virtual bool FlushGnuPlotter(void) { 
			#ifdef _DEBUG
				if (gnuplot_pipe==NULL) return(false);
			#endif 
			fflush(gnuplot_pipe);
			return(true);
		}
		virtual bool SendPlotCommand(void)=0;
		virtual bool SendPlotCommand(const char* commandstring);
		//virtual void Update(const Field *theChangedField);
		virtual void Update(const SquareGridField *theChangedField)=0;
#endif //SWIG
};

class TwoDGnuplotter : public GnuPlotter
{
	protected:
		#ifndef SWIG
			#if DIMENSION==3
				Real level;
			#endif
		#endif
	public:
#ifndef SWIG
		TwoDGnuplotter();
		TwoDGnuplotter(int x, int y);
		TwoDGnuplotter(FILE *gplotpipe);
		virtual ~TwoDGnuplotter();
		virtual bool InitializeGnuplot()=0;
		virtual bool SendData(const SquareGridField *theChangedField);
		virtual bool SendPlotCommand();
		//virtual void Update(const Field *theChangedField);
		virtual void Update(const SquareGridField *theChangedField);
		#if DIMENSION==3
			virtual bool SetLevel(Real l) { level=l; return(true); }
			virtual Real GetLevel(void) { return(level); }
		#endif
#endif //SWIG
};

class GnuContourPlot : public TwoDGnuplotter
{
	protected:
		int contour_levels;
	public:
#ifndef SWIG
		GnuContourPlot();
		GnuContourPlot(FILE * gplotpipe);
#endif //SWIG
		GnuContourPlot(int x, int y);
		virtual ~GnuContourPlot();
		virtual bool SetContourLevels(int levels);
		virtual int GetContourLevels() const { return(contour_levels); }
#ifndef SWIG
		virtual bool InitializeGnuplot();
#endif //SWIG
};

class GnuSurfacePlot : public TwoDGnuplotter
{
	protected:
		double phi_x;
		double phi_z;
	public:
#ifndef SWIG
		GnuSurfacePlot(void);
		GnuSurfacePlot(FILE * gplotpipe);
#endif //SWIG
		GnuSurfacePlot(int x, int y);
		virtual ~GnuSurfacePlot();
#ifndef SWIG
		virtual bool InitializeGnuplot();
		virtual bool SetAngles(Real phix, Real phiz);
		virtual Real GetXAngle(void) { return(phi_x); }
		virtual Real GetZAngle(void) { return(phi_z); }
#endif //SWIG
};

class GnuVectorPlot : public TwoDGnuplotter
{
	protected:
		int spacing;
		int mode;
	public:
#ifndef SWIG
		GnuVectorPlot();
		GnuVectorPlot(FILE * gplotpipe);
#endif //SWIG
		GnuVectorPlot(int x, int y);
		virtual ~GnuVectorPlot();
#ifndef SWIG
		virtual bool SendPlotCommand(void);
		virtual bool SendPlotCommand(const SquareGridField *theChangedField);
		virtual bool SendData(const SquareGridField *theChangedField);
		//virtual void Update(const Field *theChangedField);
		virtual void Update(const SquareGridField *theChangedField);
		virtual bool InitializeGnuplot();
		bool SetSpacing(int sp) { 
			if (sp>0) {
				spacing=sp; 
				return(true); 
			}
			else return(false);
		}
		int GetSpacing(void) { return(spacing); }
		bool SetMode(int m) {
			if (m==0 || m==1) {
				mode=m;
				return(true);
			}
			else return(false);
		}
		int GetMode(void) { return(mode); }
#endif //SWIG
};

class GnuScatterPlot : public TwoDGnuplotter
{
	protected:
		Real pointsize;
	public:
#ifndef SWIG
		GnuScatterPlot();
		GnuScatterPlot(FILE * gplotpipe);
#endif //SWIG
		GnuScatterPlot(int x, int y);
		virtual ~GnuScatterPlot();
		virtual bool SendPlotCommand(void);
		virtual bool SendPlotCommand(const SquareGridField *theChangedField);
		virtual bool SendData(const SquareGridField *theChangedField);
#ifndef SWIG
		//virtual void Update(const Field *theChangedField);
		virtual void Update(const SquareGridField *theChangedField);
		virtual bool InitializeGnuplot();
		bool SetPointSize(Real s) { 
			if (s>0) {
				pointsize=s;
				return(true);
			}
			else return(false);
		}
		Real GetPointSize(void) { return(pointsize); }
#endif //SWIG
};
#endif
