#ifndef INDEX_H
#define INDEX_H

#ifdef _DEBUG
#include <assert.h>
#endif //_DEBUG

#include <stdlib.h>
#include <fstream.h>

#include "DigitalMaterial.h"

class Index // : public virtual Introspective, public Cloneable // not yet!
{
private:
	int coordinate[DIMENSION];

public:
	Index();
	void Zero() { Initialize(0); }
	bool IsNonNegative() const;
	bool IsPositive() const;
	
	void Initialize(const int x, const int y);
	//Only the 2D version

	char * GetPtrToArray() { return((char*)coordinate); }
	//Only used so that coordinate can be accessed from Python

#ifndef SWIG
	Index(const int x[DIMENSION]);
	Index(const Index & point);
	Index(const int x);
	Index(const int x, const int y);
	Index(const int x, const int y, const int z);

	void Initialize(const int x);
	void Initialize(const int x, const int y, const int z);
	
	bool IsInBox(const Index &box) const;
	bool NextInBox(const Index &box);
	bool PrevInBox(const Index &box);
	bool WhereInBox(const Index &box, size_t n); // Index.WhereInBox(box, Index(box)) == Index

	size_t Volume() const;
//	size_t FFTWRealVolume() const;
//	size_t FFTWComplexVolume() const;
	bool MakeFFTWBox(const Index & box);
	void Print(const char * text) const;
	int & operator[](const int i) {
#ifdef _DEBUG
		assert(i>=0 && i<DIMENSION);
#endif //_DEBUG
		return(coordinate[i]);
	}
	int operator()(const int i) const {
#ifdef _DEBUG
		assert(i>=0 && i<DIMENSION);
#endif //_DEBUG
	return(coordinate[i]);
	}
	size_t operator()(const Index & size) const;

	int * operator()(int * pp) const;

	bool operator==(const Index &right) const;
	bool operator!=(const Index &right) const { return(!(*this==right)); }
	Index & operator%=(const Index &box);
	Index & operator%=(const int b);
	Index & operator*=(const int x);
	Index & operator/=(const int x);
	Index & operator+=(const Index & addend);
	Index & operator-=(const Index & addend);
	Index & operator=(const Index & x);
	friend ostream & operator<<(ostream &os, const Index & point);
	friend istream & operator>>(istream &is, Index & point);
#endif //SWIG

};

#endif //INDEX_H
