#ifndef SQUAREGRIDFIELD_H
#define SQUAREGRIDFIELD_H

#include <rfftw.h>
#include <typeinfo>
#include "Field.h"
#include "Index.h"

//#include "Introspective.h"
//#include "Clonable.h"

class SquareGridField : public virtual Field //, public virtual Introspective, public Cloneable // not yet!
{
// Start out by hiding everything from Swig, then opening up only the parts we need!
protected:
		Real **data;
		size_t number_of_data_points;
		Index box;
		Index fftwbox;
		Index current_position;
		fftw_real **fftw_real_data;
		fftw_complex **fftw_complex_data;
		rfftwnd_plan plan_re_to_comp;
		rfftwnd_plan plan_comp_to_re;

public:
// constructors and destructor
		SquareGridField();
		virtual ~SquareGridField();
#ifndef SWIG
		SquareGridField(const SquareGridField & field);
#endif //SWIG

//	Just for now, until Introspection is implemented with these classes
		virtual bool DumpToFile(const char * filename) const;

		virtual Operator & Apply(Operator &op) { return(op.OperateOn(this)); }
#ifndef SWIG
		virtual Operator & Apply(Operator &op) const { return(op.OperateOn(this)); }
		virtual void LookAtMe(FieldObserver &obs) const { obs.Update(this); }
#endif //SWIG
// set- and get-functions:
		virtual Index GetFFTWBox() const { return(fftwbox); }
		virtual bool SetNumberOfFields(int number);
		virtual bool RemoveField(int number);
		virtual bool InsertField(int number);

		virtual bool Initialize(const FieldInitializer & initializer, const int i=0);
		
		virtual size_t GetNumberOfDataPoints() const { return(number_of_data_points); }

		virtual Index GetLogicalSize() const { return(box); }
		virtual bool SetLogicalSize(const Index & new_box);

		virtual Real GetFieldValue(const Index &x, int i=0) const;

		virtual char * GetPtrToData(const int i) {return((char*)data[i]); }
		// Only here to give python access to the data

#ifndef SWIG
		virtual fftw_real ** GetPtrToFFTWReal(void) { return(fftw_real_data); }
		virtual fftw_complex ** GetPtrToFFTComplex(void) { return(fftw_complex_data); }
		virtual rfftwnd_plan GetReToCompPlan(void) const { return(plan_re_to_comp); }
		virtual rfftwnd_plan GetCompToRePlan(void) const { return(plan_comp_to_re); }
		virtual bool SetLogicalSize(const int new_box[DIMENSION]);

// Access field values generically
		virtual void GotoFirstPoint() { current_position=0; }
		virtual void GotoLastPoint() { Index i(-1); current_position=box; current_position+=i; }
		virtual bool GotoNextPoint() { return(current_position.NextInBox(box)); }
		virtual bool GotoPrevPoint() { return(current_position.PrevInBox(box)); }
		virtual bool IsLastPoint() { return(!current_position.IsInBox(box)); }

		virtual Point GetPointCoordinates() { return(GetPhysicalCoordinates(current_position)); }
		virtual Index GetCurrentIndex() const { return(current_position); }
		virtual bool SetFieldValue(const Real * phi);
		virtual bool SetFieldValue(const Real phii, int i=0) { return(SetFieldValue(current_position, phii, i)); }

// GridField specific Set and Get functions
		virtual bool SetFieldValue(const Index &x, Real value, int i=0);
		virtual bool SetFieldValue(size_t x, Real value, int i=0);
		virtual Real GetFieldValue(int i=0) const { return(GetFieldValue(current_position, i)); }
		virtual Real GetFieldValue(size_t x, int i=0) const;
		virtual Real * GetFieldValue(Real * phi) const;
		virtual Real GetFieldValue(const Point &x, int i=0) const;

		virtual Real & GetField(const Index &x, int i);
		virtual Real & GetField(size_t x, int i=0);

		virtual int GetLogicalSize(int i) const { return(box(i)); }

		virtual Point GetPhysicalCoordinates(const Index &index) const;
		virtual Index GetApproximateLogicalCoordinates(const Point & point) const;
		virtual Field* AllocCopy() const;
#endif //SWIG
};

#endif
