#include "AnalyticSplitEuler.h"

AnalyticSplitEuler::AnalyticSplitEuler(void) : OperatorSplitFieldMover(), step_size(0.0) 
{
	// introspective things and so on
}

AnalyticSplitEuler::AnalyticSplitEuler(Real step, SplitRightHandSide *RHS) : OperatorSplitFieldMover(RHS), step_size(step)
{
	// introspective
}

AnalyticSplitEuler::AnalyticSplitEuler(const AnalyticSplitEuler &exeuler) : OperatorSplitFieldMover(exeuler), step_size(exeuler.GetStepSize())
{ SetWorkSpace(exeuler.GetWorkSpace()); }

AnalyticSplitEuler::~AnalyticSplitEuler(void) { FreeWorkSpace(); }

void AnalyticSplitEuler::FreeWorkSpace(void)
{
	int i;
	for (i=0;i<WorkSpace.GetNumberOfFields(); i++) if (&WorkSpace[i]!=ZERO) delete &WorkSpace[i];
	WorkSpace.RemoveAllFields();
}

bool AnalyticSplitEuler::SetWorkSpace(const FieldCollection &Fields)
{
	int i, number;
	bool flag=false;
	Field *NewField;

	number=Fields.GetNumberOfFields();

	if (number!=WorkSpace.GetNumberOfFields()) 											flag=true;
	else 			for (i=0; i<number; i++) if (&WorkSpace[i]==ZERO) 					flag=true;
	if (!flag) 	for (i=0; i<number; i++) if (typeid(WorkSpace[i])!=typeid(Fields[i])) 	flag=true;

	if (flag) {
		FreeWorkSpace();
		for (i=0; i<number; i++) {
			flag=WorkSpace.AddField(NewField=Fields[i].AllocCopy());
			if (!flag || NewField==ZERO) return(false);
		}
	}
	return(true);
}
bool AnalyticSplitEuler::Move(FieldCollection & Fields)
{
	int i;
	int n;

	for (n=0; n<split_rhs->GetNumberOfRHS(); n++) {
		//cout << "AnalyticSplitEuler::Move n=" << n << endl;
		if (split_rhs->Advance(Fields, step_size, n)) {
			//cout << "AnalyticSplitEuler::Move advanced analytically" << endl;
			//for (i=0; i<Fields.GetNumberOfFields(); i++) Equal[Fields[i]][WorkSpace[i]];
		}
		else {
			if (!split_rhs->RHS(Fields, WorkSpace, n)) return(false);
			//cout << "AnalyticSplitEuler::Move advance Euler" << endl;
			for (i=0; i<Fields.GetNumberOfFields(); i++) {
				Times(step_size)[WorkSpace[i]];
				Add[Fields[i]][WorkSpace[i]];
			}
		}
	}
	return(true);
}

bool AnalyticSplitEuler::SafeMove(FieldCollection & fields)
{ 
  int i;
  if (split_rhs==NULL) return(false);
  for (i=0;i<fields.GetNumberOfFields();i++){
    if (&fields[i]==NULL) return(false);
  }
  if (!SetWorkSpace(fields)) return(false);
  
  return(Move(fields));
}
