#include "ConservedOP.h"
#include <typeinfo>
#include <stdio.h>


ConservedOPParameters::ConservedOPParameters(void) { ; }

ConservedOPParameters::ConservedOPParameters(const ConservedOPParameters &copp) : diffusivity(copp.GetDiffusivity()), potential(copp.GetPotential()) { ; }

ConservedOPParameters::ConservedOPParameters(Real d, Real p) : diffusivity(d), potential(p) { ; }

ConservedOPParameters::~ConservedOPParameters(void) { ; }

ConservedOP::ConservedOP(void) : RightHandSide(), WorkSpace(), diffusivity(0.0), potential(0.0)
{
	// introspective
}

ConservedOP::ConservedOP(const ConservedOP &cop) : RightHandSide(cop), WorkSpace(), diffusivity(cop.GetDiffusivity()), potential(cop.GetPotential())
{
	SetWorkSpace(cop.GetWorkSpace());
}

ConservedOP::~ConservedOP(void)
{	
	diffusivity=0.0;
	potential=0.0;
	if (&WorkSpace[0]!=ZERO) delete &WorkSpace[0];
	WorkSpace.RemoveAllFields();
}

bool ConservedOP::SetWorkSpace(const FieldCollection & field)
{
	int i;

	for (i=0; i<WorkSpace.GetNumberOfFields(); i++) if (&WorkSpace[i]!=ZERO) delete &WorkSpace[i];
	WorkSpace.RemoveAllFields();

	//cout << "ConservedOP::SetWorkSpace(" << field << ")" << endl;
	if (!WorkSpace.AddField(field[0].AllocCopy())) return(false);
	//cout << "ConservedOP::SetWorkSpace(" << field << ") .... workspace=" << workspace << endl;
	if (&WorkSpace[0]==ZERO) return(false);

	return(true);
}

bool ConservedOP::RHS(const FieldCollection &input, FieldCollection &output)
{
	#ifdef _DEBUG
		//if (typeid(*workspace)!=typeid(*phi[0])) {
		//	cout << "ConservedOP::RHS workspace of different type as phi" << endl;
		//	cout << "workspace: id.name=" << (typeid(*workspace)).name() << endl;
		//	cout << "      phi: id.name=" << (typeid(*phi[0])).name() << endl;
		//	exit(-1);
		//}
		assert(&input[0]!=ZERO);
		assert(&output[0]!=ZERO);
		assert(&WorkSpace[0]!=ZERO);
	#endif //_DEBUG

	Field &Phi=output[0], &Wsp=WorkSpace[0];

	Equal[Wsp][input[0]];
	Equal[Phi][input[0]];

	PhaseFieldTerm(potential)[Wsp];
	Laplace[Phi];
	Times(-diffusivity)[Phi];
	Add[Phi][Wsp];
	Laplace[Phi];

	//cout << "ConservedOP::RHS(Field ** phi)" << endl;
	//Equal[*workspace][*phi[0]]; //*workspace=*phi[0];
	//PhaseFieldTerm(potential)[*workspace]; //workspace->PhaseFieldTerm(potential);
	//Laplace[*phi[0]]; //phi[0]->Laplace();
	//Times(-diffusivity)[*phi[0]]; //*phi[0]*=-diffusivity;
	//Add[*phi[0]][*workspace]; //*phi[0]+=*workspace;
	//Laplace[*phi[0]]; //phi[0]->Laplace();

	return(true);
}
