#include "ControlEuler.h"

ControlEuler::ControlEuler(void) : OperatorSplitFieldMover(), step_size(0.0) 
{
	// introspective things and so on
}

ControlEuler::ControlEuler(Real step, SplitRightHandSide *RHS) : OperatorSplitFieldMover(RHS), step_size(step)
{
	// introspective
}

ControlEuler::ControlEuler(const ControlEuler &exeuler) : OperatorSplitFieldMover(exeuler), step_size(exeuler.GetStepSize())
{
	SetWorkSpace(exeuler.GetWorkSpace());
}

ControlEuler::~ControlEuler(void) 
{
	FreeWorkSpace();
	FreeErrorEstimate();
}

void ControlEuler::FreeWorkSpace(void)
{
	int i;
	for (i=0;i<WorkSpace.GetNumberOfFields(); i++) if (&WorkSpace[i]!=ZERO) delete &WorkSpace[i];
	WorkSpace.RemoveAllFields();
}

void ControlEuler::FreeErrorEstimate(void)
{
	int i;
	for (i=0;i<ErrorEstimate.GetNumberOfFields(); i++) if (&ErrorEstimate[i]!=ZERO) delete &ErrorEstimate[i];
	ErrorEstimate.RemoveAllFields();
}

bool ControlEuler::SetWorkSpace(const FieldCollection &Fields)
{
	int i, number;
	bool flag=false;
	Field *NewField;

	number=Fields.GetNumberOfFields();

	if (number!=WorkSpace.GetNumberOfFields()) 											flag=true;
	else 			for (i=0; i<number; i++) if (&WorkSpace[i]==ZERO) 					flag=true;
	if (!flag) 	for (i=0; i<number; i++) if (typeid(WorkSpace[i])!=typeid(Fields[i])) 	flag=true;

	if (flag) {
		FreeWorkSpace();
		for (i=0; i<number; i++) {
			//cout << "ControlEuler::SetWorkSpace(const FieldCollection &Fields): Make WorkSpace " << i << endl;
			flag=WorkSpace.AddField(NewField=Fields[i].AllocCopy());
			if (!flag || NewField==ZERO) return(false);
		}
	}

	flag=false;
	if (number!=ErrorEstimate.GetNumberOfFields()) 											flag=true;
	else 			for (i=0; i<number; i++) if (&ErrorEstimate[i]==ZERO)					flag=true;
	if (!flag) 	for (i=0; i<number; i++) if (typeid(ErrorEstimate[i])!=typeid(Fields[i])) 	flag=true;

	if (flag) {
		FreeErrorEstimate();
		for (i=0; i<number; i++) {
			//cout << "ControlEuler::SetWorkSpace(const FieldCollection &Fields): Make ErrorEstimate " << i << endl;
			flag=ErrorEstimate.AddField(NewField=Fields[i].AllocCopy());
			if (!flag || NewField==ZERO) return(false);
		}
	}
	return(true);
}
bool ControlEuler::Move(FieldCollection & Fields)
{
	int i, j, n;
	#ifdef _DEBUG
		//if (work_space==ZERO)  return(false);
		//if (n!=number_of_workspaces) return(false);
		//for (i=0; i<number_of_workspaces; i++) 
		//	if (typeid(*work_space[i])!=typeid(*field[i])) return(false);
	#endif
	//for (i=0; i<number_of_workspaces; i++) Equal[*work_space[i]][*field[i]]; 

	for (i=0; i<ErrorEstimate.GetNumberOfFields(); i++) Equal[ErrorEstimate[i]][Fields[i]]; 

	for (n=0; n<split_rhs->GetNumberOfRHS(); n++) {
		if (!split_rhs->RHS(ErrorEstimate, WorkSpace, n)) return(false);

		for (i=0; i<ErrorEstimate.GetNumberOfFields(); i++) {
			Times(step_size)[WorkSpace[i]];
			Add[ErrorEstimate[i]][WorkSpace[i]];
		}
	}
	for (j=0; j<2; j++) {
		for (n=0; n<split_rhs->GetNumberOfRHS(); n++) {
			if (!split_rhs->RHS(Fields, WorkSpace, n)) return(false);

			for (i=0; i<Fields.GetNumberOfFields(); i++) {
				Times(0.5*step_size)[WorkSpace[i]];
				Add[Fields[i]][WorkSpace[i]];
			}
		}
	}
	for (i=0; i<ErrorEstimate.GetNumberOfFields(); i++) {
		Subtract[ErrorEstimate[i]][Fields[i]];
		//Times(2.0/step_size/step_size)[ErrorEstimate[i]];
		Subtract[Fields[i]][ErrorEstimate[i]];
	}
	return(true);
}

bool ControlEuler::SafeMove(FieldCollection & fields)
{ 
  int i;
  if (split_rhs==ZERO) return(false);
  for (i=0;i<fields.GetNumberOfFields();i++){
	 if (&fields[i]==ZERO) return(false);
  }
  if (!SetWorkSpace(fields)) return(false);
  
  return(Move(fields));
}
