#include "ExplicitEuler.h"

ExplicitEuler::ExplicitEuler(void) : FieldMover(), step_size(0.0)
{ }

ExplicitEuler::ExplicitEuler(Real step, RightHandSide *RHS) : FieldMover(RHS), step_size(step) 
{ }

ExplicitEuler::ExplicitEuler(const ExplicitEuler &exeuler) : FieldMover(exeuler), step_size(exeuler.GetStepSize())
{
	bool flag;
	flag=SetWorkSpace(exeuler.GetWorkSpace());
	#ifdef _DEBUG
		assert(flag);
	#endif
}

ExplicitEuler::~ExplicitEuler(void) 
{
	FreeWorkSpace();
}

void ExplicitEuler::FreeWorkSpace(void)
{
	int i;
	for (i=0;i<WorkSpace.GetNumberOfFields(); i++) if (&WorkSpace[i]!=ZERO) delete &WorkSpace[i];
	WorkSpace.RemoveAllFields();
}

bool ExplicitEuler::SetWorkSpace(const FieldCollection &Fields)
{
	int i, number;
	bool flag=false;
	Field *NewField;

	number=Fields.GetNumberOfFields();

	if (number!=WorkSpace.GetNumberOfFields()) 											flag=true;
	else 			for (i=0; i<number; i++) if (&WorkSpace[i]==ZERO) 					flag=true;
	if (!flag) 	for (i=0; i<number; i++) if (typeid(WorkSpace[i])!=typeid(Fields[i])) 	flag=true;

	if (flag) {
		FreeWorkSpace();
		for (i=0; i<number; i++) {
			flag=WorkSpace.AddField(NewField=Fields[i].AllocCopy());
			if (!flag || NewField==ZERO) return(false);
		}
	}
	return(true);
}
bool ExplicitEuler::Move(FieldCollection & Fields)
{
	int i;

	if (!rhs->RHS(Fields, WorkSpace)) return(false); // rhs->RHS(work_space);

	for (i=0; i<Fields.GetNumberOfFields(); i++) {
		Times(step_size)[WorkSpace[i]]; //*work_space[i]*=step_size;
		Add[Fields[i]][WorkSpace[i]]; //*field[i]+=*work_space[i];
	}
	return(true);
}

bool ExplicitEuler::SafeMove(FieldCollection & fields)
{ 
  int i;
  if (rhs==ZERO) return(false);
  for (i=0;i<fields.GetNumberOfFields();i++){
    if (&fields[i]==ZERO) return(false);
  }
  if (!SetWorkSpace(fields)) return(false);
  
  return(Move(fields));
}
