#include "FieldInitializer.h"
#include <new>

using namespace std;

FieldInitializer::FieldInitializer() : 	dangerous_point(ZERO), danger_of_point(ZERO), total_no_of_dangerous_points(0) 
{
}

FieldInitializer::FieldInitializer(const FieldInitializer & initializer) : 	dangerous_point(ZERO), danger_of_point(ZERO),
																			total_no_of_dangerous_points(0)
{
	bool flag;
	int num=initializer.GetTotalNumberOfDangerousPoints();
	const Point *danP = initializer.GetDangerousPointsPoints();
	const Real *danD = initializer.GetDangers();

	flag=SetDangerousPoints(num, danP, danD);
	#ifdef _DEBUG
		assert(flag);
	#endif
}

FieldInitializer::~FieldInitializer()
{
	if (danger_of_point!=ZERO) delete []danger_of_point;
	if (dangerous_point!=ZERO) delete []dangerous_point;
}

bool FieldInitializer::SetTotalNumberOfDangerousPoints(const int number) 
{
	int i;
	//cout << "FieldInitializer::SetTotalNumberOfDangerousPoints(const int " << number << ")" << endl;

	if (number==total_no_of_dangerous_points) {
		return(true);
	}
	else {
		Real  * new_danger_of_point = new(std::nothrow) Real[number];
		Point * new_dangerous_point = new(std::nothrow) Point[number];

		#ifdef _DEBUG
			assert((dangerous_point==ZERO && danger_of_point==ZERO && total_no_of_dangerous_points==0) ||
					(dangerous_point!=ZERO && danger_of_point!=ZERO && total_no_of_dangerous_points>0));
		#endif //_DEBUG
	
		if (new_danger_of_point==ZERO || new_dangerous_point==ZERO) return(false);
		else if (dangerous_point==ZERO && danger_of_point==ZERO) {
			//cout << "SetTotalNumberOfDangerousPoints: dangerous_point==ZERO && danger_of_point==ZERO" << endl;
			dangerous_point=new_dangerous_point;
			danger_of_point=new_danger_of_point;
			total_no_of_dangerous_points=number;
			return(true);
		}
		else if (dangerous_point!=ZERO && danger_of_point!=ZERO) {
			//cout << "SetTotalNumberOfDangerousPoints: dangerous_point!=ZERO && danger_of_point!=ZERO" << endl;
			for (i=0; (i<number) && (i<total_no_of_dangerous_points); i++) {	
				new_danger_of_point[i]=danger_of_point[i];
				new_dangerous_point[i]=dangerous_point[i];
			}
			delete []danger_of_point;
			delete []dangerous_point;
			dangerous_point=new_dangerous_point;
			danger_of_point=new_danger_of_point;
			total_no_of_dangerous_points=number;
			return(true);
		}
		else {
			//cout << "FieldInitializer::SetTotalNumberOfDangerousPoints UNFORSEEN!!!" << endl;
			return(false);
		}
	}
}
bool FieldInitializer::SetDangerousPoints(int number, const Point * points, const Real *dangers)
{
	int i;
	if (SetTotalNumberOfDangerousPoints(number)) {
		for (i=0; i<total_no_of_dangerous_points; i++) {
			dangerous_point[i]=points[i];
			danger_of_point[i]=dangers[i];
		}
		return(true);
	}
	else return(false);
}

bool FieldInitializer::SetDangerousPoint(const int i, const Point & point, const Real danger)
{
	#ifdef _DEBUG
		//assert(i>=0 && i<total_no_of_dangerous_points);
	#endif //_DEBUG
	if (i>=0 && i<total_no_of_dangerous_points) {
		dangerous_point[i]=point;
		danger_of_point[i]=danger;
		return(true);
	}
	else return(false);
}


bool FieldInitializer::SetDangerOfPoint(const int i, const Real danger)
{
	if (i>=0 && i<total_no_of_dangerous_points) {
		danger_of_point[i]=danger;
		return(true);
	}
	else return(false);
}


Point  FieldInitializer::GetDangerousPoint(const int i) const
{
	#ifdef _DEBUG
		assert(i>=0 && i<total_no_of_dangerous_points);
	#endif //_DEBUG
	return(dangerous_point[i]);
}

//Real ** FieldInitializer::GetDangerousPoints() const

//Point * FieldInitializer::GetDangerousPointsPoint() const

Real FieldInitializer::GetDanger(const int i) const
{
	#ifdef _DEBUG
		assert(i>=0 && i<total_no_of_dangerous_points);
	#endif //_DEBUG
	return(danger_of_point[i]);
}

Real FieldInitializer::GetMaximumDanger() const
{
	int i;
	Real max_danger;

	for (i=0, max_danger=0.0; i<total_no_of_dangerous_points; i++)
		if (danger_of_point[i]>max_danger) max_danger=danger_of_point[i];
	
	return(max_danger);
}

