#include <typeinfo>
#ifndef _WIN32
#include <unistd.h>
#endif
#include <math.h>
#ifdef _WIN32
#include <windows.h>
#endif //_WIN32
#include "GnuPlotter.h"
#include "Field.h"
#include "SquareGridField.h"
#include "Index.h"
#include "Point.h"
#include <new>

using namespace std;

GnuPlotter::GnuPlotter(int x, int y) : FieldObserver(), buffername(NULL), gnuplot_pipe(NULL), buffer(NULL)
{
	char name[128];
	strcpy(title, "");
#ifdef _WIN32
		gnuplot_pipe = _popen("I:\\gnuplot\\pgnuplot.exe", "w");	
#else
	//#ifdef _DEBUG
	//	sprintf(name, "tee gnuplot.protocol%d |  gnuplot -geometry %dx%d", (int)this, x, y);
	//	gnuplot_pipe = popen(name, "w");
	//#else
		sprintf(name, "gnuplot -geometry %dx%d", x, y);
		gnuplot_pipe = popen(name, "w");
	///#endif // _DEBUG
#endif // _WIN32
	if (gnuplot_pipe==NULL) {
		cout << "GnuPlotter::GnuPlotter(): Gnuplot couldn't be opened!" << endl;
		exit(-1);
	}
	buffername=tempnam(NULL, "gnuplotter");
	if (buffername==NULL) {
		cout << "GnuPlotter::GnuPlotter(): Buffername couldn't be created!" << endl;
		exit(-1);
	}
	buffer=fopen(buffername, "w");
	if (buffer==NULL) {
		cout << "GnuPlotter::GnuPlotter(): Buffer couldn't be created!" << endl;
		exit(-1);
	}
}

GnuPlotter::GnuPlotter(FILE * gplotpipe) : FieldObserver(), buffername(NULL), gnuplot_pipe(gplotpipe)
{
	#ifdef _DEBUG
		assert(gnuplot_pipe!=NULL);
	#endif //_DEBUG
	buffername=tempnam(NULL, "gnuplotter");
	if (buffername==NULL) {
		cout << "GnuPlotter::GnuPlotter(): Buffername couldn't be created!" << endl;
		exit(-1);
	}
	buffer=fopen(buffername, "w");
	if (buffer==NULL) {
		cout << "GnuPlotter::GnuPlotter(): Buffer couldn't be created!" << endl;
		exit(-1);
	}
}

GnuPlotter::~GnuPlotter(void) { 
	if (gnuplot_pipe!=NULL) {
		//cout << "GnuPlotter::~GnuPlotter()" << endl;
		#ifdef _WIN32
			_pclose(gnuplot_pipe);
		#else
			pclose(gnuplot_pipe); 
		#endif //_WIN32
	}
	if (buffer!=NULL) fclose(buffer);
	if (buffername!=NULL) {
		unlink(buffername);
		free(buffername);
	}
	// else nothing to close
}

bool GnuPlotter::SendPlotCommand(const char* commandstring)
{
	if (gnuplot_pipe!=NULL) {
		fprintf(gnuplot_pipe, "%s", commandstring);
		FlushGnuPlotter();
		return(true);
	}
	else return(false);
}

TwoDGnuplotter::TwoDGnuplotter() : GnuPlotter()
{ 
}

TwoDGnuplotter::TwoDGnuplotter(int x, int y) : GnuPlotter(x, y)
{ 
}

TwoDGnuplotter::TwoDGnuplotter(FILE * gplotpipe) : GnuPlotter(gplotpipe)
{ 
}

TwoDGnuplotter::~TwoDGnuplotter()
{ ; }

bool TwoDGnuplotter::SendData(const SquareGridField *field)
{
	//if (DIMENSION!=2) return(false);
	// Throw exeption?
	Index ix(0);
	Point x;

	#ifdef _DEBUG
		assert(field!=NULL);
		assert(gnuplot_pipe!=NULL);
		assert(freopen(buffername, "w", buffer)!=NULL);
	#else
		freopen(buffername, "w", buffer);
	#endif
	Index box=field->GetLogicalSize();
	#if DIMENSION==1
		for (ix[0]=0; ix[0]<box(0); ix[0]++) {
			x=field->GetPhysicalCoordinates(ix);
			fprintf(buffer, "%g %g \n", x[0], field->GetFieldValue(ix, 0));
			fflush(buffer);
		}
	#endif
	#if DIMENSION==2
		for (ix[0]=0; ix[0]<box(0); ix[0]++) {
			for (ix[1]=0; ix[1]<box(1); ix[1]++) {
				x=field->GetPhysicalCoordinates(ix);
				fprintf(buffer, "%g %g %g \n", x[0], x[1], field->GetFieldValue(ix, 0));
			}
			fprintf(buffer, "\n");
			fflush(buffer);
		}
	#endif
	#if DIMENSION==3
		int maxi, kk;
		Real max, z0, z1, z, f0, f1;
		Point kasten=(field->GetBoundaryCondition())->GetPhysicalSize();
		for (ix[0]=0; ix[0]<box(0); ix[0]++) {
			for (ix[1]=0; ix[1]<box(1); ix[1]++) {
				for (ix[2]=0, maxi=0, max=field->GetFieldValue(ix, 0); ix[2]<box(2); ix[2]++) {
					if (max<field->GetFieldValue(ix, 0)) {
						max=field->GetFieldValue(ix, 0);
						maxi=ix[2];
					}
				}
				ix[2]=maxi;
				x=field->GetPhysicalCoordinates(ix);
				z0=x[2];
				f0=field->GetFieldValue(ix, 0);
				//cout << "TwoDGnuplotter::SendData max at ix=" << ix << " phi=" << f0 << endl;
				if (max>=level) {
					for (kk=1; kk<box(2); kk++) {
						ix[2]=(kk+maxi)%box(2);
						f1=field->GetFieldValue(ix, 0);
						x=field->GetPhysicalCoordinates(ix);
						z1=x[2];
						if (kk+maxi>=box(2)) z1+=kasten(2);
						if (f1<level) {
							z=level*(z1-z0)/(f1-f0)+(z0*f1-z1*f0)/(f1-f0);
							kk=box(2);
						}
						else {
							z0=z1;
							f0=f1;
						}
					}
				}
				else {
					z=z1=z0;
				}
				fprintf(buffer, "%g %g %g %g %g\n", x[0], x[1], z, z0, z1);
			}
			fprintf(buffer, "\n");
			fflush(buffer);
		}
	#endif
	#if DIMENSION>3
			cout << "Only 1D and 2D plots supported!" << endl;
			exit(-1);
	#endif

	//fprintf(gnuplot_pipe, "e\n");
	fflush(buffer);
	return(FlushGnuPlotter());
}

bool TwoDGnuplotter::SendPlotCommand(void)
{
	#ifdef _DEBUG
		if (gnuplot_pipe==NULL) return(false);
	#endif
	#if DIMENSION==1
		fprintf(gnuplot_pipe, "\n\nplot \"%s\"  w l\n\n", buffername);
	#elif DIMENSION==2
		fprintf(gnuplot_pipe, "\n\nsplot \"%s\"  w l\n\n", buffername);
	#elif DIMENSION==3
		fprintf(gnuplot_pipe, "\n\nsplot \"%s\"  w l\n\n", buffername);
	#else
		cout << "Only 1D and 2D plots supported!" << endl;
		exit(-1);
	#endif
	return(FlushGnuPlotter());
}


void TwoDGnuplotter::Update(const SquareGridField *theChangedSubject)
{
	InitializeGnuplot();
	SendData(theChangedSubject);
	SendPlotCommand();
}

GnuContourPlot::GnuContourPlot(void) : TwoDGnuplotter(), contour_levels(20)
{ InitializeGnuplot(); }

GnuContourPlot::GnuContourPlot(int x, int y) : TwoDGnuplotter(x, y), contour_levels(20)
{ InitializeGnuplot(); }

GnuContourPlot::GnuContourPlot(FILE * gplotpipe) : TwoDGnuplotter(gplotpipe), contour_levels(20)
{ InitializeGnuplot(); }

GnuContourPlot::~GnuContourPlot(void)
{ contour_levels=-1; }

bool GnuContourPlot::InitializeGnuplot(void)
{	
	#ifdef _DEBUG
		if (gnuplot_pipe==NULL) return(false);
		if (contour_levels<1)  return(false);
	#endif

#ifndef _WIN32
	fprintf(gnuplot_pipe, "set terminal x11\n");
#endif
	fprintf(gnuplot_pipe, "set hidden3d\n");
	fprintf(gnuplot_pipe, "set title \"%s\"\n", title);
	fprintf(gnuplot_pipe, "set nosurface\n");
	fprintf(gnuplot_pipe, "set grid\n");
	fprintf(gnuplot_pipe, "set cntrparam levels %d\n", contour_levels); 
	fprintf(gnuplot_pipe, "set view 0,0\n");
	fprintf(gnuplot_pipe, "set contour\n");
	return(FlushGnuPlotter());
}


bool GnuContourPlot::SetContourLevels(int levels)
{
	if (levels<1) return(false);
	contour_levels=levels;
	return(true);
}

GnuSurfacePlot::GnuSurfacePlot(void) : TwoDGnuplotter(), phi_x(60.0), phi_z(30.0)
{ InitializeGnuplot(); }

GnuSurfacePlot::GnuSurfacePlot(int x, int y) : TwoDGnuplotter(x, y), phi_x(60.0), phi_z(30.0)
{ InitializeGnuplot(); }

GnuSurfacePlot::GnuSurfacePlot(FILE * gplotpipe) : TwoDGnuplotter(gplotpipe), phi_x(GetXAngle()), phi_z(GetZAngle())
{ InitializeGnuplot(); }

GnuSurfacePlot::~GnuSurfacePlot(void)
{ phi_x=0.0; phi_z=0.0; }


bool GnuSurfacePlot::SetAngles(Real phix, Real phiz) 
{ 
	phi_x=phix; 
	phi_z=phiz; 
	fprintf(gnuplot_pipe, "set view %g,%g\n", phi_x, phi_z);
	return(true);
}

bool GnuSurfacePlot::InitializeGnuplot(void)
{	
	#ifdef _DEBUG
		if (gnuplot_pipe==NULL) return(false);
	#endif

#ifndef _WIN32
	fprintf(gnuplot_pipe, "set terminal x11\n");
#endif
	fprintf(gnuplot_pipe, "set title \"%s\"\n", title);
	fprintf(gnuplot_pipe, "set hidden3d\n");
	fprintf(gnuplot_pipe, "set surface\n");
	fprintf(gnuplot_pipe, "set view %g,%g\n", phi_x, phi_z);
	fprintf(gnuplot_pipe, "set contour base \n");
	return(FlushGnuPlotter());
}


GnuVectorPlot::GnuVectorPlot(void) : TwoDGnuplotter(), spacing(1), mode(1)
{ InitializeGnuplot(); }

GnuVectorPlot::GnuVectorPlot(int x, int y) : TwoDGnuplotter(x, y), spacing(1), mode(1)
{ InitializeGnuplot(); }

GnuVectorPlot::GnuVectorPlot(FILE * gplotpipe) : TwoDGnuplotter(gplotpipe), spacing(GetSpacing()), mode(GetMode())
{ InitializeGnuplot(); }

GnuVectorPlot::~GnuVectorPlot(void)
{ ; }

bool GnuVectorPlot::InitializeGnuplot(void)
{	
	#ifdef _DEBUG
		if (gnuplot_pipe==NULL) return(false);
	#endif

#ifndef _WIN32
	fprintf(gnuplot_pipe, "set terminal x11\n");
#endif
	fprintf(gnuplot_pipe, "set title \"%s\"\n", title);
	return(FlushGnuPlotter());
}


bool GnuVectorPlot::SendPlotCommand(void)
{
	#ifdef _DEBUG
		if (gnuplot_pipe==NULL) return(false);
	#endif
	switch (DIMENSION) {
	case 2: { fprintf(gnuplot_pipe, "\n\nplot \"%s\" u 5:6 with points, \"%s\" u 1:2:3:4 with vectors\n\n", buffername, buffername); } break;
	default: {
			cout << "Only 2D plots supported!" << endl;
			exit(-1);
		}
	}
	return(FlushGnuPlotter());
}

bool GnuVectorPlot::SendData(const SquareGridField *field)
{
	if (DIMENSION!=2) return(false);
	// Throw exeption?
	Index ix(0);
	Point x;
	Real y0, y1, norm, maxnorm=0.0, dx;

	#ifdef _DEBUG
		assert(field!=ZERO);
		//assert(field->GetNumberOfFields()==2);
		assert(gnuplot_pipe!=NULL);
		assert(freopen(buffername, "w", buffer)!=NULL);
	#else
		freopen(buffername, "w", buffer);
	#endif

	//cout << "GnuVectorPlot::SendData  field->GetNumberOfFields()=" << field->GetNumberOfFields() << endl;
	Point size((field->GetBoundaryCondition())->GetPhysicalSize());
	Index box=field->GetLogicalSize();
	size[0]/=box(0);
	size[1]/=box(1);
	dx=(size(0)>size(1))?size(1):size(0);
	dx*=(Real)spacing;

	if (mode==0) { //divide by maximum
		maxnorm=0.0;
		for (ix[0]=0; ix[0]<box(0); ix[0]+=spacing) {
			for (ix[1]=0; ix[1]<box(1); ix[1]+=spacing) {
				x=field->GetPhysicalCoordinates(ix);
				y0=field->GetFieldValue(x, 0);
				y1=field->GetFieldValue(x, 1);
				norm=sqrt(y0*y0+y1*y1);
				//cout << "norm(" << y0 << ", " << y1 << ")=" << norm << " maxnorm=" << maxnorm << endl;
				if (norm>maxnorm) {
					maxnorm=norm;
					//cout << "norm>maxnorm ->  maxnorm=" << maxnorm << endl;
				}
			}
		}
		//cout << "GnuVectorPlot::SendData maxnorm=" << maxnorm << endl;
	}
	for (ix[0]=0; ix[0]<box(0); ix[0]+=spacing) {
		for (ix[1]=0; ix[1]<box(1); ix[1]+=spacing) {
			x=field->GetPhysicalCoordinates(ix);
			y0=field->GetFieldValue(x, 0);
			y1=field->GetFieldValue(x, 1);
			if (mode==0) { y0=dx*y0/maxnorm; y1=dx*y1/maxnorm; }
			else {
				norm=sqrt(y0*y0+y1*y1);
				if (norm>dx*1e-20) { 
					y0=dx*y0/norm; 
					y1=dx*y1/norm; 
				}
				//else cout << "GnuVectorPlot::SendData norm(" << y0 << ", " << y1 << ")=" << norm << endl;
			}
			fprintf(buffer, "%g %g %g %g %g %g\n", x[0]-0.5*y0, x[1]-0.5*y1, y0, y1, x[0], x[1]);
		}
		fprintf(buffer, "\n");
		fflush(buffer);
	}
			
	//fprintf(gnuplot_pipe, "e\n");
	fflush(buffer);
	return(FlushGnuPlotter());
}

void GnuVectorPlot::Update(const SquareGridField *theChangedSubject)
{
	InitializeGnuplot();
	SendData(theChangedSubject);
	SendPlotCommand(theChangedSubject);
}

bool GnuVectorPlot::SendPlotCommand(const SquareGridField *field)
{
	#ifdef _DEBUG
		assert(field!=ZERO);
	#endif

	Point size((field->GetBoundaryCondition())->GetPhysicalSize());
	Index box=field->GetLogicalSize();
	Point dx;
	dx[0]=size(0)/box(0);
	dx[1]=size(1)/box(1);

	#ifdef _DEBUG
		if (gnuplot_pipe==NULL) return(false);
	#endif
	switch (DIMENSION) {
		case 2: { 
			fprintf(gnuplot_pipe, "\n\nplot [%g:%g] [%g:%g] \"%s\" u 5:6 with points, \"%s\" u 1:2:3:4 with vectors\n\n", 
						-2.0*dx(0), size(0)+2.0*dx(0), -2.0*dx(1), size(1)+2.0*dx(1), buffername, buffername); 
		} break;
		default: {
			cout << "Only 2D plots supported!" << endl;
			exit(-1);
		}
	}
	return(FlushGnuPlotter());
}


GnuScatterPlot::GnuScatterPlot(void) : TwoDGnuplotter(), pointsize(1.0)
{ InitializeGnuplot(); }

GnuScatterPlot::GnuScatterPlot(int x, int y) : TwoDGnuplotter(x, y), pointsize(1.0)
{ InitializeGnuplot(); }

GnuScatterPlot::GnuScatterPlot(FILE * gplotpipe) : TwoDGnuplotter(gplotpipe), pointsize(1.0)
{ InitializeGnuplot(); }

GnuScatterPlot::~GnuScatterPlot(void)
{ ; }

bool GnuScatterPlot::InitializeGnuplot(void)
{	
	#ifdef _DEBUG
		if (gnuplot_pipe==NULL) return(false);
	#endif

#ifndef _WIN32
	fprintf(gnuplot_pipe, "set terminal x11\n");
#endif
	fprintf(gnuplot_pipe, "set title \"%s\"\n", title);
	fprintf(gnuplot_pipe, "set pointsize %f\n", pointsize);
	fprintf(gnuplot_pipe, "set grid\n");
	return(FlushGnuPlotter());
}


bool GnuScatterPlot::SendPlotCommand(void)
{
	cout << "Only: GnuScatterPlot::SendPlotCommand(Subject *theChangedSubject) supported!" << endl;
	return(false);
}

bool GnuScatterPlot::SendData(const SquareGridField *field)
{
	int i;
	Real *y;
	Point x;
	Index index;

	#ifdef _DEBUG
		assert(field!=ZERO);
		assert(gnuplot_pipe!=NULL);
		assert(freopen(buffername, "w", buffer)!=NULL);
	#else
		freopen(buffername, "w", buffer);
	#endif
	y=new(nothrow) Real[field->GetNumberOfFields()];
	#ifdef _DEBUG
		assert(y!=ZERO);
	#endif
	//for (field->GotoFirstPoint(); !field->IsLastPoint(); field->GotoNextPoint()) {
	for (index.Zero(); index.IsInBox(field->GetLogicalSize()); index.NextInBox(field->GetLogicalSize())) {
		//x=field->GetPointCoordinates();
		x=field->GetPhysicalCoordinates(index);
		//for (i=0; i<field->GetNumberOfFields(); i++) y[i]=field->GetFieldValue(i);
		for (i=0; i<field->GetNumberOfFields(); i++) y[i]=field->GetFieldValue(index, i);
		for (i=0; i<DIMENSION; i++) fprintf(buffer, "%g ", x[i]);
		fprintf(buffer, "        ");
		for (i=0; i<field->GetNumberOfFields(); i++) fprintf(buffer, "%g ", y[i]);
		fprintf(buffer, "\n");
	}
	fflush(buffer);
	delete []y;
	return(FlushGnuPlotter());
}

void GnuScatterPlot::Update(const SquareGridField *theChangedSubject)
{
	InitializeGnuplot();
	SendData(theChangedSubject);
	SendPlotCommand(theChangedSubject);
}

bool GnuScatterPlot::SendPlotCommand(const SquareGridField *field)
{
	#ifdef _DEBUG
		assert(field!=ZERO);
		if (gnuplot_pipe==NULL) return(false);
	#endif

	switch (field->GetNumberOfFields()) {
		case 1: { 
			switch (DIMENSION) {
				case 1: { fprintf(gnuplot_pipe, "\n\nplot \"%s\" u 1:2 with points\n", buffername); } break;
				case 2: { fprintf(gnuplot_pipe, "\n\nsplot \"%s\" u 1:2:3 with points\n", buffername); } break;
				default: { cout << "GnuScatterPlot: One field and more than 2 dimensions not supported!" << endl; } break;
			}
		} break;
		case 2: { fprintf(gnuplot_pipe, "\n\nplot \"%s\" u %d:%d with points\n", buffername, DIMENSION+1, DIMENSION+2); } break;
		case 3: { fprintf(gnuplot_pipe, "\n\nsplot \"%s\" u %d:%d:%d with points\n", buffername, DIMENSION+1, DIMENSION+2, DIMENSION+3); } break;
		default: { cout << "GnuScatterPlot: Only up to NumberOfFields=3 supported!" << endl; } break;
	}
	return(FlushGnuPlotter());
}

