#include "Point.h"
#include <new>
#include <math.h>

Point::Point(const Real x[DIMENSION]) 
{
	int i; for (i=0; i<DIMENSION; i++) coordinate[i]=x[i];
}

Point::Point(const Point & point) 
{
	int i; for (i=0; i<DIMENSION; i++) coordinate[i]=point(i);
}

#if DIMENSION==2
Point::Point(const Real x, const Real y) 
{
	coordinate[0]=x; coordinate[1]=y;
}

#elif DIMENSION==3
Point::Point(const Real x, const Real y, const Real z) 
{
	coordinate[0]=x; coordinate[1]=y; coordinate[2]=z;
}

#else
		//
#endif

Point::Point() 
{ 
	// Zero(); // For efficiency
}

Point::Point(const Real x) 
{ 
	Initialize(x); 
}


Real * Point::operator()(Real *pp) const
{
	int i;
	#ifdef _DEBUG
		assert(pp!=ZERO);
	#endif
	for (i=0; i<DIMENSION; i++) pp[i]=coordinate[i];
	return(pp);
}

bool Point::operator==(const Point &right) const
{
	int i;
	bool f;
	for (i=0, f=true; i<DIMENSION; i++) if (coordinate[i]!=right(i)) f=false; 
	return(f);
}

bool Point::IsPositive() const
{
	int i;
	bool f;
	for (i=0, f=true; i<DIMENSION; i++) if (coordinate[i]<=0.0) f=false;
	return(f);
}

bool Point::IsNonNegative() const
{
	int i;
	bool f;
	for (i=0, f=true; i<DIMENSION; i++) if (coordinate[i]<0.0) f=false;
	return(f);
}

bool Point::IsInBox(const Point &box) const
{
	int i;
	bool f;
	if (!IsNonNegative() || !box.IsPositive()) f=false;
	else for (i=0, f=true; i<DIMENSION; i++) if (coordinate[i]>=box(i)) f=false;
	return(f);
}

Point & Point::operator*=(const Real x)
{
	int i;
	for (i=0; i<DIMENSION; i++) coordinate[i]*=x;
	return(*this);
}

Point & Point::operator/=(const Real x)
{
	int i;
	#ifdef _DEBUG
		assert(x!=0.0);
	#endif
	for (i=0; i<DIMENSION; i++) coordinate[i]/=x;
	return(*this);
}

Point & Point::operator+=(const Point & addend)
{
	int i;
	for (i=0; i<DIMENSION; i++) coordinate[i]+=addend(i);
	return(*this);
}

Point & Point::operator-=(const Point & addend)
{
	int i;
	for (i=0; i<DIMENSION; i++) coordinate[i]-=addend(i);
	return(*this);
}

Point & Point::operator=(const Point & x)
{
	int i;
	for (i=0; i<DIMENSION; i++) coordinate[i]=x(i);
	return(*this);
}

bool Point::Initialize(const Real x)
{
	int i;
	for (i=0; i<DIMENSION; i++) coordinate[i]=x;
	return(true);
}

//#if DIMENSION==2
bool Point::Initialize(const Real x, const Real y)
{
	coordinate[0]=x;
	coordinate[1]=y;
	return(true);
}
/*#elif DIMENSION==3
bool Point::Initialize(const Real x, const Real y, const Real z)
{
	coordinate[0]=x;
	coordinate[1]=y;
	coordinate[2]=z;
	return(true);
}
#endif
*/

Real Point::Volume() const
{
	int i;
	Real vol;
	for (i=0, vol=1.0; i<DIMENSION; i++)
	vol*=coordinate[i];
	return(vol);
}

Real Point::Norm(void) const
{
	int i;
	Real n;
	for (n=0, i=0; i<DIMENSION; i++) n+=coordinate[i]*coordinate[i];
	return(sqrt(n));
}

void Point::Print(const char * text) const
{
	int i;
	cout << text << "=[" << coordinate[0];
	for (i=1; i<DIMENSION; i++) cout << ", " << coordinate[i];
	cout << "]" << endl;
}

ostream & operator <<(ostream &os, const Point & point) {
	int i;
	os << "[" << point(0);
	for (i=1; i<DIMENSION; i++) os << ", " << point(i);
	os << "]";
	return(os);
}

istream & operator >>(istream &is, Point & point) {
	int i;
	for (i=0; i<DIMENSION; i++) is >> point[i];
	return(is);
}

