#include <math.h>
#include <new>
#include "SquareGridField.h"
#include "DigitalMaterial.h"
#include "SimplePeriodicBC.h"
#include "SimpleDirichletBC.h"

#ifdef _DEBUG
#include <assert.h>
#endif //_DEBUG

using namespace std;

SquareGridField::SquareGridField(void) : 	Field() , data(ZERO), number_of_data_points(0), box(), fftwbox(), current_position(0),
											fftw_real_data(ZERO), fftw_complex_data(ZERO), plan_re_to_comp(NULL), plan_comp_to_re(NULL)
{
	Index box(1);
	SetLogicalSize(box);
	//cout << "SquareGridField::SquareGridField() " << data << " " << fftw_real_data << " " << fftw_complex_data << endl;
	//for (int n=0; n<number_of_fields; n++) cout << data[n] << " " << fftw_real_data[n] << " " << fftw_complex_data[n] << endl;
}

SquareGridField::SquareGridField(const SquareGridField & field) : Field(field), data(ZERO), number_of_data_points(0), box(-1), fftwbox(), 
											current_position(0), fftw_real_data(ZERO), fftw_complex_data(ZERO), plan_re_to_comp(NULL), 
											plan_comp_to_re(NULL)
{
	size_t i;
	int n;
	int enn[DIMENSION];

	number_of_fields=0;
	SetNumberOfFields(field.GetNumberOfFields());
	//cout << "SquareGridField::SquareGridField(SquareGridField)" << field.GetLogicalSize() << endl;
	//cout << "SquareGridField::SquareGridField(SquareGridField) SetLogicalSize=" << SetLogicalSize(field.GetLogicalSize()) << endl;
	SetLogicalSize(field.GetLogicalSize());
	//cout << "SquareGridField::SquareGridField(SquareGridField) box=" << box << endl;

	//cout << "SquareGridField::SquareGridField(SquareGridField) " << data << " " << fftw_real_data << " " << fftw_complex_data << endl;
	for (n=0; n<number_of_fields; n++) {
		//cout << data[n] << " " << fftw_real_data[n] << " " << fftw_complex_data[n] << endl;
		for (i=0; i<number_of_data_points; i++) 
			data[n][i]=field.GetFieldValue(i, n);
	}
	
	box(enn);
	//cout << "SquareGridField::SquareGridField(const SquareGridField & field) create plans" << endl;
	plan_re_to_comp=rfftwnd_create_plan(DIMENSION, box(enn), FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE | FFTW_IN_PLACE);
	plan_comp_to_re=rfftwnd_create_plan(DIMENSION, box(enn), FFTW_COMPLEX_TO_REAL, FFTW_ESTIMATE | FFTW_IN_PLACE);
	//cout << "SquareGridField::SquareGridField(const SquareGridField & field) finished" << endl;

	// No introspectice things because field must have registered already
}

SquareGridField::~SquareGridField(void) 
{
	int i;
	// XXX: Shouldn't the individual SUB-arrays be deleted as well? YES YOU ARE RIGHT!!!!!
	for (i=0; i<number_of_fields; i++) if (data[i]!=ZERO) delete []data[i];
	if (data!=ZERO) delete []data;
	if (fftw_real_data!=ZERO) delete []fftw_real_data;
	if (fftw_complex_data!=ZERO) delete []fftw_complex_data;
	box.Zero();
	fftwbox.Zero();
	rfftwnd_destroy_plan(plan_re_to_comp);
	rfftwnd_destroy_plan(plan_comp_to_re);
	plan_re_to_comp=NULL;
	plan_comp_to_re=NULL;
}

bool SquareGridField::SetNumberOfFields(int number)
{
	int i, max;
	bool flag;

	#ifdef _DEBUG
		assert((number_of_fields==0 && data==ZERO) ||
			   (number_of_fields!=0 && data!=ZERO));
	#endif

	if (number==number_of_fields) return(true);
	else if (number<number_of_fields) {
		for (i=number, flag=true, max=number_of_fields; i<max; i++) if (!RemoveField(i)) flag=false;
		#ifdef _DEBUG
			assert(number_of_fields==number);
		#endif
		//cout << "SquareGridField::SetNumberOfFields " << number_of_fields << endl;
		return(flag);
	}
	else {
		for (i=number_of_fields, flag=true; i<number && flag; i++) {
			if (!InsertField(i)) flag=false;
		}
		#ifdef _DEBUG
			assert(number_of_fields==number || !flag);
		#endif
		//cout << "SquareGridField::SetNumberOfFields " << number_of_fields << endl;
		return(flag);
	}
	return(false);
}

 bool SquareGridField::RemoveField(int number) 
 {
	int i;
	Real **dummy;

	if (number<0 || number>=number_of_fields || data==ZERO) return(false);
	else {
		delete []fftw_real_data;
		delete []fftw_complex_data;
		dummy=new(nothrow) Real*[number_of_fields-1];
		fftw_real_data=new(nothrow) fftw_real*[number_of_fields-1];
		fftw_complex_data=new(nothrow) fftw_complex*[number_of_fields-1];
		if (dummy==ZERO || fftw_real_data==ZERO || fftw_complex_data==ZERO) {
			cout << "SquareGridField::RemoveField(" << number << ") allocation failed!" << endl;
			return(false);
		}
		for (i=0; i<number; i++) {
			dummy[i]=data[i];
			fftw_real_data[i]=(fftw_real*)data[i];
			fftw_complex_data[i]=(fftw_complex*)data[i];
		}
		if (data[number]!=ZERO) delete []data[i];
		for (i=number+1; i<number_of_fields; i++) {
			dummy[i-1]=data[i];
			fftw_real_data[i-1]=(fftw_real*)(((*this).data)[i]);
			fftw_complex_data[i-1]=(fftw_complex*)data[i];
		}
		delete []data;
		data=dummy;
		number_of_fields--;
		return(true);
	}
}

bool SquareGridField::InsertField(int number)
{
	int i;
	Real **dummy;

	if (number>=0 && number<=number_of_fields && data!=ZERO) {
		//cout << "SquareGridField::InsertField number=" << number << " in range and data=" << data << "!=ZERO" << endl;
		delete []fftw_real_data;
		delete []fftw_complex_data;
		dummy=new(nothrow) Real*[number_of_fields+1];
		fftw_real_data=new(nothrow) fftw_real*[number_of_fields+1];
		fftw_complex_data=new(nothrow) fftw_complex*[number_of_fields+1];
		if (dummy==ZERO || fftw_real_data==ZERO || fftw_complex_data==ZERO) {
			cout << "SquareGridField::InsertField(" << number << ") allocation failed!" << endl;
			return(false);
		}
		for (i=0; i<number; i++) {
			dummy[i]=data[i];
			fftw_real_data[i]=(fftw_real*)data[i];
			fftw_complex_data[i]=(fftw_complex*)data[i];
		}
		#ifdef _DEBUG 
			assert(fftwbox.Volume()==number_of_data_points || 0==number_of_data_points);
		#endif
		if (number_of_data_points>0) {
			dummy[number]=new(nothrow) Real[number_of_data_points];
			if (dummy==ZERO) {
				cout << "SquareGridField::InsertField(" << number << ") allocation failed!" << endl;
				return(false);
			}
			fftw_real_data[number]=(fftw_real*)dummy[number];
			fftw_complex_data[number]=(fftw_complex*)dummy[number];
		}
		else {
			dummy[number]=ZERO;
			fftw_real_data[number]=ZERO;
			fftw_complex_data[number]=ZERO;
		}
		//catch(bad_alloc) {
		//	return(false);
		//}
		for (i=number+1; i<number_of_fields; i++) {
			dummy[i+1]=data[i];
			fftw_real_data[i+1]=(fftw_real*)data[i];
			fftw_complex_data[i+1]=(fftw_complex*)data[i];
		}
		delete []data;
		data=dummy;
		number_of_fields++;
		return(true);
	}
	else if (number==0 && data==ZERO) {
		//cout << "SquareGridField::InsertField number=" << number << "=0 and data=" << data << "=ZERO" << endl;
		data = new(nothrow) Real*[1];
		fftw_real_data=new(nothrow) fftw_real*[1];
		fftw_complex_data=new(nothrow) fftw_complex*[1];
		if (data==ZERO || fftw_real_data==ZERO || fftw_complex_data==ZERO) {
			cout << "SquareGridField::InsertField(" << number << ") allocation failed!" << endl;	
			return(false);
		}
		if (number_of_data_points>0) {
			data[0] = new(nothrow) Real[number_of_data_points];
			if (data[0]==ZERO) {
				cout << "SquareGridField::InsertField(" << number << ") allocation failed!" << endl;
				return(false);
			}
			fftw_real_data[0]=(fftw_real*)data[0];
			fftw_complex_data[0]=(fftw_complex*)data[0];
		}
		else {
			data[0]=ZERO;
			fftw_real_data[0]=ZERO;
			fftw_complex_data[0]=ZERO;
		}
		number_of_fields++;
		return(true);
	}
	else {
		#ifdef _DEBUG
			cout << "SquareGridField::InsertField something is wrong!!!" << endl;
		#endif
		return(false);
	}
}

bool SquareGridField::SetLogicalSize(const int new_box[DIMENSION])
{
	Index dummy(new_box);
	return(SetLogicalSize(dummy));
}

bool SquareGridField::SetLogicalSize(const Index & new_box)
{
	int i;
	size_t new_size, j;
	Real **dummy;
	Index new_fftwbox;
	new_fftwbox.MakeFFTWBox(new_box);
	int enn[DIMENSION];

	if (box==new_box) {
		//cout << "Nothing to do!" << endl;
		return(true);
	}
	else if (number_of_fields==0) {
		//cout << "SquareGridField::SetLogicalSize number_ofFields=" << number_of_fields << endl;
		if (plan_re_to_comp!=NULL) rfftwnd_destroy_plan(plan_re_to_comp);
		if (plan_comp_to_re!=NULL) rfftwnd_destroy_plan(plan_comp_to_re);
		box=new_box;
		fftwbox.MakeFFTWBox(box);
		//cout << "SquareGridField::SetLogicalSize(" << new_box << ") create plans" << endl;
		plan_re_to_comp=rfftwnd_create_plan(DIMENSION, box(enn), FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE | FFTW_IN_PLACE);
		plan_comp_to_re=rfftwnd_create_plan(DIMENSION, box(enn), FFTW_COMPLEX_TO_REAL, FFTW_ESTIMATE | FFTW_IN_PLACE);
		number_of_data_points=fftwbox.Volume();
		return(true);
	}
	else if (number_of_fields!=0 && data!=ZERO) {
		//cout << "SquareGridField::SetLogicalSize number_ofFields=" << number_of_fields << "!=0 and data=";
		//cout << data << "!=ZERO" << endl;
		if (plan_re_to_comp!=NULL) rfftwnd_destroy_plan(plan_re_to_comp);
		if (plan_comp_to_re!=NULL) rfftwnd_destroy_plan(plan_comp_to_re);

		new_size=new_fftwbox.Volume();
		if (fftwbox.Volume()==new_size && number_of_data_points>0) {
			box=new_box;
			fftwbox=new_fftwbox;
			//cout << "SquareGridField::SetLogicalSize(" << new_box << ") create plans" << endl;
			plan_re_to_comp=rfftwnd_create_plan(DIMENSION, box(enn), FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE | FFTW_IN_PLACE);
			plan_comp_to_re=rfftwnd_create_plan(DIMENSION, box(enn), FFTW_COMPLEX_TO_REAL, FFTW_ESTIMATE | FFTW_IN_PLACE);
			number_of_data_points=new_size;
			return(true);
		}
		else {
			dummy=new(nothrow) Real*[number_of_fields];
			if (dummy==ZERO) {
				cout << "SquareGridField::SetLogicalSize(" << new_box << ") allocation failed!" << endl;
				return(false);
			}
			for (i=0; i<number_of_fields; i++) {
				dummy[i]=new(nothrow) Real[new_size];
				if (dummy[i]==ZERO) {
					cout << "SquareGridField::SetLogicalSize(" << new_box << ") allocation failed!" << endl;
					return(false);
				}
				fftw_real_data[i]=(fftw_real*)dummy[i];
				fftw_complex_data[i]=(fftw_complex*)dummy[i];
			}
			for (i=0; i<number_of_fields; i++) {
				for (j=0; j<new_size && j<number_of_data_points; j++) dummy[i][j]=data[i][j];
				delete []data[i];
			}
			delete []data;
			data=dummy;
			box=new_box;
			fftwbox=new_fftwbox;
			number_of_data_points=new_size;
			//cout << "SquareGridField::SetLogicalSize(" << new_box << ") create plans" << endl;
			plan_re_to_comp=rfftwnd_create_plan(DIMENSION, box(enn), FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE | FFTW_IN_PLACE);
			plan_comp_to_re=rfftwnd_create_plan(DIMENSION, box(enn), FFTW_COMPLEX_TO_REAL, FFTW_ESTIMATE | FFTW_IN_PLACE);
			return(true);
		}
	}
	else {
		#ifdef _DEBUG
			cout << "SquareGridField::SetLogicalSize(" << new_box << "): illegal situation!" << endl;
		#endif
		return(false);
	}
}

bool SquareGridField::SetFieldValue(const Real * phi)
{
	int i;
	bool flag;
	for (i=0, flag=true; (i<number_of_fields && flag); i++)
		flag=SetFieldValue(phi[i], i);
	return(flag);
}

Real SquareGridField::GetFieldValue(const Index &x, int i) const
{
	#ifdef _DEBUG
		assert(i>=0 && i<number_of_fields);
		assert(data[i]!=ZERO);
		assert(x.IsInBox(box));
	#endif //_DEBUG
	return(data[i][x(fftwbox)]);
}

Real SquareGridField::GetFieldValue(size_t x, int i) const
{
	#ifdef _DEBUG
		assert(i>=0 && i<=number_of_fields && x<=fftwbox.Volume() && x>=0);
	#endif //_DEBUG
	return(data[i][x]);
}

bool SquareGridField::SetFieldValue(size_t x, Real value, int i) 
{
	#ifdef _DEBUG
		assert(i>=0 && i<=number_of_fields && x<=fftwbox.Volume() && x>=0);
	#endif //_DEBUG
	data[i][x]=value;
	return(true);
}

Real & SquareGridField::GetField(size_t x, int i) 
{
	#ifdef _DEBUG
		assert(i>=0 && i<=number_of_fields && x<=fftwbox.Volume() && x>=0);
	#endif //_DEBUG
	return(data[i][x]);
}

Real & SquareGridField::GetField(const Index &x, int i)
{
	#ifdef _DEBUG
		assert(i>=0 && i<number_of_fields);
		assert(x.IsInBox(box));
	#endif //_DEBUG
	return(data[i][x(fftwbox)]);
}

//Real SquareGridField::GetField(const Index &x, int i) const
//{
//	#ifdef _DEBUG
//		assert(i>=0 && i<number_of_fields);
//		assert(x.IsInBox(box));
//	#endif //_DEBUG
//	return(data[i][x(box)]);
//}

Real SquareGridField::GetFieldValue(const Point &x, int i) const
{
	int j;
	Index index;
	// Throw exeption?
	if (i<0 || i>=number_of_fields || boundary_condition==ZERO) return(0.0);
	else if (!x.IsInBox(boundary_condition->GetPhysicalSize()) || data[i]==ZERO) return(0.0);
	else {
		for (j=0; j<DIMENSION; j++)
			index[j]=(int)(x(j)/(boundary_condition->GetPhysicalSize())(j)*(Real)box(j)+0.5);
		return(GetFieldValue(index, i));
	}
}

Real * SquareGridField::GetFieldValue(Real * phi) const
{
	int i;
	for (i=0; i<number_of_fields; i++) phi[i]=GetFieldValue(i);
	return(phi);
}


bool SquareGridField::Initialize(const FieldInitializer & initializer, const int i)
{
	int d;
	Real danger;
	Index index, logical_size, fftwsize;
	bool flag;

	// Throw exeption???
	if (boundary_condition==ZERO) {
		#ifdef _DEBUG
			cout << "SquareGridField::Initialize(FieldInitializer, " << i << ") No boundary condition!" << endl;
		#endif
		return(false);
	}
	else if (i<0 || i>=number_of_fields) {
		#ifdef _DEBUG
			cout << "SquareGridField::Initialize(FieldInitializer, " << i << ") Out of range!" << endl;
		#endif
		return(false);
	}
	else if (initializer.GetMaximumDanger()<=0.0 && GetLogicalSize().Volume()<=0) {
		#ifdef _DEBUG
			cout << "SquareGridField::Initialize(FieldInitializer, " << i << ")";
			cout << "Max Danger <=0 AND Log.Volumere=" << GetLogicalSize().Volume() << endl;
		#endif
		return(false);
	}
	else {
		danger=initializer.GetMaximumDanger();
		if (danger>0.0) {
			for (d=0; d<DIMENSION; d++)
				logical_size[d]=(int)((boundary_condition->GetPhysicalSize())(i)*danger+0.5);
			if (!SetLogicalSize(logical_size)) return(false);
		}
		if (GetLogicalSize().Volume()==0) return(false);
	
		logical_size=GetLogicalSize();
		fftwsize=GetFFTWBox();
		//cout << "logical_size=" << logical_size << endl;
		for (index.Zero(), flag=true; (flag && index.IsInBox(logical_size)); index.NextInBox(logical_size)) {
					//cout << index << " " << GetPhysicalCoordinates(index) << " " << initializer.Field(GetPhysicalCoordinates(index)) << endl;
					flag=SetFieldValue(index,  initializer.FieldValue(GetPhysicalCoordinates(index)), i);
		}
		return(flag);
	}
}

bool SquareGridField::SetFieldValue(const Index &x, Real value, int i)
{
	if (i<0 || i>=number_of_fields) {
		cout << "SquareGridField::SetFieldValue i=" << i << " out of range!" << endl;
		return(false);
	}
	else if (!x.IsInBox(box)) {
		cout << "SquareGridField::SetFieldValue x=" << x << " is not in box=" << box << endl;
		return(false);
	}
	else if (data==ZERO) {
		cout << "SquareGridField::SetFieldValue data=" <<  data << "=ZERO" << endl;
		return(false);
	}
	else if (data[i]==ZERO) {
		cout << "SquareGridField::SetFieldValue data[" << i << "]=" << data[i] << "=ZERO" << endl;
		return(false);
	}
	else {
		data[i][x(fftwbox)]=value;
		return(true);
	}
}

Point SquareGridField::GetPhysicalCoordinates(const Index &index) const
{
	int i;
	Point coords(-1.0);
	#ifdef _DEBUG
		assert(boundary_condition!=ZERO);
	#endif
	if (index.IsInBox(box))
		for (i=0; i<DIMENSION; i++)
			coords[i]=(Real)index(i)/((Real)box(i))*(boundary_condition->GetPhysicalSize())(i);
	return(coords);
}

Index SquareGridField::GetApproximateLogicalCoordinates(const Point & point) const
{
	int j;
	Index index(-1);
	#ifdef _DEBUG
		assert(boundary_condition!=ZERO);
	#endif
	if (point.IsInBox((boundary_condition->GetPhysicalSize()))) for (j=0; j<DIMENSION; j++)
		index[j]=(int)(point(j)/(boundary_condition->GetPhysicalSize())(j)*(Real)box(j)+0.5);
	return(index);
}

Field * SquareGridField::AllocCopy() const
{
	//cout << "SquareGridField::AllocCopy()" << endl;
	SquareGridField *ptr = new(nothrow) SquareGridField(*this);
	//cout << "SquareGridField::AllocCopy()" << endl;
	#ifdef _DEBUG
		assert(ptr!=ZERO);
	#endif
	return(ptr);
}

bool SquareGridField::DumpToFile(const char * filename) const
{
	int i;
	FILE *file;

	//if (DIMENSION!=2) {
	//	cout << "SquareGridField::DumpToFile(" << filename << ") DIMENSION=" << DIMENSION << "!=2" << endl;
	//	return(false);
	//}
	//else cout << "SquareGridField::DumpToFile(" << filename << ") DIMENSION=" << DIMENSION << "==2" << endl;

	if ((file=fopen(filename, "w"))==NULL) return(false);

	fprintf(file, "#SquareGridField in '%s'\n#\n", filename);
	fprintf(file, "#number of fields: %d\n", number_of_fields);
	fprintf(file, "#box: [%d", box(0));
	for (i=1; i<DIMENSION; i++) fprintf(file, ", %d", box(i));
	fprintf(file, "]\n");
	fprintf(file, "#system size: [%g", (boundary_condition->GetPhysicalSize())(0));
	for (i=1; i<DIMENSION; i++) fprintf(file, ", %g", (boundary_condition->GetPhysicalSize())(i));
	fprintf(file, "]\n");

	Point x;
	Index ix(0);
	for ( ; ix.IsInBox(box); ix.NextInBox(box)) {
		if (ix[DIMENSION-1]==0) fprintf(file, "\n");
		x=GetPhysicalCoordinates(ix);
		for (i=0; i<DIMENSION; i++) fprintf(file, "%g ", x(i));
		for (i=0; i<number_of_fields; i++) fprintf(file, "%g ", GetFieldValue(x, i));
		fprintf(file, "\n");
	}
	//for (ix[0]=0; ix[0]<box(0); ix[0]++) {
	//	for (ix[1]=0; ix[1]<box(1); ix[1]++) {
	//		x=GetPhysicalCoordinates(ix);
	//		//fprintf(file, "%g %g %g \n", x[0], x[1], GetFieldValue(x, 0));
	//		fprintf(file, "%g %g ", x[0], x[1]);
	//		for (i=0; i<number_of_fields; i++) fprintf(file, "%g ", GetFieldValue(x, i));
	//		fprintf(file, "\n");
	//	}
	//	fprintf(file, "\n");
	//}
	fflush(file);
	fclose(file);
	return(true);
}

