#include "UnaryOperators.h"
#include "Field.h"
#include "SquareGridField.h"
#include <math.h>
#include <new>

using namespace std;

Operator & Laplacian::OperateOn(Field *f)
{
	cout << "Laplacian::OperateOn(Field *f) not implemented!" << endl;
	exit(-1);
	return(*this);
}

Operator & Laplacian::OperateOn(SquareGridField *Phi)
{
	int i, d, vv;
	Real xre, xim, yre, yim, vol, N2m;
	Index fftwcomplexbox(Phi->GetFFTWBox()), index, box=Phi->GetLogicalSize();
	const Point pp((Phi->GetBoundaryCondition())->GetPhysicalSize());
	const Point mm((Phi->GetBoundaryCondition())->GetHelicity());
	const Point null(0.0);
	fftwcomplexbox[DIMENSION-1]/=2;
	fftw_real **real_data=Phi->GetPtrToFFTWReal();
	fftw_complex **complex_data=Phi->GetPtrToFFTComplex();

	vol=(Real)(Phi->GetLogicalSize()).Volume();

	#ifdef _DEBUG
		assert(real_data!=ZERO);
		assert(complex_data!=ZERO);
		assert(Phi->GetReToCompPlan()!=NULL);
		assert(Phi->GetCompToRePlan()!=NULL);
		for (i=0; i<Phi->GetNumberOfFields(); i++) {
			assert(real_data[i]!=ZERO);
			assert(complex_data[i]!=ZERO);
		}
	#endif

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		rfftwnd_one_real_to_complex(Phi->GetReToCompPlan(), real_data[i], NULL);
		if (mm!=null) {
			for (d=0, vv=1; d<DIMENSION; d++) vv*=box(d);
			for (d=0; d<DIMENSION; d++) {
				N2m=0.5*mm(d)*pp(d)*vv/box(d);
				index.Zero(); 
				//complex_data[i][index(fftwcomplexbox)].re-=N2m*(box(d)-1); // not needed for differentiation
				for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
					complex_data[i][index(fftwcomplexbox)].re+=N2m;
					complex_data[i][index(fftwcomplexbox)].im-=N2m/tan(PI*index(d)/box(d));
				}
			}
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) { 
		for (index.Zero(); index.IsInBox(fftwcomplexbox); index.NextInBox(fftwcomplexbox)) {
			yre=0.0; xre=complex_data[i][index(fftwcomplexbox)].re;
			yim=0.0; xim=complex_data[i][index(fftwcomplexbox)].im;
			for (d=0; d<DIMENSION; d++) {
				yre+=xre*2.0*box(d)*box(d)/(pp(d)*pp(d))*(cos(2.0*PI*index(d)/box(d))-1);
				yim+=xim*2.0*box(d)*box(d)/(pp(d)*pp(d))*(cos(2.0*PI*index(d)/box(d))-1);
			}
			complex_data[i][index(fftwcomplexbox)].re=yre/vol;
			complex_data[i][index(fftwcomplexbox)].im=yim/vol;
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) rfftwnd_one_complex_to_real(Phi->GetCompToRePlan(), complex_data[i], NULL);

	(Phi->GetBoundaryCondition())->SetHelicity(null);
	return(*this);
}



Operator & LaplacianSquared::OperateOn(Field *f)
{
	cout << "LaplacianSquared::OperateOn(Field *f) not implemented!" << endl;
	exit(-1);
	return(*this);
}

Operator & LaplacianSquared::OperateOn(SquareGridField *Phi)
{
	// To be implemented more efficiently
	Laplacian Delta;
	Phi->Apply(Delta);
	Delta[*Phi];
	return(*this);
}

DirDerivative::DirDerivative(void) { ; }
DirDerivative::DirDerivative(const Point & dir) : direction(dir) { ; }

Operator & DirDerivative::OperateOn(Field *f)
{
	cout << "DirDerivative::OperateOn(Field *f) not implemented!" << endl;
	exit(-1);
	return(*this);
}

Operator & DirDerivative::OperateOn(SquareGridField *Phi)
{
	int i, n, d, vv;
	Real xre, xim, yre, yim, vol, N2m;
	Index fftwcomplexbox(Phi->GetFFTWBox()), index, box=Phi->GetLogicalSize();
	const Point pp((Phi->GetBoundaryCondition())->GetPhysicalSize());
	const Point mm((Phi->GetBoundaryCondition())->GetHelicity());
	const Point null(0.0);
	fftwcomplexbox[DIMENSION-1]/=2;
	fftw_real **real_data=Phi->GetPtrToFFTWReal();
	fftw_complex **complex_data=Phi->GetPtrToFFTComplex();

	vol=(Real)box.Volume();

	#ifdef _DEBUG
		assert(real_data!=ZERO);
		assert(complex_data!=ZERO);
		assert(Phi->GetReToCompPlan()!=NULL);
		assert(Phi->GetCompToRePlan()!=NULL);
		for (i=0; i<Phi->GetNumberOfFields(); i++) {
			assert(real_data[i]!=ZERO);
			assert(complex_data[i]!=ZERO);
		}
	#endif

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		rfftwnd_one_real_to_complex(Phi->GetReToCompPlan(), real_data[i], NULL);
		if (mm!=null) {
			for (d=0, vv=1; d<DIMENSION; d++) vv*=box(d);
			for (d=0; d<DIMENSION; d++) {
				N2m=0.5*mm(d)*pp(d)*vv/box(d);
				index.Zero(); 
				//complex_data[i][index(fftwcomplexbox)].re-=N2m*(box(d)-1); // not needed for differentiation
				for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
					complex_data[i][index(fftwcomplexbox)].re+=N2m;
					complex_data[i][index(fftwcomplexbox)].im-=N2m/tan(PI*index(d)/box(d));
				}
			}
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		for (index.Zero(); index.IsInBox(fftwcomplexbox); index.NextInBox(fftwcomplexbox)) {
			xre=complex_data[i][index(fftwcomplexbox)].re; yre=0.0;
			xim=complex_data[i][index(fftwcomplexbox)].im; yim=0.0;
			for (n=0; n<DIMENSION; n++) {
				yre+=box(n)/pp(n)*(-direction(n)*xim*sin(2.0*PI*index(n)/box(n))
								+fabs(direction(n))*xre*(1.0-cos(2.0*PI*index(n)/box(n))));
				yim+=box(n)/pp(n)*(+direction(n)*xre*sin(2.0*PI*index(n)/box(n))
								+fabs(direction(n))*xim*(1.0-cos(2.0*PI*index(n)/box(n))));
			}
			complex_data[i][index(fftwcomplexbox)].re=yre/vol;
			complex_data[i][index(fftwcomplexbox)].im=yim/vol;
		}
	}

	index.Zero();
	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		//only needed for first derivatives:
		for (d=0; d<DIMENSION; d++) complex_data[i][index(fftwcomplexbox)].re+=mm(d)*direction(d); 
		rfftwnd_one_complex_to_real(Phi->GetCompToRePlan(), complex_data[i], NULL);
	}

	(Phi->GetBoundaryCondition())->SetHelicity(null);
	return(*this);
}


AverageTwo::AverageTwo(void) : direction(0), forwardbackward(1) { ; } 
AverageTwo::AverageTwo(int n) : direction(n), forwardbackward(1) { ; }

Operator & AverageTwo::OperateOn(Field *f)
{
	cout << "AverageTwo::OperateOn(Field *f) not implemented!" << endl;
	exit(-1);
	return(*this);
}

Operator & AverageTwo::OperateOn(SquareGridField *Phi)
{
	int i, d, vv;
	Real rePhi, imPhi, vol, reD, imD, N2m;
	Index fftwcomplexbox(Phi->GetFFTWBox()), index, box=Phi->GetLogicalSize();
	const Point pp((Phi->GetBoundaryCondition())->GetPhysicalSize());
	const Point mm((Phi->GetBoundaryCondition())->GetHelicity());
	fftwcomplexbox[DIMENSION-1]/=2;
	fftw_real **real_data=Phi->GetPtrToFFTWReal();
	fftw_complex **complex_data=Phi->GetPtrToFFTComplex();

	vol=(Real)box.Volume();

	#ifdef _DEBUG
		assert(0<=direction && direction<DIMENSION);
		assert(real_data!=ZERO);
		assert(complex_data!=ZERO);
		assert(Phi->GetReToCompPlan()!=NULL);
		assert(Phi->GetCompToRePlan()!=NULL);
		for (i=0; i<Phi->GetNumberOfFields(); i++) {
			assert(real_data[i]!=ZERO);
			assert(complex_data[i]!=ZERO);
		}
	#endif

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		rfftwnd_one_real_to_complex(Phi->GetReToCompPlan(), real_data[i], NULL);
		for (d=0, vv=1; d<DIMENSION; d++) vv*=box(d);
		d=direction;
		if (mm(d)!=0.0) {
			N2m=0.5*mm(d)*pp(d)*vv/box(d);
			index.Zero(); 
			for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
				complex_data[i][index(fftwcomplexbox)].re+=N2m;
				complex_data[i][index(fftwcomplexbox)].im-=N2m/tan(PI*index(d)/box(d));
			}
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) { 
		for (index.Zero(); index.IsInBox(fftwcomplexbox); index.NextInBox(fftwcomplexbox)) {
			rePhi=complex_data[i][index(fftwcomplexbox)].re;
			imPhi=complex_data[i][index(fftwcomplexbox)].im;
			reD=                (cos(2.0*PI*index(direction)/box(direction))+1);
			imD=forwardbackward*(sin(2.0*PI*index(direction)/box(direction)));
			complex_data[i][index(fftwcomplexbox)].re=0.5*(rePhi*reD-imPhi*imD)/vol;
			complex_data[i][index(fftwcomplexbox)].im=0.5*(rePhi*imD+imPhi*reD)/vol;
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		d=direction;
		if (mm(d)!=0.0) {
			N2m=0.5*mm(d)*pp(d)/box(d);
			index.Zero(); 
			complex_data[i][index(fftwcomplexbox)].re+=forwardbackward*mm(d)*pp(d)/box(d)/2.0; 
			for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
				complex_data[i][index(fftwcomplexbox)].re-=N2m;
				complex_data[i][index(fftwcomplexbox)].im+=N2m/tan(PI*index(d)/box(d));
			}
		}
		rfftwnd_one_complex_to_real(Phi->GetCompToRePlan(), complex_data[i], NULL);
	}
	return(*this);
}

AverageFour::AverageFour(void) : forwardbackward(1) { ; } 
Operator & AverageFour::OperateOn(Field *f)
{
	cout << "AverageFour::OperateOn(Field *f) not implemented!" << endl;
	exit(-1);
	return(*this);
}

Operator & AverageFour::OperateOn(SquareGridField *Phi)
{
	int i, d;
	Real rePhi, imPhi, vol, reDx, imDx, reDy, imDy, reD, imD, N2m;
	Index fftwcomplexbox(Phi->GetFFTWBox()), index, box=Phi->GetLogicalSize();
	const Point pp((Phi->GetBoundaryCondition())->GetPhysicalSize());
	const Point mm((Phi->GetBoundaryCondition())->GetHelicity());
	const Point null(0.0);
	fftwcomplexbox[DIMENSION-1]/=2;
	fftw_real **real_data=Phi->GetPtrToFFTWReal();
	fftw_complex **complex_data=Phi->GetPtrToFFTComplex();

	vol=(Real)box.Volume();

	#ifdef _DEBUG
		assert(real_data!=ZERO);
		assert(complex_data!=ZERO);
		assert(Phi->GetReToCompPlan()!=NULL);
		assert(Phi->GetCompToRePlan()!=NULL);
		for (i=0; i<Phi->GetNumberOfFields(); i++) {
			assert(real_data[i]!=ZERO);
			assert(complex_data[i]!=ZERO);
		}
	#endif

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		rfftwnd_one_real_to_complex(Phi->GetReToCompPlan(), real_data[i], NULL);
		if (mm!=null) {
			for (d=0; d<DIMENSION; d++) {
				N2m=0.5*mm(d)*pp(d)*box(d);
				index.Zero(); 
				for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
					complex_data[i][index(fftwcomplexbox)].re+=N2m;
					complex_data[i][index(fftwcomplexbox)].im-=N2m/tan(PI*index(d)/box(d));
				}
			}
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) { 
		for (index.Zero(); index.IsInBox(fftwcomplexbox); index.NextInBox(fftwcomplexbox)) {
			rePhi=complex_data[i][index(fftwcomplexbox)].re;
			imPhi=complex_data[i][index(fftwcomplexbox)].im;
			reDx=                (cos(2.0*PI*index(0)/box(0))+1);
			imDx=forwardbackward*(sin(2.0*PI*index(0)/box(0)));
			reDy=                (cos(2.0*PI*index(1)/box(1))+1);
			imDy=forwardbackward*(sin(2.0*PI*index(1)/box(1)));
			reD=reDx*reDy-imDx*imDy;
			imD=reDy*imDx+reDx*imDy;
			complex_data[i][index(fftwcomplexbox)].re=0.25*(rePhi*reD-imPhi*imD)/vol;
			complex_data[i][index(fftwcomplexbox)].im=0.25*(rePhi*imD+imPhi*reD)/vol;
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		if (mm!=null) {
			for (d=0; d<DIMENSION; d++) {
				N2m=0.5*mm(d)*pp(d)/box(d);
				index.Zero(); 
				complex_data[i][index(fftwcomplexbox)].re+=forwardbackward*mm(d)*pp(d)/box(d)/2.0; 
				for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
					complex_data[i][index(fftwcomplexbox)].re-=N2m;
					complex_data[i][index(fftwcomplexbox)].im+=N2m/tan(PI*index(d)/box(d));
				}
			}
		}
		rfftwnd_one_complex_to_real(Phi->GetCompToRePlan(), complex_data[i], NULL);
	}
	return(*this);
}


PartialDerivative::PartialDerivative(void) { ; }
PartialDerivative::PartialDerivative(int n) : direction(n) { ; }

Operator & PartialDerivative::OperateOn(Field *f)
{
	cout << "PartialDerivative::OperateOn(Field *f) not implemented!" << endl;
	exit(-1);
	return(*this);
}


Operator & PartialDerivative::OperateOn(SquareGridField *Phi)
{
	int i, d, vv;
	Real rePhi, imPhi, imD, vol, inv_dX, N2m;
	Index fftwcomplexbox(Phi->GetFFTWBox()), index, box=Phi->GetLogicalSize();
	const Point pp((Phi->GetBoundaryCondition())->GetPhysicalSize());
	const Point mm((Phi->GetBoundaryCondition())->GetHelicity());
	const Point null(0.0);
	fftwcomplexbox[DIMENSION-1]/=2;
	fftw_real **real_data=Phi->GetPtrToFFTWReal();
	fftw_complex **complex_data=Phi->GetPtrToFFTComplex();

	vol=(Real)box.Volume();

	#ifdef _DEBUG
		assert(0<=direction && direction<DIMENSION);
		assert(real_data!=ZERO);
		assert(complex_data!=ZERO);
		assert(Phi->GetReToCompPlan()!=NULL);
		assert(Phi->GetCompToRePlan()!=NULL);
		for (i=0; i<Phi->GetNumberOfFields(); i++) {
			assert(real_data[i]!=ZERO);
			assert(complex_data[i]!=ZERO);
		}
	#endif

	inv_dX=box(direction)/pp(direction);

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		rfftwnd_one_real_to_complex(Phi->GetReToCompPlan(), real_data[i], NULL);
		for (d=0, vv=1; d<DIMENSION; d++) vv*=box(d);
		d=direction;
		if (mm(d)!=0.0) {
			N2m=0.5*mm(d)*pp(d)*vv/box(d);
			index.Zero(); 
			for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
				complex_data[i][index(fftwcomplexbox)].re+=N2m;
				complex_data[i][index(fftwcomplexbox)].im-=N2m/tan(PI*index(d)/box(d));
			}
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) { 
		for (index.Zero(); index.IsInBox(fftwcomplexbox); index.NextInBox(fftwcomplexbox)) {
			rePhi=complex_data[i][index(fftwcomplexbox)].re;
			imPhi=complex_data[i][index(fftwcomplexbox)].im;
			imD=inv_dX*sin(2.0*PI*index(direction)/box(direction));
			//reD=0.0;
			complex_data[i][index(fftwcomplexbox)].re=-imPhi*imD/vol;
			complex_data[i][index(fftwcomplexbox)].im= rePhi*imD/vol;
		}
	}

	index.Zero();
	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		//only needed for first derivatives:
		complex_data[i][index(fftwcomplexbox)].re+=mm(direction); 
		rfftwnd_one_complex_to_real(Phi->GetCompToRePlan(), complex_data[i], NULL);
	}

	(Phi->GetBoundaryCondition())->SetHelicity(null);
	return(*this);
}

AsymmetricDerivative::AsymmetricDerivative(void) : direction(0), forwardbackward(1) { ; } 
AsymmetricDerivative::AsymmetricDerivative(int n) : direction(n), forwardbackward(1) { ; }

Operator & AsymmetricDerivative::OperateOn(Field *f)
{
	cout << "AsymmetricDerivative::OperateOn(Field *f) not implemented!" << endl;
	exit(-1);
	return(*this);
}


Operator & AsymmetricDerivative::OperateOn(SquareGridField *Phi)
{
	int i, d, vv;
	Real rePhi, imPhi, vol, reD, imD, inv_dX, N2m;
	Index fftwcomplexbox(Phi->GetFFTWBox()), index, box=Phi->GetLogicalSize();
	const Point pp((Phi->GetBoundaryCondition())->GetPhysicalSize());
	const Point mm((Phi->GetBoundaryCondition())->GetHelicity());
	const Point null(0.0);
	fftwcomplexbox[DIMENSION-1]/=2;
	fftw_real **real_data=Phi->GetPtrToFFTWReal();
	fftw_complex **complex_data=Phi->GetPtrToFFTComplex();

	vol=(Real)box.Volume();

	#ifdef _DEBUG
		assert(0<=direction && direction<DIMENSION);
		assert(real_data!=ZERO);
		assert(complex_data!=ZERO);
		assert(Phi->GetReToCompPlan()!=NULL);
		assert(Phi->GetCompToRePlan()!=NULL);
		for (i=0; i<Phi->GetNumberOfFields(); i++) {
			assert(real_data[i]!=ZERO);
			assert(complex_data[i]!=ZERO);
		}
	#endif

	inv_dX=box(direction)/pp(direction);
	
	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		rfftwnd_one_real_to_complex(Phi->GetReToCompPlan(), real_data[i], NULL);
		for (d=0, vv=1; d<DIMENSION; d++) vv*=box(d);
		d=direction;
		if (mm(d)!=0.0) {
			N2m=0.5*mm(d)*pp(d)*vv/box(d);
			index.Zero(); 
			for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
				complex_data[i][index(fftwcomplexbox)].re+=N2m;
				complex_data[i][index(fftwcomplexbox)].im-=N2m/tan(PI*index(d)/box(d));
			}
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) { 
		for (index.Zero(); index.IsInBox(fftwcomplexbox); index.NextInBox(fftwcomplexbox)) {
			rePhi=complex_data[i][index(fftwcomplexbox)].re;
			imPhi=complex_data[i][index(fftwcomplexbox)].im;
			reD=forwardbackward*(cos(2.0*PI*index(direction)/box(direction))-1);
			imD=                 sin(2.0*PI*index(direction)/box(direction));
			complex_data[i][index(fftwcomplexbox)].re=inv_dX*(rePhi*reD-imPhi*imD)/vol;
			complex_data[i][index(fftwcomplexbox)].im=inv_dX*(rePhi*imD+imPhi*reD)/vol;
		}
	}

	index.Zero();
	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		//only needed for first derivatives:
		complex_data[i][index(fftwcomplexbox)].re+=mm(direction); 
		rfftwnd_one_complex_to_real(Phi->GetCompToRePlan(), complex_data[i], NULL);
	}

	(Phi->GetBoundaryCondition())->SetHelicity(null);
	return(*this);
}

DiagonalPropagator::DiagonalPropagator(void) : time(0.0) { ; } 
DiagonalPropagator::DiagonalPropagator(Real t) : time(t) { ; }
DiagonalPropagator::DiagonalPropagator(const DiagonalPropagator & prop) : time(prop.GetTime()) { ; }

DiagonalPropagator::~DiagonalPropagator(void) { ; }

Operator & DiagonalPropagator::OperateOn(Field *f)
{
	cout << "DiagonalPropagator::OperateOn(Field *f) not implemented!" << endl;
	exit(-1);
	return(*this);
}


Operator & DiagonalPropagator::OperateOn(SquareGridField *Phi)
{
	int i, j, d, vv;
	Real rePhi, imPhi, vol, reD, imD, reExp, imExp, N2m;
	Index fftwcomplexbox(Phi->GetFFTWBox()), index, box=Phi->GetLogicalSize();
	const Point pp((Phi->GetBoundaryCondition())->GetPhysicalSize());
	const Point mm((Phi->GetBoundaryCondition())->GetHelicity());
	const Point null(0.0);
	Point kappa;
	fftwcomplexbox[DIMENSION-1]/=2;
	fftw_real **real_data=Phi->GetPtrToFFTWReal();
	fftw_complex **complex_data=Phi->GetPtrToFFTComplex();

	vol=(Real)box.Volume();
	for (i=0; i<DIMENSION; i++) delta[i]=pp(i)/(Real)box(i);

	#ifdef _DEBUG
		assert(real_data!=ZERO);
		assert(complex_data!=ZERO);
		assert(Phi->GetReToCompPlan()!=NULL);
		assert(Phi->GetCompToRePlan()!=NULL);
		for (i=0; i<Phi->GetNumberOfFields(); i++) {
			assert(real_data[i]!=ZERO);
			assert(complex_data[i]!=ZERO);
		}
	#endif
	
	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		rfftwnd_one_real_to_complex(Phi->GetReToCompPlan(), real_data[i], NULL);
		if (mm!=null) {
			for (d=0, vv=1; d<DIMENSION; d++) vv*=box(d);
			for (d=0; d<DIMENSION; d++) {
				N2m=0.5*mm(d)*pp(d)*vv/box(d);
				index.Zero(); 
				//complex_data[i][index(fftwcomplexbox)].re-=N2m*(box(d)-1); // not needed for differentiation
				for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
					complex_data[i][index(fftwcomplexbox)].re+=N2m;
					complex_data[i][index(fftwcomplexbox)].im-=N2m/tan(PI*index(d)/box(d));
				}
			}
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) { 
		for (index.Zero(); index.IsInBox(fftwcomplexbox); index.NextInBox(fftwcomplexbox)) {
			rePhi=complex_data[i][index(fftwcomplexbox)].re;
			imPhi=complex_data[i][index(fftwcomplexbox)].im;
			for (j=0; j<DIMENSION; j++) kappa[j]=2.0*PI*((Real)index(j))/((Real)box(j));
			reD=time*ReDiagonal(kappa);
			imD=time*ImDiagonal(kappa);
			reExp=exp(reD)*cos(imD);
			imExp=exp(reD)*sin(imD);
			complex_data[i][index(fftwcomplexbox)].re=(reExp*rePhi-imExp*imPhi)/vol;
			complex_data[i][index(fftwcomplexbox)].im=(reExp*imPhi-imExp*rePhi)/vol;
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		if (mm!=null) {
			for (d=0; d<DIMENSION; d++) {
				N2m=0.5*mm(d)*pp(d)/box(d);
				index.Zero(); 
				//complex_data[i][index(fftwcomplexbox)].re+=forwardbackward*mm(d)*pp(d)/box(d)/2.0; 
				for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
					complex_data[i][index(fftwcomplexbox)].re-=N2m;
					complex_data[i][index(fftwcomplexbox)].im+=N2m/tan(PI*index(d)/box(d));
				}
			}
		}
		rfftwnd_one_complex_to_real(Phi->GetCompToRePlan(), complex_data[i], NULL);
	}
	return(*this);
}

ImplicitPropagator::ImplicitPropagator(void) : time(0.0) { ; } 
ImplicitPropagator::ImplicitPropagator(Real t) : time(t) { ; }
ImplicitPropagator::ImplicitPropagator(const ImplicitPropagator & prop) : time(prop.GetTime()) { ; }

ImplicitPropagator::~ImplicitPropagator(void) { ; }

Operator & ImplicitPropagator::OperateOn(Field *f)
{
	cout << "ImplicitPropagator::OperateOn(Field *f) not implemented!" << endl;
	exit(-1);
	return(*this);
}


Operator & ImplicitPropagator::OperateOn(SquareGridField *Phi)
{
	int i, j, d, vv;
	Real rePhi, imPhi, vol, reD, imD, absD, N2m;
	Index fftwcomplexbox(Phi->GetFFTWBox()), index, box=Phi->GetLogicalSize();
	const Point pp((Phi->GetBoundaryCondition())->GetPhysicalSize());
	const Point mm((Phi->GetBoundaryCondition())->GetHelicity());
	const Point null(0.0);
	Point kappa;
	fftwcomplexbox[DIMENSION-1]/=2;
	fftw_real **real_data=Phi->GetPtrToFFTWReal();
	fftw_complex **complex_data=Phi->GetPtrToFFTComplex();

	vol=(Real)box.Volume();
	for (i=0; i<DIMENSION; i++) delta[i]=pp(i)/(Real)box(i);

	#ifdef _DEBUG
		assert(real_data!=ZERO);
		assert(complex_data!=ZERO);
		assert(Phi->GetReToCompPlan()!=NULL);
		assert(Phi->GetCompToRePlan()!=NULL);
		for (i=0; i<Phi->GetNumberOfFields(); i++) {
			assert(real_data[i]!=ZERO);
			assert(complex_data[i]!=ZERO);
		}
	#endif
	
	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		rfftwnd_one_real_to_complex(Phi->GetReToCompPlan(), real_data[i], NULL);
		if (mm!=null) {
			for (d=0, vv=1; d<DIMENSION; d++) vv*=box(d);
			for (d=0; d<DIMENSION; d++) {
				N2m=0.5*mm(d)*pp(d)*vv/box(d);
				index.Zero(); 
				for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
					complex_data[i][index(fftwcomplexbox)].re+=N2m;
					complex_data[i][index(fftwcomplexbox)].im-=N2m/tan(PI*index(d)/box(d));
				}
			}
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) { 
		for (index.Zero(); index.IsInBox(fftwcomplexbox); index.NextInBox(fftwcomplexbox)) {
			rePhi=complex_data[i][index(fftwcomplexbox)].re;
			imPhi=complex_data[i][index(fftwcomplexbox)].im;
			for (j=0; j<DIMENSION; j++) kappa[j]=2.0*PI*((Real)index(j))/((Real)box(j));
			reD=1.0-time*ReDiagonal(kappa);
			imD=   -time*ImDiagonal(kappa);
			absD=reD*reD+imD*imD;
			reD= reD/absD;
			imD=-imD/absD;
			complex_data[i][index(fftwcomplexbox)].re=(reD*rePhi-imD*imPhi)/vol;
			complex_data[i][index(fftwcomplexbox)].im=(reD*imPhi-imD*rePhi)/vol;
		}
	}

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		if (mm!=null) {
			for (d=0; d<DIMENSION; d++) {
				N2m=0.5*mm(d)*pp(d)/box(d);
				index.Zero(); 
				//complex_data[i][index(fftwcomplexbox)].re+=forwardbackward*mm(d)*pp(d)/box(d)/2.0; 
				for (index[d]=1; index[d]<fftwcomplexbox(d); index[d]++) {
					complex_data[i][index(fftwcomplexbox)].re-=N2m;
					complex_data[i][index(fftwcomplexbox)].im+=N2m/tan(PI*index(d)/box(d));
				}
			}
		}
		rfftwnd_one_complex_to_real(Phi->GetCompToRePlan(), complex_data[i], NULL);
	}
	return(*this);
}

AddReal::AddReal(void) : addend(0.0) { ; }
AddReal::AddReal(Real ad) : addend(ad) { ; }

Operator & AddReal::OperateOn(Field *Phi)
{
	int i;
	Real y1;

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		for (Phi->GotoFirstPoint(); !Phi->IsLastPoint(); Phi->GotoNextPoint()) {
			y1=Phi->GetFieldValue(i);
			y1+=addend;
			Phi->SetFieldValue(y1, i);
			// Check for exeptions?
		}
	}
	return(*this);
}

Operator & AddReal::OperateOn(SquareGridField *Phi)
{
	int i;
	size_t n;
	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		for (n=0; n<Phi->GetNumberOfDataPoints(); n++) {
			Phi->GetField(n, i)+=addend;
		}
	}
	return(*this);
}


MultiplyReal::MultiplyReal(void) : factor(1.0) { ; }
MultiplyReal::MultiplyReal(Real fac) : factor(fac) { ; }

Operator & MultiplyReal::OperateOn(Field *Phi) 
{
	int i;
	Real *y1=new(nothrow) Real[Phi->GetNumberOfFields()];
	#ifdef _DEBUG
		assert(y1);
	#endif
	for (Phi->GotoFirstPoint(); !Phi->IsLastPoint(); Phi->GotoNextPoint()) {
		Phi->GetFieldValue(y1);
		for (i=0; i<Phi->GetNumberOfFields(); i++) y1[i]*=factor;
		Phi->SetFieldValue(y1);
	}
	delete []y1;
	return(*this);
}

Operator & MultiplyReal::OperateOn(SquareGridField *Phi)
{
	int i;
	size_t n;
	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		for (n=0; n<Phi->GetNumberOfDataPoints(); n++) {
			Phi->GetField(n, i)*=factor;
		}
	}
	return(*this);
}

ApplyFunction::ApplyFunction(void) { ; }

Operator & ApplyFunction::OperateOn(Field *Phi)
{
	int i;
	Real y;

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		for (Phi->GotoFirstPoint(); !Phi->IsLastPoint(); Phi->GotoNextPoint()) {
			y=Phi->GetFieldValue(i);
			Phi->SetFieldValue(function(y), i);
		}
	}
	return(*this);
}

Operator & ApplyFunction::OperateOn(SquareGridField *Phi)
{
	int i;
	size_t n;
	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		for (n=0; n<Phi->GetNumberOfDataPoints(); n++) {
			Phi->GetField(n, i)=function(Phi->GetFieldValue(n, i));
		}
	}
	return(*this);
}


PhaseFieldPotential::PhaseFieldPotential(void) { ; }
PhaseFieldPotential::PhaseFieldPotential(Real s) : strength(s) { ; }

Operator & PhaseFieldPotential::OperateOn(Field *Phi)
{
	int i;
	Real x, y;

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		for (Phi->GotoFirstPoint(); !Phi->IsLastPoint(); Phi->GotoNextPoint()) {
			y=Phi->GetFieldValue(i);
			x=y-1.5;
			x=y*x+0.5;
			x*=y*strength;
			Phi->SetFieldValue(x, i);
		}
	}
	return(*this);
}

Operator & PhaseFieldPotential::OperateOn(SquareGridField *Phi)
{
	int i;
	size_t n;
	Real x, y;

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		for (n=0; n<Phi->GetNumberOfDataPoints(); n++) {
			y=Phi->GetFieldValue(n, i);
			x=y-1.5;
			x=y*x+0.5;
			x*=y*strength;
			Phi->GetField(n, i)=x;
		}
	}
	return(*this);
}

MultiplyFunction::MultiplyFunction(void) { ; }

Operator & MultiplyFunction::OperateOn(Field *Phi) 
{
	int i;
	Real y;
	Point x;

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		for (Phi->GotoFirstPoint(); !Phi->IsLastPoint(); Phi->GotoNextPoint()) {
			y=Phi->GetFieldValue(i);
			x=Phi->GetPointCoordinates();
			Phi->SetFieldValue(y*function(x), i);
		}
	}
	return(*this);
}

Operator & MultiplyFunction::OperateOn(SquareGridField *Phi)
{
	// To be implemented more efficiently
	return(OperateOn((Field *)Phi));
}

AddFunction::AddFunction(void) { ; }

Operator & AddFunction::OperateOn(Field *Phi) 
{
	int i;
	Real y;
	Point x;

	for (i=0; i<Phi->GetNumberOfFields(); i++) {
		for (Phi->GotoFirstPoint(); !Phi->IsLastPoint(); Phi->GotoNextPoint()) {
			y=Phi->GetFieldValue(i);
			x=Phi->GetPointCoordinates();
			Phi->SetFieldValue(y+function(x), i);
		}
	}
	return(*this);
}

Operator & AddFunction::OperateOn(SquareGridField *Phi)
{
	// To be implemented more efficiently
	return(OperateOn((Field *)Phi));
}

L2NormalizeField::L2NormalizeField(void) : Operator() { ; } 

Operator & L2NormalizeField::OperateOn(Field *Phi) 
{
	int i;
	Real norm, y;

	//cout << "L2NormalizeField::OperateOn(" << Phi << ") Phi->GetNumberOfFields()=" << Phi->GetNumberOfFields() << endl;
	for (Phi->GotoFirstPoint(); !Phi->IsLastPoint(); Phi->GotoNextPoint()) {
		norm=0;
		for (i=0; i<Phi->GetNumberOfFields(); i++) {
			y=Phi->GetFieldValue(i);
			norm+=y*y;
		}
		norm=sqrt(norm);
		if (norm==0.0) {
			Phi->SetFieldValue(1.0, 0);
			for (i=1; i<Phi->GetNumberOfFields(); i++) Phi->SetFieldValue(0.0, i);
		}
		else for (i=0; i<Phi->GetNumberOfFields(); i++) Phi->SetFieldValue(Phi->GetFieldValue(i)/norm, i);
	}
	return(*this);
}

Operator & L2NormalizeField::OperateOn(SquareGridField *Phi)
{
	int i;
	size_t n;
	Real norm, y;

	//cout << "L2NormalizeField::OperateOn(" << Phi << ") Phi->GetNumberOfFields()=" << Phi->GetNumberOfFields() << endl;
	for (n=0; n<Phi->GetNumberOfDataPoints(); n++) {
		norm=0.0;
		for (i=0; i<Phi->GetNumberOfFields(); i++) {
			y=Phi->GetFieldValue(n, i);
			norm+=y*y;
		}
		norm=sqrt(norm);
		if (norm==0.0) {
			Phi->GetField(n, 0)=1.0;
			for (i=1; i<Phi->GetNumberOfFields(); i++) Phi->GetField(n, i)=0.0;
		}
		else for (i=0; i<Phi->GetNumberOfFields(); i++) Phi->GetField(n, i)=Phi->GetFieldValue(n, i)/norm;
	}
	return(*this);
}
