#include "Face.h"
#include <assert.h>
#include <algorithm>

using namespace std ;

Face::Face(int n0, int n1, int n2)
{
    // make sure that nodes[0] < nodes[1] and nodes[0] < nodes[2], but
    // preserve the cyclic order of n0, n1 and n2:
    if (n0 < n1)
    {
        if (n0 < n2)
        {
            nodes[0] = n0; nodes[1] = n1; nodes[2] = n2;
        }
        else
        {
            nodes[0] = n2; nodes[1] = n0; nodes[2] = n1;
        }
    }
    else
        if (n1 < n2)
        {
            nodes[0] = n1; nodes[1] = n2; nodes[2] = n0;
        }
        else
        {
            nodes[0] = n2; nodes[1] = n0; nodes[2] = n1;
        }
}

bool Face::operator<(const Face &face) const
{
    for(int n = 0; n < 3; n++)
    {
        int d = face.nodes[n] - nodes[n];
        if(d > 0)
            return true;
        else if(d < 0)
            return false;
    }
    return false;
}

int Face::FindNode(int node) const
{
    int n = 0;
    for(; n < 3; n++)
	if(nodes[n] == node)
	    break;
    return n;
}

void Face::Flip()
{
    swap(nodes[1], nodes[2]);
}

ostream& operator<<(ostream& out, const Face& f)
{
    out << f.nodes[0] << ' '
	<< f.nodes[1] << ' '
	<< f.nodes[2];
    return out;
}

istream& operator>>(istream& in, Face& f)
{
    in >> f.nodes[0] >> f.nodes[1] >> f.nodes[2];
    return in;
}

