#ifndef __VECTOR_H__
#define __VECTOR_H__

#include <iostream>

template<class T>
class Vector
{
public:
    T x[3];
    Vector() {};
    Vector(T x0, T x1, T x2);
    Vector(T x0);
    Vector<T> Cross(const Vector<T>& v) const;
    T operator*(const Vector<T>& v) const;
    Vector<T> operator*(const T& s) const;
    Vector<T> operator+(const Vector<T>& v) const;
    Vector<T> operator-(const Vector<T>& v) const;
    bool operator==(const Vector<T>& v) const;
    T operator[](int n) const;
    T& operator[](int n);
    T Length2() const;
    friend std::ostream& operator<<(std::ostream& out, const Vector& v) {
        out << "(" << v[0] << ", " << v[1] << ", " << v[2] << ")" ; return out ;}
    friend std::istream& operator>>(std::istream& in, Vector& v) {
        in >> v[0] >> v[1] >> v[2]; return in ;}
};

template<class T> inline
Vector<T>::Vector(T x0, T x1, T x2)
{
    x[0] = x0;
    x[1] = x1;
    x[2] = x2;
}

template<class T> inline
Vector<T>::Vector(T x0)
{
    x[0] = x[1] = x[2] = x0;
}

template<class T> inline
T Vector<T>::operator*(const Vector& v) const
{
    return x[0] * v.x[0] + x[1] * v.x[1] + x[2] * v.x[2];
}

template<class T> inline
Vector<T> Vector<T>::operator*(const T& s) const
{
    return Vector(s * x[0], s * x[1], s * x[2]);
}

template<class T> inline
Vector<T> Vector<T>::operator+(const Vector& v) const
{
    return Vector(x[0] + v.x[0], x[1] + v.x[1], x[2] + v.x[2]);
}

template<class T> inline
Vector<T> Vector<T>::operator-(const Vector& v) const
{
    return Vector(x[0] - v.x[0], x[1] - v.x[1], x[2] - v.x[2]);
}

template<class T> inline
bool Vector<T>::operator==(const Vector& v) const
{
    return x[0] == v.x[0] && x[1] == v.x[1] && x[2] == v.x[2];
}

template<class T> inline
T Vector<T>::operator[](int n) const
{
    return x[n];
}

template<class T> inline
T& Vector<T>::operator[](int n)
{
    return x[n];
}

template<class T> inline
Vector<T> Vector<T>::Cross(const Vector& v) const
{
    return Vector(x[1] * v.x[2] - x[2] * v.x[1],
        x[2] * v.x[0] - x[0] * v.x[2],
        x[0] * v.x[1] - x[1] * v.x[0]);
}

template<class T> inline
T Vector<T>::Length2() const
{
    return x[0] * x[0] + x[1] * x[1] + x[2] * x[2];
}

#endif // __VECTOR_H__

