#!/usr/local/bin/python

from VTK import *
from Numeric import *
from Mesh3DDisplay import Mesh3DDisplay

def vtkMesh3D(Positions=None):
    """vtkMesh3D takes a set of points in positions, create Delaunay in convex region and 
    return vtkDelaunay3D"""
    points = vtkPoints()
    for i in range(len(Positions)):
        points.InsertPoint(i,Positions[i][0],Positions[i][1],Positions[i][2])

    polydata = vtkPolyData()
    polydata.SetPoints(points)
    # triangulate them
    #
    delny = vtkDelaunay3D()
    delny.SetInput(polydata)
    delny.SetTolerance(0.001)
    delny.BoundingTriangulationOff()
    delny.SetAlpha(0.0)
    delny.Update()
    return delny

def vtkQueryElements(Delaunay =  None):
    """return connectivity of elements using NumPy array"""
    pdata = vtkUnstructuredGrid()
    pdata = Delaunay.GetOutput()
    pyelems = zeros((pdata.GetNumberOfCells(),4))  # hack on 4 nodes tet
    ptIds =vtkIdList()
    for i in range(0,pdata.GetNumberOfCells()):
        pdata.GetCellPoints(i,ptIds)
        for j in range(0,ptIds.GetNumberOfIds()):
            assert (ptIds.GetNumberOfIds() == 4), "Number of nodes != 3"
            pyelems[i][j] = ptIds.GetId(j)
    return pyelems
        
    
if __name__ == '__main__':
    pynodes= array([[0,2,1],[1,2,1],[2,2,2],
                    [0,1,3],[1,1,3],[2,1,4],
                    [0,0,0],[1,0,2],[2,0,5]]) #3d
    delny = vtkMesh3D(Positions=pynodes)
# query the connectivity
    pyelems = vtkQueryElements(Delaunay=delny)
# display the meshes
    showmeshes = Mesh3DDisplay(Delaunay=delny)
    showmeshes.ShowNodes(BallSize = 0.1)
    showmeshes.ShowEdges(TubeSize = 0.05)
    showmeshes.Show()
# save image
#    showmeshes.saveTIFFImage("junk.tiff")
