#
# This is a clean up version of dis.py; we will use this to test new C++ stuff
#
# 01-August-2000
#  
import sys
sys.path.append('../WindowsBuild')

import QC
from VTKMesh2D import *
from PlotAtoms import *
from Numeric import *
from LinearAlgebra import *
from math import *
from MDFemStuff import *

########################
# STEP: Create Cluster #
########################
# potentialLJ = QC.CutLennardJonesPotential()
potential = QC.QCLJPotential()
latSpacing = potential.GetLengthScale()

lattice = QC.TriangularLattice(latSpacing)

lattice1 = QC.TriangularLattice(5*latSpacing) # used to create a coarser lattice of master atoms in boring regions


rectSize = lattice1.CartCoordFromLatticeCoord([8.000001, 8.00001]) #[160.001*latSpacing, 160.001*latSpacing]
rect = QC.RectangularClusterInitializer(rectSize, lattice1)

masters = QC.ListOfMasterAtoms()

atomsTMP = QC.ListOfMasterAtoms()
rect.Create(masters)

# this is the region where we'll introduce the dislocation
smallRectSize = [10.,10.]
rect.SetRectangleSize(smallRectSize)
rect.SetBravaisLattice(lattice)
rect.Create(atomsTMP)

masters.Merge(atomsTMP)
del atomsTMP

#set BravaisLattice for all the master atoms
for i in xrange(masters.GetNumber()):
    masters.SetBravaisLattice(i, lattice)

# this is to remove duplicates
overlapPruner = QC.OverlapPruner(0.5*latSpacing)
overlapPruner.Transform(masters)

# let's find the index of atoms on the external boundary. We imagine that the overlapPruner won't change them
mastersPos = masters.GetCartesianPositions()
bList = []

for i in xrange(len(mastersPos)):
    if (abs(abs(mastersPos[i][0])*2-rectSize[0]) < 0.01) or \
        (abs(abs(mastersPos[i][1])*2-rectSize[1]) < 0.01) :
        bList.append(i)
tmp = argsort(map(lambda i: atan2(mastersPos[i][1],mastersPos[i][0]), bList))
edgeConnectivity = []
for i in range(len(tmp)-1): edgeConnectivity.append([bList[tmp[i]], bList[tmp[i+1]]])
edgeConnectivity.append([bList[tmp[len(tmp)-1]], bList[tmp[0]]])

# make a single dislocation
disloc = QC.DislocationMaker([latSpacing, 0.], [0.,0.], [0.5*latSpacing,latSpacing*sqrt(3)/6.],0.0)
disloc.Transform(masters)
overlapPruner.Transform(masters)

mastersPos = masters.GetCartesianPositions()
iList = []

for i in range(len(mastersPos)):
    if i not in bList:
        iList.append(i)
        
##plot = PlotAtoms(masters.GetCartesianPositions())
##plot.display(phiResolution=5, thetaResolution=5)
##del plot
#tmpArr = array((len(bList), 2), Float)
#mastersPos = masters.GetCartesianPositions()
boundaryList = map(lambda i: [mastersPos[bList[i]][0], mastersPos[bList[i]][1]], range(len(bList)))
internalList = map(lambda i: [mastersPos[iList[i]][0], mastersPos[iList[i]][1]], range(len(iList)))
#plot = PlotAtoms(boundaryList)
#plot.display(phiResolution=5, thetaResolution=5)
#del plot

#################
# STEP: MESH    #
#################
delny=vtkMesh2D(Positions = mastersPos, Polygons = edgeConnectivity)
# query the connectivity
pyelems = vtkQueryElements(Delaunay=delny)

vtkMeshData = QC.VTKMeshData(len(pyelems)) 
for i in xrange(len(pyelems)):
    vtkMeshData.SetElementConnectivity(i, pyelems[i])

masters.SetReferencePositions(masters.GetCartesianPositionsPtr())

vtkMeshData.CreateElements(masters)

print "Done with Meshing"

################################################################################
# STEP: INTERPOLATE (we now use master atoms and interpolation field from fem  #
# to construct slaved atoms)                                                   #
################################################################################
rd = 1.2 * potential.GetCutoffDistance()

mastersNL = QC.CellNeighborLocator(rd, 0.05*rd,masters)
mastersNL.UpdatePosition()

slaves = QC.ListOfSlavedAtoms(masters)

overlapPruner.SetMinSeparation(0.4*latSpacing)
print masters.GetNumber(), "nodes , ",len(pyelems) , " elements"

vtkMeshData.InitializeSlavedAtomsFromMasters(rd, masters, slaves)

print slaves.GetNumber(), " in the slaves before pruning"
overlapPruner.SetMinSeparation(0.0001*latSpacing) # this is to remove duplicated slaves
overlapPruner.SetCellSize(latSpacing)
overlapPruner.Transform(slaves)
print slaves.GetNumber(), " in the slaves after pruning1"

# we also have to remove the slave atoms that lie on masters
pruner2 = QC.OverlapPruner(0.0001*latSpacing, masters.GetCartesianPositionsPtr())
pruner2.SetCellSize(latSpacing)
pruner2.Transform(slaves)
print slaves.GetNumber(), " in the slaves after pruning2"

# compute master weight

mastersWeight = zeros(len(mastersPos), Float)
mastersWeight = CalculateNodesWeightFromSlaves(masters, slaves)
for i in xrange(len(mastersWeight)):
    masters.SetWeight(i, mastersWeight[i])
#
atoms = QC.PrimitiveListOfAtoms()
atoms.Attach(masters)
masters.thisown = 0
atoms.Attach(slaves)
slaves.thisown = 0


print "Done with Interpolation"


# display the meshes and atoms
##showmeshes = Mesh2DDisplay(Delaunay=delny, AtomData = slaves.GetCartesianPositions())
##showmeshes.ShowNodes(BallSize = 0.4)
##showmeshes.ShowEdges(TubeSize = 0.05)
##showmeshes.ShowAtoms(radius=0.4)
##showmeshes.Show()
##showmeshes.saveImage('temp.ppm')
##showmeshes.saveTIFFImage('temp.tif')
##
#plot = PlotAtoms(slaves.GetCartesianPositions())
#plot.display(radius=0.6)

############################
# STEP: MOVER (MINIMIZER)  #
############################
# a QC mover would know the masters.
# each time the Move command is called, it would decorate, compute forces on the masters,
# move the masters accordingly, interpolate the slaves
neighborLoc = QC.CellNeighborLocator(potential.GetCutoffDistance(), 0.05*potential.GetCutoffDistance())

#noNL = QC.NoNeighborLocator()   # cannot be used now. But will be needed for efficiency
#slaves.SetNeighborLocator(noNL)
tmpNL = QC.CellNeighborLocator(0., potential.GetCutoffDistance())
slaves.SetNeighborLocator(tmpNL)
masters.SetNeighborLocator(neighborLoc);
nl = QC.CompositeNeighborList(potential.GetCutoffDistance(), 0.05*potential.GetCutoffDistance())
#nl = QC.CellNeighborLocator(potential.GetCutoffDistance(), 0.05*potential.GetCutoffDistance())
#nl = QC.SimpleCompositeNeighborLocator(potential.GetCutoffDistance())
atoms.SetNeighborLocator(nl)

print "#1 : " ,potential.CalculateEnergy(atoms)
Etot = 0.
for i in range(masters.GetNumber()):
    Etot = Etot + mastersWeight[i] * potential.CalculateAtomicEnergy(atoms, i)
print Etot

#stop
#forces = atoms.GetCartesianPositions().StructCopy()
#forces.Copy(atoms.GetCartesianPositions())
#potential.CalculateForces(atoms, forces)
#mover = QC.QuickMinAtomsMover(potential, 0.001, 0.000001)
#mover.SetMaxSteps(1000)
#mover.Move(atoms)

print "Starting Mover"

#mover = QC.ConjugateGradientAtomsMover(potential)
#mover.SetTolerance(1.e-16)
#mover.Verbose()
#mover.Move(atoms)

##qminMover = QC.QuickMinAtomsMover(potential,0.001,1.e-5)
##qminMover.SetMaxSteps(200)
##qminMover.Move(atoms)
##while qminMover.GetFinalTFS() > 0.1:
##    qminMover.Move(atoms)
##    print qminMover.GetFinalTFS()
##
##print "Done with Mover"
##print "Potential Energy = ", potential.CalculateEnergy(atoms)

#stop
#forces = atoms.GetCartesianPositions().StructCopy()
#forces.Copy(atoms.GetCartesianPositions())
#potential.CalculateForces(atoms, forces)
#mover = QC.QuickMinAtomsMover(potential, 0.001, 0.000001)
#mover.SetMaxSteps(1000)
#mover.Move(atoms)

#mover = QC.VerletAtomsMover(potential, 0.00001)

############################
# STEP: ERROR ESTIMATION   #
############################

mastersNew = atoms.GetSubList(0)
mastersNewPos = mastersNew.GetCartesianPositions()

defGradOfElements = zeros((len(pyelems),4), Float)
smallStrainOfElements = zeros((len(pyelems),3), Float)
lagragianStrainOfElements = zeros((len(pyelems),3), Float)

for elemIndex in xrange(len(pyelems)):
    [defGradOfElements[elemIndex], smallStrainOfElements[elemIndex], lagragianStrainOfElements[elemIndex]]  \
                    = ComputeGradientOfElement(elemIndex, lattice, pyelems, mastersPos[:], mastersNewPos[:])

smallStrainIndex = computeSecondInvariantOfStrainsOfElements(smallStrainOfElements)
#showSmallStrain = Mesh2DDisplay(Delaunay=delny, CellColorMap = smallStrainIndex)
#showSmallStrain.Show()

lagragianStrainIndex = computeSecondInvariantOfStrainsOfElements(lagragianStrainOfElements)
showLagragianStrain = Mesh2DDisplay(Delaunay=delny, AtomData = mastersNewPos, \
                                    CellColorMap = lagragianStrainIndex, ColorMapRange = [0,0.01])
showLagragianStrain.ShowEdges()
showLagragianStrain.ShowAtoms(radius=0.2)
showLagragianStrain.Show()
showLagragianStrain.saveTIFFImage('temp.tif')

############################
# STEP: ADAPTIVE MESH      #
############################
#refineRatioOfElements = ComputeRefineRatioOfElements(errorNormOfElements, dispNormOfElements)
prescNum = 0.01
#need to fix PolyClusterInitializer
##newMasters = createNewNodesForRemesh(lattice, pyelems, mastersPos[:], lagragianStrainIndex, prescNum)
##
##masters.Merge(newMasters)
##plot = PlotAtoms(masters.GetCartesianPositions())
##plot.display()


