// ListOfSlavedAtoms.cpp: implementation of the ListOfSlavedAtoms class.
//
//////////////////////////////////////////////////////////////////////
#include "DMFirstHeader.h"
#include <iostream>
#include "DigitalMaterial.h"
#include "ListOfSlavedAtoms.h"
#include "QCListOfAtoms.h"
#include "QCSlaveConstraint.h"
#include "DMLastHeader.h"
using namespace std ;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
/////////////////////////////////////////////////////////////////////


ListOfSlavedAtoms::ListOfSlavedAtoms(QCListOfAtoms *masters) :
    elementIndices(&positions), latticeIndicesInsideElement(&positions)
{
    Allocate(0);
    this->masters = masters;
    SetConstraint(new QCSlaveConstraint(this, masters));
}

ListOfSlavedAtoms::~ListOfSlavedAtoms()
{
    if(upperNode) upperNode->RemoveBranch(this); // it appears that cutting the relation with the parent is the first thing to do (do not wait for ~PrimitiveListOfAtoms)
    ASSERT(!upperNode);
    Clear();
}

void ListOfSlavedAtoms::Save(DMWriter *writer)
{
    ListOfAtoms::Save(writer);
    cerr << "Her." << endl;
    writer->Put(&elementIndices, "elementIndices");
    cerr << "Her." << endl;
    writer->Put(&latticeIndicesInsideElement, "indicesInsideElement");
    cerr << "Her." << endl;
}

void ListOfSlavedAtoms::Load(DMReader *reader)
{
    DMArray<int>* tmp1 =
        dynamic_cast<DMArray<int> *>(reader->Get("elementIndices"));
    if(!tmp1)
        throw DMException("QCListOfAtoms", "Load",
                          "Cannot read elementIndices");
    elementIndices.SetShape(tmp1->GetShape()); // we know that positions have a Shape(0), so we can reshape 
    elementIndices.Copy(tmp1);
    delete tmp1;
    DMArray<int, 3>* tmp2 =
        dynamic_cast<DMArray<int, 3> *>(reader->Get("indicesInsideElement"));
    if(!tmp2)
        throw DMException("QCListOfAtoms", "Load",
                          "Cannot read indicesInsideElement");
    latticeIndicesInsideElement.SetShape(tmp2->GetShape()); // we know that positions have a Shape(0), so we can reshape 
    latticeIndicesInsideElement.Copy(tmp2);
    delete tmp2;
}

void ListOfSlavedAtoms::Clear()
{
    ListOfAtoms::Clear();
    elementIndices.SetShape(0);
    latticeIndicesInsideElement.SetShape(0);
}

void ListOfSlavedAtoms::RemoveSubList(ListOfAtoms *atomsToRemove)
{
    ListOfSlavedAtoms *slaves = dynamic_cast<ListOfSlavedAtoms *>(atomsToRemove);
    if(!slaves) throw DMException("ListOfSlavedAtoms", "RemoveSubList", "Need a list of slave atoms");
    
    elementIndices.RemoveBranch(&(slaves->elementIndices)) ;
    latticeIndicesInsideElement.RemoveBranch(&(slaves->latticeIndicesInsideElement)) ;
    ListOfAtoms::RemoveBranch(atomsToRemove);
}

void ListOfSlavedAtoms::Allocate(const Shape &newShape)
{
    ListOfAtoms::Allocate(newShape);
    if (newShape.IsLeaf())
    {
        elementIndices.SetShape(newShape);
        latticeIndicesInsideElement.SetShape(newShape);
    }
    ASSERT(elementIndices.GetShape() == newShape);
}

void ListOfSlavedAtoms::Merge(const ListOfAtoms *atoms)
{
    const ListOfSlavedAtoms *slaves = dynamic_cast<const ListOfSlavedAtoms *>(atoms);
    if(!slaves) throw DMException("ListOfSlavedAtoms","Merge",
        "attempt to merge a non ListOfSlavedAtoms");
    ListOfAtoms::Merge(atoms);
    elementIndices.Merge(&(slaves->elementIndices));
    latticeIndicesInsideElement.Merge(&(slaves->latticeIndicesInsideElement));
}

void ListOfSlavedAtoms::Attach(ListOfAtoms *atoms)
{
    ListOfSlavedAtoms *slaves = dynamic_cast<ListOfSlavedAtoms *>(atoms);
    if(!slaves) throw DMException("ListOfSlavedAtoms", "Attach", 
        "attempt to attach a non ListOfSlavedAtoms");
    ListOfAtoms::Attach(atoms);
    elementIndices.Attach(&slaves->elementIndices);
    latticeIndicesInsideElement.Attach(&slaves->latticeIndicesInsideElement);
}

void ListOfSlavedAtoms::SetElement(int i, int elem, int localCoord[DIMENSION])
{
    elementIndices[i] = elem ;
    memcpy(latticeIndicesInsideElement[i], localCoord, DIMENSION*sizeof(int));
    //    std::cout << "Size of Elements After Allocate " << elems.Size() << std::endl ;
}

void ListOfSlavedAtoms::SetConstraint(Constraint *constraint)
{
    if(!dynamic_cast<QCSlaveConstraint *> (constraint))
        throw DMException("ListOfSlavedAtoms", "SetConstraint", 
			  "ListOfAtoms expects a QCSlaveAtomsContraint");
    ListOfAtoms::SetConstraint(constraint);
}

void ListOfSlavedAtoms::DistributeData(const double* elementData,
                                       double* slaveData)
{
    for(int slaveId = 0; slaveId < elementIndices.Size(); slaveId++)
        slaveData[slaveId] = elementData[elementIndices[slaveId]];
}

