// QCListOfAtoms.cpp: implementation of the QCListOfAtoms class.
//
//////////////////////////////////////////////////////////////////////
#include "DMFirstHeader.h"
#include <math.h>
#include <fstream>
#include <string>
#include <stdlib.h>
#include "DigitalMaterial.h"
#include "QCListOfAtoms.h"
//#include "QCNeighborList.h"

#include "CompositeNeighborLocator.h"
#include "CellNeighborLocator.h"
#include "ListOfSlavedAtoms.h"
#include "MeshData.h"
#include "Constraint.h"
#include "DMLastHeader.h"

using namespace std ;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

QCListOfAtoms::QCListOfAtoms(DynamicListOfAtoms* nonLocalAtoms,
                             DynamicListOfAtoms* localAtoms,
                             const BoundaryConditions *bc,
                             NeighborLocator *neighborLocator) :
    DynamicListOfAtoms(0, 0, neighborLocator), initialized(false)
{
    Attach(nonLocalAtoms);
    Attach(localAtoms);
    ListOfAtoms *parent = new ListOfAtoms();
    parent->Attach(this);
    ListOfSlavedAtoms *slaves = new ListOfSlavedAtoms(this);
    parent->Attach(slaves);
    upperNode->SetNeighborLocator(0);
    if(bc) SetBoundaryConditions(bc);

    allAtomsNL    = 0;
    masterAtomsNL = 0;
    localAtomsNL  = 0;
    slaveAtomsNL  = 0;
}


QCListOfAtoms::~QCListOfAtoms()
{
    ListOfAtoms *parent = upperNode;
    upperNode->RemoveBranch(this); // it appears that cutting the relation with the parent is the first thing to do (do not wait for ~PrimitiveListOfAtoms)
    ASSERT(!upperNode);
    delete parent;

    if(allAtomsNL)    delete allAtomsNL;
    if(masterAtomsNL) delete masterAtomsNL;
    if(localAtomsNL)  delete localAtomsNL;
    if(slaveAtomsNL)  delete slaveAtomsNL;
}

void QCListOfAtoms::Save(DMWriter *writer) const
{
    DynamicListOfAtoms::Save(writer);
    //GetSlaves()->Save(writer);
}

void QCListOfAtoms::Load(DMReader *reader)
{
    DynamicListOfAtoms::Load(reader);
    //GetSlaves()->Load(reader);
}

ListOfAtoms* QCListOfAtoms::GetAllAtoms() const
{
    return upperNode;
}

ListOfSlavedAtoms *QCListOfAtoms::GetSlaves() const
{
    ASSERT(0 != upperNode);
    ASSERT(0 != dynamic_cast<ListOfSlavedAtoms *>(upperNode->GetBranch(1)));
    return static_cast<ListOfSlavedAtoms *> (upperNode->GetBranch(1));
}

DynamicListOfAtoms *QCListOfAtoms::GetNonLocals() const
{
    ASSERT(0 != dynamic_cast<DynamicListOfAtoms *>((*branches)[0]));
    return static_cast<DynamicListOfAtoms *> ((*branches)[0]);
}

DynamicListOfAtoms *QCListOfAtoms::GetLocals() const
{
    ASSERT(0 != dynamic_cast<DynamicListOfAtoms *>((*branches)[1]));
    return static_cast<DynamicListOfAtoms *> ((*branches)[1]);
}

void QCListOfAtoms::SetBoundaryConditions(const BoundaryConditions *bc)
{
    ListOfAtoms::SetBoundaryConditions(bc);
    if(upperNode->GetBoundaryConditions() != bc) // this is to prevent an infinite loop
    {
        upperNode->SetBoundaryConditions(bc);   // this will set bc for the slaves.
    }
}


void QCListOfAtoms::SetNeighborLocator(NeighborLocator *neighborLocator)
{
    GetNonLocals()->SetNeighborLocator(neighborLocator);

    if(localAtomsNL) delete localAtomsNL;
    localAtomsNL = new CellNeighborLocator(0., neighborLocator->GetCutoffDistance());
    GetLocals()->SetNeighborLocator(localAtomsNL);    

    if(masterAtomsNL) delete masterAtomsNL;
    masterAtomsNL = new CompositeNeighborList(0., this);
    ListOfAtoms::SetNeighborLocator(masterAtomsNL);
    
    if(slaveAtomsNL) delete slaveAtomsNL;
    slaveAtomsNL = new CellNeighborLocator(0., neighborLocator->GetCutoffDistance());
    GetSlaves()->SetNeighborLocator(slaveAtomsNL);

    if(allAtomsNL) delete allAtomsNL;
    allAtomsNL = new CompositeNeighborList(neighborLocator->GetCutoffDistance(), GetAllAtoms());
    GetAllAtoms()->SetNeighborLocator(allAtomsNL);
//    NeighborLocator* nl = new QCNeighborList(neighborLocator, this);
//    ListOfAtoms::SetNeighborLocator(nl);
//    GetAllAtoms()->SetNeighborLocator(nl);                                                      
    
//     const double rCut = neighborLocator->GetCutoffDistance();
//     //temporary solution:
//     GetSlaves()->SetNeighborLocator(new CellNeighborList(rCut, 0.05 * rCut));
//    if(upperNode->GetNeighborLocator() != 0)
//         delete upperNode->GetNeighborLocator();
//     upperNode->SetNeighborLocator(new CompositeNeighborList(rCut, upperNode));
}

void QCListOfAtoms::SetPos(double* data)
{
    DMArray<double, DIMENSION> newPositions(positions.GetShape());
    memcpy(newPositions[0],
           data,
           GetNonLocals()->GetNumber() * DIMENSION * sizeof(double));
    memcpy(newPositions[GetNonLocals()->GetNumber()],
           data + GetNonLocals()->GetNumber() * DIMENSION,
           GetLocals()->GetNumber() * DIMENSION * sizeof(double));
    SetCartesianPositions(&newPositions);
}

void QCListOfAtoms::AdjustPositions() 
{
    ListOfAtoms::AdjustPositions(); 
    GetSlaves()->AdjustPositions();
}
