#include "DMFirstHeader.h"
#include <vector>
#include "DigitalMaterial.h"

#include "NeighborLocator.h"
#include "MeshData.h"
#include "QCListOfAtoms.h"

class QCNeighborList : public NeighborLocator
{
public:
    QCNeighborList(NeighborLocator* nl0, MeshData* mesh,
                   QCListOfAtoms* masters,
                   std::vector< std::vector<int> >& slaveNeighbors0) :
        NeighborLocator(0.0), nl(nl0), meshData(mesh),
        mybc(masters->GetBoundaryConditions()),
        masterPositions(*(masters->GetNonLocals()->GetCartesianPositions())),
        slavePositions(*(masters->GetSlaves()->GetCartesianPositions())),
        nMasters(masters->GetShape().GetSize())   // ???
        {
            slaveNeighbors.swap(slaveNeighbors0);
        }
//    ~NeighborLocator();
    bool HalfNeighbors(int atomNumber,
                       int * neighbors, 
                       int& numNeighbors,
                       double (*displacements)[DIMENSION],
                       double* displacementsSquared,
                       int& spacesLeftInArrays)
        {
            bool ok = nl->
                HalfNeighbors(atomNumber, neighbors, numNeighbors,
                              displacements, displacementsSquared,
                              spacesLeftInArrays);
            if (!ok)
                return false;
//            const vector<int>& slaves = meshData->GetNeighbors(atomNumber);
            const std::vector<int>& slaves = slaveNeighbors[atomNumber];
            int numSlaves = slaves.size();
            if (numSlaves == 0)
                return true;
            if (spacesLeftInArrays < numSlaves)
            {
                spacesLeftInArrays += numNeighbors; // ???
                numNeighbors = 0; // ???
                return false;
            }
            spacesLeftInArrays -= numSlaves;
            double* masterPos = masterPositions[atomNumber];
            int s;
            for (s = 0; s < numSlaves; s++)
            {
                neighbors[numNeighbors + s] = slaves[s] + nMasters;
                for (int coord = 0; coord < DIMENSION; coord++)
                    displacements[numNeighbors + s][coord] =
                        slavePositions[slaves[s]][coord] - masterPos[coord];
            }
            mybc->DifferenceBoundaryConditions(displacements + numNeighbors,
                                             numSlaves);
            for (s = 0; s < numSlaves; s++)
            {
                double sq = 0.0;
                for (int coord = 0; coord < DIMENSION; coord++)
                    sq += displacements[numNeighbors][coord] *
                        displacements[numNeighbors][coord];
                displacementsSquared[numNeighbors] = sq;
                numNeighbors++;
            }
            return true;
        }
    bool Neighbors(int atomNumber,
                       int * neighbors, 
                       int& numNeighbors,
                       double (*displacements)[DIMENSION],
                       double* displacementsSquared,
                       int& spacesLeftInArrays)
        {assert(false); return 0;}
    bool Neighbors(const double *center,
                   int* neighbors, 
                   int& numNeighbors,
                   double (*displacements)[DIMENSION],
                   double* displacementsSquared,
                   int& spacesLeftInArrays, double cutoff=-1.)
        {assert(false);return 0;}
    void UpdatePosition() {nl->UpdatePosition();}
    NeighborLocator* Copy() const  {assert(false);return 0;}
private:
    NeighborLocator* nl;
    MeshData* meshData;
    const BoundaryConditions* mybc;
    const DMArray<double, DIMENSION>& masterPositions;
    const DMArray<double, DIMENSION>& slavePositions;
    const int nMasters;
    std::vector< std::vector<int> > slaveNeighbors;
};
