// QCSlaveConstraint.h: interface for the QCSlaveConstraint class.
//
//////////////////////////////////////////////////////////////////////

#ifndef QCSLAVECONSTRAINT_H
#define QCSLAVECONSTRAINT_H

#include "DigitalMaterial.h"
#include "Constraint.h"
#include "ListOfSlavedAtoms.h"
#include "QCListOfAtoms.h"

//! We force slaves to follow the motion of their master using a \a QCSlaveConstraint
/** In Molecular Dynamics, constraints in general are attached to ListsOfAtoms. Their
basic interface contains \a AdjustPositions, \a AdjustVelocities and \a AdjustForces 
(see \a Constraint). These functions are called when we move atoms via ListOfAtoms's 
\a IncrementCartesianPositions() or \a SetCartesianPositions(). For quasicontinuum, slaves 
belong to a CompositeListOfAtoms (\a atoms) made of masters and slaves. Each time we move 
\a atoms slaves's \a AdjustPositions() is called and their positions are set according to
the shape functions and the masters positions.
*/
class QCSlaveConstraint : public Constraint  
{
public:
    //! The constraint has to know the slaves and the masters, which isn't usual for constraint. (-> we need one constraint per list of slave atoms)
    QCSlaveConstraint(ListOfSlavedAtoms *slaves, const QCListOfAtoms *masters);
    virtual ~QCSlaveConstraint();
    //! Set the slaves' positions according to the masters's.
    void AdjustPositions(DMArray<double,DIMENSION> *positions);
    void AdjustForces(DMArray<double, DIMENSION> *forces) {/* precondition? */}
    
private:
    const QCListOfAtoms *masters;
    ListOfSlavedAtoms *slaves;
};

#endif // !defined QCSLAVECONSTRAINT_H
