%module QC

%include MD.i

%{
#include "MeshData.h"
#include "QCListOfAtoms.h"
#include "ListOfSlavedAtoms.h"
#include "QCEMTPotential.h"
#include "BoundaryConditions.h"
#include "Element.h"

%}


%include Element.h
%addmethods Element
{
public:
    Element(int nodes[DIMENSION+1], int nNonLocals, double nodesLocalCoord[DIMENSION][DIMENSION])
	{
		TinyVector<int> tmp[DIMENSION];
		for(int node=0; node<DIMENSION; ++node)
		   for(int i=0; i<DIMENSION; ++i) tmp[node][i] = nodesLocalCoord[node][i];
		return new Element(nodes, nNonLocals, tmp);
	};
	PyObject *CalculateLattice(ListOfAtoms *atoms)
	{ // return the lattice as a NumPyArray.
		int dims[2] = {DIMENSION,DIMENSION};
		PyObject *o = PyArray_FromDims(2, dims, 'd');
		double tmp[DIMENSION][DIMENSION];
        self->CalculateLattice(atoms, tmp);
		memcpy(((PyArrayObject *)o)->data, tmp, DIMENSION*DIMENSION*sizeof(double));
		return o;
	};
}

%include "MeshData.h"

%include "ListOfSlavedAtoms.h"

%include "QCListOfAtoms.h"

%include "../../MDFiles/Constraints/Constraints.i"

%addmethods MeshData
{
 public:
    void CreateElement(PyObject* tetrahedron, 
               const QCListOfAtoms* masters,
               const BravaisLatticeWithBasis* lattice)
    {
        PyArrayObject* o = (PyArrayObject*)tetrahedron;
        assert(o->nd == 1);
        int dimension = o->dimensions[0] - 1;
        assert(dimension == 3 || dimension == 2);
        int* p = (int*)o->data;
        std::vector<int> t(p, p + dimension + 1);
        self->CreateElement(t, masters, lattice);
    }
	PyObject *GetConnectivities()
	{
		int dims[2];
		dims[0] = self->GetNumberOfElements();
		dims[1] = DIMENSION+1;
		PyObject *o = PyArray_FromDims(2, dims, 'i');
		int (*data)[DIMENSION+1] = (int (*)[DIMENSION+1])(((PyArrayObject *)o)->data);
		for(int i = 0; i<dims[0]; ++i)
			for(int coord=0; coord<DIMENSION+1; ++coord) 
				data[i][coord] = self->GetElement(i).GetNode(coord);
		return o;
	}
//  const ArrayDoubleVector *GetNodePositions() 
//  {
//      return self->GetMasters()->GetCartesianPositions();
//  }
/*  void Cut(PyObject* P0, PyObject* n1, PyObject* n2,
    int N1, int N2,
    double L,
    PyObject* atomicData, PyObject* elementData)
    {
	  self->MeshPlot((double*)((PyArrayObject*)P0)->data,
	    (double*)((PyArrayObject*)n1)->data,
		(double*)((PyArrayObject*)n2)->data,
		N1,	N2, L,
		(double*)((PyArrayObject*)atomicData)->data,
	    (double*)((PyArrayObject*)elementData)->data);
    }*/
}

%addmethods QCListOfAtoms
{
public:
    void SetGeneralizedPositions(PyObject* pos)
    {
        self->SetPos((double*)((PyArrayObject*)pos)->data);
    }
/*    PyObject* GetGeneralizedPositions()
    {
        int dims = 3 * self->GetNumberOfMasters();
        double* data = (*self->GetCartesianPositions())[0];
        PyObject *o = PyArray_FromDims(1, &dims, 'd');
        memcpy(((PyArrayObject*)o)->data, data, dims * sizeof(double));
        return o;
    }
    PyObject* GetGeneralizedForces()
    {
        DMArray<double, DIMENSION> forces(self->GetNumber());
        self->GetPotential()->CalculateForces(self, &forces);   
        self->AdjustForces(&forces);
        int dims = 3 * self->GetNumberOfMasters();
        double* data = forces[0];
        PyObject *o = PyArray_FromDims(1, &dims, 'd');
        memcpy(((PyArrayObject*)o)->data, forces[0], dims * sizeof(double));
        return o;
    }*/
//     void DecorateWithSlaves(PyObject* pySurface, int nAtomic,
//              double rc, BravaisLatticeWithBasis* lattice,
//              double L)
//     {
//  if(PyArray_Check(pySurface))
//  {
//      PyArrayObject *o = (PyArrayObject *)pySurface;
//      int nIndices = o->dimensions[0];
//      if (o->nd != 1)
//      std::cerr << " != DIMENSION !!!" << std::endl;
//      if(true)//o->descr->type_num == PyArray_INT)
//      {
//      int* data = (int*)o->data;
//      set<int> surface;
//      for(int i = 0; i < nIndices;i++)
//          surface.insert(data[i]);
//      self->DecorateWithSlaves(surface, nAtomic, rc, lattice, L);
//      return;
//      }
//      else
//      {
//      PyErr_SetString(PyExc_TypeError,"need a double NumPy Array");
//      return;
//      }
//  }
//  else
//  {
//      PyErr_SetString(PyExc_TypeError,"not a NumPy Array");
//      return;
//  }
//     }
%pragma(python) addtoclass="
    def __init__(self,*args):
        cmd = 'self.this = apply(' + __name__ + 'c.new_QCListOfAtoms, args)'
        exec cmd
        self.thisown = 0
        args[0].thisown = 0        # nonLocalAtoms
        args[0].attachedTo = self  # nonLocalAtoms increases ref count of QCListOfAtoms
        args[1].thisown = 0        # localAtoms
        args[1].attachedTo = self  # localAtoms increases ref count of QCListOfAtoms
"
}

%{
#include "QCPotential.h"
%}

class QCPotential: public Potential
{
public:
	QCPotential(Potential *potential, BravaisLatticeWithBasis *lattice);
    void SetMeshData(const MeshData *meshData);
}

%include "QCEMTPotential.h"
/*
%addmethods QCEMTPotential
{
 public:
    PyObject *CalculateElementStress(ListOfAtoms *atoms, int e)
    {
        int dims = 6;
        double *stress = new double[6];
        self->CalculateElementStressTensor(atoms, e, stress);
        PyObject *o = PyArray_FromDimsAndData(1, &dims, 'd', (char *)stress);
        return o;
    }
	void CheckForces(QCListOfAtoms *atoms, double epsilon, bool verbose = false)
	{
  		DMArray<double, DIMENSION> *forces = new DMArray<double, DIMENSION>(atoms->GetShape());
  		self->CalculateForces(atoms, forces);
		double max = 0;
  		for(int at=0; at < atoms->GetNumber(); ++at) 
			for(int coord=0; coord<DIMENSION;++coord)
    		{
     			double e1, e2;
				atoms->IncrementCartesianPosition(at, coord, epsilon);
				e1 = self->CalculateEnergy(atoms);
				atoms->IncrementCartesianPosition(at, coord, -2.*epsilon);
				e2 = self->CalculateEnergy(atoms);
				atoms->IncrementCartesianPosition(at, coord, epsilon);
				double f1 = 0.5*(e2-e1)/epsilon, f2 = (*forces)[at][coord], diff = f1-f2;
				if(verbose) std::cout << at << "," << coord << "\t" << f1 << "\t" << f2 << "\t" << diff << std::endl;
				if(fabs(diff) > max) max = fabs(diff);
			}
		delete forces;
		std::cout << "Max diff = " << max << std::endl;
	}
}
*/
#<<<<<<< QC.i
##    if(PyArray_Check(pySurface))
#    {
#        PyArrayObject *o = (PyArrayObject *)pySurface;
#        int nIndices = o->dimensions[0];
##        if (o->nd != 1)
#        std::cerr << " != DIMENSION !!!" << std::endl;
##        if(true)//o->descr->type_num == PyArray_INT)
#        {
#        int* data = (int*)o->data;
#        std::set<int> surface;
#        for(int i = 0; i < nIndices;i++)
#            surface.insert(data[i]);
#        self->DecorateWithSlaves(surface, nAtomic, rc, lattice);
#        return;
#        }
##        else
#        {
#        PyErr_SetString(PyExc_TypeError,"need a double NumPy Array");
#        return;
#        }
#    }
##    else
#    {
#        PyErr_SetString(PyExc_TypeError,"not a NumPy Array");
#        return;
#    }
#=======*/

